#!/bin/sh
#
# file tst02.sh 
# (test dg87d functions)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

################################################## TARGET 
echo "Test Target"
mem1=`dgd memory`
# add a target 
dgd ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# verify target count = 1
n=`dgd tc`
if [ $n != 1 ] ; then echo "Target count isn't 1"; exit 1; fi
# change target description
dgd tsd tg1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change description of Target"; exit 1; fi
# verify description change
if [ x`dgd ti tg1 | grep Description | awk '{print $2}'` != xNewDescr ] ; then
	echo "Description change of Target failed"; exit 1;
fi
# change target ipport 
dgd tsp tg1 6666 >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change ipport of Target"; exit 1; fi
# verify ipport change
if [ x`dgd ti tg1 | grep "IP Port" | awk '{print $3}'` != x6666 ] ; then
	echo "IPPort change of Target failed"; exit 1;
fi

# try to add a second target with same name
dgd ta tg1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two target with same name"; exit 1; fi
# delete target 
dgd tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the target"; exit 1; fi
# verify target count = 0
n=`dgd tc`
if [ $n != 0 ] ; then echo "Target count isn't 0"; exit 1; fi
# try to delete a target which doesn't exist
dgd tk tg1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete the target which doesn't exist"; exit 1; fi

mem2=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error before and after target $mem1 -> $mem2"
	exit 1
fi

################################################## QUEUE 
echo "Test Queue"
# add a target 
dgd ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# add a queue 
dgd qa tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a queue"; exit 1; fi
# change queue description
dgd qsd tg1 q1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change description of Queue"; exit 1; fi
# verify description change
if [ x`dgd qi tg1 q1 | grep Description |awk '{print $2}'` != xNewDescr ] ; then
	echo "Description change of Queue failed"; exit 1;
fi
# change queue max 
dgd qsm tg1 q1 55 >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change max property of Queue"; exit 1; fi
# verify max property change
if [ x`dgd qi tg1 q1 | grep Max|awk '{print $2}'` != x55 ] ; then
	echo "Max property change of Queue failed"; exit 1;
fi
# verify queue count = 1
n=`dgd qc tg1`
if [ $n != 1 ] ; then echo "Queue count isn't 1"; exit 1; fi
# try to add a second queue with same name
dgd qa tg1 q1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two queue with same name"; exit 1; fi
# delete queue 
dgd qk tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the queue"; exit 1; fi
# verify queue count = 0
n=`dgd qc tg1`
if [ $n != 0 ] ; then echo "Queue count isn't 0"; exit 1; fi
# try to delete a queue which doesn't exist
dgd qk tg1 q1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete a queue which doesn't exist"; exit 1; fi
# delete the target
dgd tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a target"; exit 1; fi

mem2=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error before and after queue $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a target with queues
mem1=`dgd memory`
dgd ta tg1 > /dev/null 2>&1
dgd qa tg1 q1 > /dev/null 2>&1
dgd qa tg1 q2 > /dev/null 2>&1
dgd qa tg1 q3 > /dev/null 2>&1
dgd tk tg1 > /dev/null 2>&1
mem2=`dgd memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error delete target with queues $mem1 -> $mem2"
	exit 1
fi


################################################## REF
echo "Test Ref"
# add a ref
dgd ra r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a ref"; exit 1; fi
# verify ref count = 1
n=`dgd rc`
if [ $n != 1 ] ; then echo "Ref count isn't 1"; exit 1; fi
# try to add a second ref with same name
dgd ra r1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two ref with same name"; exit 1; fi
# delete ref
dgd rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the ref"; exit 1; fi
# verify ref count = 0
n=`dgd rc`
if [ $n != 0 ] ; then echo "Ref count isn't 0"; exit 1; fi
# try to delete a ref which doesn't exist
dgd rk r1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete the ref which doesn't exist"; exit 1; fi
# verify memory
mem2=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error before and after ref $mem1 -> $mem2"
	exit 1
fi

################################################## TASK 
echo "Test Task"
# add a ref 
dgd ra r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a ref"; exit 1; fi
# add a task
dgd rta r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a task"; exit 1; fi
# verify ref count = 1
n=`dgd rtc r1`
if [ $n != 1 ] ; then echo "task count isn't 1"; exit 1; fi
# verify day name
if [ "`dgd rti -S! r1 t1 | grep \"Ref Name\"`" != "Ref Name!r1" ] ; then
	echo "Task ref is not set." ; exit 1
fi
# change task description
dgd rtsd r1 t1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change description of Task"; exit 1; fi
# verify description change
if [ x`dgd rti r1 t1 | grep Description |awk '{print $2}'` != xNewDescr ] ; then
	echo "Description change of Task failed"; exit 1;
fi

# verify that waitOldestTask is true
if [ x`dgd rti r1 t1 | grep Oldest |awk '{print $4}'` != xtrue ] ; then
	echo "waitOldestTask property of task isn't true"; exit 1;
fi
# change waitOldestTask property
dgd rtswot r1 t1 false
if [ $? != 0 ] ; then echo "Can't change waitOldestTask property"; exit 1; fi
# verify that property is changed
if [ x`dgd rti r1 t1 | grep Oldest |awk '{print $4}'` != xfalse ] ; then
	echo "waitOldestTask property of task change failed"; exit 1;
fi

# verify that waitAllDependences is true
if [ x`dgd rti r1 t1 | grep "All Dep" |awk '{print $4}'` != xtrue ] ; then
	echo "waitAllDependences property of task isn't true"; exit 1;
fi
# change waitAllDependences property
dgd rtswad r1 t1 false
if [ $? != 0 ] ; then echo "Can't change waitAllDependences property";exit 1;fi
# verify that property is changed
if [ x`dgd rti r1 t1 | grep "All Dep" |awk '{print $4}'` != xfalse ] ; then
	echo "waitAllDependences property of task change failed"; exit 1;
fi

# verify that onErrorContinue is true
if [ x`dgd rti r1 t1 | grep "On Error" |awk '{print $4}'` != xtrue ] ; then
	echo "onErrorContinue property of task isn't true"; exit 1;
fi
# change OnErrorContinue property
dgd rtsoec r1 t1 false
if [ $? != 0 ] ; then echo "Can't change onErrorContinue property";exit 1;fi
# verify that property is changed
if [ x`dgd rti r1 t1 | grep "On Error" |awk '{print $4}'` != xfalse ] ; then
	echo "OnErrorContinue property of task change failed"; exit 1;
fi

# try to add a second task with same name
dgd rta r1 t1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two tasks with same name"; exit 1; fi
# delete task 
dgd rtk r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the task"; exit 1; fi
# verify task count = 0
n=`dgd rtc r1`
if [ $n != 0 ] ; then echo "Task count isn't 0"; exit 1; fi
# try to delete a task which doesn't exist
dgd rtk r1 t1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete a task which doesn't exist"; exit 1; fi
# delete the ref
dgd rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error before and after task $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a ref with tasks 
mem1=`dgd memory`
dgd ra r1 > /dev/null 2>&1
dgd rta r1 t1 > /dev/null 2>&1
dgd rta r1 t2 > /dev/null 2>&1
dgd rta r1 t3 > /dev/null 2>&1
dgd rk r1 > /dev/null 2>&1
mem2=`dgd memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error deleting ref with tasks $mem1 -> $mem2"
	exit 1
fi

################################################## JOB
echo "Test Job"
# add a ref 
dgd ra r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a ref"; exit 1; fi
# add a task
dgd rta r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a task"; exit 1; fi
# add a job 
dgd rja r1 t1 j1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a job"; exit 1; fi
# verify ref count = 1
n=`dgd rjc r1 t1`
if [ $n != 1 ] ; then echo "job count isn't 1"; exit 1; fi
# verify task 
if [ "`dgd rji -S! r1 t1 j1 | grep \"Task Name\"`" != "Task Name!t1" ] ; then
	echo "Job task is not set." ; exit 1
fi
# verify ref 
if [ "`dgd rji -S! r1 t1 j1 | grep \"Ref Name\"`" != "Ref Name!r1" ] ; then
	echo "Job ref is not set." ; exit 1
fi
# change job description
dgd rjsd r1 t1 j1 NewDescr >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change description of job"; exit 1; fi
# verify description change
if [ x`dgd rji r1 t1 j1 |grep Description |awk '{print $2}'` != xNewDescr ]; 
then echo "Description change of job failed"; exit 1; fi
# change job path 
dgd rjsp r1 t1 j1 ls >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change path of job"; exit 1; fi
# verify description change
if [ x`dgd rji r1 t1 j1 |grep Path |awk '{print $2}'` != xls ]; 
then echo "Path change of job failed"; exit 1; fi
# change job argument 
dgd rjsa r1 t1 j1 fic >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change argument of job"; exit 1; fi
# verify description change
if [ x`dgd rji r1 t1 j1 |grep Argument |awk '{print $2}'` != xfic ]; 
then echo "Argument change of job failed"; exit 1; fi
# change job tempo 
dgd rjst r1 t1 j1 66 67 >/dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't change tempo of job"; exit 1; fi
# verify tempo change
if [ x`dgd rji r1 t1 j1 |grep "Start Tempo" |awk '{print $3}'` != x66 ]; 
then echo "Start tempo change of job failed"; exit 1; fi
if [ x`dgd rji r1 t1 j1 |grep "End Tempo" |awk '{print $3}'` != x67 ]; 
then echo "End tempo change of job failed"; exit 1; fi

# verify that waitAllDependences is true
if [ x`dgd rji r1 t1 j1 | grep "All Dep" |awk '{print $4}'` != xtrue ] ; then
	echo "waitAllDependences property of job isn't true"; exit 1;
fi
# change waitAllDependences property
dgd rjswad r1 t1 j1 false
if [ $? != 0 ] ; then echo "Can't change waitAllDependences property";exit 1;fi
# verify that property is changed
if [ x`dgd rji r1 t1 j1 | grep "All Dep" |awk '{print $4}'` != xfalse ] ; then
	echo "waitAllDependences property of job change failed"; exit 1;
fi

# verify that onErrorContinue is true
if [ x`dgd rji r1 t1 j1 | grep "On Error" |awk '{print $4}'` != xtrue ] ; then
	echo "OnErrorContinue property of job isn't true"; exit 1;
fi
# change OnErrorContinue property
dgd rjsoec r1 t1 j1 false
if [ $? != 0 ] ; then echo "Can't change OnErrorContinue property";exit 1;fi
# verify that property is changed
if [ x`dgd rji r1 t1 j1 | grep "On Error" |awk '{print $4}'` != xfalse ] ; then
	echo "OnErrorContinue property of job change failed"; exit 1;
fi

# try to add a second job with same name
dgd rja r1 t1 j1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two jobs with same name"; exit 1; fi
# delete job 
dgd rjk r1 t1 j1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the job"; exit 1; fi
# verify task count = 0
n=`dgd rjc r1 t1`
if [ $n != 0 ] ; then echo "Job count isn't 0"; exit 1; fi
# try to delete a job which doesn't exist
dgd rjk r1 t1 j1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete a job which doesn't exist"; exit 1; fi
# delete the task 
dgd rtk r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a task"; exit 1; fi
# delete the ref
dgd rk r1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a ref"; exit 1; fi
# verify memory
mem2=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error before and after job $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a ref with jobs
mem1=`dgd memory`
dgd ra r1 > /dev/null 2>&1
dgd rta r1 t1 > /dev/null 2>&1
dgd rja r1 t1 j1 > /dev/null 2>&1
dgd rja r1 t1 j2 > /dev/null 2>&1
dgd rja r1 t1 j3 > /dev/null 2>&1
dgd rk r1 > /dev/null 2>&1
mem2=`dgd memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error deleting ref with jobs $mem1 -> $mem2"
	exit 1
fi

################################################## TASK SET QUEUE 
echo "Test Task-Queue"
mem1=`dgd memory`
dgd ra r1 > /dev/null 2>&1
dgd rta r1 t1 > /dev/null 2>&1
dgd ta tg1 > /dev/null 2>&1
dgd qa tg1 q1 > /dev/null 2>&1
# set queue for task 
dgd rtsq r1 t1 tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't set queue"; exit 1; fi
# verify that queue is set
if [ x`dgd rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xq1 ] ; then
	echo "Queue is not set"
	exit 1
fi
# verify that target is set
if [ x`dgd rti r1 t1 | sed -ne "s/Target:.\(.*\)/\1/p"` != xtg1 ] ; then
	echo "Target is not set"
	exit 1
fi
# verify that task is in task list of target
if [ x`dgd qrtc tg1 q1` != x1 ] ; then
	echo "Task list count is not 1."
	exit 1
fi

if [ x`dgd qrtl tg1 q1 | awk '{print $2}'` != xt1 ] ; then
	echo "Task is not in task list of target."
	exit 1
fi
########## unset task
# set queue for task to none
dgd rtsq r1 t1 none none > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't set queue to none"; exit 1; fi
# verify that queue is unset
if [ x`dgd rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xnone ] ; then
	echo "Queue is not unset"
	exit 1
fi
# verify that target is unset
if [ x`dgd rti r1 t1 | sed -ne "s/Target:.\(.*\)/\1/p"` != xnone ] ; then
	echo "Target is not unset"
	exit 1
fi
# verify that task is not in task list of target
if [ x`dgd qrtc tg1 q1` != x0 ] ; then
	echo "Task list count is not 0."
	exit 1
fi
######## delete task
# set queue for task 
dgd rtsq r1 t1 tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't set queue"; exit 1; fi
# verify that task is in task list of target
if [ x`dgd qrtc tg1 q1` != x1 ] ; then
	echo "Task list count is not 1."
	exit 1
fi
dgd rtk r1 t1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete task"; exit 1; fi
# verify that task is not in task list of target
if [ x`dgd qrtc tg1 q1` != x0 ] ; then
	echo "Task list count for queue is not 0."
	exit 1
fi
######## delete queue 
dgd rta r1 t1 > /dev/null 2>&1
dgd rtsq r1 t1 tg1 q1 > /dev/null 2>&1
# verify that queue is set
if [ x`dgd rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xq1 ] ; then
	echo "Queue is not set"
	exit 1
fi
# delete queue
dgd qk tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete queue"; exit 1; fi
# verify that queue is set to none for task
if [ x`dgd rti r1 t1 | sed -ne "s/Queue:.\(.*\)/\1/p"` != xnone ] ; then
	echo "Queue is not unset"
	exit 1
fi
# cleanup
dgd rk r1 > /dev/null 2>&1
dgd tk tg1 > /dev/null 2>&1
mem2=`dgd memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## TASK DEPENDENCES 
echo "Test TaskDep" 
mem1=`dgd memory`
dgd ra r1 > /dev/null 2>&1
dgd rta r1 t1 > /dev/null 2>&1
dgd rta r1 t2 > /dev/null 2>&1
dgd rta r1 t3 > /dev/null 2>&1
dgd rta r1 t4 > /dev/null 2>&1
dgd rta r1 t5 > /dev/null 2>&1

# try to add a dep with a wrong type
dgd rtda r1 t1 t2 zzz > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a taskDep with wrong type"; exit 1; fi
# try to add a dep with an unknown ref 
dgd rtda rzzz1 t1 t2> /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a taskDep with unknown ref"; exit 1; fi
# try to add a dep with an unknown task 
dgd rtda r1 tzzz1 t2> /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a taskDep with unknown task"; exit 1; fi

# add a dep with different type
dgd rtda r1 t1 t2 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep"; exit 1; fi
dgd rtda r1 t2 t3 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep with type always"; exit 1; fi
dgd rtda r1 t3 t4 onerror  > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep with type onerror"; exit 1; fi
dgd rtda r1 t3 t5 onsuccess > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a taskDep with type onsucces"; exit 1; fi

# verify dep count for ref 
if [ x`dgd rtdc r1` != x4 ] ; then
	echo "TaskDep count is not 4."
	exit 1
fi
# verify taskdep list command
if [ x`dgd rtdl r1 | wc -l | awk '{print $1}'` != x4 ] ; then
	echo "Incorrect taskdep list."
	exit 1
fi

# delete a task dep
mem3=`dgd memory`
dgd rtda r1 t1 t5 always > /dev/null 2>&1
dgd rtdk r1 t1 t5 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a taskdep"; exit 1; fi
if [ x`dgd rtdl r1 | wc -l | awk '{print $1}'` != x4 ] ; then
	echo "Incorrect taskdep count."
	exit 1
fi
mem4=`dgd memory`
if [ $mem3 != $mem4 ] ; then
	echo "dg87d : memory error after taskdep delete $mem1 -> $mem2"
	exit 1
fi

# delete a task 
dgd rta r1 t6 > /dev/null 2>&1
dgd rtda r1 t1 t6 > /dev/null 2>&1
# try to delete a task witch has dependences
dgd rtk r1 t6 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete a task with taskdep 1"; exit 1; fi
dgd rtdk r1 t1 t6 always > /dev/null 2>&1
dgd rtk r6 t1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete a task with taskdep 2"; exit 1; fi
dgd rk r1 > /dev/null 2>&1
mem2=`dgd memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## JOB DEPENDENCES 
echo "Test JobDep" 
mem1=`dgd memory`

dgd ra r1 > /dev/null 2>&1
dgd rta r1 t1 > /dev/null 2>&1
dgd rja r1 t1 j1 > /dev/null 2>&1
dgd rja r1 t1 j2 > /dev/null 2>&1
dgd rja r1 t1 j3 > /dev/null 2>&1
dgd rja r1 t1 j4 > /dev/null 2>&1
dgd rja r1 t1 j5 > /dev/null 2>&1

# try to add a dep with a wrong type
dgd rjda r1 t1 j1 j2 zzz > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a jobDep with wrong type"; exit 1; fi

# try to add a dep with an unknown ref 
dgd rjda rzzz1 t1 j1 j2> /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a jobDep with unknown ref"; exit 1; fi
# try to add a dep with an unknown task 
dgd rjda r1 tzzz1 j1 j2> /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a jobDep with unknown task"; exit 1; fi
# try to add a dep with an unknown job 
dgd rjda r1 t1 jzzz1 j2> /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a jobDep with unknown job"; exit 1; fi
# try to add a dep with an unknown type 
dgd rjda r1 t1 j1 j2 zzz > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add a jobDep with unknown type"; exit 1; fi

# add a dep with different type
dgd rjda r1 t1 j1 j2 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep"; exit 1; fi
dgd rjda r1 t1 j2 j3 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep with type always"; exit 1; fi
dgd rjda r1 t1 j3 j4 onerror  > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep with type onerror"; exit 1; fi
dgd rjda r1 t1 j3 j5 onsuccess > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a jobDep with type onsucces"; exit 1; fi

# verify dep count for task 
if [ x`dgd rjdc r1 t1` != x4 ] ; then
	echo "JobDep count is not 4."
	exit 1
fi
# verify jobdep list command
if [ x`dgd rjdl r1 t1 | wc -l | awk '{print $1}'` != x4 ] ; then
	echo "Incorrect jobdep list."
	exit 1
fi

# delete a job dep
mem3=`dgd memory`
dgd rjda r1 t1 j1 j5 always > /dev/null 2>&1
# verify dep count for task 
if [ x`dgd rjdc r1 t1` != x5 ] ; then
	echo "JobDep count is incorrect before delete."
	exit 1
fi
dgd rjdk r1 t1 j1 j5 always > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a jobdep"; exit 1; fi
# verify dep count for task 
if [ x`dgd rjdc r1 t1` != x4 ] ; then
	echo "JobDep count is incorrect after delete."
	exit 1
fi

mem4=`dgd memory`
if [ $mem3 != $mem4 ] ; then
	echo "dg87d : memory error after jobdep delete $mem1 -> $mem2"
	exit 1
fi

# delete a job 
dgd rja r1 t1 j6 > /dev/null 2>&1
dgd rjda r1 t1 j1 j6 > /dev/null 2>&1
dgd rjda r1 t1 j6 j2 > /dev/null 2>&1
# verify dep count for task 
if [ x`dgd rjdc r1 t1` != x6 ] ; then
	echo "JobDep count is incorrect after delete."
	exit 1
fi
# delete a job witch has dependences
dgd rjk r1 t1 j6 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can' delete a job with jobdep 1"; exit 1; fi
# verify dep count for task 
if [ x`dgd rjdc r1 t1` != x4 ] ; then
	echo "JobDep count is incorrect after delete."
	exit 1
fi
# verify job count
if [ x`dgd rjc r1 t1` != x5 ] ; then
	echo "Job count is incorrect after delete."
	exit 1
fi

# delete ref
dgd rk r1 > /dev/null 2>&1
mem2=`dgd memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi


################################################## CONDITIONS
echo "Test Conditions" 
mem1=`dgd memory`

dgd ra r1 > /dev/null 2>&1
dgd rta r1 t1 > /dev/null 2>&1

# add a condition with an unknown ref
dgd rtca rzzz1 t1 manual "Ready?" > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can't add a condition with unknown ref"; exit 1; fi
# add a condition with an unknown task 
dgd rtca r1 tzzz1 manual "Ready?" > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can't add a condition with unknown task"; exit 1; fi
# add a condition with an unknown type 
dgd rtca r1 t1 zzz "Ready?" > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can't add a condition with unknown type"; exit 1; fi
# add a condition without data argument
dgd rtca r1 t1 manual > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can't add a condition with without data"; exit 1; fi

# add a manual condition 
dgd rtca r1 t1 manual "Ready?" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a manual condition"; exit 1; fi
# add a file condition 
dgd rtca r1 t1 file "/tmp/fictop" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a file condition"; exit 1; fi
# add a hmin condition 
dgd rtca r1 t1 hmin "20:30" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmin condition"; exit 1; fi
# add a hmax condition 
dgd rtca r1 t1 hmax "23:30" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmax condition"; exit 1; fi
# add a hlate condition 
dgd rtca r1 t1 hlate "22:30" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hlate condition"; exit 1; fi

# verify condition count for task 
if [ x`dgd rtcc r1 t1` != x5 ] ; then
	echo "Conditions count is incorrect."
	exit 1
fi
# verify condition list 
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x5 ] ; then
	echo "Condition list is incorrect ."
	exit 1
fi

# add and remove a manual condition 
mem3=`dgd memory`
dgd rtca r1 t1 manual "OK?" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a manual condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x6 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
dgd rtck r1 t1 manual "OK?" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a manual condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x5 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
mem4=`dgd memory`
if [ $mem3 != $mem4 ] ; then
	echo "dg87d : memory error after manual condition delete $mem3 -> $mem4"
	exit 1
fi


# add and remove a file condition 
mem3=`dgd memory`
dgd rtca r1 t1 file "/tmp/fic" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a file condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x6 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
dgd rtck r1 t1 file "/tmp/fic" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a file condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x5 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
mem4=`dgd memory`
if [ $mem3 != $mem4 ] ; then
	echo "dg87d : memory error after file condition delete $mem3 -> $mem4"
	exit 1
fi

# add and remove a hmin condition 
mem3=`dgd memory`
dgd rtca r1 t1 hmin "11:00" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmin condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x6 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
dgd rtck r1 t1 hmin "11:00" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a hmin condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x5 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
mem4=`dgd memory`
if [ $mem3 != $mem4 ] ; then
	echo "dg87d : memory error after hmin condition delete $mem3 -> $mem4"
	exit 1
fi

# add and remove a hmax condition 
mem3=`dgd memory`
dgd rtca r1 t1 hmax "11:00" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hmax condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x6 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
dgd rtck r1 t1 hmax "11:00" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a hmax condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x5 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
mem4=`dgd memory`
if [ $mem3 != $mem4 ] ; then
	echo "dg87d : memory error after hmax condition delete $mem3 -> $mem4"
	exit 1
fi


# add and remove a hlate condition 
mem3=`dgd memory`
dgd rtca r1 t1 hlate "11:00" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a hlate condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x6 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
dgd rtck r1 t1 hlate "11:00" > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a hlate condition"; exit 1; fi
if [ x`dgd rtcl r1 t1 | wc -l | awk '{print $1}'` != x5 ] ; then
	echo "Condition list is incorrect ." exit 1
fi
mem4=`dgd memory`
if [ $mem3 != $mem4 ] ; then
	echo "dg87d : memory error after hlate condition delete $mem3 -> $mem4"
	exit 1
fi

dgd rk r1
mem2=`dgd memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## REF COPY 
echo "Test Ref Copy" 

# create a target and a queue
dgd ta tg1 > /dev/null 2>&1
dgd qa tg1 q1 > /dev/null 2>&1

mem1=`dgd memory`
# create a base ref
dgd ra r1 > /dev/null 2>&1

dgd rta r1 t1 "task1" > /dev/null 2>&1
dgd rta r1 t2 "task2" > /dev/null 2>&1
dgd rta r1 t3 "task3" > /dev/null 2>&1
dgd rta r1 t4 "task4" > /dev/null 2>&1
dgd rtda r1 t1 t2 onerror > /dev/null 2>&1
dgd rtda r1 t1 t3 always > /dev/null 2>&1
dgd rtda r1 t3 t4 onsuccess > /dev/null 2>&1
dgd rtswad r1 t2 false || exit 1
dgd rtswad r1 t3 true || exit 1
dgd rtsoec r1 t1 false || exit 1
dgd rtsoec r1 t4 true || exit 1

dgd rja r1 t1 j1 "task1-job1" > /dev/null 2>&1
dgd rja r1 t1 j2 "task1-job2" > /dev/null 2>&1
dgd rja r1 t1 j3 "task1-job3" > /dev/null 2>&1
dgd rja r1 t1 j4 "task1-job4" > /dev/null 2>&1
dgd rjda r1 t1 j1 j2 onerror > /dev/null 2>&1
dgd rjda r1 t1 j1 j3 always > /dev/null 2>&1
dgd rjda r1 t1 j3 j4 onsuccess > /dev/null 2>&1
dgd rjsp r1 t1 j1 "ls"
dgd rjsa r1 t1 j1 "arg"
dgd rjst r1 t1 j1 10 20
dgd rjswad r1 t1 j2 false || exit 1
dgd rjswad r1 t1 j3 true || exit 1
dgd rjsoec r1 t1 j1 false || exit 1
dgd rjsoec r1 t1 j4 true || exit 1

dgd rja r1 t2 j1 "task2-job1" > /dev/null 2>&1
dgd rja r1 t2 j2 "task2-job2" > /dev/null 2>&1
dgd rjda r1 t2 j1 j2 always > /dev/null 2>&1

dgd rja r1 t3 j1 "task3-job1" > /dev/null 2>&1
dgd rja r1 t3 j2 "task3-job2" > /dev/null 2>&1
dgd rjda r1 t3 j1 j2 always > /dev/null 2>&1

dgd rja r1 t4 j1 "task4-job1" > /dev/null 2>&1
dgd rja r1 t4 j2 "task4-job2" > /dev/null 2>&1
dgd rjda r1 t4 j1 j2 onerror > /dev/null 2>&1

dgd rtca r1 t1 hmin "11:00"
dgd rtca r1 t1 hmax "12:00"
dgd rtca r1 t2 manual "ok?"
dgd rtca r1 t4 file "/tmp/fic"
dgd rtsq r1 t1 tg1 q1 > /dev/null 2>&1

# make a copy of the ref
mem2=`dgd memory`
dgd rd r1 r2
if [ $? != 0 ] ; then echo "Can't copy a ref"; exit 1; fi
mem3=`dgd memory`

# verify memory 
delta1=$(( $mem2 - $mem1))
delta2=$(( $mem3 - $mem2))
if [ $delta1 -ne $delta2 ] ; then 
	echo "memory delta different after ref duplicate"
	exit 1
fi

# verify task copy
nbr=`dgd rtc r2`
if [ $nbr -ne 4 ] ; then echo "Wrong task count"; exit 1; fi
# verify condition count
nbr=`dgd rtcc r2 t1` 
if [ $nbr -ne 2 ] ; then echo "Wrong condition count"; exit 1; fi
# verify task dependence count
nbr=`dgd rtdc r2` 
if [ $nbr -ne 3 ] ; then echo "Wrong task dependence count"; exit 1; fi
# verify job count
nbr=`dgd rjc r2 t1`
if [ $nbr -ne 4 ] ; then echo "Wrong job count"; exit 1; fi
# verify job dependence count
nbr=`dgd rjdc r2 t1`
if [ $nbr -ne 3 ] ; then echo "Wrong job dependence count"; exit 1; fi
# verify a task
if [ x`dgd rtl r2 | grep t1` != "xt1" ] ; then
	echo "Can't find a task."; exit 1
fi
if [ x`dgd rti r2 t1 | grep Description | awk '{print $2;}'` != "xtask1" ] 
then
	echo "Wrong task description."; exit 1
fi
# verify task which queue is set 
if [ x`dgd rti r2 t1 | grep Queue | awk '{print $2;}'` != "xq1" ] ; then
	echo "Queue is not set"; exit 1
fi
# verify task which queue is not set 
if [ x`dgd rti r2 t2 | grep Queue | awk '{print $2;}'` != "xnone" ] ; then
	echo "Queue is set"; exit 1
fi
# verify task waitAllDependences property 
if [ x`dgd rti r2 t2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "waitAllDependences property is not false"; exit 1
fi
if [ x`dgd rti r2 t3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "waitAllDependences property is not true"; exit 1
fi
# verify task OnErrorContinue property 
if [ x`dgd rti r2 t1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "OnErrorContinue property is not false"; exit 1
fi
if [ x`dgd rti r2 t4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "OnErrorContinue property is not true"; exit 1
fi

# verify a job 
if [ x`dgd rjl r2 t1 | grep j1` != "xj1" ] ; then
	echo "Can't find a job."; exit 1
fi
if [ x`dgd rji r2 t1 j1 | grep Description | awk '{print $2}'` != "xtask1-job1" ] ; then
	echo "Wrong job description."; exit 1
fi
if [ x`dgd rji r2 t1 j1 | grep Path | awk '{print $2}'` != "xls" ] ; then
	echo "Wrong job path."; exit 1
fi
if [ x`dgd rji r2 t1 j1 | grep Argument | awk '{print $2}'` != "xarg" ] ; then
	echo "Wrong job argument."; exit 1
fi
if [ x`dgd rji r2 t1 j1 | grep Start | awk '{print $3}'` != "x10" ] ; then
	echo "Wrong start tempo."; exit 1
fi
if [ x`dgd rji r2 t1 j1 | grep End | awk '{print $3}'` != "x20" ] ; then
	echo "Wrong end tempo."; exit 1
fi
# verify job waitAllDependences property 
if [ x`dgd rji r2 t1 j2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; then
	echo "waitAllDependences property of job is not false"; exit 1
fi
if [ x`dgd rji r2 t1 j3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; then
	echo "waitAllDependences property of job is not true"; exit 1
fi
# verify job OnErrorContinue property 
if [ x`dgd rji r2 t1 j1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ] ; then
	echo "OnErrorContinue property of job is not false"; exit 1
fi
if [ x`dgd rji r2 t1 j4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ] ; then
	echo "OnErrorContinue property of job is not true"; exit 1
fi

# verify task dependence
if [ "`dgd rtdl -S! -t r2 | grep onerror`" != "t1!t2!onerror" ] ; then
	echo "Incorrect task dependence (onerror)"; exit 1
fi
if [ "`dgd rtdl -S! -t r2 | grep always`" != "t1!t3!always" ] ; then
	echo "Incorrect task dependence (always)"; exit 1
fi
if [ "`dgd rtdl -S! -t r2 | grep onsuccess`" != "t3!t4!onsuccess" ] ; then
	echo "Incorrect task dependence (onsuccess)"; exit 1
fi

# verify job dependence
if [ "`dgd rjdl -S! -t r2 t1 | grep onerror`" != "j1!j2!onerror" ] ; then
	echo "Incorrect job dependence (onerror)"; exit 1
fi
if [ "`dgd rjdl -S! -t r2 t1 | grep always`" != "j1!j3!always" ] ; then
	echo "Incorrect job dependence (always)"; exit 1
fi
if [ "`dgd rjdl -S! -t r2 t1 | grep onsuccess`" != "j3!j4!onsuccess" ] ; then
	echo "Incorrect job dependence (onsuccess)"; exit 1
fi
# verify condition
if [ "`dgd rtcl -S! r2 t1 | grep hmin`" != "hmin!11:00" ] ; then
	echo "Incorrect hmin condition"; exit 1
fi
if [ "`dgd rtcl -S! r2 t1 | grep hmax`" != "hmax!12:00" ] ; then
	echo "Incorrect hmax condition"; exit 1
fi
if [ "`dgd rtcl -S! r2 t2 | grep manual`" != "manual!ok?" ] ; then
	echo "Incorrect manual condition"; exit 1
fi
if [ "`dgd rtcl -S! r2 t4 | grep file`" != "file!/tmp/fic" ] ; then
	echo "Incorrect file condition"; exit 1
fi

# delete ref copy
dgd rk r2 > /dev/null 2>&1
mem3=`dgd memory`
if [ $mem3 -ne $mem2 ] ; then 
	echo "memory delta different after ref copy delete"
	exit 1
fi
# delete ref orig 
dgd rk r1 > /dev/null 2>&1
mem3=`dgd memory`
if [ $mem3 -ne $mem1 ] ; then 
	echo "memory delta different after ref orig delete"
	exit 1
fi

echo "Stop dg87d"
stopDg87d || exit 1
