#!/bin/sh
#
# file tst03.sh 
# (test dg87x functions)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87x"
startDg87x || exit 1

# dg87x : test 
mem1=`dgx memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

################################################## TARGET 
echo "Test Target"
mem1=`dgx memory`

# add a target 
dgx ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# verify target count = 1
n=`dgx tc`
if [ $n != 1 ] ; then echo "Target count isn't 1"; exit 1; fi
# verify target list
if [ "`dgx tl`" != "tg1" ] ; then
	echo "Target is not in list."; exit 1
fi

# change target description
dgx tsd tg1 newDesc
if [ $? != 0 ] ; then echo "Can't change target description !"; exit 1; fi
if [ "`dgx ti -S! tg1 | grep Desc`" != "Description!newDesc" ] ; then
	echo "Target description is not changed"; exit 1
fi
# change open property
dgx tso tg1 true 
if [ $? != 0 ] ; then echo "Can't change target open property !"; exit 1; fi
if [ "`dgx ti -S! tg1 | grep Open`" != "Opened!true" ] ; then
	echo "Target max property is not changed"; exit 1
fi
# change ip port property
dgx tsp tg1 8888 
if [ $? != 0 ] ; then echo "Can't change IP Port property !"; exit 1; fi
if [ "`dgx ti -S! tg1 | grep Port`" != "IP Port!8888" ] ; then
	echo "Target IP Port property is not changed"; exit 1
fi

# try to add a second target with same name
dgx ta tg1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two target with same name"; exit 1; fi

# delete target
dgx tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the target"; exit 1; fi

# verify target count = 0
n=`dgx tc`
if [ $n != 0 ] ; then echo "Target count isn't 0"; exit 1; fi

# try to delete a target which doesn't exist
dgx tk tg1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete the target which doesn't exist"; exit 1; fi

mem2=`dgx memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error before and after target $mem1 -> $mem2"
	exit 1
fi

################################################## QUEUE 
echo "Test Queue"

# add a target 
dgx ta tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a target"; exit 1; fi
# add a queue 
dgx qa tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't add a queue"; exit 1; fi

# verify queue count = 1
n=`dgx qc tg1`
if [ $n != 1 ] ; then echo "Queue count isn't 1"; exit 1; fi
# verify queue list
if [ "`dgx ql tg1`" != "q1" ]; then
	echo "Queue is not in queue list of the target"; exit 1
fi

# change queue description
dgx qsd tg1 q1 newDesc
if [ $? != 0 ] ; then echo "Can't change queue description !"; exit 1; fi
if [ "`dgx qi -S! tg1 q1 | grep Desc`" != "Description!newDesc" ] ; then
	echo "Queue description is not changed"; exit 1
fi

# change queue max 
dgx qsm tg1 q1 55
if [ $? != 0 ] ; then echo "Can't change max property of Queue"; exit 1; fi
# verify max property change
if [ "`dgx qi tg1 q1 | grep Max|awk '{print $2}'`" != "55" ] ; then
	echo "Queue max property is not changed"; exit 1;
fi

# change open property
dgx qso tg1 q1 true
if [ $? != 0 ] ; then echo "Can't change queue open property !"; exit 1; fi
if [ "`dgx qi -S! tg1 q1 | grep Open`" != "Opened!true" ] ; then
	echo "Queue max property is not changed"; exit 1
fi

# try to add a second queue with same name
dgx qa tg1 q1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two queue with same name"; exit 1; fi

# delete queue 
dgx qk tg1 q1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete the queue"; exit 1; fi

# verify queue count = 0
n=`dgx qc tg1`
if [ $n != 0 ] ; then echo "Queue count isn't 0"; exit 1; fi

# try to delete a queue which doesn't exist
dgx qk tg1 q1 > /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can delete a queue which doesn't exist"; exit 1; fi

# delete the target
dgx tk tg1 > /dev/null 2>&1
if [ $? != 0 ] ; then echo "Can't delete a target"; exit 1; fi

mem2=`dgx memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error before and after queue $mem1 -> $mem2"
	exit 1
fi

# verify memory when deleting a target with queues
mem1=`dgx memory`
dgx ta tg1 > /dev/null 2>&1
dgx qa tg1 q1 > /dev/null 2>&1
dgx qa tg1 q2 > /dev/null 2>&1
dgx qa tg1 q3 > /dev/null 2>&1
dgx tk tg1 > /dev/null 2>&1
mem2=`dgx memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error delete target with queues $mem1 -> $mem2"
	exit 1
fi

################################################## DAY
echo "Test Day"

mem1=`dgx memory`
# add a day
dgx xda d1 "$dayDate"
if [ $? != 0 ] ; then echo "Can't add a day"; exit 1; fi
# verify day count
n=`dgx xdc`
if [ $n -ne 1 ] ; then echo "Day count is not 1."; exit 1; fi
# verify day list
if [ "`dgx xdl`" != "d1" ] ; then echo "Day not in day list."; exit 1;fi

# verify start time
if [ "`dgx xdi -S! d1 | grep Start`" != "Start Time!$dayDate" ] ; then
	echo "Day start time is not set ($dayDate)."; exit 1
fi

# verify that day is locked
if [ "`dgx xdi -S! d1 | grep Locked`" != "Locked!true" ] ; then
	echo "Day is not locked."; exit 1
fi

# validate day
dgx xdv d1
if [ $? -ne 0 ] ; then echo "Can't validate day."; exit 1; fi
# verify that day is unlocked
if [ "`dgx xdi -S! d1 | grep Locked`" != "Locked!false" ] ; then
	echo "Day is not unlocked."; exit 1
fi

# verify that day is disabled
if [ "`dgx xdi -S! d1 | grep Enabled`" != "Enabled!false" ] ; then
	echo "Day is not disabled."; exit 1
fi
# enable day
dgx xdse d1 true 
if [ $? -ne 0 ] ; then echo "Can't enable day."; exit 1; fi
# verify that day is enabled
if [ "`dgx xdi -S! d1 | grep Enabled`" != "Enabled!true" ] ; then
	echo "Day is not enabled."; exit 1
fi

# try to add another day with the same name
dgx xda d1 "$dayDate" >  /dev/null 2>&1
if [ $? == 0 ] ; then echo "Can add two days with same name"; exit 1; fi

# delete day
dgx xdk d1
if [ $? != 0 ]; then echo "Can't delete day"; exit 1; fi

mem2=`dgx memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error after day delete $mem1 -> $mem2"
	exit 1
fi

# try to delete an inexistant day
dgx xdk jx > /dev/null 2>&1
if [ $? == 0 ]; then echo "Can delete an inexistant day"; exit 1; fi

################################################## TASK 
echo "Test Task"

# add a target and a queue
dgx ta tg1
if [ $? != 0 ]; then echo "Can't add target"; exit 1; fi
dgx qa tg1 q1
if [ $? != 0 ]; then echo "Can't add queue"; exit 1; fi

mem1=`dgx memory`
# add a day
dgx xda d1 "$dayDate"
if [ $? != 0 ]; then echo "Can't add day"; exit 1; fi

# add a task
dgx xta d1 t1 tg1 q1 "task1" true true true
if [ $? != 0 ]; then echo "Can't add a task"; exit 1; fi

# verify task count and list
if [ `dgx xtc d1` -ne 1 ] ; then echo "Task count isn't 1."; exit 1; fi
if [ "`dgx xtl d1`" != "t1" ]; then 
	echo "Task is not in task list of day"; exit 1
fi

if [ "`dgx xti -S! d1 t1 | grep \"Day Name\"`" != "Day Name!d1" ] ; then
	echo "Task day is not set." ; exit 1
fi
# verify task description
if [ "`dgx xti -S! d1 t1 | grep Description`" != "Description!task1" ] ; then
	echo "Task description is not set"; exit 1
fi
# verify queue 
if [ "`dgx xti -S! d1 t1 | grep Queue`" != "Queue!q1" ] ; then
	echo "Queue is not set"; exit 1
fi


# verify waitAllDependences property
dgx xta d1 t2 tg1 q1 "" true false true
dgx xta d1 t3 tg1 q1 "" true true true
if [ x`dgx xti d1 t2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "waitAllDependences property is not false"; exit 1
fi
if [ x`dgx xti d1 t3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "waitAllDependences property is not true"; exit 1
fi

# try to add task with inexistant day/target/queue
dgx xta d2 t2 tg1 q1 "task2" true true true >/dev/null 2>&1
if [ $? == 0 ]; then echo "Can add a task whith an inexistant day"; exit 1; fi
dgx xta d1 t2 tg2 q1 "task2" true true true>/dev/null 2>&1
if [ $? == 0 ]; then echo "Can add a task whith an inexistant target"; exit 1; fi
dgx xta d1 t2 tg1 q2 "task2" true true true>/dev/null 2>&1
if [ $? == 0 ]; then echo "Can add a task whith an inexistant queue"; exit 1; fi

# delete day (with a task)
dgx xdk d1
if [ $? != 0 ]; then echo "Can't delete a day with tasks"; exit 1; fi

mem2=`dgx memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error after day delete $mem1 -> $mem2"
	exit 1
fi
dgx tk tg1

################################################## JOB 
echo "Test Job"

# add a target and a queue
dgx ta tg1 || exit 1
dgx qa tg1 q1 || exit 1

mem1=`dgx memory`
# add a day
dgx xda d1 "$dayDate"
if [ $? != 0 ]; then echo "Can't add day"; exit 1; fi

# add a task
dgx xta d1 t1 tg1 q1 "task1" true true true
if [ $? != 0 ]; then echo "Can't add a task"; exit 1; fi

# add a job
dgx xja d1 t1 j1 "job1" "/bin/ls" "arg" 25 35 false true
if [ $? != 0 ]; then echo "Can't add a job"; exit 1; fi

# verify job fields
if [ "`dgx xji -S! d1 t1 j1 | grep \"Task Name\"`" != "Task Name!t1" ] ; then
	echo "Job task is not set." ; exit 1
fi
if [ "`dgx xji -S! d1 t1 j1 | grep \"Day Name\"`" != "Day Name!d1" ] ; then
	echo "Job day is not set." ; exit 1
fi
if [ "`dgx xji -S! d1 t1 j1 | grep Description`" != "Description!job1" ] ; then
	echo "Job description is not set." ; exit 1
fi
if [ "`dgx xji -S! d1 t1 j1 | grep Queue`" != "Target Queue!q1" ] ; then
	echo "Job queue is not set." ; exit 1
fi
if [ "`dgx xji -S! d1 t1 j1 | grep Path`" != "Path!/bin/ls" ] ; then
	echo "Job path is not set." ; exit 1
fi
if [ "`dgx xji -S! d1 t1 j1 | grep Argument`" != "Argument!arg" ] ; then
	echo "Job argument is not set." ; exit 1
fi
if [ "`dgx xji -S! d1 t1 j1 | grep Start`" != "Start Tempo!25" ] ; then
	echo "Job start tempo is not set." ; exit 1
fi
if [ "`dgx xji -S! d1 t1 j1 | grep End`" != "End Tempo!35" ] ; then
	echo "Job end tempo is not set." ; exit 1
fi

# try to add a job with inexistant day/task
dgx xja dx t1 j2 "job1" "/bin/ls" "arg" 25 35 false true > /dev/null 2>&1
if [ $? == 0 ]; then echo "Can add a job with inexistant day"; exit 1; fi
dgx xja d1 tx j2 "job1" "/bin/ls" "arg" 25 35 false true > /dev/null 2>&1
if [ $? == 0 ]; then echo "Can add a job with inexistant task"; exit 1; fi

# verify waitAllDependences property
dgx xja d1 t1 j2 "" "/bin/ls" "arg" 25 35 false true
dgx xja d1 t1 j3 "" "/bin/ls" "arg" 25 35 true true
if [ x`dgx xji d1 t1 j2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; then
	echo "waitAllDependences property of job is not false"; exit 1
fi
if [ x`dgx xji d1 t1 j3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; then
	echo "waitAllDependences property of job is not true"; exit 1
fi

# delete day (with tasks and jobs)
dgx xdk d1
if [ $? != 0 ]; then echo "Can't delete a day with jobs"; exit 1; fi

mem2=`dgx memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error after day delete $mem1 -> $mem2"
	exit 1
fi
dgx tk tg1

################################################## TASK DEP 
echo "Test task dependences" 

# add a target and a queue
dgx ta tg1 || exit 1
dgx qa tg1 q1 || exit 1

mem1=`dgx memory`
# add a day
dgx xda d1 "$dayDate"
if [ $? != 0 ]; then echo "Can't add day"; exit 1; fi

# add tasks
dgx xta d1 t1 tg1 q1 "task1" true true true || exit 1
dgx xta d1 t2 tg1 q1 "task2" true true true || exit 1
dgx xta d1 t3 tg1 q1 "task3" true true true || exit 1
dgx xta d1 t4 tg1 q1 "task4" true true true || exit 1
dgx xta d1 t5 tg1 q1 "task5" true true true || exit 1

dgx xtda d1 t1 t2 always
if [ $? != 0 ]; then echo "Can't add task dependence (type always)"; exit 1; fi
dgx xtda d1 t2 t3 onerror 
if [ $? != 0 ]; then echo "Can't add task dependence (type onerror)"; exit 1; fi
dgx xtda d1 t3 t4 onsuccess
if [ $? != 0 ]; then echo "Can't add task dependence (type onsuccess)"; exit 1; fi

# try to add dependence with inexistant day, task, type
dgx xtda dx t3 t4 onsuccess > /dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add task dependence with inexistant day"; exit 1; 
fi
dgx xtda d1 tx t4 onsuccess > /dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add task dependence with inexistant task (before)"; exit 1; 
fi
dgx xtda d1 t3 tx onsuccess > /dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add task dependence with inexistant task (after)"; exit 1; 
fi
dgx xtda d1 t3 t4 onxxx > /dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add task dependence with bad dependence type"; exit 1; 
fi

# verify task dependence list
if [ "`dgx xtdc d1`" != "3" ] ; then 
	echo "Task dependence count isn't 3"; exit 1
fi
if [ "`dgx xtdl -tS! d1 | grep always`" != "t1!t2!always" ] ; then
	echo "Task dependence is not in list (always)"; exit 1
fi
if [ "`dgx xtdl -tS! d1 | grep onerror`" != "t2!t3!onerror" ] ; then
	echo "Task dependence is not in list (onerror)"; exit 1
fi
if [ "`dgx xtdl -tS! d1 | grep onsuccess`" != "t3!t4!onsuccess" ] ; then
	echo "Task dependence is not in list (onsuccess)"; exit 1
fi

# delete day (with task dependences)
dgx xdk d1
if [ $? != 0 ]; then echo "Can't delete a day with task dependences"; exit 1; fi

mem2=`dgx memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error after day delete $mem1 -> $mem2"
	exit 1
fi
dgx tk tg1

################################################## JOB DEP
echo "Test job dependences" 

# add a target and a queue
dgx ta tg1 || exit 1
dgx qa tg1 q1 || exit 1

mem1=`dgx memory`
# add a day and a task
dgx xda d1 "$dayDate" || exit 1
dgx xta d1 t1 tg1 q1 "task1" true true true || exit 1

# add jobs
dgx xja d1 t1 j1 "job1" "ls" "" 1 1 false true || exit 1
dgx xja d1 t1 j2 "job2" "ls" "" 1 1 false true || exit 1
dgx xja d1 t1 j3 "job3" "ls" "" 1 1 false true || exit 1
dgx xja d1 t1 j4 "job4" "ls" "" 1 1 false true || exit 1

dgx xjda d1 t1 j1 j2 always
if [ $? != 0 ]; then echo "Can't add job dependence (type always)"; exit 1; fi
dgx xjda d1 t1 j2 j3 onerror 
if [ $? != 0 ]; then echo "Can't add job dependence (type onerror)"; exit 1; fi
dgx xjda d1 t1 j3 j4 onsuccess
if [ $? != 0 ]; then echo "Can't add job dependence (type onsuccess)"; exit 1; fi


# verify task dependence list
if [ "`dgx xjdc d1 t1`" != "3" ] ; then 
	echo "Job dependence count isn't 3"; exit 1
fi
if [ "`dgx xjdl -tS! d1 t1 | grep always`" != "j1!j2!always" ] ; then
	echo "Job dependence is not in list (always)"; exit 1
fi
if [ "`dgx xjdl -tS! d1 t1 | grep onerror`" != "j2!j3!onerror" ] ; then
	echo "Job dependence is not in list (onerror)"; exit 1
fi
if [ "`dgx xjdl -tS! d1 t1 | grep onsuccess`" != "j3!j4!onsuccess" ] ; then
	echo "Job dependence is not in list (onsuccess)"; exit 1
fi

# try to add dependence with inexistant day, task, job, type
dgx xjda dx t1 j1 j2 onsuccess >/dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add job dependence with inexistant day"; exit 1; 
fi
dgx xjda d1 tx j1 j2 onsuccess >/dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add job dependence with inexistant task"; exit 1; 
fi
dgx xjda d1 t1 jx j2 onsuccess >/dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add job dependence with inexistant job (before)"; exit 1; 
fi
dgx xjda d1 t1 j1 jx onsuccess >/dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add job dependence with inexistant job (after)"; exit 1; 
fi
dgx xjda d1 t1 j1 j2 onxxx > /dev/null 2>&1
if [ $? == 0 ]; then 
	echo "Can add job dependence with bad dependence type"; exit 1; 
fi

# delete day (with task dependences)
dgx xdk d1
if [ $? != 0 ]; then echo "Can't delete a with job dependences"; exit 1; fi

mem2=`dgx memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error after day delete $mem1 -> $mem2"
	exit 1
fi

################################################## CONDITIONS 
echo "Test conditions" 

mem1=`dgx memory`
# add a day and a task
dgx xda d1 "$dayDate" || exit 1
dgx xta d1 t1 tg1 q1 "task1" true true true || exit 1


# add conditions
dgx xtca d1 t1 file "/tmp/fic"
if [ $? != 0 ]; then echo "Can't add file condition"; exit 1; fi
dgx xtca d1 t1 manual "ok" 
if [ $? != 0 ]; then echo "Can't add manual condition"; exit 1; fi
dgx xtca d1 t1 hmin "08:00" 
if [ $? != 0 ]; then echo "Can't add hmin condition"; exit 1; fi
dgx xtca d1 t1 hmax "09:00" 
if [ $? != 0 ]; then echo "Can't add hmax condition"; exit 1; fi
dgx xtca d1 t1 hlate "12:00" 
if [ $? != 0 ]; then echo "Can't add hlate condition"; exit 1; fi

# verify condition list
if [ "`dgx xtcc d1 t1`" != "5" ]; then
	echo "Condition count is not 5."; exit 1
fi
if [ "`dgx xtcl -S! d1 t1 | grep file`" != "file!/tmp/fic" ]; then
	echo "File condition is not in list."; exit 1
fi
if [ "`dgx xtcl -S! d1 t1 | grep manual`" != "manual!ok" ]; then
	echo "File condition is not in list."; exit 1
fi
if [ "`dgx xtcl -S! d1 t1 | grep hmin`" != "hmin!08:00" ]; then
	echo "HMin condition is not in list."; exit 1
fi
if [ "`dgx xtcl -S! d1 t1 | grep hmax`" != "hmax!09:00" ]; then
	echo "HMax condition is not in list."; exit 1
fi
if [ "`dgx xtcl -S! d1 t1 | grep hlate`" != "hlate!12:00" ]; then
	echo "HLate condition is not in list."; exit 1
fi
	
# try to add condition with inexistant day, task, type
dgx xtca dx t1 file "/tmp/fic" >/dev/null 2>&1
if [ $? == 0 ]; then echo "Can add condition with inexistant day"; exit 1; fi
dgx xtca d1 tx file "/tmp/fic" >/dev/null 2>&1
if [ $? == 0 ]; then echo "Can add condition with inexistant task"; exit 1; fi
dgx xtca d1 t1 xxx "/tmp/fic" >/dev/null 2>&1
if [ $? == 0 ]; then echo "Can add condition with bad type"; exit 1; fi

# delete day (with task conditions)
dgx xdk d1
if [ $? != 0 ]; then echo "Can't delete a day with conditions"; exit 1; fi

mem2=`dgx memory`
if [ $mem1 != $mem2 ] ; then
	echo "dg87x : memory error after day delete $mem1 -> $mem2"
	exit 1
fi

echo "Stop dg87x"
stopDg87x || exit 1


