#!/bin/sh
#
# file tst04.sh 
# (Test day planification)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1

memd1=`dgd memory`
memx1=`dgx memory`

# add target and queues
dgd ta tg1 || exit 1
dgx ta tg1 || exit 1
for i in 1 2 3 4 ; do
	dgd qa tg1 q$i || exit 1
	dgx qa tg1 q$i || exit 1
done

echo "Add ref"
# add new ref
dgd ra r1 || exit 1

# add tasks 
echo "Add tasks"
for i in 1 2 3 4 ; do
	dgd rta r1 t$i task$i || exit 1
	dgd rtsq r1 t$i tg1 q$i || exit 1
	dgd rtoa r1 t$i add always || exit 1
done
dgd rtsoec r1 t1 false || exit 1
dgd rtsoec r1 t4 true || exit 1
dgd rtswad r1 t2 false || exit 1
dgd rtswad r1 t3 true || exit 1

# add task dependences
dgd rtda r1 t1 t2 always || exit 1
dgd rtda r1 t2 t3 onsuccess  || exit 1
dgd rtda r1 t2 t4 onerror || exit 1

# add conditions
dgd rtca r1 t1 hmin "08:00" || exit 1
dgd rtca r1 t1 hmax "09:00" || exit 1
dgd rtca r1 t1 hlate "10:00" || exit 1
dgd rtca r1 t1 file "/tmp/fic" || exit 1
dgd rtca r1 t1 manual "ok" || exit 1

# add jobs 
echo "Add jobs"
for i in 1 2 3 4 ; do
	dgd rja r1 t1 j$i job$i || exit 1
	dgd rjsp r1 t1 j$i "/job$i.sh" || exit 1
	dgd rjsa r1 t1 j$i "arg$i" || exit 1
	dgd rjst r1 t1 j$i $(( $i + 10)) $(( $i + 20 )) || exit 1
done
dgd rjsoec r1 t1 j1 false || exit 1
dgd rjsoec r1 t1 j4 true || exit 1
dgd rjswad r1 t1 j2 false || exit 1
dgd rjswad r1 t1 j3 true || exit 1

# add job dependences
dgd rjda r1 t1 j1 j2 always || exit 1
dgd rjda r1 t1 j2 j3 onsuccess || exit 1
dgd rjda r1 t1 j2 j4 onerror || exit 1

echo "Plan ref"
dgd rp r1 d1 "$dayDate" || exit 1
dgd ese true || exit 1
sleep 1

# verify that day is plan
echo "Verify day"
if [ "`dgx xdc`" != "1" ] ; then echo "Day count isn't 1"; exit 1; fi
if [ "`dgx xdl`" != "d1" ] ; then echo "Can't find the day"; exit 1; fi

if [ "`dgx xdi -S! d1 | grep \"Start Time\"`" != "Start Time!$dayDate" ] ; then
	echo "Day date is not set." ; exit 1
fi
if [ "`dgx xdi -S! d1 | grep Enabled`" != "Enabled!false" ] ; then
	echo "Day date is not disabled." ; exit 1
fi
if [ "`dgx xdi -S! d1 | grep Locked`" != "Locked!false" ] ; then
	echo "Day date is locked." ; exit 1
fi

# verify task properties
echo "Verify tasks"
if [ "`dgx xtc d1`" != "4" ] ; then echo "Task count isn't 4"; exit 1; fi
for i in 1 2 3 4 ; do
	if [ "`dgx xtl -S! -d d1 | grep t$i`" != "t$i!task$i" ] ; then 
		echo "Can't find task t$i in task list"; exit 1; 
	fi
	if [ "`dgx xti -S! d1 t$i | grep Queue`" != "Queue!q$i" ]; then
		echo "Queue is not set for task t$i"; exit 1; 
	fi
done
# verify waitAllDependences property
if [ x`dgx xti d1 t2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "waitAllDependences property is not false"; exit 1
fi
if [ x`dgx xti d1 t3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "waitAllDependences property is not true"; exit 1
fi
# verify OnErrorContinue property
if [ x`dgx xti d1 t1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ] ; 
then
	echo "OnErrorContinue property is not false"; exit 1
fi
if [ x`dgx xti d1 t4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ] ; 
then
	echo "OnErrorContinue property is not true"; exit 1
fi

# verify task dependences
echo "Verify task dependences"
if [ "`dgx xtdc d1`" != "3" ] ; then 
	echo "Task dependences count isn't 3"; exit 1; 
fi
if [ "`dgx xtdl -tS! d1 | grep always`" != "t1!t2!always" ] ; then 
	echo "Can't find task dependences (type always)"; exit 1; 
fi
if [ "`dgx xtdl -tS! d1 | grep onsuccess`" != "t2!t3!onsuccess" ] ; then 
	echo "Can't find task dependences (type onsuccess)"; exit 1; 
fi
if [ "`dgx xtdl -tS! d1 | grep onerror`" != "t2!t4!onerror" ] ; then 
	echo "Can't find task dependences (type onerror)"; exit 1; 
fi

# verify task conditions 
echo "Verify task conditions"
if [ "`dgx xtcc d1 t1`" != "5" ] ; then 
	echo "Task condition count isn't 5"; exit 1; 
fi
if [ "`dgx xtcl -S! d1 t1 | grep hmin`" != "hmin!08:00" ] ; then 
	echo "Can't find condition (type hmin)"; exit 1; 
fi
if [ "`dgx xtcl -S! d1 t1 | grep hmax`" != "hmax!09:00" ] ; then 
	echo "Can't find condition (type hmax)"; exit 1; 
fi
if [ "`dgx xtcl -S! d1 t1 | grep hlate`" != "hlate!10:00" ] ; then 
	echo "Can't find condition (type hlate)"; exit 1; 
fi
if [ "`dgx xtcl -S! d1 t1 | grep file`" != "file!/tmp/fic" ] ; then 
	echo "Can't find condition (type file)"; exit 1; 
fi
if [ "`dgx xtcl -S! d1 t1 | grep manual`" != "manual!ok" ] ; then 
	echo "Can't find condition (type manual)"; exit 1; 
fi

# verify job properties
echo "Verify jobs"
if [ "`dgx xjc d1 t1`" != "4" ] ; then echo "Job count isn't 4"; exit 1; fi
for i in 1 2 3 4 ; do
	if [ "`dgx xjl -S! -d d1 t1 | grep j$i`" != "j$i!job$i" ] ; then 
		echo "Can't find job j$i in task list"; exit 1; 
	fi
	if [ "`dgx xji -S! d1 t1 j$i | grep Path`" != "Path!/job$i.sh" ]; then
		echo "Path is not set for job j$i"; exit 1; 
	fi
	if [ "`dgx xji -S! d1 t1 j$i | grep Arg`" != "Argument!arg$i" ]; then
		echo "Argument is not set for job j$i"; exit 1; 
	fi
	if [ "`dgx xji -S! d1 t1 j$i | grep Start`" != \
			"Start Tempo!$(( $i + 10 ))" ]; then
		echo "Start tempo is not set for job j$i"; exit 1; 
	fi
	if [ "`dgx xji -S! d1 t1 j$i | grep End`" != \
			"End Tempo!$(( $i + 20 ))" ]; then
		echo "End tempo is not set for job j$i"; exit 1; 
	fi
done

# verify job dependences
echo "Verify job dependences"
if [ "`dgx xjdc d1 t1`" != "3" ] ; then 
	echo "Job dependences count isn't 3"; exit 1; 
fi
if [ "`dgx xjdl -tS! d1 t1 | grep always`" != "j1!j2!always" ] ; then 
	echo "Can't find job dependences (type always)"; exit 1; 
fi
if [ "`dgx xjdl -tS! d1 t1 | grep onsuccess`" != "j2!j3!onsuccess" ] ; then 
	echo "Can't find job dependences (type onsuccess)"; exit 1; 
fi
if [ "`dgx xjdl -tS! d1 t1 | grep onerror`" != "j2!j4!onerror" ] ; then 
	echo "Can't find job dependences (type onerror)"; exit 1; 
fi

# verify waitAllDependences property
if [ x`dgx xji d1 t1 j2 | grep "Wait All Dep"| awk '{print $4;}'` != "xfalse" ]; then
	echo "waitAllDependences property of job is not false"; exit 1
fi
if [ x`dgx xji d1 t1 j3 | grep "Wait All Dep"| awk '{print $4;}'` != "xtrue" ]; then
	echo "waitAllDependences property of job is not true"; exit 1
fi

# verify OnErrorContinue property
if [ x`dgx xji d1 t1 j1 | grep "On Error"| awk '{print $4;}'` != "xfalse" ]; then
	echo "OnErrorContinue property of job is not false"; exit 1
fi
if [ x`dgx xji d1 t1 j4 | grep "On Error"| awk '{print $4;}'` != "xtrue" ]; then
	echo "OnErrorContinue property of job is not true"; exit 1
fi

# kill ref and day
dgd rk r1 || exit 1
dgx xdk d1 || exit 1

# delete target and queue
dgd tk tg1 || exit 1
dgx tk tg1 || exit 1
memd2=`dgd memory`
if [ $memd2 -ne $memd1 ] ; then 
	echo "memory delta different for dg87d $memd1 -> $memd2"; exit 1
fi
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1
