#!/bin/sh
#
# file tst05.sh 
# (test locked property for referential and day)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1

memd1=`dgd memory`

echo "Add ref"
# add new ref
dgd ta tg1 || exit 1
dgd qa tg1 q1 || exit 1
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rta r1 t3 || exit 1
dgd rtoa r1 t3 add always || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rja r1 t1 j2 || exit 1
dgd rja r1 t1 j3 || exit 1
dgd rtda r1 t1 t2 always || exit 1
dgd rjda r1 t1 j1 j2 always || exit 1
dgd rtca r1 t1 manual "test"

# disable dg87d engine
dgd ese false || exit 1
if [ "`dgd ei -S! | grep Enabled`" != "Enabled!false" ]; then
	echo "dg87d engine is not disabled."; exit 1
fi

# plan day
dgd rp r1 d1 "$dayDate" || exit 1
if [ "`dgd ei -S! | grep Ref`" != "Ref to plan!r1" ]; then
	echo "r1 is not the ref to plan"; exit 1
fi

# verify that ref is locked
if [ "`dgd ri -S! r1 | grep Locked`" != "Locked!true" ]; then
	echo "r1 is not locked"; exit 1
fi

echo "Test task functions"
# try to add a task on a locked ref
dgd rta r1 tx >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task on a locked ref !"; exit 1; fi
# try to change task description
dgd rtsd r1 t1 xxx >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can change task description on a locked ref !"; exit 1; fi
# try to change task queue 
dgd rtsq r1 t1 tg1 q1 >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can change task queue on a locked ref !"; exit 1; fi
# try to kill a task
dgd rtk r1 t1 >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can kill a task on a locked ref !"; exit 1; fi


echo "Test job functions"
# try to add a job on a locked ref
dgd rja r1 t1 jx >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a job on a locked ref !"; exit 1; fi
# try to change job description
dgd rjsd r1 t1 j1 xxx >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can change job description on a locked ref !"; exit 1; fi
# try to change job path 
dgd rjsp r1 t1 j1 xxx >/dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can change job path on a locked ref !"; exit 1; fi
# try to change job argument 
dgd rjsa r1 t1 j1 xxx >/dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can change job argument on a locked ref !"; exit 1; fi
# try to change job tempo 
dgd rjst r1 t1 j1 22 33 >/dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can change job tempo on a locked ref !"; exit 1; fi
# try to kill a job 
dgd rjk r1 t1 j1 >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can kill a job on a locked ref !"; exit 1; fi

echo "Test task dependence functions"
# try to add a task dependence on a locked ref
dgd rtda r1 t1 t3 always >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task dependence on a locked ref !"; exit 1; fi
# try to kill a task dependence on a locked ref
dgd rtdk r1 t1 t2 always >/dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can delete a task dependence on a locked ref !"; exit 1; fi


echo "Test job dependence functions"
# try to add a job dependence on a locked ref
dgd rjda r1 t1 j1 j3 always >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a job dependence on a locked ref !"; exit 1; fi
# try to kill a job dependence on a locked ref
dgd rjdk r1 t1 j1 j2 always >/dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can delete a job dependence on a locked ref !"; exit 1; fi

echo "Test task condition functions"

dgd rtca r1 t1 hmin "08:00" >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task condition (hmin) on a locked ref !"; exit 1; fi
dgd rtca r1 t1 hmax "09:00" >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task condition (hmax) on a locked ref !"; exit 1; fi
dgd rtca r1 t1 hlate "10:00" >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task condition (hlate) on a locked ref !"; exit 1; fi
dgd rtca r1 t1 file "/tmp/fic" >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task condition (file) on a locked ref !"; exit 1; fi
dgd rtca r1 t1 file "ok" >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task condition (manual) on a locked ref !"; exit 1; fi
# try to delete a task condition
dgd rtck r1 t1 manual "test" >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can kill a task condition on a locked ref !"; exit 1; fi

echo "Test cleanup"
# kill ref
dgd rk r1 >/dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can kill a locked ref !"; exit 1; fi

# cancel plan process and kill the ref
dgd ecp 
if [ $? -ne 0 ]; then echo "Can't cancel plan process !"; exit 1; fi
dgd ese true 
if [ $? -ne 0 ]; then echo "Can't enable engine !"; exit 1; fi
sleep 1
# verify that ref is unlocked
if [ "`dgd ri -S! r1 | grep Locked`" != "Locked!false" ]; then
	echo "r1 is not unlocked"; exit 1
fi
dgd rk r1 >/dev/null 2>&1 
if [ $? -ne 0 ]; then echo "Can't kill the ref !"; exit 1; fi


# delete target and queue
dgd tk tg1 || exit 1

memd2=`dgd memory`
if [ $memd2 -ne $memd1 ] ; then 
	echo "memory delta different for dg87d $memd1 -> $memd2"; exit 1
fi


memx1=`dgx memory`


echo "Test dg87x"

dgx ese false
dgx ta tg1 || exit 1
dgx qa tg1 q1 || exit 1
dgx xda d1 "$dayDate" || exit 1
dgx xta d1 t1 tg1 q1 "task1" true true true || exit 1
dgx xta d1 t2 tg1 q1 "task1" true true true || exit 1
dgx xja d1 t1 j1 "job1" "/bin/ls" "arg" 25 35 true true || exit 1
dgx xja d1 t1 j2 "job2" "/bin/ls" "arg" 25 35 true true || exit 1
dgx xtda d1 t1 t2 always || exit 1
dgx xjda d1 t1 j1 j2 onsuccess || exit 1
dgx xtca d1 t1 file "/tmp/fic" || exit 1 


echo "Verify lock protection"
# try to update task and job state on a locked day
dgx xtu d1 t1 terminated > /dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can update task state on a locked day !"; exit 1; fi
dgx xju d1 t1 j1 terminated > /dev/null 2>&1 
if [ $? -eq 0 ]; then echo "Can update job state on a locked day !"; exit 1; fi
# try to change job enabled property
dgx xjse d1 t1 j1 true > /dev/null 2>&1 
if [ $? -eq 0 ]; then 
	echo "Can change job enabled property on a locked ref !"; exit 1; 
fi
# try to change task enabled property
dgx xtse d1 t1 true > /dev/null 2>&1 
if [ $? -eq 0 ]; then 
	echo "Can change task enabled property on a locked ref !"; exit 1; 
fi
# try to change a job dependence state
dgx xjdu d1 t1 j1 j2 onsuccess validated > /dev/null 2>&1
if [ $? -eq 0 ]; then 
	echo "Can update job dependence state on a locked ref !"; exit 1; 
fi
# try to change a task dependence state
dgx xtdu d1 t1 t2 always validated > /dev/null 2>&1
if [ $? -eq 0 ]; then 
	echo "Can update task dependence state on a locked ref !"; exit 1; 
fi

echo "Validate day"
# validate day
dgx xdv d1

echo "Verify unlock protection"

# try to add a task on a unlocked day
dgx xta d1 tx tg1 q1 "task1" true true true >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task on unlocked day !"; exit 1; fi
dgx xja d1 t1 jx "job1" "/bin/ls" "arg" 25 35 true true >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a job on unlocked day !"; exit 1; fi
dgx xtda d1 t1 t2 always >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task dependence on unlocked day !"; exit 1; fi
dgx xjda d1 t1 j1 j2 onsuccess >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a job dependence on unlocked day !"; exit 1; fi
dgx xtca d1 t2 file "/tmp/fic" >/dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a task condition on unlocked day !"; exit 1; fi

# change enabled property
echo "Verify enabled property for task and job"
dgx xtse d1 t1 false 
if [ $? != 0 ]; then echo "Can't change enabled property of task"; exit 1; fi
if [ "`dgx xti -S! d1 t1 | grep Enabled`" != "Enabled!false" ] ; then
	echo "Enabled property of task is not false"; exit 1
fi
dgx xtse d1 t1 true
if [ $? != 0 ]; then echo "Can't change enabled property of task"; exit 1; fi
if [ "`dgx xti -S! d1 t1 | grep Enabled`" != "Enabled!true" ] ; then
	echo "Enabled property of task is not true"; exit 1
fi
dgx xjse d1 t1 j1 false 
if [ $? != 0 ]; then echo "Can't change enabled property of job"; exit 1; fi
if [ "`dgx xji -S! d1 t1 j1 | grep Enabled`" != "Enabled!false" ] ; then
	echo "Enabled property of job is not false"; exit 1
fi
dgx xjse d1 t1 j1 true
if [ $? != 0 ]; then echo "Can't change enabled property of job"; exit 1; fi
if [ "`dgx xji -S! d1 t1 j1 | grep Enabled`" != "Enabled!true" ] ; then
	echo "Enabled property of job is not true"; exit 1
fi

# delete day
dgx xdk d1 
dgx tk tg1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1
