#!/bin/sh

#
# file tst07.sh 
# (test open/close property and max running jobs for target and queue */
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1


# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1

######################################## target open/close 
echo "Test target open/close property" 
memx1=`dgx memory`
# close target 
dgx tso localhost false
if [ "`dgx ti localhost | grep Opened | awk '{print $2}'`" != "false" ]; then
	echo "Target is not closed."; exit 1
fi
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
# verify that job is not running
if [ "`dgx xji d1 t1 j1 |grep State |awk '{print $2}'`" != "ready_to_submit" ]; 
then
	echo "Job isn't ready_to_submit."; exit 1
fi
if [ "`dgx xti d1 t1 | grep State | awk '{print $2}'`" != "running" ]; then
	echo "Task isn't running."; exit 1
fi
if [ "`dgx qxjc localhost q1`" != "1" ]; then
	echo "Number of running job in queue is not 1 !"; exit 1
fi
if [ "`dgx qxjl localhost q1 -S!`" != "d1!t1!j1" ]; then
	echo "Job isn't in running job list of the queue !"; exit 1
fi
# open the target 
dgx tso localhost true 
if [ "`dgx ti localhost | grep Opened | awk '{print $2}'`" != "true" ]; then
	echo "Target is not opened."; exit 1
fi
sleep 1
# verify that job is terminated
if [ "`dgx xji d1 t1 j1 | grep State | awk '{print $2}'`" != "terminated" ]; 
then
	echo "Job isn't terminated."; exit 1
fi
if [ "`dgx xti d1 t1 | grep State | awk '{print $2}'`" != "terminated" ]; then
	echo "Task isn't terminated."; exit 1
fi
if [ "`dgx xji d1 t1 j1 | grep "Exit code"| awk '{print $3}'`" != "0" ]; then
	echo "Exit code is not good."; exit 1
fi
if [ "`dgx qxjc localhost q1`" != "0" ]; then
	echo "Number of running job in queue is not zero !"; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; 
	#exit 1
fi

######################################## queue open/close 
echo "Test queue open/close property" 
memx1=`dgx memory`
# close queue
dgx qso localhost q1 false
if [ "`dgx qi localhost q1 | grep Opened | awk '{print $2}'`" != "false" ]; then
	echo "Queue is not closed."; exit 1
fi
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
# verify that job is not running
if [ "`dgx xji d1 t1 j1 |grep State |awk '{print $2}'`" != "ready_to_submit" ]; 
then
	echo "Job isn't ready_to_submit."; exit 1
fi
if [ "`dgx xti d1 t1 | grep State | awk '{print $2}'`" != "running" ]; then
	echo "Task isn't running."; exit 1
fi
if [ "`dgx qxjc localhost q1`" != "1" ]; then
	echo "Number of running job in queue is not 1 !"; exit 1
fi
if [ "`dgx qxjl localhost q1 -S!`" != "d1!t1!j1" ]; then
	echo "Job isn't in running job list of the queue !"; exit 1
fi
# open the queue
dgx qso localhost q1 true 
if [ "`dgx qi localhost q1 | grep Opened | awk '{print $2}'`" != "true" ]; then
	echo "Queue is not opened."; exit 1
fi
sleep 1
# verify that job is terminated
if [ "`dgx xji d1 t1 j1 | grep State | awk '{print $2}'`" != "terminated" ]; 
then
	echo "Job isn't terminated."; exit 1
fi
if [ "`dgx xti d1 t1 | grep State | awk '{print $2}'`" != "terminated" ]; then
	echo "Task isn't terminated."; exit 1
fi
if [ "`dgx xji d1 t1 j1 | grep "Exit code"| awk '{print $3}'`" != "0" ]; then
	echo "Exit code is not good."; exit 1
fi
if [ "`dgx qxjc localhost q1`" != "0" ]; then
	echo "Number of running job in queue is not zero !"; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; 
	#exit 1
fi

######################################## queue max running job (max=1)
echo "Test maximum running jobs (max=1)" 
memx1=`dgx memory`
# set maximum running jobs
dgx qsm localhost q1 1 
if [ "`dgx qi localhost q1 | grep Max | awk '{print $2}'`" != "1" ]; then
	echo "Maximum running job isn't 1."; exit 1
fi
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job4.sh"
dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job5.sh"
dgd rja r1 t1 j3 || exit 1
dgd rjsp r1 t1 j3 "$JOBPATH/job6.sh"
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
echo " Verify that first job is running"
# verify that only one job is running
if [ "`dgx xjl d1 t1 -s | grep running | wc -l | awk '{print $1}'`" != "1" ];
	then echo "Number of running jobs isn't 1."; exit 1; fi
# verify that two jobs are waiting
if [ "`dgx xjl d1 t1 -s | grep ready | wc -l | awk '{print $1}'`" != "2" ];
	then echo "Number of waiting jobs isn't 2."; exit 1; fi

sleep 2
echo " Verify that second job is running"
# verify that only one job is running
if [ "`dgx xjl d1 t1 -s | grep running | wc -l | awk '{print $1}'`" != "1" ];
	then echo "Number of running jobs isn't 1."; exit 1; fi
# verify that a job is terminated 
if [ "`dgx xjl d1 t1 -s | grep terminated | wc -l | awk '{print $1}'`" != "1" ];
	then echo "Number of terminated jobs isn't 1."; exit 1; fi
# verify that a job is waiting
if [ "`dgx xjl d1 t1 -s | grep ready | wc -l | awk '{print $1}'`" != "1" ];
	then echo "Number of waiting jobs isn't 1."; exit 1; fi

sleep 3
echo " Verify that third job is running"
# verify that only one job is running
if [ "`dgx xjl d1 t1 -s | grep running | wc -l | awk '{print $1}'`" != "1" ];
	then echo "Number of running jobs isn't 1."; exit 1; fi
# verify that two jobs are terminated 
if [ "`dgx xjl d1 t1 -s | grep terminated | wc -l | awk '{print $1}'`" != "2" ];
	then echo "Number of terminated jobs isn't 2."; exit 1; fi

sleep 2
if [ "`dgx xti d1 t1 | grep State | awk '{print $2}'`" != "terminated" ] ; then
	echo "Task isn't terminated."; exit 1
fi

dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; 
	#exit 1
fi




######################################## queue max running job (max=2)
echo "Test maximum running jobs (max=2)" 
memx1=`dgx memory`
# set maximum running jobs
dgx qsm localhost q1 2 
if [ "`dgx qi localhost q1 | grep Max | awk '{print $2}'`" != "2" ]; then
	echo "Maximum running job isn't 2."; exit 1
fi
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job4.sh"
dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job5.sh"
dgd rja r1 t1 j3 || exit 1
dgd rjsp r1 t1 j3 "$JOBPATH/job6.sh"
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
echo " Verify that two jobs are running"
# verify that two jobs are running
if [ "`dgx xjl d1 t1 -s | grep running | wc -l | awk '{print $1}'`" != "2" ];
	then echo "Number of running jobs isn't 2."; exit 1; fi
# verify that a job is waiting
if [ "`dgx xjl d1 t1 -s | grep ready | wc -l | awk '{print $1}'`" != "1" ];
	then echo "Number of waiting jobs isn't 1."; exit 1; fi
sleep 2
echo " Verify that last job is running"
# verify that only one job is running
if [ "`dgx xjl d1 t1 -s | grep running | wc -l | awk '{print $1}'`" != "1" ];
	then echo "Number of running jobs isn't 1."; exit 1; fi
# verify that two jobs are terminated 
if [ "`dgx xjl d1 t1 -s | grep terminated | wc -l | awk '{print $1}'`" != "2" ];
	then echo "Number of terminated jobs isn't 2."; exit 1; fi

sleep 2
if [ "`dgx xti d1 t1 | grep State | awk '{print $2}'`" != "terminated" ] ; then
	echo "Task isn't terminated."; exit 1
fi

dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; 
	#exit 1
fi





echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1
