#!/bin/sh

#
# file tst08.sh 
# (test task conditions)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1


# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1

######################################## manual condition 
echo "Test manual condition" 
memx1=`dgx memory`
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtca r1 t1 manual "OK"
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtcl d1 t1 -sS!`" != "manual!OK!waiting" ] ; then
	echo "Manual condition is not waiting"; exit 1
fi
if [ "`dgx xtl d1 -sS!`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
dgx xtcv d1 t1 1 || exit 1
if [ "`dgx xtcl d1 t1 -sS!`" != "manual!OK!finished" ] ; then
	echo "Manual condition is not finished"; exit 1
fi
if [ "`dgx xtl d1 -sS!`" != "t1!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## file condition 
echo "Test file condition" 
memx1=`dgx memory`
file="/tmp/fictop$$"
if [ -f $file ]; then
	echo "Ftop $file already exists"; exit 1
fi
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtca r1 t1 file $file 
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtcl d1 t1 -sS!`" != "file!$file!waiting" ] ; then
	echo "File condition is not waiting"; exit 1
fi
if [ "`dgx xtl d1 -sS!`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
touch $file 
if [ $? -ne 0 ]; then
	echo "Can't create file $file"; exit 1
fi
sleep 1
if [ "`dgx xtcl d1 t1 -sS!`" != "file!$file!finished" ] ; then
	echo "File condition is not finished"; rm $file; exit 1
fi
if [ "`dgx xtl d1 -sS!`" != "t1!terminated" ]; then
	echo "Task is not terminated"; rm $file; exit 1
fi
rm $file
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## hmin condition 
echo "Test hmin condition" 
memx1=`dgx memory`
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtca r1 t1 hmin "23:59"
dgd rtca r1 t1 manual OK
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtca r1 t2 hmin "00:00"
dgd rtca r1 t2 manual OK
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
# t1 is waiting
if [ "`dgx xtcl d1 t1 -sS! | grep hmin`" != "hmin!23:59!waiting" ] ; then
	echo "HMin condition is not waiting"; exit 1
fi
if [ "`dgx xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# t2 is finished
if [ "`dgx xtcl d1 t2 -sS! | grep hmin`" != "hmin!00:00!finished" ] ; then
	echo "HMin condition is not finished"; exit 1
fi
if [ "`dgx xtl d1 -sS! | grep t2`" != "t2!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# validate manual condition
dgx xtcv d1 t1 2 || exit 1
dgx xtcv d1 t2 2 || exit 1
sleep 1
if [ "`dgx xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
if [ "`dgx xtl d1 -sS! | grep t2`" != "t2!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## hmax condition 
echo "Test hmax condition" 
memx1=`dgx memory`
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtca r1 t1 hmax "23:59"
dgd rtca r1 t1 manual OK
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtca r1 t2 hmax "00:00"
dgd rtca r1 t2 manual OK
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
# t1 is waiting
if [ "`dgx xtcl d1 t1 -sS! | grep hmax`" != "hmax!23:59!waiting" ] ; then
	echo "HMax condition is not waiting"; exit 1
fi
if [ "`dgx xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# t2 is finished
if [ "`dgx xtcl d1 t2 -sS! | grep hmax`" != "hmax!00:00!finished" ] ; then
	echo "HMax condition is not finished"; exit 1
fi
if [ "`dgx xtl d1 -sS! | grep t2`" != "t2!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


######################################## hlate condition 
echo "Test hlate condition" 
memx1=`dgx memory`
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtca r1 t1 hlate "23:59"
dgd rtca r1 t1 manual OK
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtca r1 t2 hlate "00:00"
dgd rtca r1 t2 manual OK
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
# t1 is waiting
if [ "`dgx xtcl d1 t1 -sS! | grep hlate`" != "hlate!23:59!waiting" ] ; then
	echo "HLate condition is not waiting"; exit 1
fi
if [ "`dgx xtl d1 -sS! | grep t1`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
# t2 is canceled 
if [ "`dgx xtcl d1 t2 -sS! | grep hlate`" != "hlate!00:00!finished" ] ; then
	echo "HLate condition is not finished"; exit 1
fi
if [ "`dgx xtl d1 -sS! | grep t2`" != "t2!canceled" ]; then
	echo "Task is not canceled"; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## force a file condition 
echo "Force a file condition" 
memx1=`dgx memory`
file="/tmp/fictop$$"
if [ -f $file ]; then
	echo "Ftop $file already exists"; exit 1
fi
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtca r1 t1 file $file 
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtcl d1 t1 -sS!`" != "file!$file!waiting" ] ; then
	echo "File condition is not waiting"; exit 1
fi
if [ "`dgx xtl d1 -sS!`" != "t1!wait_for_condition" ]; then
	echo "Task is not waiting for condition"; exit 1
fi
dgx xtcv d1 t1 1 || exit 1
if [ "`dgx xtcl d1 t1 -sS!`" != "file!$file!finished" ] ; then
	echo "File condition is not finished"; exit 1
fi
if [ "`dgx xtl d1 -sS!`" != "t1!terminated" ]; then
	echo "Task is not terminated"; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1
