#!/bin/sh
#
# file tst09.sh 
# (test task dependances)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1

# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1

######################################## test two tasks
echo "Test two tasks" 
memx1=`dgx memory`
dgd ra r1 || exit 1
# first task
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
# second task
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 2
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!terminated" ]; then
	echo "The job of first task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!terminated" ]; then
	echo "First task isn't terminated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t2 | grep j1`" != "j1!terminated" ]; then
	echo "The job of the second task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!terminated" ]; then
	echo "Second task isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## test task dependence (always+norm.term)
echo "Test task dependence (type always with normal termination)" 
memx1=`dgx memory`
dgd ra r1 || exit 1
# first task
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd rtca r1 t1 manual "OK"
# second task
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"
# task dep
dgd rtda r1 t1 t2 always || exit 1
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_for_condition" ]; then
	echo "First task isn't waiting for condition."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!wait_for_dep" ]; then
	echo "Second task isn't waiting for dependence."; exit 1
fi
dgx xtcv d1 t1 1
sleep 2
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!terminated" ]; then
	echo "The job of first task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!terminated" ]; then
	echo "First task isn't terminated."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!validated" ]; then
	echo "Task dependence isn't validated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t2 | grep j1`" != "j1!terminated" ]; then
	echo "The job of the second task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!terminated" ]; then
	echo "Second task isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi
######################################## test task dependence (always+abort)
echo "Test task dependence (type always with abort)" 
memx1=`dgx memory`
dgd ra r1 || exit 1
# first task
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job3.sh"
# second task
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"
# task dep
dgd rtda r1 t1 t2 always || exit 1
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 2
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!aborted" ]; then
	echo "First task isn't aborted."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!validated" ]; then
	echo "Task dependence isn't validated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!terminated" ]; then
	echo "Second task isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## test dependence onsuccess 
echo "Test task dependence (onsuccess with normal termination)" 
memx1=`dgx memory`
dgd ra r1 || exit 1
# first task
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd rtca r1 t1 manual "OK"
# second task
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"
# task dep
dgd rtda r1 t1 t2 onsuccess || exit 1
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_for_condition" ]; then
	echo "First task isn't waiting for condition."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!wait_for_dep" ]; then
	echo "Second task isn't waiting for dependence."; exit 1
fi
dgx xtcv d1 t1 1
sleep 2
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!terminated" ]; then
	echo "The job of first task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!terminated" ]; then
	echo "First task isn't terminated."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!validated" ]; then
	echo "Task dependence isn't validated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t2 | grep j1`" != "j1!terminated" ]; then
	echo "The job of the second task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!terminated" ]; then
	echo "Second task isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


######################################## test task dependence (onsuccess+abort)
echo "Test task dependence (type onsuccess with abort)" 
memx1=`dgx memory`
dgd ra r1 || exit 1
# first task
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job3.sh"
# second task
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"
# task dep
dgd rtda r1 t1 t2 onsuccess || exit 1
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!aborted" ]; then
	echo "First task isn't aborted."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!canceled" ]; then
	echo "Task dependence isn't canceled."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!canceled" ]; then
	echo "Second task isn't canceled."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


######################################## test dependence onerror
echo "Test task dependence (onerror with normal termination)" 
memx1=`dgx memory`
dgd ra r1 || exit 1
# first task
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd rtca r1 t1 manual "OK"
# second task
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"
# task dep
dgd rtda r1 t1 t2 onerror || exit 1
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_for_condition" ]; then
	echo "First task isn't waiting for condition."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!wait_for_dep" ]; then
	echo "Second task isn't waiting for dependence."; exit 1
fi
dgx xtcv d1 t1 1
sleep 2
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!terminated" ]; then
	echo "The job of first task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!terminated" ]; then
	echo "First task isn't terminated."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!canceled" ]; then
	echo "Task dependence isn't canceled."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!canceled" ]; then
	echo "Second task isn't canceled."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


######################################## test dependence onerror
echo "Test task dependence (onerror with aborted termination)" 
memx1=`dgx memory`
dgd ra r1 || exit 1
# first task
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job3.sh"
dgd rtca r1 t1 manual "OK"
# second task
dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"
# task dep
dgd rtda r1 t1 t2 onerror || exit 1
dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_for_condition" ]; then
	echo "First task isn't waiting for condition."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!wait_for_dep" ]; then
	echo "Second task isn't waiting for dependence."; exit 1
fi
dgx xtcv d1 t1 1
sleep 2
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!aborted" ]; then
	echo "The job of first task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!aborted" ]; then
	echo "First task isn't aborted."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!validated" ]; then
	echo "Task dependence isn't validated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t2 | grep j1`" != "j1!terminated" ]; then
	echo "The job of the second task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!terminated" ]; then
	echo "Second task isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1
memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1
