#!/bin/sh
#
# file tst11.sh 
# (test 'ignored' state for job and task)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1

# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1


######################################## job ignored state 
echo "Test Job ignored state" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job1.sh"
dgd rja r1 t1 j3 || exit 1
dgd rjsp r1 t1 j3 "$JOBPATH/job1.sh"
dgd rja r1 t1 j4 || exit 1
dgd rjsp r1 t1 j4 "$JOBPATH/job1.sh"
dgd rja r1 t1 j5 || exit 1
dgd rjsp r1 t1 j5 "$JOBPATH/job1.sh"

dgd rjda r1 t1 j1 j2 always || exit 1
dgd rjda r1 t1 j2 j3 always || exit 1
dgd rjda r1 t1 j2 j4 onsuccess || exit 1
dgd rjda r1 t1 j2 j5 onerror || exit 1

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xju d1 t1 j2 ignore 
if [ $? -ne 0 ]; then echo "Can't ignore job."; exit 1; fi
if [ "`dgx xjl -sS! d1 t1 | grep j2`" != "j2!ignore" ]; then
	echo "The job isn't in ignore state."; exit 1
fi

dgx xdse d1 true || exit 1
sleep 1
# verify dependences
if [ "`dgx xjdl -sS! d1 t1 | grep j1`" != "j1!j2!validated" ]; then
	echo "The dependence j1-j2 isn't validated."; exit 1
fi
if [ "`dgx xjdl -sS! d1 t1 | grep j3`" != "j2!j3!validated" ]; then
	echo "The dependence j2-j3 isn't validated."; exit 1
fi
if [ "`dgx xjdl -sS! d1 t1 | grep j4`" != "j2!j4!validated" ]; then
	echo "The dependence j2-j4 isn't validated."; exit 1
fi
if [ "`dgx xjdl -sS! d1 t1 | grep j5`" != "j2!j5!canceled" ]; then
	echo "The dependence j2-j5 isn't canceled."; exit 1
fi


# verify jobs
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!terminated" ]; then
	echo "The first job isn't terminated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j2`" != "j2!ignored" ]; then
	echo "The second job isn't ignored."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j3`" != "j3!terminated" ]; then
	echo "The job (always) isn't terminated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j4`" != "j4!terminated" ]; then
	echo "The job (onsuccess) isn't terminated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j5`" != "j5!canceled" ]; then
	echo "The job (onerror) isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## task ignored state 
echo "Test Task ignored state" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"

dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"

dgd rta r1 t3 || exit 1
dgd rtoa r1 t3 add always || exit 1
dgd rtsq r1 t3 localhost q1 || exit 1
dgd rja r1 t3 j1 || exit 1
dgd rjsp r1 t3 j1 "$JOBPATH/job1.sh"

dgd rta r1 t4 || exit 1
dgd rtoa r1 t4 add always || exit 1
dgd rtsq r1 t4 localhost q1 || exit 1
dgd rja r1 t4 j1 || exit 1
dgd rjsp r1 t4 j1 "$JOBPATH/job1.sh"

dgd rta r1 t5 || exit 1
dgd rtoa r1 t5 add always || exit 1
dgd rtsq r1 t5 localhost q1 || exit 1
dgd rja r1 t5 j1 || exit 1
dgd rjsp r1 t5 j1 "$JOBPATH/job1.sh"

dgd rtda r1 t1 t2 always || exit 1
dgd rtda r1 t2 t3 always || exit 1
dgd rtda r1 t2 t4 onsuccess || exit 1
dgd rtda r1 t2 t5 onerror || exit 1

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xtu d1 t2 ignore 
if [ $? -ne 0 ]; then echo "Can't ignore task."; exit 1; fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!ignore" ]; then
	echo "The task isn't in ignore state."; exit 1
fi

dgx xdse d1 true || exit 1
sleep 1

# verify dependences
if [ "`dgx xtdl -sS! d1 | grep t1`" != "t1!t2!validated" ]; then
	echo "The dependence t1-t2 isn't validated."; exit 1
fi
if [ "`dgx xtdl -sS! d1 | grep t3`" != "t2!t3!validated" ]; then
	echo "The dependence t2-t3 isn't validated."; exit 1
fi
if [ "`dgx xtdl -sS! d1 | grep t4`" != "t2!t4!validated" ]; then
	echo "The dependence t2-t4 isn't validated."; exit 1
fi
if [ "`dgx xtdl -sS! d1 | grep t5`" != "t2!t5!canceled" ]; then
	echo "The dependence t2-t5 isn't canceled."; exit 1
fi


# verify jobs
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!terminated" ]; then
	echo "The first task isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!ignored" ]; then
	echo "The second task isn't ignored."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t3`" != "t3!terminated" ]; then
	echo "The task (always) isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t4`" != "t4!terminated" ]; then
	echo "The task (onsuccess) isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t5`" != "t5!canceled" ]; then
	echo "The task (onerror) isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi



echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1


