#!/bin/sh
#
# file tst12.sh 
# (test job tempo)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1

# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1

######################################## job start tempo
echo "Test Job start tempo" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job4.sh"
dgd rjst r1 t1 j1 2 0

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
# verify job is waiting 
echo " verify that job is waiting"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!wait_start_tempo" ]; then
	echo "The job isn't waiting for start tempo."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!running" ]; then
	echo "The task isn't running (while start tempo)."; exit 1
fi
sleep 2
# verify job is running 
echo " verify that job is running"
dgx xjl -sS! d1 t1 | grep j1
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!running" ]; then
	echo "The job isn't running."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!running" ]; then
	echo "The task isn't running (while job is running)."; exit 1
fi
sleep 3
# verify job is terminated 
echo " verify that job is terminated"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!terminated" ]; then
	echo "The job isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!terminated" ]; then
	echo "The task isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


######################################## job end tempo
echo "Test Job end tempo (with normal termination)" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job4.sh"
dgd rjst r1 t1 j1 0 3

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1

# verify job is running 
echo " verify that job is running"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!running" ]; then
	echo "The job isn't running."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!running" ]; then
	echo "The task isn't running (while job is running)."; exit 1
fi
sleep 2
# verify job is waiting 
echo " verify that job is waiting"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!wait_end_tempo" ]; then
	echo "The job isn't waiting for end tempo."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!running" ]; then
	echo "The task isn't running (while start tempo)."; exit 1
fi
sleep 4
# verify job is terminated 
echo " verify that job is terminated"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!terminated" ]; then
	echo "The job isn't terminated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!terminated" ]; then
	echo "The task isn't terminated."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## job end tempo with abort
echo "Test Job end tempo (with abort)" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job9.sh"
dgd rjst r1 t1 j1 0 3

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true || exit 1
sleep 1

# verify job is running 
echo " verify that job is running"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!running" ]; then
	echo "The job isn't running."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!running" ]; then
	echo "The task isn't running (while job is running)."; exit 1
fi
sleep 2
# verify job is waiting 
echo " verify that job is waiting"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!wait_end_tempo" ]; then
	echo "The job isn't waiting for end tempo."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!running" ]; then
	echo "The task isn't running (while start tempo)."; exit 1
fi
sleep 4
# verify job is aborted 
echo " verify that job is aborted"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!aborted" ]; then
	echo "The job isn't aborted."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!aborted" ]; then
	echo "The task isn't aborted."; exit 1
fi
dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi



echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1


