#!/bin/sh
#
# file tst13.sh 
# (test xtdu and xjdu commands)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1

# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1

######################################## validate a job dependence 
echo "Test manual job dependence validation" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job1.sh"
dgd rjda r1 t1 j1 j2 always

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xjse d1 t1 j1 false || exit 1
dgx xdse d1 true || exit 1

# verify job is waiting 
echo " verify that jobs are waiting"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!wait_enabled" ]; then
	echo "The first job isn't waiting."; exit 1
fi
if [ "`dgx xjdl -sS! d1 t1 `" != "j1!j2!waiting" ]; then
	echo "The job dependence isn't waiting."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j2`" != "j2!wait_for_dep" ]; then
	echo "The second job isn't waiting."; exit 1
fi
echo " force dependence validation"
dgx xjdu d1 t1 j1 j2 always validated || exit 1
sleep 1
echo " verify that second job is terminated"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!wait_enabled" ]; then
	echo "The first job isn't waiting."; exit 1
fi
if [ "`dgx xjdl -sS! d1 t1 `" != "j1!j2!validated" ]; then
	echo "The job dependence isn't validated."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j2`" != "j2!terminated" ]; then
	echo "The second job isn't terminated."; exit 1
fi

dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## cancel a job dependence 
echo "Test manual job dependence cancellation" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"
dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job1.sh"
dgd rjda r1 t1 j1 j2 always

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xjse d1 t1 j1 false || exit 1
dgx xdse d1 true || exit 1

# verify job is waiting 
echo " verify that jobs are waiting"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!wait_enabled" ]; then
	echo "The first job isn't waiting."; exit 1
fi
if [ "`dgx xjdl -sS! d1 t1 `" != "j1!j2!waiting" ]; then
	echo "The job dependence isn't waiting."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j2`" != "j2!wait_for_dep" ]; then
	echo "The second job isn't waiting."; exit 1
fi
echo " force dependence cancellation"
dgx xjdu d1 t1 j1 j2 always canceled || exit 1
sleep 1

echo " verify that second job is canceled"
if [ "`dgx xjl -sS! d1 t1 | grep j1`" != "j1!wait_enabled" ]; then
	echo "The first job isn't waiting."; exit 1
fi
if [ "`dgx xjdl -sS! d1 t1 `" != "j1!j2!canceled" ]; then
	echo "The job dependence isn't canceled."; exit 1
fi
if [ "`dgx xjl -sS! d1 t1 | grep j2`" != "j2!canceled" ]; then
	echo "The second job isn't canceled."; exit 1
fi

dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## validate a task dependence 
echo "Test manual task dependence validation" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"

dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"

dgd rtda r1 t1 t2 always

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xtse d1 t1 false || exit 1
dgx xdse d1 true || exit 1

# verify task is waiting 
echo " verify that tasks are waiting"
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_enabled" ]; then
	echo "The first task isn't waiting."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!waiting" ]; then
	echo "The task dependence isn't waiting."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!wait_for_dep" ]; then
	echo "The second task isn't waiting."; exit 1
fi

echo " force dependence validation"
dgx xtdu d1 t1 t2 always validated || exit 1
sleep 1

echo " verify that second task is terminated"
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_enabled" ]; then
	echo "The first task isn't waiting."; exit 1
fi
if [ "`dgx xtdl -sS! d1 `" != "t1!t2!validated" ]; then
	echo "The task dependence isn't validated."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!terminated" ]; then
	echo "The second task isn't terminated."; exit 1
fi

dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## cancel a task dependence 
echo "Test manual task dependence cancellation" 
memx1=`dgx memory`
dgd ra r1 || exit 1

dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1
dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job1.sh"

dgd rta r1 t2 || exit 1
dgd rtoa r1 t2 add always || exit 1
dgd rtsq r1 t2 localhost q1 || exit 1
dgd rja r1 t2 j1 || exit 1
dgd rjsp r1 t2 j1 "$JOBPATH/job1.sh"

dgd rtda r1 t1 t2 always

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xtse d1 t1 false || exit 1
dgx xdse d1 true || exit 1

# verify task is waiting 
echo " verify that tasks are waiting"
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_enabled" ]; then
	echo "The first task isn't waiting."; exit 1
fi
if [ "`dgx xtdl -sS! d1`" != "t1!t2!waiting" ]; then
	echo "The task dependence isn't waiting."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!wait_for_dep" ]; then
	echo "The second task isn't waiting."; exit 1
fi

echo " force dependence cancellation"
dgx xtdu d1 t1 t2 always canceled || exit 1
sleep 1

echo " verify that second task is cancelled"
if [ "`dgx xtl -sS! d1 | grep t1`" != "t1!wait_enabled" ]; then
	echo "The first task isn't waiting."; exit 1
fi
if [ "`dgx xtdl -sS! d1 `" != "t1!t2!canceled" ]; then
	echo "The task dependence isn't canceled."; exit 1
fi
if [ "`dgx xtl -sS! d1 | grep t2`" != "t2!canceled" ]; then
	echo "The second task isn't canceled."; exit 1
fi

dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1


