#!/bin/sh
#
# file tst14.sh 
# (test ejm commands)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1

# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx qsm localhost q1 99 | exit 1 
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1
dgd qsm localhost q1 99 | exit 1 

######################################## 
echo "Test monitor commands" 
memx1=`dgx memory`

# first referential
# j1 -> (j2,j3,j4)
dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1

dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/job4.sh"

dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job4.sh"
dgd rjda r1 t1 j1 j2 always

dgd rja r1 t1 j3 || exit 1
dgd rjsp r1 t1 j3 "$JOBPATH/job4.sh"
dgd rjda r1 t1 j2 j3 onerror 

dgd rja r1 t1 j4 || exit 1
dgd rjsp r1 t1 j4 "$JOBPATH/job4.sh"
dgd rjda r1 t1 j2 j4 onsuccess

# second referential
# (j2,j2)->j3
# first task
dgd ra r2 || exit 1
dgd rta r2 t1 || exit 1
dgd rtoa r2 t1 add always || exit 1
dgd rtsq r2 t1 localhost q1 || exit 1

dgd rja r2 t1 j1 || exit 1
dgd rjsp r2 t1 j1 "$JOBPATH/job4.sh"

dgd rja r2 t1 j2 || exit 1
dgd rjsp r2 t1 j2 "$JOBPATH/job4.sh"

dgd rja r2 t1 j3 || exit 1
dgd rjsp r2 t1 j3 "$JOBPATH/job4.sh"

dgd rjda r2 t1 j1 j3 always 
dgd rjda r2 t1 j2 j3 always

# second task
dgd rta r2 t2 || exit 1
dgd rtoa r2 t2 add always || exit 1
dgd rtsq r2 t2 localhost q1 || exit 1

# j1 aborts after 2 seconds
dgd rja r2 t2 j1 || exit 1
dgd rjsp r2 t2 j1 "$JOBPATH/job9.sh"

dgd rja r2 t2 j2 || exit 1
dgd rjsp r2 t2 j2 "$JOBPATH/job4.sh"

dgd rjda r2 t2 j1 j2 always 

dgd rtda r2 t1 t2 always

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgd rp r2 d2 "$dayDate" || exit 1
sleep 1

# hold first job
dgx xjse d1 t1 j1 false || exit 1
# hold second task
dgx xtse d2 t2 false || exit 1
dgx xdse d1 true || exit 1
dgx xdse d2 true || exit 1

######################################### Step 1 
sleep 1
echo "  step 1"
# verify that d1!t1!j1 is waiting
if [ "`dgx ejm -hS! | grep \"d1!t1!j1\"`" != "H!d1!t1!j1!wait_enabled" ];
then
	echo "Job d1!t1!j1 is not wait_enabled."; exit 1;
fi
# active d1!t1!j1
dgx xjse d1 t1 j1 true || exit 1

# verify that j1 and j2 are running
if [ "`dgx ejm -rS! | grep \"d2!t1!j1\"`" != "R!d2!t1!j1!running" ];
then
	echo "Job d2!t1!j1 is not running."; exit 1;
fi
if [ "`dgx ejm -rS! | grep \"d2!t1!j2\"`" != "R!d2!t1!j2!running" ];
then
	echo "Job d2!t1!j2 is not running."; exit 1;
fi
######################################### Step 2
sleep 2
echo "  step 2"
# verify that j1 and j2 are terminated 
if [ "`dgx ejm -tS! | grep \"d2!t1!j1\"`" != "T!d2!t1!j1!terminated" ];
then
	echo "Job d2!t1!j1 is not terminated."; exit 1;
fi
if [ "`dgx ejm -tS! | grep \"d2!t1!j2\"`" != "T!d2!t1!j2!terminated" ];
then
	echo "Job d2!t1!j2 is not terminated."; exit 1;
fi
# verify that j3 is running
if [ "`dgx ejm -rS! | grep \"d2!t1!j3\"`" != "R!d2!t1!j3!running" ];
then
	echo "Job d2!t1!j3 is not running."; exit 1;
fi

######################################### Step 3
sleep 2
echo "  step 3"
if [ "`dgx etm -tS! | grep \"d2!t1\"`" != "T!d2!t1!terminated" ];
then
	echo "Task d2!t1 isn't terminated."; exit 1;
fi
if [ "`dgx etm -hS! | grep \"d2!t2\"`" != "H!d2!t2!wait_enabled" ];
then
	echo "Task d2!t2 isn't wait_enabled."; exit 1;
fi
# d2!t2 activation
dgx xtse d2 t2 true || exit 1

######################################### Step 4
sleep 3
echo "  step 4"

# verify that d2!t2!j1 is aborted
if [ "`dgx ejm -eS! | grep \"d2!t2!j1\"`" != "E!d2!t2!j1!aborted" ];
then
	echo "Job d2!t2!j1 is not aborted."; exit 1;
fi

######################################### Step 5
sleep 3
echo "  step 5"
# verify that d2!t2 is aborted
if [ "`dgx etm -eS! | grep \"d2!t2\"`" != "E!d2!t2!aborted" ];
then
	echo "Task d2!t2 is not aborted."; exit 1;
fi


dgd rk r1 || exit 1
dgd rk r2 || exit 1
dgx xdk d1 || exit 1
dgx xdk d2 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1


