#!/bin/sh
#
# file tst15.sh 
# (test 'deleted' state)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#

if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1
echo "Start dg87x"
startDg87x || exit 1
echo "Start dg87r"
startDg87r || exit 1

# dg87x
dgx ta localhost || exit 1
dgx qa localhost q1 || exit 1
dgx tso localhost true || exit 1
dgx qso localhost q1 true || exit 1
dgx qsm localhost q1 99 | exit 1 
dgx ese true || exit 1

# dg87d
dgd ta localhost || exit 1
dgd qa localhost q1 || exit 1
dgd qsm localhost q1 99 | exit 1 

######################################## 
echo "Test deleted state for job" 
memx1=`dgx memory`

dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1

dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/jobxxx.sh"

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true
sleep 1

if [ "`dgx xjl d1 t1 -sS!`" != "j1!deleted" ]; then
	echo "Job is not deleted."; exit 1;
fi
if [ "`dgx xtl d1 -sS!`" != "t1!aborted" ]; then
	echo "Task is not aborted."; exit 1;
fi


dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi

######################################## 
echo "Test dependences with deleted state" 
memx1=`dgx memory`

dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1

dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/jobxxx.sh"

dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job5.sh"
dgd rjda r1 t1 j1 j2 always || exit 1

dgd rja r1 t1 j3 || exit 1
dgd rjsp r1 t1 j3 "$JOBPATH/job5.sh"
dgd rjda r1 t1 j1 j3 onerror || exit 1

dgd rja r1 t1 j4 || exit 1
dgd rjsp r1 t1 j4 "$JOBPATH/job5.sh"
dgd rjda r1 t1 j1 j4 onsuccess || exit 1

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true
sleep 1

if [ "`dgx xjl d1 t1 -sS! | grep j1`" != "j1!deleted" ]; then
	echo "Job1 is not deleted."; exit 1;
fi
if [ "`dgx xjl d1 t1 -sS! | grep j2`" != "j2!running" ]; then
	echo "Job2 is not deleted."; exit 1;
fi
if [ "`dgx xjl d1 t1 -sS! | grep j3`" != "j3!running" ]; then
	echo "Job3 is not deleted."; exit 1;
fi
if [ "`dgx xjl d1 t1 -sS! | grep j4`" != "j4!canceled" ]; then
	echo "Job4 is not canceled."; exit 1;
fi
if [ "`dgx xtl d1 -sS! | grep t1`" != "t1!running" ]; then
	echo "Task1 is not running."; exit 1;
fi
sleep 2
if [ "`dgx xjl d1 t1 -sS! | grep j2`" != "j2!terminated" ]; then
	echo "Job2 is not terminated."; exit 1;
fi
if [ "`dgx xjl d1 t1 -sS! | grep j3`" != "j3!terminated" ]; then
	echo "Job3 is not terminated."; exit 1;
fi
if [ "`dgx xtl d1 -sS! | grep t1`" != "t1!aborted" ]; then
	echo "Task1 is not aborted."; exit 1;
fi

dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi


######################################## 
echo "Test onErrorContinue with deleted state for job" 
memx1=`dgx memory`

dgd ra r1 || exit 1
dgd rta r1 t1 || exit 1
dgd rtoa r1 t1 add always || exit 1
dgd rtsq r1 t1 localhost q1 || exit 1

dgd rja r1 t1 j1 || exit 1
dgd rjsp r1 t1 j1 "$JOBPATH/jobxxx.sh"
dgd rjsoec r1 t1 j1 false || exit 1

dgd rja r1 t1 j2 || exit 1
dgd rjsp r1 t1 j2 "$JOBPATH/job5.sh"
dgd rjda r1 t1 j1 j2 always || exit 1

dgd ese true || exit 1
dgd rp r1 d1 "$dayDate" || exit 1
sleep 1
dgx xdse d1 true
sleep 1

if [ "`dgx xjl d1 t1 -sS! | grep j1`" != "j1!delete_stopped" ]; then
	echo "Job1 is not delete_stopped."; exit 1;
fi
if [ "`dgx xjl d1 t1 -sS! | grep j2`" != "j2!wait_for_dep" ]; then
	echo "Job2 is not waiting for dependence."; exit 1;
fi
if [ "`dgx xtl d1 -sS!`" != "t1!running" ]; then
	echo "Task is not running."; exit 1;
fi


dgd rk r1 || exit 1
dgx xdk d1 || exit 1

memx2=`dgx memory`
if [ $memx2 -ne $memx1 ] ; then 
	echo "memory delta different for dg87x $memx1 -> $memx2"; exit 1
fi






echo "Stop dg87r"
stopDg87r || exit 1
echo "Stop dg87d"
stopDg87d || exit 1
echo "Stop dg87x"
stopDg87x || exit 1


