#!/bin/sh
#
# file tst21.sh 
# (test circular reference detection)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi


################################################## circular reference for task
echo "test circular reference detection for tasks"
echo " step 1"
# t1->t2 : t2->t1?
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rta ref t2 || exit 1
dgd rtda ref t1 t2 || exit 1
dgd rtda ref t2 t1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1

echo " step 2"
# t1->t2->t3 : t3->t1?
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rta ref t2 || exit 1
dgd rta ref t3 || exit 1
dgd rtda ref t1 t2 || exit 1
dgd rtda ref t2 t3 || exit 1
dgd rtda ref t3 t1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1

echo " step 3"
# t1->t2->t3->t4 : t4->t1?
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rta ref t2 || exit 1
dgd rta ref t3 || exit 1
dgd rta ref t4 || exit 1
dgd rtda ref t1 t2 || exit 1
dgd rtda ref t2 t3 || exit 1
dgd rtda ref t3 t4 || exit 1
dgd rtda ref t4 t1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1

echo " step 4"
# (t1,t2)->t3->(t4,t5) : t5->t1? t4->t2
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rta ref t2 || exit 1
dgd rta ref t3 || exit 1
dgd rta ref t4 || exit 1
dgd rta ref t5 || exit 1
dgd rtda ref t1 t3 || exit 1
dgd rtda ref t2 t3 || exit 1
dgd rtda ref t3 t4 || exit 1
dgd rtda ref t3 t5 || exit 1 
dgd rtda ref t5 t1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rtda ref t4 t2 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1


################################################## circular reference for job 
echo "test circular reference detection for jobs"
echo " step 1"
# j1->j2 : j2->j1?
dgd ra ref || exit 1
dgd rta ref t || exit 1
dgd rja ref t j1 || exit 1
dgd rja ref t j2 || exit 1
dgd rjda ref t j1 j2 || exit 1
dgd rjda ref t j2 j1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1

echo " step 2"
# j1->j2->j3 : j3->j1?
dgd ra ref || exit 1
dgd rta ref t || exit 1
dgd rja ref t j1 || exit 1
dgd rja ref t j2 || exit 1
dgd rja ref t j3 || exit 1
dgd rjda ref t j1 j2 || exit 1
dgd rjda ref t j2 j3 || exit 1
dgd rjda ref t j3 j1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1

echo " step 3"
# j1->j2->j3->j4 : j4->j1?
dgd ra ref || exit 1
dgd rta ref t || exit 1
dgd rja ref t j1 || exit 1
dgd rja ref t j2 || exit 1
dgd rja ref t j3 || exit 1
dgd rja ref t j4 || exit 1
dgd rjda ref t j1 j2 || exit 1
dgd rjda ref t j2 j3 || exit 1
dgd rjda ref t j3 j4 || exit 1
dgd rjda ref t j4 j1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1

echo " step 4"
# (j1,j2)->j3->(j4,j5) : j5->j1? j4->j2
dgd ra ref || exit 1
dgd rta ref t || exit 1
dgd rja ref t j1 || exit 1
dgd rja ref t j2 || exit 1
dgd rja ref t j3 || exit 1
dgd rja ref t j4 || exit 1
dgd rja ref t j5 || exit 1
dgd rjda ref t j1 j3 || exit 1
dgd rjda ref t j2 j3 || exit 1
dgd rjda ref t j3 j4 || exit 1
dgd rjda ref t j3 j5 || exit 1 
dgd rjda ref t j5 j1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rjda ref t j4 j2 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "circular reference was not detected."; exit 1
fi
dgd rk ref || exit 1




# verify memory
mem2=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

echo "Stop dg87d"
stopDg87d || exit 1
