#!/bin/sh
#
# file tst24.sh 
# (test user functions) 
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied wartanty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

################################################## 
echo "Verify system users" 

# verify system users
if [ "`dgd uc`" -ne 2 ] ; then
	echo "Can't find two system users."; exit 1
fi
if [ "`dgd ul | grep diogene | wc -l`" -ne 1 ]; then
	echo "Can't find 'diogene' user."; exit 1
fi
if [ "`dgd ul | grep admin | wc -l`" -ne 1 ]; then
	echo "Can't find 'admin' user."; exit 1
fi
# try to delete system user
dgd uk diogene > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete 'diogene' user !"; exit 1
fi
if [ "`dgd ul | grep diogene | wc -l`" -ne 1 ]; then
	echo "'diogene' user is killed."; exit 1
fi
dgd uk admin > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete 'admin' user !"; exit 1
fi
if [ "`dgd ul | grep admin | wc -l`" -ne 1 ]; then
	echo "'admin' user is killed."; exit 1
fi

# try to change system user privileges
for family in generic communication user engine target referential job day 
do
	dgd usp admin none $family > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "Can change privilege of 'admin' system user"; exit 1
	fi
	dgd usp diogene none $family > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "Can change privilege of 'diogene' system user"; exit 1
	fi
done

################################################## 
echo "Test users commands" 

# add user
dgd ua user1
if [ $? -ne 0 ]; then echo "Can't add user."; exit 1; fi
if [ "`dgd uc`" -ne 3 ] ; then
	echo "Users count isn't 3."; exit 1
fi
if [ "`dgd ul | grep user1 | wc -l`" -ne 1 ]; then
	echo "Can't find 'user1' user."; exit 1
fi
# add another user with the same name
dgd ua user1 > /dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a second user with the same name !"; exit 1;fi
if [ "`dgd uc`" -ne 3 ] ; then
	echo "Users count isn't 3."; exit 1
fi
# add a second user
dgd ua user2
if [ $? -ne 0 ]; then echo "Can't add a second user."; exit 1; fi
if [ "`dgd uc`" -ne 4 ] ; then
	echo "Users count isn't 4."; exit 1
fi
if [ "`dgd ul | grep user2 | wc -l`" -ne 1 ]; then
	echo "Can't find 'user2' user."; exit 1
fi

# delete user
dgd uk user1
if [ $? -ne 0 ]; then echo "Can't delete user."; exit 1; fi
if [ "`dgd uc`" -ne 3 ] ; then
	echo "Users count isn't 3."; exit 1
fi
dgd uk user2 || exit 1


################################################## 
echo "Try to kill a connected user" 

dgd ua user || exit 1
# user sends a command with a tempo of 1 second
$PathDgd -u user -p "" -w 1 help >/dev/null 2>&1 & 
if [ "`dgd ul -cS! | grep user`" != "user!1" ]; then
	echo "User isn't connected !"; exit 1
fi
dgd uk user > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a connected user !"; exit 1
fi
if [ "`dgd ul | grep user | wc -l`" -ne 1 ]; then
	echo "Can't find user !"; exit 1
fi
sleep 2
if [ "`dgd ul -cS! | grep user`" != "user!0" ]; then
	echo "User is still connected !"; exit 1
fi
dgd uk user || exit 1 

# verify memory
mem2=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test user privileges change" 
dgd ua user || exit 1 
# test user privileges change 
for privilege in none consult manage admin 
do
for family in generic communication user engine target referential job day 
do
	dgd usp user $privilege $family 
	if [ $? -eq 1 ]; then
		echo "Can change privilege of a user"; exit 1
	fi
	if [ "`dgd ui user -S! | grep \"$family!$privilege\" | wc -l`" -ne 1 ]
	then
		echo "Privilege $privilege for family $family isn't changed"
		exit 1
	fi	
done
done
dgd uk user || exit 1 


echo "test 'referential' family"
dgd ua user || exit 1 
echo "  'none' privilege"
dgd usp user none referential || exit 1
lstcmd="`$PathDgd -u user -p \"\"  -- help`"
# authorized functions
for cmd in help version; do
	echo "$lstcmd" | grep $cmd > /dev/null 2>&1
	if [ $? -eq 1 ]; then
		echo "Error $cmd is not available"
		exit 1
	fi
done
# unauthorized functions
for cmd in rc rl ra rk rta rtc rtl rta rtk rtsd; do
	echo "$lstcmd" | grep $cmd > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "Error $cmd is available"
		exit 1
	fi
done

echo "  'consult' privilege"
dgd usp user consult referential || exit 1
lstcmd="`$PathDgd -u user -p \"\"  -- help`"
# authorized functions
for cmd in help version rc rl rtc rtl; do
	echo "$lstcmd" | grep $cmd > /dev/null 2>&1
	if [ $? -eq 1 ]; then
		echo "Error $cmd is not available"
		exit 1
	fi
done
# unauthorized functions
for cmd in ra rk rta rtk rtsd ; do
	echo "$lstcmd" | grep $cmd > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "Error $cmd is available"
		exit 1
	fi
done

echo "  'manage' privilege"
dgd usp user manage referential || exit 1
lstcmd="`$PathDgd -u user -p \"\"  -- help`"
# authorized functions
for cmd in help version rc rl rtc rtl rtsd; do
	echo "$lstcmd" | grep $cmd > /dev/null 2>&1
	if [ $? -eq 1 ]; then
		echo "Error $cmd is not available"
		exit 1
	fi
done
# unauthorized functions
for cmd in ra rk rta rtk; do
	echo "$lstcmd" | grep $cmd > /dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "Error $cmd is available"
		exit 1
	fi
done

echo "  'admin' privilege"
dgd usp user admin referential || exit 1
lstcmd="`$PathDgd -u user -p \"\"  -- help`"
# authorized functions
for cmd in help version ra rk rc rl rtc rtl rtsd; do
	echo "$lstcmd" | grep $cmd > /dev/null 2>&1
	if [ $? -eq 1 ]; then
		echo "Error $cmd is not available"
		exit 1
	fi
done

dgd uk user || exit 1 


#
################################################## 
echo "Test password change" 
dgd ua user || exit 1 


# access with blank password
$PathDgd -u user -p ""  -- help > /dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "User can't access with blank password"; exit 1
fi

# admin change password
dgd password user zzz
if [ $? -ne 0 ]; then
	echo "Admin can't change user password"; exit 1
fi

# try access with old password
$PathDgd -u user -p ""  -- help > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "User can access with old password"; exit 1
fi
# access with new password
$PathDgd -u user -p "zzz"  -- help > /dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "User can't access with new password"; exit 1
fi
# user changes his password by himself
$PathDgd -u user -p "zzz"  -- password yyy > /dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "User can't change his password"; exit 1
fi
# try access with old password
$PathDgd -u user -p "zzz"  -- help > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "User can access with old password"; exit 1
fi
# access with new password
$PathDgd -u user -p "yyy"  -- help > /dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "User can't access with new password"; exit 1
fi
dgd uk user || exit 1 


echo "Stop dg87d"
stopDg87d || exit 1
