#!/bin/sh
#
# file tst25.sh 
# (test variable for dg87d) 
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied wartanty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

echo "Start dg87d"
startDg87d || exit 1

################################################## 
echo "Test variables commands (dg87d)" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

# add variable 
dgd va var1
if [ $? -ne 0 ]; then echo "Can't add a variable."; exit 1; fi
if [ "`dgd vc`" -ne 1 ] ; then
	echo "Variables count isn't 1."; exit 1
fi
if [ "`dgd vl | grep var1 | wc -l`" -ne 1 ]; then
	echo "Can't find 'var1' variable."; exit 1
fi
# add another variable with the same name
dgd va var1 > /dev/null 2>&1
if [ $? -eq 0 ]; then echo "Can add a second variable with the same name !"; exit 1;fi
if [ "`dgd vc`" -ne 1 ] ; then
	echo "Variable count isn't 1."; exit 1
fi
# add a second variable 
dgd va var2
if [ $? -ne 0 ]; then echo "Can't add a second variable."; exit 1; fi
if [ "`dgd vc`" -ne 2 ] ; then
	echo "Variables count isn't 2."; exit 1
fi
if [ "`dgd vl | grep var2 | wc -l`" -ne 1 ]; then
	echo "Can't find 'var2' variable."; exit 1
fi

# delete variable
dgd vk var1
if [ $? -ne 0 ]; then echo "Can't delete variable."; exit 1; fi
if [ "`dgd vc`" -ne 1 ] ; then
	echo "Users count isn't 1."; exit 1
fi
dgd vk var2 || exit 1

# add a variable with an initial value
dgd va var1 value1
if [ $? -ne 0 ]; then echo "Can't add a variable with initial value."; exit 1; fi
if [ "`dgd vc`" -ne 1 ] ; then
	echo "Variables count isn't 1."; exit 1
fi
if [ "`dgd vl -vS! | grep var1`" != "var1!value1" ]; then
	echo "Can't find 'var1' variable with initial value."; exit 1
fi
if [ "`dgd vi -S! var1 | grep Value`" != "Value!value1" ]; then
	echo "Can't find 'var1' initial value."; exit 1
fi

# change value of initial variable
dgd vsv var1 value2
if [ $? -ne 0 ]; then echo "Can't change variable value."; exit 1; fi
if [ "`dgd vi -S! var1 | grep Value`" != "Value!value2" ]; then
	echo "Can't find 'var1' value."; exit 1
fi
dgd vk var1 || exit 1

# verify memory
mem2=`dgd memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_equal rule operations (dg87d)" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dgd ra ref || exit 1
dgd va v1 || exit 1
dgd rra ref r1 || exit 1

# try to add a 'if_equal' operation whith unknown variable
dgd rroa ref r1 add if_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_equal' operation
dgd rroa ref r1 add if_equal v1 33 
if [ "`dgd rrol ref r1 -S! | grep v1`" != "add!if_equal!v1!33" ]; then
	echo "Can't find if_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dgd vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with rules !"; exit 1
fi
# verify that variable is not deleted
if [ "`dgd vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dgd rrok ref r1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dgd vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with rules !"; exit 1
fi
# verify that variable is deleted
if [ "`dgd vl | grep v1`" == "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dgd rk ref || exit 1

# verify that variable is unlinked after referential deletion
dgd va v2 || exit 1
dgd ra ref || exit 1
dgd rra ref r1 || exit 1
dgd rroa ref r1 add if_equal v2 5 || exit 1
dgd rra ref r2 || exit 1
dgd rroa ref r2 add if_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dgd rk ref || exit 1
# verify that variable is not linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v2 variable"; exit 1
fi
dgd vk v2 || exit 1


# test 'if_equal' operation with 'add' and 'remove' operators
dgd va v1 || exit 1
dgd ra ref || exit 1

echo "  test 'add' operator"
dgd rra ref r1 || exit 1
dgd rroa ref r1 add if_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rrsr ref r1 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rrsr ref r1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi

echo "  test 'remove' operator"
dgd rra ref r2 || exit 1
dgd rroa ref r2 add always || exit 1
dgd rroa ref r2 remove if_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rrsr ref r2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rrsr ref r2 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi
dgd rk ref || exit 1
dgd vk v1 || exit 1

# verify memory
mem2=`dgd memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_not_equal rule operations (dg87d)" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dgd ra ref || exit 1
dgd va v1 || exit 1
dgd rra ref r1 || exit 1

# try to add a 'if_not_equal' operation whith unknown variable
dgd rroa ref r1 add if_not_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_not_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_not_equal' operation
dgd rroa ref r1 add if_not_equal v1 33 
if [ "`dgd rrol ref r1 -S! | grep v1`" != "add!if_not_equal!v1!33" ]; then
	echo "Can't find if_not_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dgd vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with rules !"; exit 1
fi
# verify that variable is not delete
if [ "`dgd vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dgd rrok ref r1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_not_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dgd vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with rules !"; exit 1
fi
# verify that variable is deleted
if [ "`dgd vl | grep v1`" == "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dgd rk ref || exit 1

# verify that variable is unlinked after referential deletion
dgd va v2 || exit 1
dgd ra ref || exit 1
dgd rra ref r1 || exit 1
dgd rroa ref r1 add if_not_equal v2 5 || exit 1
dgd rra ref r2 || exit 1
dgd rroa ref r2 add if_not_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dgd rk ref || exit 1
# verify that variable is not linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v2 variable"; exit 1
fi
dgd vk v2 || exit 1

# test 'if_not_equal' operation with 'add' and 'remove' operators
dgd va v1 || exit 1
dgd ra ref || exit 1

echo "  test 'add' operator"
dgd rra ref r1 || exit 1
dgd rroa ref r1 add if_not_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rrsr ref r1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rrsr ref r1 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi

echo "  test 'remove' operator"
dgd rra ref r2 || exit 1
dgd rroa ref r2 add always || exit 1
dgd rroa ref r2 remove if_not_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rrsr ref r2 \"2004-01-01\"`" != "N" ] ; then
	echo "Rule is not unselected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rrsr ref r2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Rule is not selected !"; exit 1
fi
dgd rk ref || exit 1
dgd vk v1 || exit 1


# verify memory
mem2=`dgd memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_equal task operations (dg87d)" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dgd ra ref || exit 1
dgd va v1 || exit 1
dgd rta ref t1 || exit 1

# try to add a 'if_equal' operation whith unknown variable
dgd rtoa ref t1 add if_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_equal' operation
dgd rtoa ref t1 add if_equal v1 33 
if [ "`dgd rtol ref t1 -S! | grep v1`" != "add!if_equal!v1!33" ]; then
	echo "Can't find if_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dgd vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dgd vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dgd rtok ref t1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dgd vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dgd vl | grep v1`" == "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dgd rk ref || exit 1

# verify that variable is unlinked after referential deletion
dgd va v2 || exit 1
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rtoa ref t1 add if_equal v2 5 || exit 1
dgd rta ref t2 || exit 1
dgd rtoa ref t2 add if_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dgd rk ref || exit 1
# verify that variable is not linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked Rules is not 0 for v2 variable"; exit 1
fi
dgd vk v2 || exit 1


# test 'if_equal' operation with 'add' and 'remove' operators
dgd va v1 || exit 1
dgd ra ref || exit 1

echo "  test 'add' operator"
dgd rta ref t1 || exit 1
dgd rtoa ref t1 add if_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rtsr ref t1 \"2004-01-01\"`" != "N" ] ; then
	echo "Tasks is not unselected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rtsr ref t1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Task is not selected !"; exit 1
fi

echo "  test 'remove' operator"
dgd rta ref t2 || exit 1
dgd rtoa ref t2 add always || exit 1
dgd rtoa ref t2 remove if_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rtsr ref t2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Task is not selected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rtsr ref t2 \"2004-01-01\"`" != "N" ] ; then
	echo "Task is not unselected !"; exit 1
fi
dgd rk ref || exit 1
dgd vk v1 || exit 1

# verify memory
mem2=`dgd memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_not_equal task operations (dg87d)" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dgd ra ref || exit 1
dgd va v1 || exit 1
dgd rta ref t1 || exit 1

# try to add a 'if_not_equal' operation whith unknown variable
dgd rtoa ref t1 add if_not_equal vxxx 33 >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can add an 'if_not_equal' operation with an unknown variable !"
	exit 1
fi
# add a 'if_not_equal' operation
dgd rtoa ref t1 add if_not_equal v1 33 
if [ "`dgd rtol ref t1 -S! | grep v1`" != "add!if_not_equal!v1!33" ]; then
	echo "Can't find if_not_equal operation"; exit 1
fi
# verify that variable is linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!1" ]; then
	echo "Linked rules is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dgd vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dgd vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dgd rtok ref t1 1
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_not_equal' operation !"; exit 1
fi
# verify that variable is not linked
if [ "`dgd vi v1 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked rules is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dgd vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dgd vl | grep v1`" == "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dgd rk ref || exit 1

# verify that variable is unlinked after referential deletion
dgd va v2 || exit 1
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rtoa ref t1 add if_not_equal v2 5 || exit 1
dgd rta ref t2 || exit 1
dgd rtoa ref t2 add if_not_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!2" ]; then
	echo "Linked rules is not 2 for v2 variable"; exit 1
fi
dgd rk ref || exit 1
# verify that variable is not linked
if [ "`dgd vi v2 -S! | grep Rules`" != "Linked Rules!0" ]; then
	echo "Linked Rules is not 0 for v2 variable"; exit 1
fi
dgd vk v2 || exit 1


# test 'if_not_equal' operation with 'add' and 'remove' operators
dgd va v1 || exit 1
dgd ra ref || exit 1

echo "  test 'add' operator"
dgd rta ref t1 || exit 1
dgd rtoa ref t1 add if_not_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rtsr ref t1 \"2004-01-01\"`" != "Y" ] ; then
	echo "Tasks is not selected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rtsr ref t1 \"2004-01-01\"`" != "N" ] ; then
	echo "Task is not unselected !"; exit 1
fi

echo "  test 'remove' operator"
dgd rta ref t2 || exit 1
dgd rtoa ref t2 add always || exit 1
dgd rtoa ref t2 remove if_not_equal v1 OK || exit 1
dgd vsv v1 NOK || exit 1
if [ "`dgd rtsr ref t2 \"2004-01-01\"`" != "N" ] ; then
	echo "Task is not unselected !"; exit 1
fi
dgd vsv v1 OK || exit 1
if [ "`dgd rtsr ref t2 \"2004-01-01\"`" != "Y" ] ; then
	echo "Task is not selected !"; exit 1
fi
dgd rk ref || exit 1
dgd vk v1 || exit 1

# verify memory
mem2=`dgd memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_equal condition (dg87d)" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dgd va v1 || exit 1
dgd ra ref || exit 1
dgd rta ref tsk || exit 1

# try to add a condition with an unknown variable
dgd rtca ref tsk if_equal vxx OK  > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Can add an 'if_equal' condition with an unknown variable !"
	exit 1
fi
# add an 'if_equal' condition 
dgd rtca ref tsk if_equal v1 OK 
if [ $? -ne 0 ] ; then
	echo "Can't add an 'if_equal' condition !"; exit 1
fi
if [ "`dgd rtcl ref tsk -S! | grep v1`" != "if_equal!v1!OK" ]; then
	echo "Can't find if_equal condition"; exit 1
fi
# verify that variable is linked
if [ "`dgd vi v1 -S! | grep Conditions`" != "Linked Conditions!1" ]; then
	echo "Linked conditions is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dgd vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dgd vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dgd rtck ref tsk if_equal v1 OK 
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_equal' condition!"; exit 1
fi
# verify that variable is not linked
if [ "`dgd vi v1 -S! | grep Conditions`" != "Linked Conditions!0" ]; then
	echo "Linked conditions is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dgd vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dgd vl | grep v1`" == "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dgd rk ref || exit 1

# verify that variable is unlinked after referential deletion
dgd va v2 || exit 1
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rtca ref t1 if_equal v2 5 || exit 1
dgd rta ref t2 || exit 1
dgd rtca ref t2 if_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dgd vi v2 -S! | grep Conditions`" != "Linked Conditions!2" ]; then
	echo "Linked conditions is not 2 for v2 variable"; exit 1
fi
dgd rk ref || exit 1
# verify that variable is not linked
if [ "`dgd vi v2 -S! | grep Conditions`" != "Linked Conditions!0" ]; then
	echo "Linked Conditions is not 0 for v2 variable"; exit 1
fi
dgd vk v2 || exit 1


# verify memory
mem2=`dgd memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi

################################################## 
echo "Test if_not_equal condition (dg87d)" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dgd va v1 || exit 1
dgd ra ref || exit 1
dgd rta ref tsk || exit 1

# try to add a condition with an unknown variable
dgd rtca ref tsk if_not_equal vxx OK  > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	echo "Can add an 'if_not_equal' condition with an unknown variable !"
	exit 1
fi
# add an 'if_not_equal' condition 
dgd rtca ref tsk if_not_equal v1 OK 
if [ $? -ne 0 ] ; then
	echo "Can't add an 'if_not_equal' condition !"; exit 1
fi
if [ "`dgd rtcl ref tsk -S! | grep v1`" != "if_not_equal!v1!OK" ]; then
	echo "Can't find if_not_equal condition"; exit 1
fi
# verify that variable is linked
if [ "`dgd vi v1 -S! | grep Conditions`" != "Linked Conditions!1" ]; then
	echo "Linked conditions is not 1 for v1 variable"; exit 1
fi
# try to delete a linked variable
dgd vk v1 > /dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Can delete a variable which is linked with tasks !"; exit 1
fi
# verify that variable is not delete
if [ "`dgd vl | grep v1`" != "v1" ]; then
	echo "Variable is deleted !"; exit 1
fi
# delete operation
dgd rtck ref tsk if_not_equal v1 OK 
if [ $? -ne 0 ]; then
	echo "Can't delete a 'if_not_equal' condition!"; exit 1
fi
# verify that variable is not linked
if [ "`dgd vi v1 -S! | grep Conditions`" != "Linked Conditions!0" ]; then
	echo "Linked conditions is not 0 for v1 variable"; exit 1
fi
# delete a variable which was linked
dgd vk v1 
if [ $? -ne 0 ]; then
	echo "Can't delete a variable which was linked with tasks !"; exit 1
fi
# verify that variable is deleted
if [ "`dgd vl | grep v1`" == "v1" ]; then
	echo "Variable is not deleted !"; exit 1
fi
dgd rk ref || exit 1

# verify that variable is unlinked after referential deletion
dgd va v2 || exit 1
dgd ra ref || exit 1
dgd rta ref t1 || exit 1
dgd rtca ref t1 if_not_equal v2 5 || exit 1
dgd rta ref t2 || exit 1
dgd rtca ref t2 if_not_equal v2 5 || exit 1
# verify that variable is linked
if [ "`dgd vi v2 -S! | grep Conditions`" != "Linked Conditions!2" ]; then
	echo "Linked conditions is not 2 for v2 variable"; exit 1
fi
dgd rk ref || exit 1
# verify that variable is not linked
if [ "`dgd vi v2 -S! | grep Conditions`" != "Linked Conditions!0" ]; then
	echo "Linked Conditions is not 0 for v2 variable"; exit 1
fi
dgd vk v2 || exit 1

################################################## 
echo "Test referential duplicate" 

# dg87d : test 
mem1=`dgd memory`
if [ $? != 0 ] ; then echo "error"; exit 1; fi

dgd va v1 || exit 1
dgd va v2 || exit 1
dgd ra refori || exit 1
# rule operations
dgd rra refori rule1 || exit 1
dgd rroa refori rule1 add if_equal v1 OK || exit 1
dgd rroa refori rule1 add if_not_equal v2 OK || exit 1
dgd rra refori rule2 || exit 1
dgd rroa refori rule2 remove if_equal v1 OK || exit 1
dgd rroa refori rule2 remove if_not_equal v2 OK || exit 1
# task operations
dgd rta refori task1 || exit 1
dgd rtoa refori task1 add if_equal v1 OK || exit 1
dgd rtoa refori task1 add if_not_equal v2 OK || exit 1
dgd rta refori task2 || exit 1
dgd rtoa refori task2 remove if_equal v1 OK || exit 1
dgd rtoa refori task2 remove if_not_equal v2 OK || exit 1
# task conditions
dgd rta refori task3 || exit 1
dgd rtca refori task3 if_equal v1 OK || exit 1
dgd rtca refori task3 if_not_equal v2 OK || exit 1

dgd rd refori refdup 
if [ $? -ne 0 ]; then
	echo "Can't duplicate referential !"; exit 1
fi
if [ "`dgd rl | grep refdup`" != "refdup" ]; then
	echo "Can't find Duplicate referential !"; exit 1
fi
# verify rules copy
echo "  verify rule copy"
for r in rule1 rule2 ; do
	if [ "`dgd rrl refdup | grep $r`" != "$r" ]; then
		echo "Can't find '$r' rule in duplicate referential !"; exit 1
	fi
	if [ "`dgd rrol refdup $r | grep v1`" != "`dgd rrol refori $r | grep v1`" ]; then
		echo "Error in rule copy !" ; exit 1
	fi
	if [ "`dgd rrol refdup $r | grep v2`" != "`dgd rrol refori $r | grep v2`" ]; then
		echo "Error in rule copy !" ; exit 1
	fi
done
# verify task operation copy
echo "  verify task operations copy"
for r in task1 task2 task3; do
	if [ "`dgd rtl refdup | grep $r`" != "$r" ]; then
		echo "Can't find '$r' task in duplicate referential !"; exit 1
	fi
	if [ "`dgd rtol refdup $r | grep v1`" != "`dgd rtol refori $r | grep v1`" ]; then
		echo "Error in task operation copy !" ; exit 1
	fi
	if [ "`dgd rtol refdup $r | grep v2`" != "`dgd rtol refori $r | grep v2`" ]; then
		echo "Error in task operation copy !" ; exit 1
	fi
done

# verify task condition copy
echo "  verify task conditions copy"
if [ "`dgd rtcl refdup task3 | grep v1`" != "`dgd rtcl refori task3 | grep v1`" ]; then
	echo "Error in task condition copy !" ; exit 1
fi
if [ "`dgd rtcl refdup task3 | grep v2`" != "`dgd rtcl refori task3 | grep v2`" ]; then
	echo "Error in task condition copy !" ; exit 1
fi

# verify variable informations
for v in v1 v2; do
	if [ "`dgd vi -S! $v | grep Rules`" != "Linked Rules!8" ]; then
		echo "Linked rules is not 8 !"; exit 1
	fi
	if [ "`dgd vi -S! $v | grep Conditions`" != "Linked Conditions!2" ]; then
		echo "Linked conditions is not 2 !"; exit 1
	fi
done

dgd rk refori || exit 1
dgd rk refdup || exit 1
# verify variable informations
for v in v1 v2; do
	if [ "`dgd vi -S! $v | grep Rules`" != "Linked Rules!0" ]; then
		echo "Linked rules is not 0 !"; exit 1
	fi
	if [ "`dgd vi -S! $v | grep Conditions`" != "Linked Conditions!0" ]; then
		echo "Linked conditions is not 0 !"; exit 1
	fi
done
dgd vk v1 || exit 1
dgd vk v2 || exit 1

# verify memory
mem2=`dgd memory` || exit 1
if [ $mem1 != $mem2 ] ; then
	echo "dg87d : memory error $mem1 -> $mem2"
	exit 1
fi


echo "Stop dg87d"
stopDg87d || exit 1
