#!/bin/sh
#
# file tst30.sh 
# (test SIGHUP signal)
#
#      Diogene87
#	Copyright (C) 2003 - Dominique Parisot
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307 USA
#


if [ "$srcdir" == "" ]; then srcdir="."; fi
. $srcdir/functions.sh
checkNoDaemon

tmppidfile=`mktemp /tmp/dg87.XXXXXX`
tmppidfile2=`mktemp /tmp/dg87.XXXXXX`
tmpconffile=`mktemp /tmp/dg87.XXXXXX`

trap "rm $tmpconffile $tmppidfile $tmppidfile2 2>/dev/null" SIGINT

#################################### Verify dynamic change of IP Port

for daemon in dg87d dg87x dg87r
do

echo "Test dynamic change of IP port of $daemon"

# create original configuration file
echo "  create configuration file ($tmpconffile)"
cat > $tmpconffile <<EOF
IPPORT=9009
PID_FILE=$tmppidfile
EOF

# start daemon
echo "  start $daemon on IP port 9009"
$PathDg87d -dNC $tmpconffile
sleep 1

# verify PID file exists
if [ ! -f $tmppidfile ] ; then
	echo "PID file '$tmppidfile' doesn't exist !"; exit 1
fi

# verify communication 
echo "  verify communication on IP port 9009"
$PathDiocmd -P 9009 -- version > /dev/null 2>&1
if [ $? -ne 0 ] ; then
	echo "Can't communicate with $daemon on IP port 9009"; exit 1
fi
if [ "`$PathDiocmd -P 9009 -- ei -S! | grep '^IP port'`" != "IP port!9009" ] ; then
	echo "IP Port isn't 9009"; exit 1
fi


# change IP Port
echo "  change IP Port (9109)"
cat > $tmpconffile <<EOF
IPPORT=9109
PID_FILE=$tmppidfile
EOF

# send SIGHUP signal
echo "  send SIGHUP signal"
kill -s SIGHUP `cat $tmppidfile` || exit 1
sleep 1

# verify new IP port
echo "  verify communication on new IP port"
$PathDiocmd -P 9109 -- version > /dev/null 2>&1
if [ $? -ne 0 ] ; then
	echo "Can't communicate with $daemon on IP port 9109"; exit 1
fi
if [ "`$PathDiocmd -P 9109 -- ei -S! | grep '^IP port'`" != "IP port!9109" ] ; then
	echo "IP Port isn't 9109"; exit 1
fi


echo "  stop $daemon"
$PathDiocmd -P 9109 -- shutdown > /dev/null || exit 1

rm $tmpconffile $tmppidfile 2>/dev/null

done

#################################### Verify dynamic change of PID file 

for daemon in dg87d dg87x dg87r
do

echo "Test dynamic change of PID file of $daemon"

# remove previous PID files
rm $tmppidfile $tmppidfile2 2>/dev/null 

# create original configuration file
echo "  create configuration file ($tmpconffile)"
cat > $tmpconffile <<EOF
IPPORT=9009
PID_FILE=$tmppidfile
EOF
# start daemon
echo "  start $daemon on IP port 9009"
$PathDg87d -dNC $tmpconffile
sleep 1

echo "  verify communication"
$PathDiocmd -P 9009 -- version > /dev/null || exit 1
if [ "`$PathDiocmd -P 9009 -- ei -S! | grep '^Pid file'`" != "Pid file!$tmppidfile" ] ; then
echo "PID file is not '$tmppidfile'"; exit 1
fi

# verify PID file exists
if [ ! -f $tmppidfile ] ; then
	echo "PID file '$tmppidfile' doesn't exist !"; exit 1
fi

# change PID file 
echo "  change PID file"
cat > $tmpconffile <<EOF
IPPORT=9009
PID_FILE=$tmppidfile2
EOF

# send SIGHUP signal
echo "  send SIGHUP signal"
kill -s SIGHUP `cat $tmppidfile` || exit 1
sleep 1

echo "  verify communication"
$PathDiocmd -P 9009 -- version > /dev/null 2>&1 || exit 1
if [ "`$PathDiocmd -P 9009 -- ei -S! | grep '^Pid file'`" != "Pid file!$tmppidfile2" ] ; then
echo "PID file is not '$tmppidfile2'"; exit 1
fi


# verify new PID file exists
echo "  verify PID files"
if [ ! -f $tmppidfile2 ] ; then
	echo "New PID file '$tmppidfile2' doesn't exist !"; exit 1
fi
if [ -f $tmppidfile ] ; then
	echo "Old PID file '$tmppidfile' is not deleted !"; exit 1
fi

echo "  stop $daemon"
$PathDiocmd -P 9009 -- shutdown > /dev/null || exit 1

rm $tmpconffile $tmppidfile $tmppidfile2 2>/dev/null

done

#################################### Verify dynamic change of password 

for daemon in dg87d dg87x dg87r
do

echo "Test dynamic change of diogene password of $daemon"
DIOGENE_PASSWORD=zdjj88dl

# create original configuration file
echo "  create configuration file with diogene password ($DIOGENE_PASSWORD)"
cat > $tmpconffile <<EOF
PID_FILE=$tmppidfile
DIOGENE_PASSWORD=$DIOGENE_PASSWORD
EOF

# start daemon
echo "  start $daemon"
$PathDg87d -dNC $tmpconffile
sleep 1

# verify PID file exists
if [ ! -f $tmppidfile ] ; then
	echo "PID file '$tmppidfile' doesn't exist !"; exit 1
fi

# verify communication 
echo "  verify communication with diogene account"
$PathDiocmd -- version > /dev/null 2>&1 || exit 1
$PathDiocmd -u diogene -p xxx -- version > /dev/null 2>&1 
if [ $? -eq 0 ] ; then
	echo "Can communicate with $daemon with a wrong password !"; exit 1
fi
$PathDiocmd -u diogene -p $DIOGENE_PASSWORD -- version > /dev/null 2>&1
if [ $? -ne 0 ] ; then
	echo "Can't communicate with $daemon with diogene account"; exit 1
fi

# change password 
DIOGENE_PASSWORD=d80d_skoo
echo "  change password ($DIOGENE_PASSWORD)" 
cat > $tmpconffile <<EOF
PID_FILE=$tmppidfile
DIOGENE_PASSWORD=$DIOGENE_PASSWORD
EOF

# send SIGHUP signal
echo "  send SIGHUP signal"
kill -s SIGHUP `cat $tmppidfile` || exit 1
sleep 1

# verify communication 
echo "  verify communication with diogene account"
$PathDiocmd -- version > /dev/null 2>&1 || exit 1
$PathDiocmd -u diogene -p xxx -- version > /dev/null 2>&1 
if [ $? -eq 0 ] ; then
	echo "Can communicate with $daemon with a wrong password !"; exit 1
fi
$PathDiocmd -u diogene -p $DIOGENE_PASSWORD -- version > /dev/null 2>&1
if [ $? -ne 0 ] ; then
	echo "Can't communicate with $daemon with diogene account"; exit 1
fi

echo "  stop $daemon"
$PathDiocmd -- shutdown > /dev/null || exit 1

rm $tmpconffile $tmppidfile 2>/dev/null

done


exit 0
