/* file cmdjob.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdjob.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

/******************************************************************************/
/* rjk
 * Ref Job Kill
 * Usage : rjk RefName TaskName JobName
 */
int cmd_rjk(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjk RefName TaskName JobName\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't delete a job in task '%s' because "\
			" ref %s is locked !\n",
			pTask->name, pRef->name);	
		return 1;
	}

	/* delete the job */	
	dg87_err_clearError();
	if (dg87_task_jobDeleteByName(pTask, pCommand->argv[2]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't delete the job '%s' in the task '%s' ! %s\n",
			pCommand->argv[1], pTask,
			dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}
/******************************************************************************
 * cmd_rja
 * Ref Job Add
 * Usage : rja RefName TaskName Taskjob [description]
 */
int cmd_rja(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_job* pJob;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 3 || pCommand->argc > 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rja RefName TaskName JobName [description]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't add a job in task '%s' because "\
			" ref %s is locked !\n",
			pTask->name, pRef->name);	
		return 1;
	}

	/* add the job to the task */
	dg87_err_clearError();
	if ((pJob = dg87_task_jobAdd(pTask, pCommand->argv[2])) == NULL) {
		dg87_connection_printf(pConnection,
			"Can't add job '%s' to the task '%s' ! %s\n",
			pCommand->argv[2], pTask->name,
			dg87_err_getErrorStr());
		return 1;
	}

	/* change description */
	if (pCommand->argc == 4) {
		dg87_err_clearError();
		if (dg87_job_setDescription(pJob, pCommand->argv[3])
				== FALSE)
		{
			dg87_connection_printf(pConnection,
				"Can't add job '%s' to the task '%s' ! %s\n",
				pJob->name, pTask->name,
				dg87_err_getErrorStr());
			/* delete the job before exit */
			dg87_task_jobDelete(pTask, pJob);
			return 1;
		}
	}
	return 0;
}
/******************************************************************************/
/* cmd_rjc
 * Usage : rjc RefName TaskName
 * Task Job Count
 * return the number of jobs in the TASK.
 */
int cmd_rjc(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjc RefName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* return jobs count */
	dg87_connection_printf(pConnection,"%i\n", pTask->pJobVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_rjl
 * rjl [options] RefName TaskName
 * Ref Job List
 * Options
 * -d : print description of job 
 * -t : print task name of job
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first job to display (first index is 0)
 * -Nnumber : number of job to display (no limit if -N < 1)
 */
int cmd_rjl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "dtB:S:E:I:N:";*/
	int optch, iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_ref* pRef;
	dg87_element* pElement = NULL;
	dg87_task* pTask;
	dg87_job* pJob;
	BOOL showDescription = FALSE;
	BOOL showTaskName = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjl [options] RefName TaskName");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	
	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg); 
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg); 
				break;
			case 'd': /* description */
				showDescription = TRUE; 
				break;
			case 't': /* Task */
				showTaskName = TRUE; 
				break;
				
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	
	/* find the first job to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pTask->pJobVector, start);
	else
		pElement = pTask->pJobVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pJob = pElement->pObject;

		/* response */
		dg87_connection_printf(pConnection, pBeg);
		if (showTaskName ==  TRUE){
			dg87_connection_printf(pConnection, "%s%s",
				pJob->pTask->name, pSep);
		}
		dg87_connection_printf(pConnection, pJob->name);
		if (showDescription == TRUE) {
			dg87_connection_printf(pConnection, "%s%s",
				pSep, pJob->description);
		} 
		dg87_connection_printf(pConnection, "%s\n",pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* rji
 * ref job information 
 * get information for a job
 * Usage : rji [options] RefName TaskName JobName
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int cmd_rji(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch, iopt;
	char* optarg;
	dg87_ref* pRef;
	dg87_task *pTask;
	dg87_job *pJob;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 3){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rji [options] RefName TaskName JobName");
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	
	/* find the job */
	pJob = dg87_util_findJob(pConnection, pTask, pCommand->argv[2]);
	if (pJob == NULL) return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
				
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* job name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Job Name", pSep, pJob->name, pEnd);
	/* Task name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Task Name", pSep, pJob->pTask->name, pEnd);
	/* day name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Ref Name", pSep, pJob->pTask->pRef->name, pEnd);
	/* job desription */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Description", pSep, pJob->description, pEnd);
	/* Task Target */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Task Target", pSep, 
		(pJob->pTask->pQueue==NULL) ? "none" :
	       		pJob->pTask->pQueue->pTarget->name, pEnd);
	/* Target Queue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Target Queue", pSep, 
		(pJob->pTask->pQueue==NULL) ? "none" :
	       		pJob->pTask->pQueue->name, pEnd);
	/* job Path */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Path", pSep, 
		(pJob->pPath==NULL) ? "none" : pJob->pPath, pEnd);
	/* argument */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Argument", pSep, 
		(pJob->pArgument==NULL) ? "none" : pJob->pArgument, pEnd);
	/* start tempo */
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg,"Start Tempo",pSep, pJob->startTempo, pEnd);
	/* end tempo */
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg,"End Tempo",pSep, pJob->endTempo, pEnd);
	/* waitAllDependences*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Wait All Dependences", pSep,
		dg87_boolToAscii(pJob->waitAllDependences), pEnd);
	/* onErrorContinue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "On Error Continue", pSep,
		dg87_boolToAscii(pJob->onErrorContinue), pEnd);
	
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
/* rjsd
 * Ref Job Set Description
 * change the description of a job
 * Usage : rjsd RefName TaskName JobName description
 */
int cmd_rjsd(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task *pTask;
	dg87_job *pJob;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjsd RefName TaskName JobName description");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	
	/* find the job */
	pJob = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJob == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set description for job '%s' in task '%s' "
			" because ref %s is locked !\n",
			pJob->name, pTask->name, pRef->name);	
		return 1;
	}
	dg87_err_clearError();
	if (dg87_job_setDescription(pJob, pCommand->argv[3]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set description for the job '%s' " \
			"in the task '%s' ! %s\n",
			pJob->name, pJob->pTask->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rjsp
 * Ref Job Set Path 
 * change the path of a job.
 * Usage : rjsp RefName TaskName JobName Path
 */
int cmd_rjsp(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task *pTask;
	dg87_job *pJob;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjsp RefName TaskName JobName Path");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* find the job */
	pJob = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJob == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set path for job '%s' in task '%s' "
			"because ref %s is locked !\n",
			pJob->name, pTask->name, pRef->name);	
		return 1;
	}
	dg87_err_clearError();
	dg87_err_clearError();
	if (dg87_job_setPath(pJob, pCommand->argv[3]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set path for the job '%s' " \
		        "in the task '%s' ! %s\n",
			pJob->name, pJob->pTask->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rjsa
 * Ref Job Set Argument 
 * change the argument of a job.
 * Usage : rjsa RefName TaskName JobName argument
 */
int cmd_rjsa(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task *pTask;
	dg87_job *pJob;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjsa RefName TaskName JobName argument");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* find the job */
	pJob = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJob == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set argument for job '%s' in task '%s' "
			"because ref %s is locked !\n",
			pJob->name, pTask->name, pRef->name);	
		return 1;
	}
	dg87_err_clearError();
		  
	dg87_err_clearError();
	if (dg87_job_setArgument(pJob, pCommand->argv[3]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set argument for the job '%s' " \
			"in the task '%s' ! %s\n",
			pJob->name, pJob->pTask->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rjst
 * Ref Job Set Tempo
 * change the startTempo and endTempo of a job
 * Usage : rjst RefName TaskName JobName startTempo endTempo
 */
int cmd_rjst(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task *pTask;
	dg87_job *pJob;
	int startTempo, endTempo;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjst RefName TaskName JobName startTempo endTempo");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* find the job */
	pJob = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJob == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set tempo for job '%s' in task '%s' "
			"because ref %s is locked !\n",
			pJob->name, pTask->name, pRef->name);	
		return 1;
	}
	dg87_err_clearError();
	/* change tempo */
	startTempo = atoi(pCommand->argv[3]);
	endTempo   = atoi(pCommand->argv[4]);
	dg87_err_clearError();
	if (dg87_job_setTempo(pJob, startTempo, endTempo) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set tempo for the job '%s' " \
			"in the task '%s' ! %s\n",
			pJob->name, pJob->pTask->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rjswad
 * Ref Job Set WaitAllDependences
 * Usage : rjswad RefName TaskName JobName true/false
 */
int cmd_rjswad(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_job* pJob;
	BOOL value;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjswad RefName TaskName JobName true/false");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* find the job */
	pJob = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJob == NULL) return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set waitAllDependences for the job '%s' "\
		        " of the task '%s' because ref '%s' is locked !\n",
			pTask->name, pJob->name, pRef->name);
		return 1;
	}

	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[3])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[3]);
		return 1;
	}

	/* change the property waitAllDependences*/
	dg87_err_clearError();
	if (dg87_job_setWaitAllDependences(pJob, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set waitAllDependences for the job '%s' "\
			"of the task '%s' ! %s\n",
			pJob->name, pTask->name, dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rjsoec
 * Ref Job Set OnErrorContinue
 * Usage : rjsoec RefName TaskName JobName true/false
 */
int cmd_rjsoec(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_job* pJob;
	BOOL value;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjsoec RefName TaskName JobName true/false");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* find the job */
	pJob = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJob == NULL) return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set OnErrorContinue for the job '%s' "\
		        " of the task '%s' because ref '%s' is locked !\n",
			pTask->name, pJob->name, pRef->name);
		return 1;
	}

	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[3])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[3]);
		return 1;
	}

	/* change the property OnErrorContinue*/
	dg87_err_clearError();
	if (dg87_job_setOnErrorContinue(pJob, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set OnErrorContinue for the job '%s' "\
			"of the task '%s' ! %s\n",
			pJob->name, pTask->name, dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}


/******************************************************************************/
/* rjdl
 * Ref Job Dependence List
 * Usage : rjdl [options] RefName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of jobdep to display (no limit if -N < 1)
 *  -a jobName : only jobdep after this job
 *  -b jobName : only jobdep before this job 
 *  -t : print dependence type
 */
int cmd_rjdl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "tB:S:E:I:N:b:a:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first job to display */
	int number = 0;/* number of jobs to display */
	int count = 0; /* number of job actually display */
	dg87_item* pItem= NULL;
	dg87_jobdep* pJobDep;
	int filter = 0; /* 0 = no; 1 = before; 2 = after */
	dg87_job* pJobFilter = NULL;
	char* pJobFilterName = NULL;
	dg87_ref* pRef;
	dg87_task* pTask = NULL;
	BOOL showDepType = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjdl [options] RefName TaskName");
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first job to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of job to display */
				number = atoi(optarg);
				break;
			case 'b': /* only job before */
				pJobFilterName = strdup(optarg);
				filter = 1;
				break;
			case 'a': /* only job after */
				pJobFilterName = strdup(optarg);
				filter = 2;
				break;
			case 't': /* type of dependence*/
				showDepType = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				if (pJobFilterName != NULL)
					free(pJobFilterName);
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* find the filter job */
	if (filter > 0){
		pJobFilter = dg87_util_findJob(pConnection, 
				pTask, pJobFilterName);
		if (pJobFilter == NULL){
			free(pJobFilterName);
			free(pSep); free(pBeg); free(pEnd);
			return 1;
		}
	}
	
	/* find the first JobDep to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pTask->pJobDepList, start);
	else
		pItem = pTask->pJobDepList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pJobDep = pItem->pObject;
		if (filter == 0 ||
			(filter == 1 && pJobDep->pJobAfter == pJobFilter) ||
			(filter == 2 && pJobDep->pJobBefore == pJobFilter)){
			/* print job before and after */
			dg87_connection_printf(pConnection,"%s%s%s%s", pBeg,
					pJobDep->pJobBefore->name, pSep, 
					pJobDep->pJobAfter->name);
			/* print job dependence type */
			if (showDepType == TRUE){
				dg87_connection_printf(pConnection,
					"%s%s", pSep,
					dg87_jobdep_typeToAscii(pJobDep->type));
			}
			dg87_connection_printf(pConnection,"%s\n", pEnd);
		}
		pItem = pItem->pNext;
	}
	/* free memory */
	if (pJobFilterName != NULL) free(pJobFilterName);
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* rjdc
 * Ref Job Dependence Count
 * Usage : rjdc RefName TaskName
 */
int cmd_rjdc(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task *pTask;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rjdc RefName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	dg87_connection_printf(pConnection,"%i\n", pTask->pJobDepList->count);
	return 0;
}
/******************************************************************************/
/* rjda
 * Ref Job Dependence Add
 * Usage RefName TaskName JobBeforeName JobAfterName [type]
 */
int cmd_rjda(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_job *pJobBefore, *pJobAfter;
	dg87_jobdep_type depType;
	dg87_jobdep* pJobdep;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 4 || pCommand->argc > 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
		"rjda RefName TaskName JobBeforeName JobAfterName [type]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't add a job dependence in task '%s' "
			"because ref %s is locked !\n",
			pTask->name, pRef->name);	
		return 1;
	}
	dg87_err_clearError();
	/* find the job before */
	pJobBefore = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJobBefore == NULL) return 1;

	/* find the job after */
	pJobAfter = dg87_util_findJob(pConnection,pTask, pCommand->argv[3]);
	if (pJobAfter == NULL) return 1;

	/* set the type of dependances */
	depType = dg87_jobdep_unknown;
	if (pCommand->argc == 5) {
		depType = dg87_jobdep_asciiToType(pCommand->argv[4]);
		if (depType == dg87_jobdep_unknown){
			dg87_connection_printf(pConnection,
				"Incorrect dependence type '%s' !",
				pCommand->argv[4]);
			return 1;
		}
	}

	/* add a dependence between jobs */
	dg87_err_clearError();
	pJobdep = dg87_task_jobdepAdd(pTask, pJobBefore, pJobAfter);
	if (pJobdep == NULL){
		dg87_connection_printf(pConnection,
			"Can't add the dependence between '%s' " \
			"and '%s' in the task '%s' ! %s\n",
			pJobBefore->name, pJobAfter->name, pTask->name,
			dg87_err_getErrorStr());
		return 1;
	}

	/* set dependence type */
	if (depType != dg87_jobdep_unknown){
		dg87_err_clearError();
		if (dg87_jobdep_setType(pJobdep, depType) == FALSE){
			dg87_connection_printf(pConnection,
				"Can't set dependence type between "\
				"'%s' and '%s' in task '%s' ! %s\n",
				pJobBefore->name, pJobAfter->name,
				pTask->name, dg87_err_getErrorStr());
			/* TODO kill jobdep before exit ? */
			return 1;
		}
	}
	return 0;
}
/******************************************************************************/
/* rjdk
 * Ref Job Dependences Kill
 * Usage : rjdk RefName TaskName JobBeforeName JobAfterName
 */
int cmd_rjdk(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_job *pJobBefore, *pJobAfter;
	dg87_jobdep *pJobDep;
	dg87_item *pItem;
	dg87_task *pTask;
	dg87_command* pCommand;
	dg87_jobdep_type deptype;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
		"rjdk RefName TaskName JobBeforeName JobAfterName DepType");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find dependence type */
	if ((deptype = dg87_jobdep_asciiToType(pCommand->argv[4])) == FALSE){
		dg87_connection_printf(pConnection, 
			"Incorrect dependence type '%s'\n",
			pCommand->argv[4]);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't kill a job dependence in task '%s' "
			"because ref %s is locked !\n",
			pTask->name, pRef->name);	
		return 1;
	}

	/* find the job before */
	pJobBefore = dg87_util_findJob(pConnection,pTask, pCommand->argv[2]);
	if (pJobBefore == NULL) return 1;

	/* find the job after */
	pJobAfter = dg87_util_findJob(pConnection,pTask, pCommand->argv[3]);
	if (pJobAfter == NULL) return 1;

	/* TODO move this code in a new function dg87_task_jobdepDelete */
	/* find a dependence between jobs and delete it */
	pItem = pTask->pJobDepList->pFirst;
	while (pItem != NULL){
		pJobDep = pItem->pObject;
		if (pJobDep->pJobBefore == pJobBefore &&
				pJobDep->pJobAfter == pJobAfter &&
				pJobDep->type == deptype){
			dg87_err_clearError();
			if (dg87_list_removeItem(
				pTask->pJobDepList, pItem) == NULL){
				dg87_connection_printf(pConnection,
					"Can't delete the dependence between "\
					"'%s' and '%s' in the task '%s' ! %s\n",
					pJobBefore->name, pJobAfter->name, 
					pTask->name, dg87_err_getErrorStr());
				return 1;
			}
			/* success */
			dg87_jobdep_delete(pJobDep);
			return 0;
		}
		pItem = pItem->pNext;
	}
	/* we don't find the dependence */
	dg87_connection_printf(pConnection, 
		"Can't find the dependence between "\
		"'%s' and '%s' in the task '%s' with type '%s' !\n",
		pJobBefore->name, pJobAfter->name, pTask->name,
		dg87_jobdep_typeToAscii(deptype));
	return 1;
}
/******************************************************************************/
/* End of file */
