/* file cmdoperation.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "context.h"
#include "cmdoperation.h"
#include "rule.h"
#include "util.h"
#include "engine.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

char* dayname[]= {"Su", "Mo", "Tu", "We", "Th", "Fr", "Sa", ".."};
char* monthname[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

/******************************************************************************/
dg87_rule_operation* cmd_operation_ra(dg87_connection* pConnection, 
		dg87_ref* pRef)
{
	dg87_rule* pComplexRule;
	dg87_rule_operation* pOperation;
	dg87_command* pCommand;
	dg87_rule_operator oper;
	dg87_rule_type type;
	dg87_context* pContext;
	struct dg87_variable_struct *pVariable;
	int integer;
	time_t date;
	int day, month;
	char* pValue;

	if (pConnection == NULL) return NULL;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* interpret operator */
	oper = dg87_rule_asciiToOperator(pCommand->argv[2]); 
	if (oper == -1){
		dg87_connection_printf(pConnection,
			"Unknown operator '%s'\n",
			pCommand->argv[2]);
		return NULL;
	}
	/* interpret type*/
	type = dg87_rule_asciiToType(pCommand->argv[3]); 
	if (type == -1){
		dg87_connection_printf(pConnection,
			"Unknown type '%s'\n",
			pCommand->argv[3]);
		return NULL;
	}
	/* control last argument */
	if (type == dg87_rule_type_always){
		if (pCommand->argc != 4){
			dg87_connection_printf(pConnection,
				"Data argument is not requiered\n");
			return NULL;
		}
	} else if (type == dg87_rule_type_ifequal ||
			type == dg87_rule_type_ifnequal){
		if (pCommand->argc != 6){
			dg87_connection_printf(pConnection,
				"Two data arguments are requiered\n");
			return NULL;
		}
	}else{
		if (pCommand->argc != 5){
			dg87_connection_printf(pConnection,
				"Data argument is requiered\n");
			return NULL;
		}
	}

	/* create operation */
	pOperation = NULL;
	switch (type){
		case dg87_rule_type_dow:
			integer = atoi(pCommand->argv[4]);
			pOperation = dg87_operation_dayofweek_new(oper, 
					integer - 1);
			break;
		case dg87_rule_type_dom:
			integer = atoi(pCommand->argv[4]);
			pOperation = dg87_operation_dayofmonth_new(oper, 
					integer);
			break;
		case dg87_rule_type_doy:
			integer = atoi(pCommand->argv[4]);
			pOperation = dg87_operation_dayofyear_new(oper, 
					integer);
			break;
		case dg87_rule_type_wom:
			integer = atoi(pCommand->argv[4]);
			pOperation = dg87_operation_weekofmonth_new(oper, 
					integer);
			break;
		case dg87_rule_type_woy:
			integer = atoi(pCommand->argv[4]);
			pOperation = dg87_operation_weekofyear_new(oper, 
					integer);
			break;
		case dg87_rule_type_moy:
			integer = atoi(pCommand->argv[4]);
			pOperation = dg87_operation_monthofyear_new(oper, 
					integer - 1);
			break;
		case dg87_rule_type_day:
			if (dg87_rule_asciiToDay(pCommand->argv[4],
						&month,&day) == FALSE){
				dg87_connection_printf(pConnection,
					"Incorrect value for day '%s'\n",
					pCommand->argv[4]);
				return NULL;
			}
			pOperation = dg87_operation_day_new(oper,
					month, day);
			break;
		case dg87_rule_type_date:
			date = dg87_rule_asciiToDate(pCommand->argv[4]);
			if (date == -1){
				dg87_connection_printf(pConnection,
					"Incorrect value for date '%s'\n",
					pCommand->argv[4]);
				return NULL;
			}
			pOperation = dg87_operation_date_new(oper, date); 
			break;
		case dg87_rule_type_always:
			if (oper != dg87_rule_operator_add){
				dg87_connection_printf(pConnection,
					"Operator must be 'add' with an "\
					"'always' rule.\n");
				return NULL;
			}
			pOperation = dg87_operation_always_new();
			break;
		case dg87_rule_type_complex:
			if (oper != dg87_rule_operator_add &&
					oper != dg87_rule_operator_remove &&
					oper != dg87_rule_operator_shiftAfter &&
					oper != dg87_rule_operator_shiftBefore){
				dg87_connection_printf(pConnection,
					"Operator must be 'add' or 'remove' "\
					"or 'shift_after' or 'shift_before' "\
					"with a 'complex' rule.\n");
				return NULL;
			}
			pComplexRule = dg87_ref_ruleGetByName(pRef,
					pCommand->argv[4]);
			if (pComplexRule == NULL){
				dg87_connection_printf(pConnection,
				"Can't find the rule '%s' in the ref '%s' ! "\
				"%s\n",
				pCommand->argv[4], pRef->name,
				dg87_err_getErrorStr());
				return NULL;
			}
			pOperation = dg87_operation_complex_new(oper, 
					pComplexRule);
			break;
		case dg87_rule_type_ifequal:
			if (oper != dg87_rule_operator_add &&
					oper != dg87_rule_operator_remove){
				dg87_connection_printf(pConnection,
					"Operator must be 'add' or 'remove' "\
					"with a 'if_equal' rule.\n");
				return NULL;
			}
			pVariable = dg87_engine_variableGetByName(
					pContext->pEngine,
					pCommand->argv[4]);
			if (pVariable == NULL){
				dg87_connection_printf(pConnection,
				"Can't find the variable '%s' ! %s \n",
				pCommand->argv[4],
				dg87_err_getErrorStr());
				return NULL;
			}
			pValue = pCommand->argv[5];
			pOperation = dg87_operation_ifequal_new(oper, 
					pVariable, pValue); 
			break;
		case dg87_rule_type_ifnequal:
			if (oper != dg87_rule_operator_add &&
					oper != dg87_rule_operator_remove){
				dg87_connection_printf(pConnection,
					"Operator must be 'add' or 'remove' "\
					"with a 'if_not_equal' rule.\n");
				return NULL;
			}
			pVariable = dg87_engine_variableGetByName(
					pContext->pEngine,
					pCommand->argv[4]);
			if (pVariable == NULL){
				dg87_connection_printf(pConnection,
				"Can't find the variable '%s' ! %s \n",
				pCommand->argv[4],
				dg87_err_getErrorStr());
				return NULL;
			}
			pValue = pCommand->argv[5];
			pOperation = dg87_operation_ifnequal_new(oper, 
					pVariable, pValue); 
			break;

	}

	return pOperation;
}
/******************************************************************************/
int cmd_operation_rsr(dg87_connection* pConnection, dg87_list* pOperationList)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	int optch, iopt;
	char* optarg;
	dg87_command* pCommand;
	struct tm dc;
	time_t day, firstDay, lastDay; /* of the Range */
	BOOL result;
	BOOL showDayName = FALSE; 
	BOOL showDate = FALSE;
	char* pString;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* interpret date1 argument */
	firstDay = dg87_rule_asciiToDate(pCommand->argv[2]);
	if (firstDay == -1){
		dg87_connection_printf(pConnection, 
			"Incorrect value '%s' for date1 argument.\n",
			pCommand->argv[2]);
		return 1;
	}

	/* interpret date2 argument (which is optional) */
	if (pCommand->argc != 4){
		lastDay = firstDay;
	} else {
		lastDay = dg87_rule_asciiToDate(pCommand->argv[3]);
		if (lastDay == -1){
			dg87_connection_printf(pConnection, 
				"Incorrect value '%s' for date2 argument.\n",
				pCommand->argv[3]);
			return 1;
		}
	}

	/* reverse day if necessary */
	if (lastDay < firstDay){
		day = firstDay;
		firstDay = lastDay;
		lastDay = day;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'n': /* name of week day */
				showDayName = TRUE;
				break;
			case 'd': /* name of date */
				showDate = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* calculate */
	for (day = firstDay; day <= lastDay; day+=24*3600){
		dc = *localtime(&day);
		result = FALSE;

		if (dg87_operationlist_calculate(pOperationList, 
					day, &result,-1) == FALSE){
			dg87_connection_printf(pConnection,
				"Error while rule calculation ! %s\n",
				dg87_err_getErrorStr());
			/* free memory */
			free(pSep); free(pBeg); free(pEnd);
			return 1;
		}

		dg87_connection_printf(pConnection,"%s", pBeg);
		if (showDayName == TRUE)
			dg87_connection_printf(pConnection,"%s%s",
					dayname[dc.tm_wday],pSep);
		if (showDate == TRUE){
			pString = dg87_rule_dateToAscii(day);
			dg87_connection_printf(pConnection,"%s%s",
					pString,pSep);
			dg87_free(pString, DG87_CHAR);
		}

		dg87_connection_printf(pConnection,"%s%s\n", 
				(result==TRUE)?"Y":"N", pEnd);
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
int cmd_operation_rsy(dg87_connection* pConnection, dg87_list* pOperationList)
{
	dg87_command* pCommand;
	int year;
	struct tm dc;
	time_t day, firstDay, lastDay; /* of the year */
	int d[12][31];
	int imonth, iday;
	char c;
	BOOL result;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* interpret year argument */
	year = atoi(pCommand->argv[2]);
	if (year < 1900){
		dg87_connection_printf(pConnection, 
			"Incorrect value '%s' for year argument.\n",
			pCommand->argv[2]);
		return 1;
	}
	/* calculate first day of the year */
	dc.tm_sec = 0;
	dc.tm_min = 0;
	dc.tm_hour = 0;
	dc.tm_mday = 1;
	dc.tm_mon = 0;
	dc.tm_wday = 0;
	dc.tm_yday = 0;
	dc.tm_isdst = -1;
	dc.tm_year = year -1900;
	firstDay = mktime(&dc); 

	/* calculate last day of the year */
	dc.tm_sec = 1;
	dc.tm_min = 0;
	dc.tm_hour = 0;
	dc.tm_mday = 1;
	dc.tm_mon = 0;
	dc.tm_wday = 0;
	dc.tm_yday = 0;
	dc.tm_isdst = -1;
	dc.tm_year = year +1 -1900;
	lastDay = mktime(&dc) - 24*3600 ;

	/* initialize array */
	for (imonth = 0; imonth < 12; imonth++)
		for (iday = 0; iday < 31; iday++)
			d[imonth][iday] = 7;

	/* calculate */
	for (day = firstDay; day <= lastDay; day+=24*3600){
		dc = *localtime(&day);
		result = FALSE;
		d[dc.tm_mon][dc.tm_mday-1] = dc.tm_wday;
		if (dg87_operationlist_calculate(pOperationList, 
					day, &result, -1) == FALSE){
			dg87_connection_printf(pConnection,
				"Error while rule calculation ! %s\n",
				dg87_err_getErrorStr());
			return 1;
		}
		if (result == TRUE)
			d[dc.tm_mon][dc.tm_mday-1] += 8;
	}

	/* print calendar */
	dg87_connection_printf(pConnection,
		"Year : %i\n  ", year);
	/* print header (month name) */
	for (imonth = 0; imonth < 12; imonth++)
		dg87_connection_printf(pConnection,"   %s", monthname[imonth]);
	/* each line is a day */
	for (iday = 0; iday < 31; iday++){
		if (iday < 9)
			dg87_connection_printf(pConnection,"\n %i  ", iday+1);
		else
			dg87_connection_printf(pConnection,"\n%i  ", iday+1);
		/* each column is a month */
		for (imonth = 0; imonth < 12; imonth++){
			if (d[imonth][iday] >= 8){
				c = '*';
				d[imonth][iday] -= 8;
			} else {
				c = ' ';
			}
			dg87_connection_printf(pConnection," %s %c ",
					dayname[d[imonth][iday]], c);
		}
	}
	dg87_connection_printf(pConnection,"\n");
	return 0;
}
/******************************************************************************/
int cmd_operation_ol(dg87_connection* pConnection, dg87_list* pOperationList)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "dtB:S:E:I:N:";*/
	int optch, iopt;
	char* pString;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of Rules to display */
	int count = 0; /* number of Rule actually display */
	dg87_item* pItem= NULL;
	dg87_rule_operation* pOperation;
	dg87_command* pCommand;
	BOOL showIndex = FALSE;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
			case 'I': /* index of the first operation to display */
				start = atoi(optarg); 
				break;
			case 'N': /* number of operations to display */
				number = atoi(optarg); 
				break;
			case 'i': /* print index */
				showIndex = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !\n");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	
	/* find the first operation to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pOperationList, start);
	else
		pItem = pOperationList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pOperation = pItem->pObject;

		dg87_connection_printf(pConnection, "%s", pBeg);
		/* index */
		if (showIndex == TRUE){
			dg87_connection_printf(pConnection, "%i%s",
				count, pSep);
		}
		/* operator */
		dg87_connection_printf(pConnection, "%s%s",
			dg87_rule_operatorToAscii(pOperation->oper),pSep);
		/* type of rule */
		dg87_connection_printf(pConnection, "%s%s",
			dg87_rule_typeToAscii(pOperation->type),pSep);
		/* print data */
		switch (pOperation->type){
			case dg87_rule_type_dow:
				dg87_connection_printf(pConnection, "%i",
					pOperation->rule.dow.dayOfWeek+1);
				break;
			case dg87_rule_type_dom:
				dg87_connection_printf(pConnection, "%i",
					pOperation->rule.dom.dayOfMonth);
				break;
			case dg87_rule_type_doy:
				dg87_connection_printf(pConnection, "%i",
					pOperation->rule.doy.dayOfYear);
				break;
			case dg87_rule_type_wom:
				dg87_connection_printf(pConnection, "%i",
					pOperation->rule.wom.weekOfMonth);
				break;
			case dg87_rule_type_woy:
				dg87_connection_printf(pConnection, "%i",
					pOperation->rule.woy.weekOfYear);
				break;
			case dg87_rule_type_moy:
				dg87_connection_printf(pConnection, "%i",
					pOperation->rule.moy.monthOfYear + 1);
				break;
			case dg87_rule_type_day:
				pString = dg87_rule_dayToAscii(
						pOperation->rule.day.month,
						pOperation->rule.day.day);
				if (pString == NULL)
					break;
				dg87_connection_printf(pConnection, "%s",
					pString);
				dg87_free(pString,DG87_CHAR);
				break;
			case dg87_rule_type_date:
				pString = dg87_rule_dateToAscii(
						pOperation->rule.date.date);
				if (pString == NULL)
					break;
				dg87_connection_printf(pConnection, "%s",
					pString);
				dg87_free(pString,DG87_CHAR);
				break;
			case dg87_rule_type_always:
				/* nothing to print */
				break;
			case dg87_rule_type_complex:
				dg87_connection_printf(pConnection, "%s",
					pOperation->rule.pRule->name);
				break;
			case dg87_rule_type_ifequal:
			case dg87_rule_type_ifnequal:
				dg87_connection_printf(pConnection, "%s%s%s",
					pOperation->rule.ifequal.pVariable->name
					, pSep,
					pOperation->rule.ifequal.pValue
					);
				break;
		}
		dg87_connection_printf(pConnection, "%s\n",pEnd);
		pItem = pItem->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/******************************************************************************/
/* End of file */
