/* file cmdref.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdref.h"
#include "util.h"
#include "context.h"
#include "engine.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

/******************************************************************************/
/* rk
 * Ref Kill
 * Usage : rk RefName
 */
int cmd_rk(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_ref* pRef;
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rk RefName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;
	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection, 
			"Can't delete the ref '%s', because planning "\
			"is in progress !\n", 
			pRef->name);
		return 1;
	}
	/* delete the ref */
	dg87_err_clearError();
	if (dg87_context_refDeleteByName(pContext, 
				pCommand->argv[0]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't delete the ref '%s' ! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rd
 * Ref Duplicate
 * Usage : rd OriginalRefName DuplicateRefName
 */
int cmd_rd(dg87_connection* pConnection)
{
	dg87_ref* pRef, *pOriginalRef;
	dg87_context* pContext;
	dg87_command* pCommand;
	if (pConnection == NULL) return 1;
	pContext = pConnection->pServer->pData;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 2) {
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rd OriginalRefName DuplicatedRefName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the original ref */
	pOriginalRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pOriginalRef == NULL) return 1;

	/* add the ref */
	dg87_err_clearError();
	pRef = dg87_context_refDuplicate(pContext, 
			pOriginalRef, pCommand->argv[1]);
	if (pRef == NULL){
		dg87_connection_printf(pConnection,
				"Can't duplicate the ref '%s' ! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* ra
 *  Ref Add
 * Usage : ra RefName
 */
int cmd_ra(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_context* pContext;
	dg87_command* pCommand;
	if (pConnection == NULL) return 1;
	pContext = pConnection->pServer->pData;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 1) {
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ra RefName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* add the ref */
	dg87_err_clearError();
	pRef = dg87_context_refAdd(pContext, pCommand->argv[0]);
	if (pRef == NULL){
		dg87_connection_printf(pConnection,
				"Can't add a ref '%s' ! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* cmd_rc
 * Ref Count
 * return the number of Refs.
 * Usage : rc 
 */
int cmd_rc(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 0){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "rc");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	dg87_connection_printf(pConnection,"%i\n", 
			pContext->pRefVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_rl
 *  Ref List
 * Usage : rl [options] 
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int cmd_rl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_element* pElement = NULL;
	dg87_task* pRef;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* control number of arguments */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rl [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* find the first Task to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pContext->pRefVector, start);
	else
		pElement = pContext->pRefVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pRef = pElement->pObject;
		/* print ref's name */
		dg87_connection_printf(pConnection, "%s%s",
			pBeg, pRef->name);
		dg87_connection_printf(pConnection, "%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/******************************************************************************/
/* ri
 * Ref Information
 * get information for a ref
 * Usage : ri RefName
 */
int cmd_ri(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch,iopt;
	char* optarg;
	dg87_ref *pRef;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ri [options] RefName");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* Task name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Ref's Name", pSep, pRef->name, pEnd);
	
	/* locked property */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Locked", pSep, 
		dg87_boolToAscii(pRef->locked), pEnd);

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
static dg87_connection* pLocalConnection;
/* callback function for messages printing */
int callback(char* message){
	dg87_connection_printf(pLocalConnection, message);
	return 0;
}
int cmd_rp(dg87_connection* pConnection)
{
	dg87_ref *pRef;
	dg87_command* pCommand;
	dg87_context* pContext;

	

	/* TODO add option to change default dg87x deamon to contact */
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control the argument number */
	if (pCommand->argc != 3){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rp [options] RefName DayName Date");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* plan the ref */
	pLocalConnection = pConnection;
	pContext = pConnection->pServer->pData;
	if (dg87_engine_plan(pContext->pEngine, pRef, pCommand->argv[1], 
			pCommand->argv[2], pContext->dg87x_address,
			pContext->dg87x_ipPort, callback) == FALSE){
		dg87_connection_printf(pConnection, 
				"Can't plan the ref '%s' !\n",
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}

/******************************************************************************/
/* rs
 * Ref Simulate 
 * Simulate a day planification and print name which will be planed
 * DayName parameter format is YYYY-MM-DD
 * Usage : rs [options] RefName DayName
 */
int cmd_rs(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of tasks to display */
	int count = -1;
	int index = 0;

	dg87_ref *pRef;
	dg87_element *pTaskElement;
	dg87_task* pTask;
	dg87_command* pCommand;
	time_t day;
	BOOL result;
	BOOL showDescription = FALSE;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control the argument number */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rs [options] RefName Date");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* interpret date argument */
	day = dg87_rule_asciiToDate(pCommand->argv[1]);
	if (day == -1){
		dg87_connection_printf(pConnection, 
			"Incorrect value '%s' for date argument.\n",
			pCommand->argv[1]);
		return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg);
				count = -1;
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg);
				break;
			case 'd' : /* show description */ 
				showDescription = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* for each task */
	pTaskElement = pRef->pTaskVector->pFirst;
	while (pTaskElement != NULL){
		pTask = pTaskElement->pObject;
		pTaskElement = pTaskElement->pNext;
		result = FALSE;
		if (dg87_operationlist_calculate(pTask->pRuleOperationList,
					day, &result, -1) == FALSE){
			dg87_connection_printf(pConnection,
				"Error while rule calculation ! %s\n",
				dg87_err_getErrorStr());
			/* free memory */
			free(pSep); free(pBeg); free(pEnd);
			return 1;
		}

		/* skip tasks which are not planed */
		if (result == FALSE)
			continue;

		/* skip until first task to display */
		if (count < 0){
			index++;
			if (index <= start)
				continue;
			count = 0;
		}
		/* stop if number of task to display is reached */
		if (count >= 0){
			count++;
			if (number > 0 && count > number)
				break;
		}
		dg87_connection_printf(pConnection,"%s%s", 
				pBeg, pTask->name);
		if (showDescription == TRUE)
			dg87_connection_printf(pConnection,"%s%s", 
					pSep, pTask->description);
		dg87_connection_printf(pConnection,"%s\n", pEnd);
	}

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;

}
/******************************************************************************/
/* End of file */

