/* file cmdrule.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "context.h"
#include "cmdrule.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

/******************************************************************************/
/* rrsr
 * Ref Rule Simulate Range
 * Usage : rrsr RefName RuleName date1 [date2] 
 */
int cmd_rrsr(dg87_connection* pConnection)
{	
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_rule* pRule;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 3 || pCommand->argc > 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rrsr RefName RuleName date1 [date2]");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the rule */
	pRule = dg87_util_findRule(pConnection, pRef, pCommand->argv[1]);
	if (pRule == NULL)
		return 1;

	return cmd_operation_rsr(pConnection, pRule->pOperationList);
}
/******************************************************************************/
/* rrsy
 * Ref Rule Simulate Year
 * Usage : rrsy RefName RuleName Year
 */
int cmd_rrsy(dg87_connection* pConnection)
{	
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_rule* pRule;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rrsy RefName RuleName Year");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the rule */
	pRule = dg87_util_findRule(pConnection, pRef, pCommand->argv[1]);
	if (pRule == NULL)
		return 1;

	return cmd_operation_rsy(pConnection, pRule->pOperationList);
}
/******************************************************************************/
/* rrk
 * Ref Rule Kill
 * Usage : rrk RefName RuleName
 */
int cmd_rrk(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_rule* pRule;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rrk RefName RuleName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the rule */
	pRule = dg87_util_findRule(pConnection, pRef, pCommand->argv[1]);
	if (pRule == NULL)
		return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't delete rule '%s' because ref '%s' is locked !\n",
			pRule->name, pRef->name);
		return 1;
	}
	/* delete the rule */
	dg87_err_clearError();
	pContext = pConnection->pServer->pData;
	if (dg87_ref_ruleDeleteByName(pRef, pCommand->argv[1]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't delete the rule '%s' ! %s\n",
				pCommand->argv[1],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rra
 * Ref Rule Add
 * Usage : rra RefName RuleName
 */
int cmd_rra(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_rule* pRule;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 2 || pCommand->argc > 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rra RefName RuleName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't add a rule because ref '%s' is locked !\n",
			pRef->name);
		return 1;
	}
	/* add the rule */
	dg87_err_clearError();
	pContext = pConnection->pServer->pData;
	pRule = dg87_ref_ruleAdd(pRef, pCommand->argv[1]);
	if (pRule == NULL){
		dg87_connection_printf(pConnection,
				"Can't add a rule '%s' ! %s\n",
				pCommand->argv[1],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* cmd_rrc
 * Ref Rule Count
 * return the number of Rules in a referentiel.
 * Usage : rrc RefName
 */
int cmd_rrc(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "rrc RefName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	dg87_connection_printf(pConnection,"%i\n", pRef->pRuleVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_rrl
 * Ref Rule List
 * Usage : rrl [options] RefName
 * options
 * -u : print usage counters
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first rule to display (first index is 0)
 * -Nnumber : number of rule to display (no limit if -N < 1)
 */
int cmd_rrl(dg87_connection* pConnection)
{
	dg87_context* pContext;
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	int optch, iopt;
	char* optarg;
	int start = 0; /* index of the first rule to display */
	int number = 0;/* number of RULEs to display */
	int count = 0; /* number of RULE actually display */
	dg87_element* pElement = NULL;
	dg87_rule* pRule;
	dg87_command* pCommand;
	dg87_ref* pRef;
	BOOL showCounters = FALSE;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rrl [options] RefName");
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first rule to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of rule to display */
				number = atoi(optarg);
				break;
			case 'u': /* show use counters  */
				showCounters = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* find the first Rule to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
				pRef->pRuleVector, start);
	else
		pElement = pRef->pRuleVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of rule to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pRule = pElement->pObject;
		/* print rule name */
		dg87_connection_printf(pConnection, "%s%s",
			pBeg, pRule->name);
		/* print counters */
		if (showCounters == TRUE){
			dg87_connection_printf(pConnection, "%s%i%s%i",
				pSep, pRule->linkedRules,
				pSep, pRule->linkedTasks);
		}

		dg87_connection_printf(pConnection, "%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}

/******************************************************************************/
/* rrok
 * Referential Rule Operation Kill
 * Usage : rrok RefName RuleName OperationIndex 
 */
int cmd_rrok(dg87_connection* pConnection)
{
	int index;
	dg87_ref* pRef;
	dg87_rule* pRule;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rrok RefName RuleName OperationIndex");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the rule */
	pRule = dg87_util_findRule(pConnection,pRef, pCommand->argv[1]);
	if (pRule == NULL) return 1;
	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't delete an operation in rule '%s' because "\
			" ref %s is locked !\n",
			pRule->name, pRef->name);	
		return 1;
	}
	/* find the operation */
	dg87_err_clearError();
	index = atoi(pCommand->argv[2]) -1;
	if (dg87_rule_operationDelete(pRule, index) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't delete the operation for the rule '%s'" \
			"in the ref '%s' ! %s\n",
			pRule->name, pRef->name,
			dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}
/******************************************************************************
 * cmd_rroa
 * Ref Rule Operation Add
 * Usage : rroa RefName RuleName operator type [data]
 */
int cmd_rroa(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_rule* pRule;
	dg87_rule_operation* pOperation;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 4 || pCommand->argc > 6 ){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rroa RefName RuleName operator type [data1] [data2]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the rule */
	pRule = dg87_util_findRule(pConnection,pRef, pCommand->argv[1]);
	if (pRule == NULL) return 1;

	/* create operation */
	pOperation = cmd_operation_ra(pConnection, pRef); 
	if (pOperation == NULL){
		dg87_connection_printf(pConnection,
			"Can't add an operation to the rule '%s' "\
			"of the referential '%s' ! %s\n",
			pRule->name, pRef->name,
			dg87_err_getErrorStr());
		return 1;
	}

	/* add operation to the list */
	if (dg87_rule_operationAdd(pRule, pOperation) == FALSE) {
		dg87_connection_printf(pConnection,
			"Can't add an operation to the rule '%s' "\
			"of the referential '%s' ! %s\n",
			pRule->name, pRef->name,
			dg87_err_getErrorStr());
			dg87_operation_delete(pOperation);
		return 1;
	}

	return 0;
}
/******************************************************************************/
/* cmd_rroc
 * Usage : rroc RefName RuleName
 * Rule Operation Count
 * return the number of operations in the rule.
 */
int cmd_rroc(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_rule* pRule;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rroc RefName RuleName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the rule */
	pRule = dg87_util_findRule(pConnection,pRef, pCommand->argv[1]);
	if (pRule == NULL) return 1;

	/* return operations count */
	dg87_connection_printf(pConnection,"%i\n", 
			pRule->pOperationList->count);
	return 0;
}
/******************************************************************************/
/* cmd_rrol
 * rrol [options] RefName RuleName
 * Ref Rule Operation List
 * Options
 * -i : print operation index
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first operation to display (first index is 0)
 * -Nnumber : number of operation to display (no limit if -N < 1)
 */
int cmd_rrol(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_rule* pRule;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rrol [options] RefName RuleName");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the rule */
	pRule = dg87_util_findRule(pConnection,pRef, pCommand->argv[1]);
	if (pRule == NULL) return 1;
	
	return cmd_operation_ol(pConnection, pRule->pOperationList);
}
/******************************************************************************/
/* cmd_rrom
 * Usage : rrom RefName RuleName OperationIndex delta
 * Referential Rule Operation Move 
 * Move an operation in the list
 */
int cmd_rrom(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_rule* pRule;
	dg87_command* pCommand;
	int index, delta;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rrom RefName RuleName OperationIndex Delta");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the rule */
	pRule = dg87_util_findRule(pConnection,pRef, pCommand->argv[1]);
	if (pRule == NULL) return 1;

	/* get index and delta from parameters */
	index = atoi(pCommand->argv[2]) - 1;
	delta = atoi(pCommand->argv[3]);

	if (dg87_rule_operationMove(pRule, index, delta) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't move an operation in the rule '%s' "\
			"of the referential '%s' ! %s\n",
			pRule->name, pRef->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* End of file */
