/* file cmdtarget.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdtarget.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>


/******************************************************************************/
/* tk
 * Target Kill
 * Usage : tk TargetName
 */
int cmd_tk(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"tk TargetName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	if (dg87_context_targetDeleteByName(pContext, 
				pCommand->argv[0]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't delete the target '%s' ! %s!\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* ta
 * Target Add
 * Usage : ta TargetName [description]
 */
int cmd_ta(dg87_connection* pConnection)
{
	dg87_target* pTarget;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc < 1 || pCommand->argc > 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ta TargetName [description]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* add the target */
	dg87_err_clearError();
	pTarget = dg87_context_targetAdd(pContext, 
			pCommand->argv[0]);
	if (pTarget == NULL){
		dg87_connection_printf(pConnection,
				"Can't add the target '%s'! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	/* set description */
	if (pCommand->argc == 2){
		dg87_err_clearError();
		if (dg87_target_setDescription(pTarget, pCommand->argv[1])
				== FALSE)
		{
			dg87_connection_printf(pConnection,
				"Can't add the target '%s'! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
			dg87_context_targetDelete(pContext, pTarget);
			return 1;
		}
	}
	return 0;
}
/******************************************************************************/
/* cmd_tc
 * target count
 * Usage : tc
 * return the number of target.
 */
int cmd_tc(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 0){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "tc");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	dg87_connection_printf(pConnection,
		"%i\n", pContext->pTargetVector->count);
	return 0;
}
/******************************************************************************/
/* ti
 * Target information 
 * Usage : ti [options] TargetName
 * Options 
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int cmd_ti(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator begin end */
	/*static char optstring[] = "B:S:E:";*/
	int optch,iopt;
	char* optarg;
	dg87_target *pTarget;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	
	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ti [Options] TargetName");
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
					"Unknown option !\n");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* Target name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Target name", pSep, pTarget->name, pEnd);
	/* target description*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Description", pSep, pTarget->description, pEnd);
	/* target hostname */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Host Name", pSep, 
		(pTarget->pHostName == NULL)?"":pTarget->pHostName, pEnd);
	/* target ipPort's property */
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
		pBeg, "IP Port", pSep, pTarget->ipPort, pEnd);

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
/* cmd_tl
 * target list
 * options
 * -d : print description of target
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first target to display (first index is 0)
 * -Nnumber : number of target to display (no limit if -N < 1)
 */
int cmd_tl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator begin end */
	/*static char optstring[] = "dB:S:E:I:N:";*/
	int optch, iopt;
	char* optarg;
	int start = 0; /* index of the first target to display */
	int number = 0;/* number of targets to display */
	int count = 0; /* number of target actually display */
	dg87_element* pElement = NULL;
	dg87_target* pTarget;
	BOOL showDescription = FALSE;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* control number of arguments */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"tl [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");
	showDescription = FALSE;

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first target to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of target to display */
				number = atoi(optarg);
				break;
			case 'd': /* description */
				showDescription = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
					"Unknown option !\n");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	
	/* find the first TASK to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pContext->pTargetVector, start);
	else
		pElement = pContext->pTargetVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pTarget = pElement->pObject;
		dg87_connection_printf(pConnection,"%s%s", 
			pBeg, pTarget->name, pEnd);
		if (showDescription == TRUE)
			dg87_connection_printf(pConnection,"%s%s", 
				pSep, pTarget->description);
		dg87_connection_printf(pConnection,"%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/***************************************************************************/
/* tsh
 * Target Set Hostname 
 * Usage : tsh TargetName Hostname
 */
int cmd_tsh(dg87_connection* pConnection)
{
	dg87_target *pTarget;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"tsh TargetName HostName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* change the hostname */
	dg87_err_clearError();
	if (dg87_target_setHostName(pTarget, pCommand->argv[1]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't change hostname for the target '%s' ! %s\n",
			pTarget->name, dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}
/***************************************************************************/
/* tsd
 * Target Set Description
 * Usage : tsd TargetName Description
 */
int cmd_tsd(dg87_connection* pConnection)
{
	dg87_target *pTarget;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"tsd TargetName description");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* change the description */
	dg87_err_clearError();
	if (dg87_target_setDescription(pTarget, pCommand->argv[1]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't change description for the target '%s' ! %s\n",
			pTarget->name, dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}
/***************************************************************************/
/* tsp
 * Target Set ipPort
 * Usage : tsp TargetName IntegerValue 
 */
int cmd_tsp(dg87_connection* pConnection)
{
	dg87_target *pTarget;
	int value;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"tsp TargetName IpPort");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	value = atoi(pCommand->argv[1]); 
	if (value < 1) {
		dg87_connection_printf(pConnection,
			"Incorrect value '%s' for IpPort !\n",
			pCommand->argv[1]);
		return 1;
	}

	/* change the description */
	dg87_err_clearError();
	if (dg87_target_setIpPort(pTarget, value) == FALSE){
		dg87_connection_printf(pConnection,
		"Can't change IpPort's property for the target '%s' ! %s\n",
			pTarget->name, dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}

/***************************************************************************/
/* End of file */
