/* file cmdtask.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "context.h"
#include "cmdtask.h"
#include "util.h"
#include "variable.h"
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>


/******************************************************************************/
/* rtk
 * Ref Task Kill
 * Usage : rtk RefName TaskName
 */
int cmd_rtk(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_task* pTask;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtk RefName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL)
		return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't delete task '%s' because ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}
	/* delete the task */
	dg87_err_clearError();
	pContext = pConnection->pServer->pData;
	if (dg87_ref_taskDeleteByName(pRef, pCommand->argv[1]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't delete the task '%s' ! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rta
 * Ref Task Add
 * Usage : rta RefName TaskName [description]
 */
int cmd_rta(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_task* pTask;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 2 || pCommand->argc > 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rta RefName TaskName [description]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't add a task because ref '%s' is locked !\n",
			pRef->name);
		return 1;
	}
	/* add the task */
	dg87_err_clearError();
	pContext = pConnection->pServer->pData;
	pTask = dg87_ref_taskAdd(pRef, pCommand->argv[1]);
	if (pTask == NULL){
		dg87_connection_printf(pConnection,
				"Can't add a task '%s' ! %s\n",
				pCommand->argv[1],
				dg87_err_getErrorStr());
		return 1;
	}
	/* change the description */
	if (pCommand->argc == 3) {
		dg87_err_clearError();
		if (dg87_task_setDescription(pTask, pCommand->argv[2]) 
				== FALSE){
			dg87_connection_printf(pConnection,
				"Can't add a task '%s' ! %s\n",
				pTask->name, dg87_err_getErrorStr());
			dg87_ref_taskDelete(pRef, pTask);
			return 1;
		}
	}
	return 0;
}
/******************************************************************************/
/* cmd_rtc
 * Ref Task Count
 * return the number of Tasks in a referentiel.
 * Usage : rtc RefName
 */
int cmd_rtc(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "rtc RefName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	dg87_connection_printf(pConnection,"%i\n", pRef->pTaskVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_rtl
 * Ref Task List
 * Usage : rtl [options] RefName
 * options
 * -d : print description of task
 * -t : show target and queue
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int cmd_rtl(dg87_connection* pConnection)
{
	dg87_context* pContext;
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/* "dtB:S:E:I:N:" */
	int optch, iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_element* pElement = NULL;
	dg87_task* pTask;
	BOOL showDescription = FALSE;
	BOOL showTarget = FALSE;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtl [options] RefName");
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");
	showDescription = FALSE;

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg);
				break;
			case 'd': /* description */
				showDescription = TRUE;
				break;
			case 't': /* show target*/
				showTarget = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* find the first Task to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pRef->pTaskVector, start);
	else
		pElement = pRef->pTaskVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pTask = pElement->pObject;
		/* print taks name */
		dg87_connection_printf(pConnection, "%s%s",
			pBeg, pTask->name);
		/* print target and queue */
		if (showTarget == TRUE)
			dg87_connection_printf(pConnection, 
				"%s%s%s%s", pSep,
				(pTask->pQueue ==NULL) ? "none" :
					pTask->pQueue->pTarget->name,
				pSep,
				(pTask->pQueue ==NULL) ? "none" :
					pTask->pQueue->name);
		/* print description */
		if (showDescription == TRUE)
			dg87_connection_printf(pConnection, 
					"%s%s", pSep, pTask->description);

		dg87_connection_printf(pConnection, "%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/******************************************************************************/
/* rti
 * Ref Task Information 
 * get information for a Task 
 * Usage : rti RefName TaskName
 */
int cmd_rti(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch,iopt;
	char* optarg;
	dg87_task *pTask;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rti RefName TaskName");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL)
		return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* Task name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Task Name", pSep, pTask->name, pEnd);
	/* Ref name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Ref Name", pSep, pTask->pRef->name, pEnd);
	/* Task description*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Description", pSep, pTask->description, pEnd);
	/* waitOldestTask */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Wait Oldest Task", pSep,
		dg87_boolToAscii(pTask->waitOldestTask),
		pEnd);
	/* waitAllDependences*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Wait All Dependences", pSep,
		dg87_boolToAscii(pTask->waitAllDependences),
		pEnd);
	/* onErrorContinue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "On Error Continue", pSep,
		dg87_boolToAscii(pTask->onErrorContinue),
		pEnd);

	/* Task Target */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Target", pSep, 
		(pTask->pQueue==NULL) ? "none" : pTask->pQueue->pTarget->name,
		pEnd);
	/* Target Queue */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Queue", pSep, 
		(pTask->pQueue==NULL) ? "none" : pTask->pQueue->name,
		pEnd);
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
/* rtsq
 * Ref Task Set Queue
 * Usage : rtsq RefName TaskName TargetName QueueName
 */
int cmd_rtsq(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_target *pTarget = NULL;
	dg87_queue* pQueue= NULL;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtsq RefName TaskName TargetName QueueName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set queue for task '%s' because "\
			"ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}
	/* Find the target */
	if (strcmp(pCommand->argv[2], "none") != 0){
		pTarget = dg87_util_findTarget(pConnection, 
				pCommand->argv[2]);
		if (pTarget == NULL) return 1;
	}
	/* Find the queue */
	if (strcmp(pCommand->argv[3], "none") != 0){
		pQueue= dg87_util_findQueue(pConnection, pTarget,
				pCommand->argv[3]);
		if (pQueue == NULL) return 1;
	}
	/* pQueue can be null if queue is "none" */
	dg87_err_clearError();
	if (dg87_task_setQueue(pTask, pQueue) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set queue '%s' for the task '%s' ! %s\n",
			(pQueue == NULL) ? "none" : pQueue->name,
			pTask->name,
			dg87_err_getErrorStr());	
		return 1;
	}
	return 0;
}

/******************************************************************************/
/* rtsd
 * Ref Task Set Description
 * Usage : rtsd RefName TaskName description
 */
int cmd_rtsd(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtsd RefName TaskName description");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set description for task '%s' because "\
			"ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}

	/* change the description */
	dg87_err_clearError();
	if (dg87_task_setDescription(pTask, pCommand->argv[2]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set description of the task '%s' ! %s\n",
			pTask->name, dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rtswot
 * Ref Task Set WaitOldestTask
 * Usage : rtswot RefName TaskName true/false
 */
int cmd_rtswot(dg87_connection* pConnection)
{
	dg87_task *pTask;
	BOOL value;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtswot RefName TaskName true/false");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set waitOldestTask for task '%s' because "\
			"ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}

	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[2])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[2]);
		return 1;
	}

	/* change the property waitOldestTask */
	dg87_err_clearError();
	if (dg87_task_setWaitOldestTask(pTask, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set waitOldestTask of the task '%s' ! %s\n",
			pTask->name, dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rtswad
 * Ref Task Set Wait All Dependences
 * Usage : rtswad RefName TaskName true/false
 */
int cmd_rtswad(dg87_connection* pConnection)
{
	dg87_task *pTask;
	BOOL value;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtswad RefName TaskName true/false");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set waitAllDependences for task '%s' because "\
			"ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}

	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[2])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[2]);
		return 1;
	}

	/* change the property waitAllDependences*/
	dg87_err_clearError();
	if (dg87_task_setWaitAllDependences(pTask, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set waitAllDependences of the task '%s' ! %s\n",
			pTask->name, dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* rtsoec
 * Ref Task Set On Error Continue 
 * Usage : rtsoec RefName TaskName true/false
 */
int cmd_rtsoec(dg87_connection* pConnection)
{
	dg87_task *pTask;
	BOOL value;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtsoec RefName TaskName true/false");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't set OnErrorContinue for task '%s' because "\
			"ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}

	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[2])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[2]);
		return 1;
	}

	/* change the property OnErrorContinue*/
	dg87_err_clearError();
	if (dg87_task_setOnErrorContinue(pTask, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't set OnErrorContinue of the task '%s' ! %s\n",
			pTask->name, dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}


/******************************************************************************/
/* rtdl
 * Ref Task Dependence List
 * Usage : rtdl [options] RefName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first taskdep to display (first index is 0)
 *  -Nnumber : number of taskdep to display (no limit if -N < 1)
 *  -a TaskName : only TaskDep after this TASK
 *  -b TaskName : only TaskDep before this TASK
 *  -t : print dependence type
 */
int cmd_rtdl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "tB:S:E:I:N:b:a:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_item* pItem= NULL;
	dg87_taskdep* pTaskDep;
	dg87_task* pTaskFilter = NULL;
	int filter = 0; /* 0 = no; 1 = before; 2 = after */
	BOOL showDeptype = FALSE;
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtdl [options] RefName");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;


	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg);
				break;
			case 'b': /* only task before */
				pTaskFilter = dg87_ref_taskGetByName(
					pRef, optarg);
				filter = 1;
				break;
			case 'a': /* only task after */
				pTaskFilter = dg87_ref_taskGetByName(
					pRef, optarg);
				filter = 2;
				break;
			case 't': /* type of dependence*/
				showDeptype = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* error if filter and task is null */
	if (filter > 0 && pTaskFilter == NULL){
		dg87_connection_printf(pConnection, 
				"Can't find the filter task !\n");
		free(pSep); free(pBeg); free(pEnd);
		return 1;
	}

	/* find the first Taskdep to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pRef->pTaskdepList, start);
	else
		pItem = pRef->pTaskdepList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pTaskDep = pItem->pObject;
		if (filter == 0 ||
			(filter == 1 && pTaskDep->pTaskAfter == pTaskFilter) ||
			(filter == 2 && pTaskDep->pTaskBefore == pTaskFilter)){
			/* print name of TaskBefore and TaskAfter */
			dg87_connection_printf(pConnection,"%s%s%s%s", pBeg,
					pTaskDep->pTaskBefore->name, pSep, 
					pTaskDep->pTaskAfter->name);
			if (showDeptype == TRUE){
				dg87_connection_printf(pConnection,
					"%s%s", pSep,
					dg87_taskdep_typeToAscii
						(pTaskDep->type));
			}
			dg87_connection_printf(pConnection,"%s\n", pEnd);
		}
		pItem = pItem->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* rtdc
 * Ref Task Dependence Count
 * Usage : rtdc RefName
 */
int cmd_rtdc(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "rtdc RefName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	dg87_connection_printf(pConnection,"%i\n", pRef->pTaskdepList->count);
	return 0;
}
/******************************************************************************/
/* rtda
 * Ref Task Dependence Add
 * Usage : rtdc RefName TaskBeforeName TaskAfterName [type]
 */
int cmd_rtda(dg87_connection* pConnection)
{
	dg87_task *pTaskBefore, *pTaskAfter;
	dg87_taskdep *pTaskDep;
	dg87_taskdep_type depType;
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc < 3 || pCommand->argc > 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtda RefName TaskBeforeName TaskAfterName [type]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't add a task dependence because "\
			"ref '%s' is locked !\n",
			pRef->name);
		return 1;
	}
	/* find the task before */
	pTaskBefore = dg87_util_findRefTask(pConnection,pRef,pCommand->argv[1]);
	if (pTaskBefore == NULL) return 1;

	/* find the task after */
	pTaskAfter = dg87_util_findRefTask(pConnection,pRef,pCommand->argv[2]);
	if (pTaskAfter == NULL) return 1;

	/* set the type of dependences */
	depType = dg87_taskdep_unknown;
	if (pCommand->argc == 4) {
		depType = dg87_taskdep_asciiToType(pCommand->argv[3]);
		if (depType == dg87_taskdep_unknown){
			dg87_connection_printf(pConnection,
				"Incorrect dependence type '%s' !",
				pCommand->argv[3]);
			return 1;
		}
	}

	/* add a dependence between tasks */
	dg87_err_clearError();
	pContext = pConnection->pServer->pData;
	pTaskDep = dg87_ref_taskdepAdd(pRef, pTaskBefore, pTaskAfter);
	if (pTaskDep == NULL){
		dg87_connection_printf(pConnection,
			"Can't add a dependence between "\
			"task '%s' and task '%s' ! %s\n",
			pTaskBefore->name, pTaskAfter->name,
			dg87_err_getErrorStr());
		return 1;
	}

	/* set dependence type */
	if (depType != dg87_taskdep_unknown){
		dg87_err_clearError();
		if (dg87_taskdep_setType(pTaskDep, depType) == FALSE){
			dg87_connection_printf(pConnection,
				"Can't set dependence type between "\
				"tasks '%s' and '%s' ! %s",
				pTaskBefore->name, pTaskAfter->name,
				dg87_err_getErrorStr());
			/* TODO kill taskdep before exit ? */
			return 1;
		}
	}
	
	return 0;
}
/******************************************************************************/
/* rtdk
 * Ref Task Dependence Kill
 * Usage : rtdk RefName TaskBeforeName TaskAfterName Type
 */
int cmd_rtdk(dg87_connection* pConnection)
{
	dg87_task *pTaskBefore, *pTaskAfter;
	dg87_taskdep *pTaskDep;
	dg87_item *pItem;
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_taskdep_type deptype;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtdk RefName TaskBeforeName TaskAfterName DepType");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find dependence type */
	if ((deptype = dg87_taskdep_asciiToType(pCommand->argv[3])) == -1){
		dg87_connection_printf(pConnection, 
			"Incorrect dependence type '%s'\n",
			pCommand->argv[3]);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;

	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't kill a task dependence because "\
			"ref '%s' is locked !\n",
			pRef->name);
		return 1;
	}
	/* find the task before */
	pTaskBefore = dg87_util_findRefTask(pConnection,pRef,pCommand->argv[1]);
	if (pTaskBefore == NULL) return 1;

	/* find the task after */
	pTaskAfter = dg87_util_findRefTask(pConnection,pRef,pCommand->argv[2]);
	if (pTaskAfter == NULL) return 1;

	/* TODO move this code in a new function dg87_ref_taskdepDelete */
	pContext = pConnection->pServer->pData;
	pItem = pRef->pTaskdepList->pFirst;
	while (pItem != NULL){

		pTaskDep = pItem->pObject;
		if (pTaskDep->pTaskBefore == pTaskBefore &&
				pTaskDep->pTaskAfter == pTaskAfter &&
				pTaskDep->type == deptype){
			dg87_err_clearError();
			if (dg87_list_removeItem(
				pRef->pTaskdepList,
				pItem) == NULL){
				dg87_connection_printf(pConnection, 
					"Can't delete the dependence "\
					"between task '%s' and '%s' ! %s\n",
					pTaskBefore->name, pTaskAfter->name,
					dg87_err_getErrorStr());
				return 1;
			}
			/* success */
			dg87_taskdep_delete(pTaskDep);
			return 0;
		}
		pItem = pItem->pNext;
	}
	/* we don't find the dependence */
	dg87_connection_printf(pConnection, 
		"Can't find a dependence between task '%s' and '%s' "\
		"with type '%s' !\n",
		pTaskBefore->name, pTaskAfter->name,
		dg87_taskdep_typeToAscii(deptype));
	return 0;
}

/******************************************************************************/
/* rtcl
 * Ref Task Condition List
 * Usage : rtcl [options] RefName TaskName
 * options
 *  -Svalue : set the separator use between to fields
 *  -Bvalue : set the begin of the line
 *  -Evalue : set the end of the line 
 *  -Inumber : index of the first condition to display (first index is 0)
 *  -Nnumber : number of condition to display (no limit if -N < 1)
 *  -a TaskName : only condition after this TASK
 *  -b TaskName : only condition before this TASK
 */
int cmd_rtcl(dg87_connection* pConnection)
{
	char* pHour;
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first condition to display */
	int number = 0;/* number of conditions to display */
	int count = 0; /* number of conditions actually display */
	dg87_item* pItem= NULL;
	dg87_condition* pCondition;
	dg87_task* pTask;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtcl [options] RefName TaskName");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;

	/* find the task */
	pTask= dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask== NULL)
		return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first condition to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of condition to display */
				number = atoi(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* find the first condition to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pTask->pConditionList, start);
	else
		pItem = pTask->pConditionList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pCondition = pItem->pObject;
		switch(pCondition->type){
			case condition_hmin:
				pHour = dg87_condition_minuteToHour(
				pCondition->data.pConditionHmin->minutes);	
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s\n", pBeg,
					"hmin", pSep, pHour,pEnd);
				dg87_free(pHour, DG87_CHAR);
				break;
			case condition_hmax:
				pHour = dg87_condition_minuteToHour(
				pCondition->data.pConditionHmax->minutes);	
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s\n", pBeg,
					"hmax", pSep, pHour,pEnd);
				dg87_free(pHour, DG87_CHAR);
				break;
			case condition_hlate:
				pHour = dg87_condition_minuteToHour(
				pCondition->data.pConditionHlate->minutes);	
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s\n", pBeg,
					"hlate", pSep, pHour,pEnd);
				dg87_free(pHour, DG87_CHAR);
				break;
			case condition_file:
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s\n", pBeg, "file",pSep,
					pCondition->data.pConditionFile->pPath,
					pEnd);
				break;
			case condition_manual:
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s\n", pBeg, "manual",pSep,
				pCondition->data.pConditionManual->pMessage,
					pEnd);
				break;
			case condition_ifequal:
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s%s%s\n", pBeg, 
					dg87_condition_typeToAscii(
						pCondition->type), pSep,
					pCondition->data.pConditionIfEqual
						->pVariable->name, pSep,
					pCondition->data.pConditionIfEqual
						->pValue, pEnd);
				break;
			case condition_ifnequal:
				dg87_connection_printf(pConnection,
					"%s%s%s%s%s%s%s\n", pBeg, 
					dg87_condition_typeToAscii(
						pCondition->type), pSep,
					pCondition->data.pConditionIfEqual
						->pVariable->name, pSep,
					pCondition->data.pConditionIfEqual
						->pValue, pEnd);
				break;

			default:
				dg87_connection_printf(pConnection,
					"Unknown type of condition !\n");
				break;
		}
		pItem = pItem->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* rtcc
 * Ref Task Condition Count
 * Usage : rtcc RefName TaskName
 */
int cmd_rtcc(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtcc RefName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask= dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask== NULL) return 1;

	dg87_connection_printf(pConnection,"%i\n", 
			pTask->pConditionList->count);
	return 0;
}

/******************************************************************************/
/* rtca
 * Ref Task Condition Add
 * Usage : rtca RefName TaskName Type Argument [Argument2]
 * with type = hmin, hmax, hlate, file, manual, if_equal or if_not_equal 
 * - with type=hmin, hmax, hlate, argument is an hour 
 * - with type=file, argument is a file string 
 * - with type=manual, argument is a message 
 * - with type=if_equal, if_not_equal, argument are a variable name and a value
 */
int cmd_rtca(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_condition *pCondition = NULL;
	dg87_condition_type type;
	int minutes;
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_variable* pVariable;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 4 || pCommand->argc > 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtca RefName TaskName type argument [argument2]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask= dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask== NULL) return 1;

	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't add a condition to the task '%s' because "\
			"ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}
	/* interpret condition type argument */
	type = dg87_condition_asciiToType(pCommand->argv[2]);

	/* control number of arguments */
	if (type == condition_ifequal || type == condition_ifnequal){
		if (pCommand->argc == 4){
			dg87_connection_printf(pConnection,
				"With 'if_equal' and 'if_not_equal' conditions"\
				", a variable value "\
				"must be passed as optional parameter.\n");
			return 1;
		}
	} else {
		if (pCommand->argc == 5){
			dg87_connection_printf(pConnection,
			"The last optional argument is not required.\n");
			return 1;
		}
	}
	/* add the condition */
	dg87_err_clearError();
	switch(type)
	{
	case condition_hmin:
		minutes = dg87_condition_hourToMinute(pCommand->argv[3]);
		pCondition = dg87_task_conditionHminAdd(pTask, minutes);
		break;
	case condition_hmax:
		minutes = dg87_condition_hourToMinute(pCommand->argv[3]);
		pCondition = dg87_task_conditionHmaxAdd(pTask, minutes);
		break;
	case condition_hlate:
		minutes = dg87_condition_hourToMinute(pCommand->argv[3]);
		pCondition = dg87_task_conditionHlateAdd(pTask, minutes);
		break;
	case condition_file:
		pCondition = dg87_task_conditionFileAdd(pTask, 
				pCommand->argv[3]);
		break;
	case condition_manual:
		pCondition = dg87_task_conditionManualAdd(pTask, 
				pCommand->argv[3]);
	break;
	case condition_ifequal:
		pVariable = dg87_util_findVariable(pConnection, 
				pCommand->argv[3]);
		if (pVariable == NULL)
			return 1;
		pCondition = dg87_task_conditionIfEqualAdd(pTask,
				pVariable, pCommand->argv[4]);
	break;
	case condition_ifnequal:
		pVariable = dg87_util_findVariable(pConnection, 
				pCommand->argv[3]);
		if (pVariable == NULL)
			return 1;
		pCondition = dg87_task_conditionIfNotEqualAdd(pTask,
				pVariable, pCommand->argv[4]);
	break;

	default :
		dg87_connection_printf(pConnection,
				"Incorrect type of condition '%s' !\n",
				pCommand->argv[2]);
		return 1;
	}

	if (pCondition == NULL) {
		dg87_connection_printf(pConnection,
				"Can't add the condition ! %s\n",
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}

/******************************************************************************/
/* rtck
 * Ref Task Condition Kill 
 * rtck RefName TaskName type Data
 * with type = hmin, hmax, hlate, file or manuel
 * argument is an hour, a file string or a message for manuel condition
 */
int cmd_rtck(dg87_connection* pConnection)
{
	dg87_task *pTask;
	dg87_condition *pCondition;
	dg87_item *pItem;
	BOOL findCondition = FALSE;	
	int index = 0;
	int type = 0;
	int minutes = 0;
	dg87_command* pCommand;
	dg87_ref* pRef;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	
	if (pCommand->argc < 4 || pCommand->argc > 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtck RefName TaskName typeOfCondition data1 [data2]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask= dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask== NULL) return 1;

	/* control the locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't kill a condition in the task '%s' because "\
			"ref '%s' is locked !\n",
			pTask->name, pRef->name);
		return 1;
	}

	/* find the type of condition */
	type = dg87_condition_asciiToType(pCommand->argv[2]);
	if (type == condition_error){
		dg87_connection_printf(pConnection,
			"Incorrect condition type '%s' !\n",
			pCommand->argv[1]);
		return 1;
	}

	/* TODO move this code in dg87_condition */
       	/* find the condition */
	pItem = pTask->pConditionList->pFirst;
	index = 0;
	while (pItem != NULL){
		pCondition = pItem->pObject;
		if (pCondition->type == type)
		switch (pCondition->type){

		case condition_hmin:
			minutes = dg87_condition_hourToMinute
				(pCommand->argv[3]);
			if (pCondition->data.pConditionHmin->minutes == minutes)
				findCondition = TRUE;	
			break;
		case condition_hmax:
			minutes = dg87_condition_hourToMinute
				(pCommand->argv[3]);
			if (pCondition->data.pConditionHmax->minutes == minutes)
				findCondition = TRUE;	
			break;
		case condition_hlate:
			minutes = dg87_condition_hourToMinute
				(pCommand->argv[3]);
			if (pCondition->data.pConditionHlate->minutes 
					== minutes)
				findCondition = TRUE;	
			break;
		case condition_file:
			if (strcmp(pCondition->data.pConditionFile->pPath,
					pCommand->argv[3]) == 0)
				findCondition = TRUE;
			break;
		case condition_manual:
			if (strcmp(pCondition->data.pConditionManual->pMessage,
				pCommand->argv[3]) == 0)
				findCondition = TRUE;
			break;
		case condition_ifequal:
			if (strcmp(pCondition->data.pConditionIfEqual
				->pVariable->name, pCommand->argv[3]) == 0 &&
				strcmp(pCondition->data.pConditionIfEqual
					->pValue, pCommand->argv[4]) == 0)
				findCondition = TRUE;
			break;
		case condition_ifnequal:
			if (strcmp(pCondition->data.pConditionIfNotEqual
				->pVariable->name, pCommand->argv[3]) == 0 &&
				strcmp(pCondition->data.pConditionIfNotEqual
					->pValue, pCommand->argv[4]) == 0)
				findCondition = TRUE;
			break;

		case condition_error: /* to remove compilators warning */
			break;

		}
		if (findCondition == TRUE){
			dg87_err_clearError();
			if ( dg87_task_conditionDelete(pTask, index) == FALSE){
				dg87_connection_printf(pConnection,
					"Can't delete the dependence !\n",
					dg87_err_getErrorStr());
				return 1;
			}
			/* success */
			return 0;
		}
		index++;
		pItem = pItem->pNext;
	}
	dg87_connection_printf(pConnection, 
			"Can't find this condition in task '%s' !\n",
			pTask->name);
	return 1;
}
/******************************************************************************/
/* rtsr
 * Ref Task Simulate Range
 * Usage : rts RefName TaskName date1 [date2]
 */
int cmd_rtsr(dg87_connection* pConnection)
{	
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_task* pTask;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 3 || pCommand->argc > 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtsr RefName TaskName date1 [date2]");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL)
		return 1;

	return cmd_operation_rsr(pConnection, pTask->pRuleOperationList);
}
/******************************************************************************/
/* rtsy
 * Ref Task Simulate Year
 * Usage : rts RefName TaskName Year
 */
int cmd_rtsy(dg87_connection* pConnection)
{	
	dg87_command* pCommand;
	dg87_ref* pRef;
	dg87_task* pTask;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtsy RefName TaskName Year");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL)
		return 1;
	/* find the task */
	pTask = dg87_util_findRefTask(pConnection, pRef, pCommand->argv[1]);
	if (pTask == NULL)
		return 1;

	return cmd_operation_rsy(pConnection, pTask->pRuleOperationList);
}
/******************************************************************************/
/* rtok
 * Referential Task Operation Kill
 * Usage : rtok RefName TaskName OperationIndex 
 */
int cmd_rtok(dg87_connection* pConnection)
{
	int index;
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtok RefName TaskName OperationIndex");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't delete an operation in task '%s' because "\
			" ref %s is locked !\n",
			pTask->name, pRef->name);	
		return 1;
	}
	/* find the operation */
	dg87_err_clearError();
	index = atoi(pCommand->argv[2]) -1;
	if (dg87_task_operationDelete(pTask, index) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't delete the operation for the task '%s'" \
			"in the ref '%s' ! %s\n",
			pTask->name, pRef->name,
			dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}
/******************************************************************************
 * cmd_rtoa
 * Ref Task Operation Add
 * Usage : rtoa RefName TaskName operator type [data]
 */
int cmd_rtoa(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_rule_operation* pOperation;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc < 4 || pCommand->argc > 6 ){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtoa RefName TaskName operator type [data1] [data2]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* control locked property */
	if (pRef->locked == TRUE){
		dg87_connection_printf(pConnection,
			"Can't add a rule operation in task'%s' because "\
			" ref %s is locked !\n",
			pTask->name, pRef->name);	
		return 1;
	}
	/* create operation */
	pOperation = cmd_operation_ra(pConnection, pRef);
	if (pOperation == NULL){
		dg87_connection_printf(pConnection,
			"Can't add an operation to the task '%s' "\
			"of the referential '%s' ! %s\n",
			pTask->name, pRef->name,
			dg87_err_getErrorStr());
		return 1;
	}

	/* add operation to the list */
	if (dg87_task_operationAdd(pTask, pOperation) == FALSE) {
		dg87_connection_printf(pConnection,
			"Can't add an operation to the task '%s' "\
			"of the referential '%s' ! %s\n",
			pTask->name, pRef->name,
			dg87_err_getErrorStr());
		dg87_operation_delete(pOperation);
		return 1;
	}

	return 0;
}

/******************************************************************************/
/* cmd_rtoc
 * Usage : rtoc RefName TaskName
 * Task Operation Count
 * return the number of operations in the task.
 */
int cmd_rtoc(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtoc RefName TaskName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* return operations count */
	dg87_connection_printf(pConnection,"%i\n", 
			pTask->pRuleOperationList->count);
	return 0;
}
/******************************************************************************/
/* cmd_rtol
 * rtol [options] RefName TaskName
 * Ref Task Operation List
 * Options
 * -i : print index of operation
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first operation to display (first index is 0)
 * -Nnumber : number of operation to display (no limit if -N < 1)
 */
int cmd_rtol(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtol [options] RefName TaskName");
		return 1;
	}
	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;
	
	return cmd_operation_ol(pConnection, pTask->pRuleOperationList);
}
/******************************************************************************/
/* cmd_rtom
 * Usage : rtom RefName TaskName OperationIndex delta
 * Referential Task Operation Move 
 * Move an operation in the list
 */
int cmd_rtom(dg87_connection* pConnection)
{
	dg87_ref* pRef;
	dg87_task* pTask;
	dg87_command* pCommand;
	int index, delta;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 4){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"rtom RefName TaskName OperationIndex Delta");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the ref */
	pRef = dg87_util_findRef(pConnection, pCommand->argv[0]);
	if (pRef == NULL) return 1;

	/* find the task */
	pTask = dg87_util_findRefTask(pConnection,pRef, pCommand->argv[1]);
	if (pTask == NULL) return 1;

	/* get index and delta from parameters */
	index = atoi(pCommand->argv[2]) - 1;
	delta = atoi(pCommand->argv[3]);

	if (dg87_task_operationMove(pTask, index, delta) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't move an operation in the task '%s' "\
			"of the referential '%s' ! %s\n",
			pTask->name, pRef->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* End of file */
