/* file engine.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef ENGINE_H
#define ENGINE_H 1

#include <libdg87/strbuf.h>
#include <libdg87/vector.h>
#include <libdg87/server.h>
#include "global.h"
#include "ref.h"
#include "context.h"
#include "target.h"

typedef enum dg87_engine_step_enum
{
	dg87_engine_step_disabled,
	dg87_engine_step_prepare,
	dg87_engine_step_connection,
	dg87_engine_step_identification,
	dg87_engine_step_createDay,
	dg87_engine_step_copyTask,
	dg87_engine_step_copyCondition,
	dg87_engine_step_copyJob,
	dg87_engine_step_copyJobDep,
	dg87_engine_step_copyTaskDep,
	dg87_engine_step_validateDay,
	dg87_engine_step_terminate,
	dg87_engine_step_error
} dg87_engine_step;

typedef struct dg87_engine_struct
{
	dg87_context* pContext;
	BOOL enabled;
	dg87_ref* pRefToPlan;
	dg87_vector* pTaskToPlanVector;
	dg87_list* pTaskDepToPlanList;
	char* pNewDayName;
	char* pDate;
	dg87_connection* pConnection;
	dg87_engine_step step;
	dg87_element* pTaskElement;
	dg87_element* pJobElement;
	dg87_item* pJobDepItem;
	dg87_item* pTaskDepItem;
	dg87_item* pConditionItem;
	dg87_strbuf* pLog;
	BOOL dayCreated;
	dg87_vector* pVariableVector;
} dg87_engine;

/*
 * dg87_engine_new
 */
dg87_engine* dg87_engine_new(dg87_context* pContext);

/*
 * dg87_engine_delete
 */
BOOL dg87_engine_delete(dg87_engine* pEngine);

/*
 * dg87_engine_run
 * This function is call by dg87_server.
 */
BOOL dg87_engine_run(dg87_engine* pEngine, dg87_server* pServer);

/*
 * dg87_engine_plan
 */
BOOL dg87_engine_plan(dg87_engine* pEngine, dg87_ref* pRef, 
		char* pNewDayName, char* pDate,
		char* pHost, int ipPort ,
		int (*print)(char*));

/*
 * dg87_engine_cancelPlan
 */
BOOL dg87_engine_cancelPlan(dg87_engine* pEngine);

/*************************************************************************/
/*  functions for variable                                               */
/*************************************************************************/

/*
 * dg87_engine_variableAdd
 * create a Variable object, add this object in the VariableVector 
 * of the engine and return a pointer on this object.
 * Return NULL if the variable object can't be create.
 */
struct dg87_variable_struct* dg87_engine_variableAdd
		(dg87_engine* pEngine, char* pVariableName,
		 char* pValue);

/*
 * dg87_engine_variableCount
 * return the number of variables in the engine.
 * return -1 if pEngine is NULL.
 */
int dg87_engine_variableCount(dg87_engine* pEngine);


/*
 * dg87_engine_variableGetByName
 * return a pointer on the variable which has the name
 * given in parameter. Return NULL if it can't be find.
 */
struct dg87_variable_struct* dg87_engine_variableGetByName
		(dg87_engine* pEngine, char* pName);

/*
 * dg87_engine_variableGetByIndex
 * return a pointer on the variable at the given index.
 */
struct dg87_variable_struct* dg87_engine_variableGetByIndex
		(dg87_engine* pEngine, int index);


/*
 * dg87_engine_variableDeleteByName
 * delete the variable which has the name given in parameter.
 */
BOOL dg87_engine_variableDeleteByName(dg87_engine* pEngine,char* pName);

/*
 * dg87_engine_variableDeleteByIndex
 * delete the variable at the given index.
 */
BOOL dg87_engine_variableDeleteByIndex(dg87_engine* pEngine,int index);

/*
 * dg87_engine_save
 */
BOOL dg87_engine_save(dg87_engine* pEngine, dg87_storage* pStorage);

/*
 * dg87_engine_load
 */
BOOL dg87_engine_load(dg87_engine* pEngine, dg87_storage* pStorage);

#ifdef DEBUG
/*
 * dg87_engine_ckeck
 */
int dg87_engine_check(dg87_engine* pEngine, int (*print)(char*));
#endif /*DEBUG*/

#endif /*ENGINE_H*/
/* End of file */
