/* file job.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef JOB_H 
#define JOB_H 1

#include <config.h>
#include <libdg87/list.h>
#include <libdg87/storage.h>
#include "global.h"
#include "task.h"

#define JOB_MAX_TEMPO 3600

typedef struct dg87_job_struct
{
	char name[DG87_JOB_NAME_LENGTH];
	char description[DG87_JOB_DESC_LENGTH];
	struct dg87_task_struct* pTask;
	char* pPath; /* path for the script */
	char* pArgument; /* argument for the script */
	dg87uint startTempo; /* temporization before execution in seconds */
	dg87uint endTempo; /* temporization after execution in seconds */
	BOOL waitAllDependences;
	BOOL onErrorContinue;
} dg87_job;

/*
 * dg87_job_new 
 * create a new job and initialize the name property
 */
dg87_job* dg87_job_new(struct dg87_task_struct* pTask, char* name);

/*
 * dg87_job_duplicate
 * create a new job by copying another
 */
dg87_job* dg87_job_duplicate(struct dg87_task_struct* pNewTask, 
		dg87_job* pOriginalJob);

/*
 * dg87_job_delete
 */
BOOL dg87_job_delete(dg87_job* pJob);

/*
 * dg87_job_load
 */
BOOL dg87_job_load(dg87_job* pJob, dg87_storage* pStorage);

/*
 * dg87_job_save
 */
BOOL dg87_job_save(dg87_job* pJob, dg87_storage* pStorage);

/*
 * dg87_job_setDescription
 * set the description of the job.
 */
BOOL dg87_job_setDescription(dg87_job* pJob, char* pDescription);

/*
 * dg87_job_setTempo
 * set the startTempo and EndTempo for the job.
 */
BOOL dg87_job_setTempo(dg87_job* pJob, int startTempo, int endTempo);

/*
 * dg87_job_setPath
 * set the path of the job.
 * pPath could be NULL or zero length.
 */
BOOL dg87_job_setPath(dg87_job* pJob,char* pPath);

/*
 * dg87_job_setArgument
 * set the argument of the job.
 * pArgument could be NULL or zero length.
 */
BOOL dg87_job_setArgument(dg87_job* pJob,char* pArgument);

/*
 * dg87_job_setWaitAllDependences
 */
BOOL dg87_job_setWaitAllDependences(dg87_job* pJob, BOOL value);

/*
 * dg87_job_setOnErrorContinue
 */
BOOL dg87_job_setOnErrorContinue(dg87_job* pJob, BOOL value);

/*
 * dg87_job_findUpStreamJobs
 * This function adds to the list given in parameter all upstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_job_findUpStreamJobs(dg87_job* pJob, dg87_list* pJobList);

/*
 * dg87_job_findDownStreamJobs
 * This function adds to the list given in parameter all downstream jobs
 * of a job.
 * The function return FALSE if an error occurres.
 */
BOOL dg87_job_findDownStreamJobs(dg87_job* pJob, dg87_list* pJobList);


#ifdef DEBUG
/*
 * dg87_job_ckeck
 */
int dg87_job_check(dg87_job* pJob, int (*print)(char*));
#endif /*DEBUG*/

#endif
/* End of file */
