/* file jobdep.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef JOBDEP_H
#define JOBDEP_H 1
#include <libdg87/global.h>
#include <libdg87/storage.h>
#include "global.h"

/* type of jobs dependences */
typedef enum {
	dg87_jobdep_unknown,
	dg87_jobdep_always,
	dg87_jobdep_onSuccess,
	dg87_jobdep_onError
} dg87_jobdep_type;

/* structure for jobdep objects */
typedef struct dg87_jobdep_struct
{
	struct dg87_job_struct* pJobBefore;
	struct dg87_job_struct* pJobAfter;
	dg87_jobdep_type type;
} dg87_jobdep;

/*
 * dg87_jobdep_new
 * create a new jobdep and return a pointer on it.
 */
dg87_jobdep* dg87_jobdep_new
	(struct dg87_job_struct* pJobBefore, struct dg87_job_struct* pJobAfter);

/*
 * dg87_jobdep_delete
 * free memory allocate for the object.
 */
BOOL dg87_jobdep_delete(dg87_jobdep* pJobDep);

/*
 * dg87_jobdep_load
 */
BOOL dg87_jobdep_load(dg87_jobdep* pJobDep, dg87_storage* pStorage);

/*
 * dg87_jobdep_save
 */
BOOL dg87_jobdep_save(dg87_jobdep* pJobDep, dg87_storage* pStorage);

/*
 * dg87_jobdep_setType
 * set the type of dependence for the jobdep.
 */
BOOL dg87_jobdep_setType(dg87_jobdep* pJobDep, dg87_jobdep_type type);

/*
 * dg87_jobdep_asciiToType
 * convert a string into a dependence type.
 * return -1 if error.
 */
dg87_jobdep_type dg87_jobdep_asciiToType(char* pStrType);

/*
 * dg87_jobdep_typeToAscii
 * convert a dependence type into a string
 */
char* dg87_jobdep_typeToAscii(dg87_jobdep_type deptype);

#ifdef DEBUG
/*
 * dg87_jobdep_ckeck
 */
int dg87_jobdep_check(dg87_jobdep* pJobdep, int (*print)(char*));
#endif /*DEBUG*/

#endif /*JOBDEP_H*/

/* End of file */
