/* file main.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include <config.h>
#include <libdg87/server.h>
#include <libdg87/daemon.h>
#include <libdg87/interpret.h>
#include <libdg87/dgerr.h>
#include "ref.h"
#include "engine.h"
#include "cmdtask.h"
#include "cmdtarget.h"
#include "cmdqueue.h"
#include "cmdjob.h"
#include "cmdref.h"
#include "cmdengine.h"
#include "cmdrule.h"
#include "cmdvariable.h"

extern char *optarg;
extern int optind;
extern int opterr;

/* local prototypes */
int cmd_shutdown(dg87_connection* pConnection);
#ifdef DEBUG	
int cmd_check(dg87_connection* pConnection);
#endif

/* array for Diogene's commands */
static dg87_interpret_cmd pCmdArray[] = {

#ifdef DEBUG
	{"check", cmd_check,"Check data (debug release only)","",
		dg87_command_generic, dg87_command_access_none},
#endif

	/* Engine functions */
	{"ei",     cmd_ei,  "Engine Information","B:S:E:",
		dg87_command_engine, dg87_command_access_consult},
	{"ecp",    cmd_ecp, "Engine Cancel Plan","",
		dg87_command_day, dg87_command_access_manage},
	{"el",     cmd_el,  "Engine Log","B:S:E:",
		dg87_command_day, dg87_command_access_consult},
	{"ese",    cmd_ese, "Engine Set Enabled","",
		dg87_command_engine, dg87_command_access_manage},

	/* Queue functions */
	{"qa",     cmd_qa,  "Queue add","",
		dg87_command_target, dg87_command_access_admin},
	{"qc",     cmd_qc,  "Queue count","",
		dg87_command_target, dg87_command_access_consult},
	{"qi",     cmd_qi,  "Queue information","B:S:E:",
		dg87_command_target, dg87_command_access_consult},
	{"qk",     cmd_qk,  "Queue kill","",
		dg87_command_target, dg87_command_access_admin},
	{"ql",     cmd_ql,  "Queue list","dtB:S:E:I:N:",
		dg87_command_target, dg87_command_access_consult},
	{"qrtl",   cmd_qrtl,"Queue ref Task List","dB:S:E:I:N:",
		dg87_command_target, dg87_command_access_consult},
	{"qrtc",   cmd_qrtc,"Queue ref Task Count","",
		dg87_command_target, dg87_command_access_consult},
	{"qsd",    cmd_qsd, "Queue Set Description","",
		dg87_command_target, dg87_command_access_manage},
	{"qsm",    cmd_qsm ,"Queue Set Max","",
		dg87_command_target, dg87_command_access_manage},

	/* Ref commands */
	{"ra",    cmd_ra,  "Ref Add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rc",    cmd_rc,  "Ref Count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rd",    cmd_rd,  "Ref Duplicate","",
		dg87_command_ref, dg87_command_access_admin},
	{"ri",    cmd_ri,  "Ref Information","B:S:E:",
		dg87_command_ref, dg87_command_access_consult},
	{"rk",    cmd_rk,  "Ref Kill","",
		dg87_command_ref, dg87_command_access_admin},
	{"rl",    cmd_rl,  "Ref List","B:S:E:I:N:",
		dg87_command_ref, dg87_command_access_consult},
	{"rp",    cmd_rp,  "Ref Plan","H:P:",
		dg87_command_day, dg87_command_access_admin},
	{"rs",    cmd_rs,  "Ref Simulate","B:S:E:I:N:d",
		dg87_command_ref, dg87_command_access_consult},

	/* Ref Job functions */
	{"rja",    cmd_rja,   "Ref Job Add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rjc",    cmd_rjc,   "Ref Job Count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rjda",   cmd_rjda,  "Ref Job Dep Add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rjdc",   cmd_rjdc,  "Ref Job Dep count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rjdk",   cmd_rjdk,  "Ref Job Dep Kill","",
		dg87_command_ref, dg87_command_access_admin},
	{"rjdl",   cmd_rjdl,  "Ref Job Dep List","tB:S:E:I:N:b:a:",
		dg87_command_ref, dg87_command_access_consult},
	{"rji",    cmd_rji,   "Ref Job Information", "B:S:E:",
		dg87_command_ref, dg87_command_access_consult},
	{"rjk",    cmd_rjk,   "Ref Job Kill", "",
		dg87_command_ref, dg87_command_access_admin},
	{"rjl",    cmd_rjl,   "Ref Job List", "dtB:S:E:I:N:",
		dg87_command_ref, dg87_command_access_consult},
	{"rjsa",   cmd_rjsa,  "Ref Job Set Argument", "",
		dg87_command_ref, dg87_command_access_manage},
	{"rjsd",   cmd_rjsd,  "Ref Job Set Description", "",
		dg87_command_ref, dg87_command_access_manage},
	{"rjsoec", cmd_rjsoec,"Ref Job Set On Error Continue", "",
		dg87_command_ref, dg87_command_access_manage},
	{"rjsp",   cmd_rjsp,  "Ref Job Set Path", "",
		dg87_command_ref, dg87_command_access_manage},
	{"rjst",   cmd_rjst,  "Ref Job Set Tempo", "",
		dg87_command_ref, dg87_command_access_manage},
	{"rjswad", cmd_rjswad,"Ref Job Set Wait All Dependences", "",
		dg87_command_ref, dg87_command_access_manage},

	/* Ref Rule functions */
	{"rra",    cmd_rra,   "Ref Rule add", "",
		dg87_command_ref, dg87_command_access_admin},
	{"rrc",    cmd_rrc,   "Ref Rule count",
		"",dg87_command_ref, dg87_command_access_consult},
	{"rrk",    cmd_rrk,   "Ref Rule kill", "",
		dg87_command_ref, dg87_command_access_admin},
	{"rrl",    cmd_rrl,   "Ref Rule list", "uB:S:E:I:N:",
		dg87_command_ref, dg87_command_access_consult},
	{"rrsr",   cmd_rrsr,  "Ref Rule Simulate Range","ndB:S:E:",
		dg87_command_ref, dg87_command_access_consult},
	{"rrsy",   cmd_rrsy,  "Ref Rule Simulate Year","",
		dg87_command_ref, dg87_command_access_consult},

	/* Ref Rule Operation functions */
	{"rroa",   cmd_rroa,  "Ref Rule Operation add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rroc",   cmd_rroc,  "Ref Rule Operation count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rrok",   cmd_rrok,  "Ref Rule Operation kill","",
		dg87_command_ref, dg87_command_access_admin},
	{"rrol",   cmd_rrol,  "Ref Rule Operation list","iB:S:E:I:N:",
		dg87_command_ref, dg87_command_access_consult},
	{"rrom",   cmd_rrom,  "Ref Rule Operation Move","",
		dg87_command_ref, dg87_command_access_manage},


	/* Ref Task functions */
	{"rta",    cmd_rta,   "Ref Task add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rtc",    cmd_rtc,   "Ref Task count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rtca",   cmd_rtca,  "Ref Task Condition Add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rtcc",   cmd_rtcc,  "Ref Task Condition count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rtck",   cmd_rtck,  "Ref Task Condition Kill","",
		dg87_command_ref, dg87_command_access_admin},
	{"rtcl",   cmd_rtcl,  "Ref Task Condition List","B:S:E:I:N:",
		dg87_command_ref, dg87_command_access_consult},
	{"rtda",   cmd_rtda,  "Ref Task Dep Add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rtdc",   cmd_rtdc,  "Ref Task Dep count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rtdk",   cmd_rtdk,  "Ref Task Dep Kill","",
		dg87_command_ref, dg87_command_access_admin},
	{"rtdl",   cmd_rtdl,  "Ref Task Dep List","tB:S:E:I:N:b:a:",
		dg87_command_ref, dg87_command_access_consult},
	{"rti",    cmd_rti,   "Ref Task Information","B:S:E:",
		dg87_command_ref, dg87_command_access_consult},
	{"rtk",    cmd_rtk,   "Ref Task kill","",
		dg87_command_ref, dg87_command_access_admin},
	{"rtl",    cmd_rtl,   "Ref Task list","dtB:S:E:I:N:",
		dg87_command_ref, dg87_command_access_consult},
	{"rtsr",   cmd_rtsr,  "Ref Task Simulate Range","ndB:S:E:",
		dg87_command_ref, dg87_command_access_consult},
	{"rtsy",   cmd_rtsy,  "Ref Task Simulate Year","",
		dg87_command_ref, dg87_command_access_consult},

	/* Ref Task Operation functions */
	{"rtoa",   cmd_rtoa,  "Ref Task Operation add","",
		dg87_command_ref, dg87_command_access_admin},
	{"rtoc",   cmd_rtoc,  "Ref Task Operation count","",
		dg87_command_ref, dg87_command_access_consult},
	{"rtok",   cmd_rtok,  "Ref Task Operation kill","",
		dg87_command_ref, dg87_command_access_manage},
	{"rtol",   cmd_rtol,  "Ref Task Operation list","iB:S:E:I:N:",
		dg87_command_ref, dg87_command_access_consult},
	{"rtom",   cmd_rtom,  "Ref Task Operation Move","",
		dg87_command_ref, dg87_command_access_manage},

	{"rtsd",   cmd_rtsd,  "Ref Task Set Description","",
		dg87_command_ref, dg87_command_access_manage},
	{"rtsoec", cmd_rtsoec,"Ref Task Set On Error Continue","",
		dg87_command_ref, dg87_command_access_manage},
	{"rtsq",   cmd_rtsq,  "Ref Task Set Queue","",
		dg87_command_ref, dg87_command_access_manage},
	{"rtswot", cmd_rtswot,"Ref Task Set Wait Oldest Task","",
		dg87_command_ref, dg87_command_access_manage},
	{"rtswad", cmd_rtswad,"Ref Task Set Wait All Dependences","",
		dg87_command_ref, dg87_command_access_manage},

	/* Save function */
	{"save",    cmd_save, "Save","",
		dg87_command_engine, dg87_command_access_manage},

	/* Shutdown function */
	{"shutdown", cmd_shutdown,"Stop the execution of dg87d","",
		dg87_command_engine, dg87_command_access_manage},

	/* Target functions */
	{"ta",     cmd_ta,  "Target Add","",
		dg87_command_target, dg87_command_access_admin},
	{"tc",     cmd_tc,  "Target Count","",
		dg87_command_target, dg87_command_access_consult},
	{"ti",     cmd_ti,  "Target Information","B:S:E:",
		dg87_command_target, dg87_command_access_consult},
	{"tk",     cmd_tk,  "Target Kill","",
		dg87_command_target, dg87_command_access_manage},
	{"tl",     cmd_tl,  "Target List","dB:S:E:I:N:",
		dg87_command_target, dg87_command_access_consult},
	{"tsd",    cmd_tsd ,"Target Set Description","",
		dg87_command_target, dg87_command_access_manage},
	{"tsh",   cmd_tsh,  "Target Set HostName","",
		dg87_command_target, dg87_command_access_manage},
	{"tsp",    cmd_tsp ,"Target Set ipPort","",
		dg87_command_target, dg87_command_access_manage},

	/* Variable functions */
	{"va",     cmd_va,  "Variable Value Add","",
		dg87_command_variable, dg87_command_access_admin},
	{"vc",     cmd_vc,  "Variable Count","",
		dg87_command_variable, dg87_command_access_consult},
	{"vi",     cmd_vi,  "Variable Information","B:S:E:",
		dg87_command_variable, dg87_command_access_consult},
	{"vk",     cmd_vk,  "Variable Kill","",
		dg87_command_variable, dg87_command_access_manage},
	{"vl",     cmd_vl,  "Variable List","vB:S:E:I:N:",
		dg87_command_variable, dg87_command_access_consult},
	{"vsv",    cmd_vsv ,"Variable Set Description","",
		dg87_command_variable, dg87_command_access_manage},

	{NULL,NULL,NULL,NULL,0, 0}
};
/******************************************************************************/
#ifdef DEBUG	
/* TODO find a way to remove this local variable */
static dg87_connection* pLocalConnection;
/* callback function for messages printing */
int callBack(char* message){
	dg87_connection_printf(pLocalConnection, message);
	return 0;
}
int cmd_check(dg87_connection* pConnection)
{
	dg87_context* pContext;
	
	if (pConnection == NULL) return 1;
	
	pContext = pConnection->pServer->pData;
	if (pContext == NULL){
		dg87_connection_printf(pConnection,
				"ERROR : Context is NULL !\n");
		return 1;
	}
	pLocalConnection = pConnection;
	if (dg87_context_check(pContext, callBack) != 0){
		dg87_connection_printf(pConnection,"ERROR !\n");
		return 1;
	}
	
	dg87_connection_printf(pConnection,"All is right ;-)\n", PACKAGE);
	return 0;
}
#endif
/******************************************************************************/
int cmd_shutdown(dg87_connection* pConnection)
{
	dg87_connection_printf(pConnection,
			"%s will stop...\n", DG87D_NAME);
	return -1;
}
/******************************************************************************/
static void usage(const char* name){
	char* msg =
		"Diogene87 daemon for referentials managment.\n\n"\
		"Usage : %s [options]\n\n" \
		"Options :\n" \
		" -C path   : path of the configuration file\n" \
		" -d        : start process as daemon\n" \
		" -N        : don't try to load database at startup\n" \
		" -I        : create database file and exit\n" \
		"\n"; 
	fprintf(stderr,msg, name);
}
/******************************************************************************/
static char* getBaseName(const char* path){
	char *baseName = 0;
	if ((baseName = strrchr(path,'/')) != 0)
		++baseName;
	else
		baseName = (char*) path;
	return baseName;
}
/******************************************************************************/
int signal_function(dg87_server* pServer,int signo)
{	
	dg87_context *pContext;
	if (pServer == NULL)
		return FALSE;
	pContext = pServer->pData;

	switch (signo)
	{
		case SIGHUP:
			dg87_context_readConfig(pContext);
			dg87_daemon_log("SIGNAL SIGHUP");
			break;
		case SIGTERM:
			dg87_daemon_log("SIGNAL SIGTERM");
			break;
		case SIGINT:
			dg87_daemon_log("SIGNAL SIGINT");
			break;
		case SIGPIPE:
			dg87_daemon_log("SIGNAL SIGPIPE");
			break;
		case SIGUSR1: /* save database */
			dg87_daemon_log("Signal SIGUSR1 : save database");
			if (dg87_context_save(pContext) == FALSE)
				dg87_daemon_log("Save error !");
			else 
				dg87_daemon_log("Save done !");
			break;
		default:
			dg87_daemon_log("SIGNAL NOT HANDLED");
			break;
	}

	return FALSE;
}
/******************************************************************************/
BOOL work_function(dg87_server* pServer)
{	
	dg87_context* pContext;
	if (pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pContext = pServer->pData;
	return dg87_engine_run(pContext->pEngine, pServer);
}
/******************************************************************************/
int main(int argc, char* argv[])
{
	int optch;
	int res;
	static char optstring[] = "C:hvdNI";
	dg87_daemon* pDaemon = NULL;
	dg87_context* pContext;
	dg87_server* pServer;
	void* pData;
	char* baseName= getBaseName(argv[0]);
	BOOL startDaemon = FALSE;
	BOOL newDatabase = FALSE;
	BOOL initDatabase = FALSE;
	char* pConfFile = NULL;
	struct stat sb;
		
	/* interpret options */
	while ((optch = getopt(argc, argv, optstring)) != -1){
		switch (optch){
			case 'I': /* init database */
				initDatabase = TRUE;
				break;
			case 'N': /* new database */
				newDatabase = TRUE;
				break;
			case 'C': /* configuration file */
				pConfFile = dg87_strdup(optarg,DG87_CHAR);
				break;
			case 'h': /* help */
				usage(baseName);
				return 1;
			case 'v': /* version */
				fprintf(stderr,"%s-%s\n", PACKAGE, VERSION);
				return 1;
			case 'd': /* start as daemon */
				startDaemon = TRUE;
				break;
			default:
				fprintf(stderr,"Use -h for help.\n");
				return 1;
		}
	}


	/* create the context object */
	if (pConfFile == NULL)
		pConfFile = dg87_strdup(DG87D_CONFIGFILE, DG87_CHAR);
	if ((pContext = dg87_context_new(pConfFile)) == NULL){
		dg87_daemon_log("Can't create context !");
		exit(1);
	}
	pData = pContext;
	dg87_free(pConfFile, DG87_CHAR);

	/* daemonize */
	if (startDaemon == TRUE){
		pDaemon = dg87_daemon_new(DG87D_NAME, 
				dg87_conf_getFieldValue(pContext->pConf,
					"USER_ACCOUNT"));
		if (pDaemon == NULL){
			fprintf(stderr,"Can't create daemon object\n");
			dg87_context_delete(pContext);
			return 1;
		}
		
		res = dg87_daemon_start(pDaemon);
		if (res < 0){
			fprintf(stderr,"Can't daemonize !\n");
			dg87_daemon_delete(pDaemon);
			dg87_context_delete(pContext);
			return 1;
		}
		
		if (res == 0){
			/* the parent process terminates here */
			dg87_daemon_delete(pDaemon);
			dg87_context_delete(pContext);
			return 0;
		}
		pContext->pDaemon = pDaemon;
	}

	/* create the server object */
	pServer = dg87_server_new(PACKAGE, DG87D_NAME, VERSION, 
			pContext->ipPort,
			work_function, signal_function, pData, pCmdArray,
			pContext->pPidFile,
			pContext->pDiogenePassword);
	if (pServer == NULL){
		fprintf(stderr,"Can't create server object\n");
		dg87_context_delete(pContext);
		return 1;
	}
	if (dg87_context_setServer(pContext, pServer) == FALSE){
		fprintf(stderr, "Can't set server object\n");
		return 1;
	}

	/* option -I : init database and exit */
	if (initDatabase == TRUE){
		if (stat(pContext->pStorage->pDataFile, &sb) != -1 ||
				errno != ENOENT){
			fprintf(stderr, "Database already exists : %s\n",
				pContext->pStorage->pDataFile);
			dg87_server_delete(pServer);
			dg87_context_delete(pContext);
			return 1;
		}
		fprintf(stderr, "Create database :%s\n", 
				pContext->pStorage->pDataFile);
		if (dg87_context_save(pContext) == FALSE){
			fprintf(stderr,"Can't save database '%s' ! %s\n",
					pContext->pStorage->pDataFile,
					dg87_err_getErrorStr());
			dg87_server_delete(pServer);
			dg87_context_delete(pContext);
			return 1;
		}
		dg87_server_delete(pServer);
		dg87_context_delete(pContext);
		return 0;
	}

	/* load database */
	if (newDatabase == TRUE) {
		fprintf(stderr, "Don't load database, because -N option "
				"was specified.\n");
	} else {
		fprintf(stderr, "Load database :%s\n", 
				pContext->pStorage->pDataFile);
		if (dg87_context_load(pContext) == FALSE){
			fprintf(stderr,"Can't load database '%s' ! %s\n",
					pContext->pStorage->pDataFile,
					dg87_err_getErrorStr());
			dg87_context_delete(pContext);
			return 1;
		}
	}

	fprintf(stderr, "server ready.\n");
	if (dg87_server_run(pServer) == TRUE){
		fprintf(stderr, "Save database :%s\n", 
				pContext->pStorage->pDataFile);
		if (dg87_context_save(pContext) == FALSE){
			fprintf(stderr,"Can't save database '%s' ! %s\n",
					pContext->pStorage->pDataFile,
					dg87_err_getErrorStr());
			dg87_context_delete(pContext);
			dg87_server_delete(pServer);
			return 1;
		}
	}

	dg87_server_delete(pServer);
	dg87_context_delete(pContext);
	if (startDaemon == TRUE){
		dg87_daemon_stop(pDaemon);
		dg87_daemon_delete(pDaemon);
	}
	return 0;
}
/******************************************************************************/
/* End of file */
