/* file queue.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef QUEUE_H 
#define QUEUE_H 1

#include "global.h"
#include "target.h"

typedef struct dg87_queue_struct
{
	char name[DG87_QUEUE_NAME_LENGTH];
	char description[DG87_QUEUE_DESC_LENGTH];
	struct dg87_target_struct* pTarget;
	dg87uint max; /* maxiumum of jobs which can be run at the same time */
	dg87_list* pRefTaskList; /* Tasks which run in this queue */
} dg87_queue;

/*
 * dg87_queue_new 
 * create a new queue and initialize the name property
 */
dg87_queue* dg87_queue_new(struct dg87_target_struct* pTarget, char* name);

/*
 * dg87_queue_delete
 */
BOOL dg87_queue_delete(dg87_queue* pQueue);

/*
 * dg87_queue_load
 */
BOOL dg87_queue_load(dg87_queue* pQueue, dg87_storage* pStorage);

/*
 * dg87_queue_save
 */
BOOL dg87_queue_save(dg87_queue* pQueue, dg87_storage* pStorage);

/*
 * dg87_queue_setDescription
 * set the description of the queue.
 */
BOOL dg87_queue_setDescription(dg87_queue* pQueue, char* pDescription);

/*
 * dg87_queue_setMax
 * set the value of the "max" property.
 * return FALSE if the value can't be change.
 */
BOOL dg87_queue_setMax(dg87_queue* pQueue, int value);


#ifdef DEBUG
/*
 * dg87_queue_ckeck
 */
int dg87_queue_check(dg87_queue* pQueue, int (*print)(char*));
#endif /*DEBUG*/

#endif
/* End of file */
