/* file ref.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef REF_H 
#define REF_H 1

#include "libdg87/vector.h"
#include "task.h"
#include "taskdep.h"
#include "rule.h"

 
typedef struct dg87_ref_struct{
	char name[DG87_REF_NAME_LENGTH];
	BOOL locked;
	dg87_vector* pTaskVector;
	dg87_list* pTaskdepList;
	dg87_vector* pRuleVector;
} dg87_ref;

/*
 * dg87_ref_new
 * create a new ref and return a pointer on this.
 */
dg87_ref* dg87_ref_new(char* name);

/*
 * dg87_ref_delete
 */
BOOL dg87_ref_delete(dg87_ref* pRef);

/*
 * dg87_ref_load
 */
BOOL dg87_ref_load(dg87_ref* pRef, dg87_storage* pStorage);

/*
 * dg87_ref_save
 */
BOOL dg87_ref_save(dg87_ref* pRef, dg87_storage* pStorage);

/*
 * dg87_ref_duplicate
 */
dg87_ref* dg87_ref_duplicate(dg87_ref* pOrignialRef, char* pNewRefName);

/*************************************************************************/
/*  functions for tasks                                                  */
/*************************************************************************/

/*
 * dg87_ref_taskAdd
 * create an TASK object, add this object in the TaskList
 * of the ref and return a pointer on this object.
 * Return NULL if the TASK object can't be create.
 */
dg87_task* dg87_ref_taskAdd(dg87_ref* pRef, char* pTaskName);

/*
 * dg87_ref_TaskCount
 * return the number of TASK.
 * return -1 if pRef is NULL.
 */
int dg87_ref_taskCount(dg87_ref* pRef);

/*
 * dg87_ref_taskGetByName
 * return a pointer on the TASK which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_task* dg87_ref_taskGetByName(dg87_ref* pRef, char* pName);

/*
 * dg87_taskGetByIndex
 * return a pointer on the TASK at the given position.
 */
dg87_task* dg87_ref_taskGetByIndex(dg87_ref* pRef, int index);

/*
 * dg87_ref_taskDeleteByName
 * delete the task which has the name given in parameter.
 */
BOOL dg87_ref_taskDeleteByName(dg87_ref* pRef,char* pName);

/*
 * dg87_ref_taskDeleteByIndex
 * delete the task at the given index.
 */
BOOL dg87_ref_taskDeleteByIndex(dg87_ref* pRef,int index);

/*
 * dg87_ref_taskDelete
 */
BOOL dg87_ref_taskDelete(dg87_ref* pRef,dg87_task* pTask);

/*************************************************************************/
/*  functions for taskdep                                                  */
/*************************************************************************/

/*
 * dg87_ref_taskdepAdd
 * add a dependance between two tasks of the ref. The name of the tasks must
 * be pass in parameters.
 */
dg87_taskdep* dg87_ref_taskdepAdd (dg87_ref* pRef, 
	 struct dg87_task_struct* pTaskBefore, 
	 struct dg87_task_struct* pTaskAfter);

/*
 * dg87_ref_taskdepCount
 * return the number of taskdep the ref.
 * return -1 if pRef is NULL.
 */
int dg87_ref_taskdepCount(dg87_ref* pRef);

/*
 * dg87_ref_taskdepGet
 * return a pointer on the taskdep at the given index.
 */
dg87_taskdep* dg87_ref_taskdepGet(dg87_ref* pRef, int index);

/*
 * dg87_ref_taskdepDelete
 * delete the taskdep at the given index.
 */
BOOL dg87_ref_taskdepDelete(dg87_ref* pRef,int index);

/*************************************************************************/
/*  functions for rules                                                  */
/*************************************************************************/

/*
 * dg87_ref_ruleAdd
 * create an rule object, add this object in the rulevector
 * of the ref and return a pointer on this object.
 * Return NULL if the rule object can't be create.
 */
dg87_rule* dg87_ref_ruleAdd(dg87_ref* pRef, char* pRuleName);

/*
 * dg87_ref_ruleCount
 * return the number of rule.
 * return -1 if pRef is NULL.
 */
int dg87_ref_RuleCount(dg87_ref* pRef);

/*
 * dg87_ref_ruleGetByName
 * return a pointer on the rule which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_rule* dg87_ref_ruleGetByName(dg87_ref* pRef, char* pName);

/*
 * dg87_ruleGetByIndex
 * return a pointer on the rule at the given position.
 */
dg87_rule* dg87_ref_ruleGetByIndex(dg87_ref* pRef, int index);

/*
 * dg87_ref_ruleDeleteByName
 * delete the rule which has the name given in parameter.
 */
BOOL dg87_ref_ruleDeleteByName(dg87_ref* pRef,char* pName);

/*
 * dg87_ref_ruleDeleteByIndex
 * delete the rule at the given index.
 */
BOOL dg87_ref_ruleDeleteByIndex(dg87_ref* pRef,int index);

#ifdef DEBUG
/*
 * dg87_ref_ckeck
 */
int dg87_ref_check(dg87_ref* pRef, int (*print)(char*));
#endif /*DEBUG*/

#endif

/* End of file */
