/* file target.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef TARGET_H 
#define TARGET_H 1

#include "global.h"
#include <libdg87/vector.h>
#include <libdg87/list.h>
#include <libdg87/storage.h>
#include "task.h"

typedef struct dg87_target_struct
{
	char name[DG87_TARGET_NAME_LENGTH];
	char description[DG87_TARGET_DESC_LENGTH];
	char* pHostName; /* can be null */
	dg87uint ipPort;
	dg87_vector* pQueueVector;
} dg87_target;

/*
 * dg87_target_new 
 * create a new target and initialize the name property
 */
dg87_target* dg87_target_new(char* name);

/*
 * dg87_target_delete
 */
BOOL dg87_target_delete(dg87_target* pTarget);

/*
 * dg87_target_load
 */
BOOL dg87_target_load(dg87_target* pTarget, dg87_storage* pStorage);

/*
 * dg87_target_save
 */
BOOL dg87_target_save(dg87_target* pTarget, dg87_storage* pStorage);

/*
 * dg87_target_setHostName
 * set the HostName of the target. The pHostName can be null : the
 * target name will be used to contact remote computer.
 */
BOOL dg87_target_setHostName(dg87_target* pTarget, char* pHostName);

/*
 * dg87_target_setDescription
 * set the description of the target.
 */
BOOL dg87_target_setDescription(dg87_target* pTarget, char* pDescription);

/* dg87_target_setIpPort 
 * change the IpPort on which the tareget is listen
 * return FALSE if it can't be change
 */
BOOL dg87_target_setIpPort(dg87_target* pTarget, dg87int value);

/*************************************************************************/
/*  functions for queue                                                  */
/*************************************************************************/

/*
 * dg87_target_queueAdd
 * create a Queue object, add this object in the QueueVector
 * of the target and return a pointer on this object.
 * Return NULL if the Queue object can't be create.
 */
struct dg87_queue_struct* dg87_target_queueAdd
		(dg87_target* pTarget, char* pQueueName);

/*
 * dg87_target_queueCount
 * return the number of queue in the target.
 * return -1 if pTarget is NULL.
 */
int dg87_target_queueCount(dg87_target* pTarget);


/*
 * dg87_target_queueGetByName
 * return a pointer on the queue which has the name
 * given in parameter. Return NULL if it can't be find.
 */
struct dg87_queue_struct* dg87_target_queueGetByName
		(dg87_target* pTarget, char* pName);

/*
 * dg87_target_queueGetByIndex
 * return a pointer on the queue at the given index.
 */
struct dg87_queue_struct* dg87_target_queueGetByIndex
		(dg87_target* pTarget, int index);


/*
 * dg87_target_queueDeleteByName
 * delete the queue which has the name given in parameter.
 */
BOOL dg87_target_queueDeleteByName(dg87_target* pTarget,char* pName);

/*
 * dg87_target_queueDeleteByIndex
 * delete the queue at the given index.
 */
BOOL dg87_target_queueDeleteByIndex(dg87_target* pTarget,int index);

/*
 * dg87_target_queueDelete
 */
BOOL dg87_target_queueDelete(dg87_target* pTarget,
		struct dg87_queue_struct* pQueue);

#ifdef DEBUG
/*
 * dg87_target_ckeck
 */
int dg87_target_check(dg87_target* pTarget, int (*print)(char*));
#endif /*DEBUG*/

#endif /*TARGET_H*/
/* End of file */
