/* file util.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "util.h"
#include "context.h"
#include "engine.h"

/******************************************************************************/
dg87_ref* dg87_util_findRef(dg87_connection *pConnec,char* pName)
{
	dg87_ref* pRef;
	dg87_context* pContext;
	if (pConnec == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pContext = pConnec->pServer->pData;
	dg87_err_clearError();
	pRef = dg87_context_refGetByName(pContext, pName);
	if (pRef == NULL){
		dg87_connection_printf(pConnec,
			"Can't find the ref '%s' ! %s\n",
			pName, dg87_err_getErrorStr());
		return NULL;
	}
	return pRef;
}

/******************************************************************************/
dg87_target* dg87_util_findTarget(dg87_connection *pConnec,char* pName)
{
	dg87_target* pTarget;
	dg87_context* pContext;
	if (pConnec == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pContext = pConnec->pServer->pData;
	dg87_err_clearError();
	pTarget = dg87_context_targetGetByName(pContext, pName);
	if (pTarget == NULL){
		dg87_connection_printf(pConnec,
			"Can't find the target '%s' ! %s\n",
			pName, dg87_err_getErrorStr());
		return NULL;
	}
	return pTarget;
}

/******************************************************************************/
dg87_queue* dg87_util_findQueue(dg87_connection *pConnec,
		dg87_target* pTarget, char* pQueueName)
{
	dg87_queue* pQueue;
	if (pConnec == NULL || pTarget == NULL || pQueueName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	dg87_err_clearError();
	pQueue = dg87_target_queueGetByName(pTarget, pQueueName);
	if (pQueue == NULL){
		dg87_connection_printf(pConnec,
			"Can't find the queue '%s' in the target '%s' ! %s\n",
			pQueueName, pTarget->name, dg87_err_getErrorStr());
		return NULL;
	}
	return pQueue;
}

/******************************************************************************/
dg87_task* dg87_util_findRefTask(dg87_connection *pConnec, 
		dg87_ref* pRef, char* pName)
{
	dg87_task* pTask;
	if (pConnec == NULL || pRef == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	dg87_err_clearError();
	pTask = dg87_ref_taskGetByName(pRef, pName);
	if (pTask == NULL){
		dg87_connection_printf(pConnec,
			"Can't find the task '%s' in the ref '%s' ! %s\n", 
			pName, pRef->name, dg87_err_getErrorStr());
		return NULL;
	}
	return pTask;
}
/******************************************************************************/
dg87_job* dg87_util_findJob(dg87_connection *pConnec,
		dg87_task* pTask, char* pJobName)
{
	dg87_job* pJob;
	if (pConnec == NULL || pTask == NULL || pJobName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	dg87_err_clearError();
	pJob = dg87_task_jobGetByName(pTask, pJobName);
	if (pJob == NULL){
		dg87_connection_printf(pConnec,
			"Can't find the job '%s' in the task '%s' ! %s\n",
			pJobName, pTask->name,
			dg87_err_getErrorStr());
		return NULL;
	}
	return pJob;
}
/******************************************************************************/
dg87_rule* dg87_util_findRule(dg87_connection *pConnec, 
		dg87_ref* pRef, char* pName)
{
	dg87_rule* pRule;
	if (pConnec == NULL || pRef == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	dg87_err_clearError();
	pRule = dg87_ref_ruleGetByName(pRef, pName);
	if (pRule == NULL){
		dg87_connection_printf(pConnec,
			"Can't find the rule '%s' in the ref '%s' ! %s\n", 
			pName, pRef->name, dg87_err_getErrorStr());
		return NULL;
	}
	return pRule;
}
/******************************************************************************/
dg87_variable* dg87_util_findVariable(dg87_connection *pConnec, char* pName)
{
	dg87_variable* pVariable;
	dg87_context* pContext;
	if (pConnec == NULL || pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pContext = pConnec->pServer->pData;
	dg87_err_clearError();
	pVariable = dg87_engine_variableGetByName(pContext->pEngine, pName);
	if (pVariable == NULL){
		dg87_connection_printf(pConnec,
			"Can't find the variable '%s' ! %s\n",
			pName, dg87_err_getErrorStr());
		return NULL;
	}
	return pVariable;

}
/******************************************************************************/
/* End of file */
