/* file cmdengine.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdengine.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

/******************************************************************************/
/* ei
 * Engine Information
 * Usage : ei
 */
int cmd_ei(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch, iopt;
	char* optarg;
	dg87_engine *pEngine;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	pEngine = pContext->pEngine;

	/* control the argument number */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ei [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
		pBeg, "IP port", pSep, 
		pContext->pServer->ipPort, pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Pid file", pSep, 
		(pContext->pPidFile==NULL)?"None":pContext->pPidFile, pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Configuration file", pSep, 
		pContext->pConf->pConfigFile, pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Data file", pSep, 
		pContext->pStorage->pDataFile, pEnd);

	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "dg87x address", pSep, 
		pContext->dg87x_address, pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
		pBeg, "dg87x IP port", pSep, 
		pContext->dg87x_ipPort, pEnd);

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
int cmd_save(dg87_connection* pConnection)
{
	dg87_engine *pEngine;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	pEngine = pContext->pEngine;

	/* control the argument number */
	if (pCommand->argc != 0 || pCommand->optc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"save");
		return 1;
	}
	
	dg87_connection_printf(pConnection,"Save\n");
	return 0;
}
/******************************************************************************/
/* End of file */
