/* file cmdjob.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include <libdg87/daemon.h>
#include <libdg87/command.h>
#include <libdg87/daemon.h> /* log function */

#include "context.h"
#include "util.h"
#include "cmdjob.h"


/******************************************************************************/
/* jk
 * Job Kill
 * Usage : jk TaskName JobName DayName
 */
int cmd_jk(dg87_connection* pConnection)
{
	dg87_job *pJob;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"jk TaskName JobName DayName\n");
		return 1;
	}

	/* find the task */
	pJob = dg87_util_findJob(pConnection, 
			pCommand->argv[0],  /* task's name */
			pCommand->argv[1],  /* job's name */
			pCommand->argv[2]); /* day's name */
	if (pJob == NULL) return 1;

	/* delete the job */	
	dg87_err_clearError();
	pContext = pConnection->pServer->pData;
	if (dg87_context_jobDelete(pContext, pJob) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't delete the job '%s' '%s' '%s' ! %s\n",
			pCommand->argv[0], 
			pCommand->argv[1], 
			pCommand->argv[2], 
			dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}
/******************************************************************************
 * cmd_ja
 * Ref Job Add
 * Usage : rja DayName TaskName JobName Path [Argument]
 */
int cmd_ja(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_job* pJob;
	char* pArg;
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc < 4 || pCommand->argc > 5){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ja DayName TaskName JobName Path [Argument]");
		return 1;
	}

	/* argument is optional */
	if (pCommand->argc == 5)
		pArg = pCommand->argv[4];
	else
		pArg = "";
	
	/* TODO control if the same job is already submited :
	 * same day, same task, same job*/

	/* add the job to the task */
	dg87_err_clearError();
	pContext = pConnection->pServer->pData;
	pJob = dg87_context_jobAdd(pContext, 
			pCommand->argv[0], /* day's name */
			pCommand->argv[1], /* task's name */
			pCommand->argv[2], /* job's name */
			pCommand->argv[3], /* path */
			pArg);                /* argument */
	 if (pJob == NULL){
		dg87_connection_printf(pConnection,
			"Can't add job '%s' '%s' '%s' ! %s\n",
			pCommand->argv[0],
			pCommand->argv[1],
			pCommand->argv[2],
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* cmd_jc
 * Usage : rjc
 * Task Job Count
 * return the number of jobs in the TASK.
 */
int cmd_jc(dg87_connection* pConnection)
{
	dg87_context* pContext;
	if (pConnection == NULL) return 1;
	pContext = pConnection->pServer->pData;
	dg87_connection_printf(pConnection,"%i\n", pContext->pJobList->count);
	return 0;
}
/******************************************************************************/
/* cmd_jl
 * Usage : jl [options]
 * Ref Job List
 * Options
 * -d : print description of job 
 * -t : print task name of job
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first job to display (first index is 0)
 * -Nnumber : number of job to display (no limit if -N < 1)
 * -s : state
 */
int cmd_jl(dg87_connection* pConnection)
{	
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/* static char optstring[] = "sB:S:E:I:N:"; */
	int optch, iopt;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_item* pItem = NULL;
	dg87_job* pJob;
	dg87_context* pContext;
	dg87_command* pCommand;
	BOOL showState = FALSE;

	if (pConnection == NULL) return 1;
	pContext = pConnection->pServer->pData;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"jl [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg); 
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg); 
				break;
			case 's': /* show state */
				showState = TRUE;
				break;
				
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
					"Unknown option '%c'!", optopt);
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	
	/* find the first job to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem( pContext->pJobList, start);
	else
		pItem = pContext->pJobList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pJob = pItem->pObject;

		/* print DayName, TaskName, JobName */
		dg87_connection_printf(pConnection, "%s%s%s%s%s%s",
				pBeg, pJob->dayName,
				pSep, pJob->taskName,
				pSep, pJob->jobName);

		/* print state */
		if (showState == TRUE){
			dg87_connection_printf(pConnection, "%s%s", pSep,
				dg87_job_stateToAscii(pJob->state));
		}

		dg87_connection_printf(pConnection, "%s\n",pEnd);
		pItem = pItem->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}


/******************************************************************************/
/* cmd_ji
 * job information 
 * get information for a job
 * Usage : ji [options] DayName TaskName JobName 
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int cmd_ji(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	int optch,iopt;
	dg87_job *pJob;
	dg87_command* pCommand;
	
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	
	/* control number of arguments */
	if (pCommand->argc != 3){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ji [options] DayNameTaskName JobName");
		return 1;
	}

	/* find the job */
	pJob = dg87_util_findJob(pConnection, pCommand->argv[0],
			pCommand->argv[1], pCommand->argv[2]);
	if (pJob == NULL) return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* day name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Day Name", pSep, pJob->dayName, pEnd);
	/* Task name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Task Name", pSep, pJob->taskName, pEnd);
	/* job name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Job Name", pSep, pJob->jobName, pEnd);
	/* Path */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Path", pSep, pJob->pPath, pEnd);
	/* Argument */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Argument", pSep, pJob->pArgument, pEnd);
	/* pid */
	dg87_connection_printf(pConnection,"%s%s%s%ld%s\n",
		pBeg, "Pid", pSep, pJob->pid, pEnd);
	/* state */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "State", pSep, dg87_job_stateToAscii(pJob->state), pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
		pBeg, "State", pSep, pJob->status, pEnd);
	
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);

	return 0;
}
/******************************************************************************/
/* End of file */
