/* file context.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef CONTEXT_H
#define CONTEXT_H 1

#include <libdg87/daemon.h>
#include <libdg87/list.h>
#include <libdg87/server.h>
#include <libdg87/conf.h>
#include <libdg87/storage.h>
#include <config.h>
#include "global.h"
#include "context.h"
#include "job.h"

typedef struct dg87_context_struct
{
	int ipPort;
	dg87_list* pJobList;
	struct dg87_engine_struct* pEngine;
	int dg87x_ipPort;
	char* dg87x_address;
	dg87_server* pServer;
	dg87_conf *pConf; /* configuration file */
	dg87_storage *pStorage;
	char* pPidFile;
	char* pDiogenePassword;
	time_t saveTime;
	dg87uint saveTempo;
} dg87_context;

/*
 * dg87_context_new
 * Create a dg87_context object and return a pointer on it.
 * pConfFile is the path of the configuration file (/etc/diogene87/dg87?.conf)
 */
dg87_context* dg87_context_new(char* pConfFile );

/*
 * dg87_context_delete
 */
BOOL dg87_context_delete(dg87_context* pContext);

/*
 * dg87_context_setServer
 */
BOOL dg87_context_setServer(dg87_context* pContext, 
		struct dg87_server_struct* pServer);

/*
 * dg87_context_makePeriodicSave
 */
BOOL dg87_context_makePeriodicSave(dg87_context* pContext);

/*
 * dg87_context_save
 */
BOOL dg87_context_save(dg87_context* pContext);

/*
 * dg87_context_load
 */
BOOL dg87_context_load(dg87_context* pContext);

/*
 * dg87_context_readConfig
 * This function reads (or re-reads) the configuration file variables
 */
BOOL dg87_context_readConfig(dg87_context* pContext);

/*
 * dg87_context_jobAdd
 */
struct dg87_job_struct*
dg87_context_jobAdd(dg87_context* pContext, char* pTaskName, char* pJobName, 
		char* pDayName, char* pPath, char* pArgument);

/*
 * dg87_context_jobDelete
 */
BOOL dg87_context_jobDelete(dg87_context* pContext, dg87_job *pJob);


#ifdef DEBUG
/*
 * dg87_context_ckeck
 */
int dg87_context_check(dg87_context* pContext, int (*print)(char*));
#endif /*DEBUG*/

#endif /*CONTEXT_H*/
/* End of file */
