/* file engine.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <syslog.h>
#include <string.h>
#include <libdg87/list.h>
#include <libdg87/daemon.h>
#include "context.h"
#include "job.h"
#include "engine.h"

/******************************************************************************/
dg87_engine* dg87_engine_new(dg87_context* pContext)
{
	dg87_engine* pEngine;
	if (pContext == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pEngine = dg87_malloc(sizeof(dg87_engine), DG87_ENGINE);
	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pEngine->pContext = pContext;
	return pEngine;
}
/******************************************************************************/
BOOL dg87_engine_delete(dg87_engine* pEngine)
{

	if (pEngine == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_free(pEngine, DG87_ENGINE);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_engine_run(dg87_engine* pEngine, dg87_server* pServer)
{
	int status;
	dg87_job* pJob;
	dg87_item* pItem;
	pid_t wpid;
	if (pEngine == NULL || pServer == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* save database */
	dg87_context_makePeriodicSave(pEngine->pContext);

	/* for each job */
	pItem = pEngine->pContext->pJobList->pFirst;
	while (pItem != NULL){
		pJob = pItem->pObject;
		pItem = pItem->pNext;
		if (pJob->state == dg87_job_state_uninitialize){
			dg87_job_changeState(pJob,dg87_job_state_readytorun);
		}
		if (pJob->state == dg87_job_state_readytorun){
			dg87_job_run(pJob);
		}
		if (pJob->state == dg87_job_state_old){
			/* job can be remove from the list */
			if (dg87_list_removeObject(pEngine->pContext->pJobList, 
					pJob) == TRUE)
				dg87_job_delete(pJob);

		}
	}

	/* test if one job is finished */
	wpid = waitpid(0,&status,WNOHANG);
	if (wpid > 0){
		/* find the job with the same pid */
		pJob = NULL;
		pItem = pEngine->pContext->pJobList->pFirst;
		while (pItem != NULL){
			pJob = pItem->pObject;
			if (pJob->pid == wpid) 
				break;
			pItem = pItem->pNext;
		}
		if (pJob != NULL){
			if (WIFEXITED(status) != 0){
				/* normal termination */
				pJob->status = WEXITSTATUS(status);
				dg87_job_changeState(pJob, dg87_job_state_exit);
			} else {
				/* job could not be run */
				dg87_job_changeState(pJob,
					dg87_job_state_deleted);
			}
		}
	}

	/* for each job */
	pItem = pEngine->pContext->pJobList->pFirst;
	while (pItem != NULL){
		pJob = pItem->pObject;
		dg87_job_transmitState(pEngine->pContext,pJob);
		pItem = pItem->pNext;
	}


	/* remove old jobs */
	/*dg87_context_jobClean(pEngine->pContext);*/
	/* return TRUE to have a long time before next call */
	return FALSE;

}
/****************************************************************************/
#ifdef DEBUG
int dg87_engine_check(dg87_engine* pEngine, int (*print)(char*))
{
	return 0;
}
#endif /*DEBUG*/

/****************************************************************************/
/* End of file */
