/* file job.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef JOB_H 
#define JOB_H 1

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <config.h>
#include <libdg87/global.h>
#include <libdg87/storage.h>
#include "global.h"

typedef enum{
	dg87_job_state_uninitialize,
	dg87_job_state_readytorun,
	dg87_job_state_deleted, /* job could not be executed */
	dg87_job_state_running,
	dg87_job_state_exit, /* job is finished */
	dg87_job_state_old /* can be remove from the job list */
} dg87_job_state;

typedef struct dg87_job_struct
{
	char jobName[DG87_JOB_NAME_LENGTH];
	char taskName[DG87_TASK_NAME_LENGTH];
	char dayName[DG87_DAY_NAME_LENGTH];
	char* pPath; /* path for the script */
	char* pArgument; /* argument for the script */
	dg87_job_state state;
	pid_t pid; /* process id */
	int status;
	BOOL transmitState;
} dg87_job;

/*
 * dg87_job_new 
 * create a new job and initialize the name property
 */
dg87_job* dg87_job_new(char* pTaskName, char* pJobName, char* pDayName,
		char* pPath, char* pArgument);

/*
 * dg87_job_delete
 */
BOOL dg87_job_delete(dg87_job* pJob);

/*
 * dg87_job_load
 */
BOOL dg87_job_load(dg87_job* pJob, dg87_storage* pStorage);

/*
 * dg87_job_save
 */
BOOL dg87_job_save(dg87_job* pJob, dg87_storage* pStorage);

/*
 * dg87_job_run
 */
BOOL dg87_job_run(dg87_job* pJob);

/*
 * dg87_job_changeState
 * change the value of the state and tell
 * to transfer this state to diogene
 */
BOOL dg87_job_changeState(dg87_job* pJob, dg87_job_state state);

/*
 * dg87_job_transmitState
 * contact diogene's server if necessary to update the state of the job
 */
BOOL dg87_job_transmitState(struct dg87_context_struct* pContext, dg87_job* pJob);

/*
 * dg87_job_stateToAscii
 * convert a job's state to a string
 */
char* dg87_job_stateToAscii(dg87_job_state state);

#ifdef DEBUG
/*
 * dg87_job_ckeck
 */
int dg87_job_check(dg87_job* pJob, int (*print)(char*));
#endif /*DEBUG*/

#endif
/* End of file */
