/* file caller.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <config.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#include <libdg87/daemon.h>
#include <libdg87/server.h>
#include <libdg87/strbuf.h>

#include "engine.h"
#include "context.h"
#include "caller.h"
#include "day.h"
#include "variable.h"

/******************************************************************************/
/* dg87_replaceVariables
 * this functions parses InBuffer to replace variables $<VarName> by
 * its value.
 * If a variable is not declared, the $<VarName> is copied.
 */
/*TODO this function is duplicated in dg87d and dg87x */
static char* dg87_replaceVariables(char* pInBuffer,dg87_vector* pVariableVector)
{
	char *pSubBeg, *pSubEnd;
	char *pVarBeg, *pVarEnd;
	char *pBuffer;
	char *pVarName;
	dg87_variable *pVariable;
	dg87_element* pElement;
	dg87_strbuf* pOutBuf;
	int varSize;

	if (pInBuffer == NULL || pVariableVector == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}

	/* optimization if no variables */
	if (strstr(pInBuffer, "$<") == NULL)
		return dg87_strdup(pInBuffer, DG87_CHAR);

	/* initialize out buffer */
	pOutBuf = dg87_strbuf_new();
	if (pOutBuf == NULL)
		return NULL;

	/* substring points on begin of string */
	pSubBeg = pInBuffer;
	pSubEnd = pSubBeg;

	while( *pSubBeg != '\0') {
		pVariable = NULL;
		pVarBeg = NULL;
		pVarEnd = NULL;

		/* find begin of variable */
		if ((pVarBeg = strstr(pSubEnd, "$<")) == NULL){
			/* skip to end of string */
			pSubEnd = strchr(pSubEnd, '\0');
		} 
		/* find end of variable */
		else if ((pVarEnd = strchr(pVarBeg, '>')) == NULL){
			/* skip to end of string */
			pSubEnd = strchr(pSubEnd, '\0');
		} else {
			varSize = pVarEnd - pVarBeg - 2;
			/* extract variable name */
			pVarName = dg87_malloc(varSize + 1, DG87_CHAR);
			if (pVarName == NULL){
				dg87_strbuf_delete(pOutBuf);
				return NULL;
			}
			strncpy(pVarName, pVarBeg+2, pVarEnd-pVarBeg-2);
			pVarName[varSize] = '\0';

			/* find variable */
			pElement = pVariableVector->pFirst;
			while (pElement != NULL){
				pVariable = pElement->pObject;
				if (strcmp(pVariable->name, pVarName) == 0)
					break;
				pVariable = NULL;
				pElement = pElement->pNext;
			}
			dg87_free(pVarName, DG87_CHAR);

			/* unknown variable */
			if (pVariable == NULL){
				/* search next variable */
				pSubEnd = pVarEnd;
				continue;
			}
			/* end of substring is start of variable */
			pSubEnd = pVarBeg;
		}

		/* copy substring */
		if (dg87_strbuf_ncat(pOutBuf, pSubBeg,
					pSubEnd - pSubBeg) == FALSE){
			dg87_strbuf_delete(pOutBuf);
			return NULL;
		}
		pSubBeg = pSubEnd;

		/* continue if no variable value to copy */
		if (pVariable == NULL)
			continue;

		/* copy variable value */
		if (dg87_strbuf_cat(pOutBuf, 
					pVariable->pValue) == FALSE){
			dg87_strbuf_delete(pOutBuf);
			return NULL;
		}

		/* next sub string starts after end of variable */
		pSubBeg = pVarEnd + 1;
		pSubEnd = pSubBeg;

	}

	/*pBuffer = dg87_strbuf_getBuffer(pOutBuf);*/
	pBuffer = dg87_strdup(pOutBuf->pBuffer, DG87_CHAR);
	dg87_strbuf_delete(pOutBuf);
	return pBuffer;
}
/******************************************************************************/
BOOL dg87_caller_submitJob(dg87_context* pContext, dg87_job* pJob)
{
	dg87_target* pTarget;
	char* pBuffer;
	int size;
	dg87_engine *pEngine;
	char* pArgBuffer;
	char* pPatBuffer;

	if (pJob == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	pEngine = pContext->pEngine;

	/* replace argument variables by values */
	pArgBuffer = dg87_replaceVariables(
			(pJob->pArgument==NULL)?"":pJob->pArgument, 
			pEngine->pVariableVector);
	if (pArgBuffer == NULL)
		return FALSE;

	/* replace path variables by values */
	pPatBuffer = dg87_replaceVariables(
			(pJob->pPath==NULL)?"":pJob->pPath, 
			pEngine->pVariableVector);
	if (pPatBuffer == NULL){
		dg87_free(pArgBuffer, DG87_CHAR);
		return FALSE;
	}

	/*ja "TaskName" "JobName" "DayName" "Path" "[Argument]"*/
	size = strlen(pJob->pTask->name) + strlen(pJob->name) +
		strlen(pJob->pTask->pDay->name) +
		strlen(pPatBuffer) + strlen(pArgBuffer) + 19;
	pBuffer = dg87_malloc(size, DG87_CHAR);
	if (pBuffer == NULL){
		dg87_free(pArgBuffer, DG87_CHAR);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return FALSE;
	}
	snprintf(pBuffer, size ,"ja \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
		       pJob->pTask->pDay->name,
		       pJob->pTask->name, pJob->name,	
		       pPatBuffer, pArgBuffer);
	
	pTarget = pJob->pTask->pQueue->pTarget;

	if (dg87_server_postMessage(pContext->pServer, 
		(pTarget->pHostName == NULL)?pTarget->name:pTarget->pHostName, 
		pTarget->ipPort, pBuffer) == TRUE)
		pJob->state = dg87_job_state_submit;

	dg87_free(pBuffer, DG87_CHAR);
	dg87_free(pArgBuffer, DG87_CHAR);
	dg87_free(pPatBuffer, DG87_CHAR);

	return TRUE;
}
/******************************************************************************/

/* End of file */
