/* file cmdengine.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdengine.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>


/******************************************************************************/
/* ei
 * Engine Information
 * Usage : ei
 */
int cmd_ei(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch, iopt;
	char* optarg;
	dg87_engine *pEngine;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	pEngine = pContext->pEngine;

	/* control the argument number */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ei [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Enabled", pSep, 
		dg87_boolToAscii(pEngine->enabled), pEnd);

	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
		pBeg, "IP port", pSep, 
		pContext->pServer->ipPort, pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Pid file", pSep, 
		(pContext->pPidFile==NULL)?"None":pContext->pPidFile, pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Configuration file", pSep, 
		pContext->pConf->pConfigFile, pEnd);
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Data file", pSep, 
		pContext->pStorage->pDataFile, pEnd);

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
int cmd_ese(dg87_connection* pConnection)
{
	dg87_engine *pEngine;
	dg87_context* pContext;
	dg87_command* pCommand;
	BOOL value;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 1) {
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ese BooleanValue");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* intepret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[0])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[0]);
		return 1;
	}

	pEngine = pContext->pEngine;
	pEngine->enabled = value;

	return 0;
}
/******************************************************************************/
/* etm
 * eXploitation Tasks Monitor 
 * return statistics on tasks 
 * Usage : ets [options] [DayName]
 */
int cmd_etm(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	int optch,iopt;
	char* optarg;
	char state;
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_element* pTaskElement;
	dg87_element* pDayElement;
	dg87_day *pSelectedDay = NULL;
	dg87_day *pDay;
	dg87_task* pTask;

	int nTotal = 0;
	int nWaiting = 0;
	int nRunning = 0;
	int nHolded = 0;
	int nTerminated = 0;
	int nError = 0;
	int nCancel = 0; 

	BOOL showHolded = FALSE;
	BOOL showRunning = FALSE;
	BOOL showError = FALSE;
	BOOL showCancel = FALSE;
	BOOL showTerminated = FALSE;
	BOOL showWaiting = FALSE;

	/* by default, shows job counters and not job list */
	BOOL showCounters = TRUE;
	BOOL showList = FALSE;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	if (pCommand->argc > 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"etm [options] [DayName]");
		return 1;
	}

	/* if DayName argument was supplied */
	if (pCommand->argc == 1){
		/* find the day */
		pSelectedDay = dg87_util_findDay(pConnection,pCommand->argv[0]);
		if (pSelectedDay == NULL) return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'h': /* holded jobs */
				showHolded = TRUE;
				break;
			case 'r': /* running jobs */
				showRunning = TRUE;
				break;
			case 'e': /* jobs on error */
				showError = TRUE;
				break;
			case 'c': /* canceled jobs */
				showCancel = TRUE;
				break;
			case 't': /* terminated jobs */
				showTerminated = TRUE;
				break;
			case 'w': /* waiting jobs */
				showWaiting = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* if filter option was given then shows task list */
	if (showHolded == TRUE || showRunning == TRUE || 
			showError == TRUE || showCancel == TRUE ||
			showTerminated == TRUE || showWaiting == TRUE)
	{
		showCounters = FALSE;
		showList = TRUE;
	}

	/* for each day */
	pDayElement = pContext->pDayVector->pFirst; 
	while (pDayElement != NULL){
		pDay = pDayElement->pObject;
		pDayElement = pDayElement->pNext;

		/* skip unwanted days if day filter was given */
		if (pSelectedDay != NULL && pSelectedDay != pDay)
			continue;

		/* for each task of the day */
		pTaskElement = pDay->pTaskVector->pFirst;
		while (pTaskElement != NULL){
			pTask = pTaskElement->pObject;
			pTaskElement= pTaskElement->pNext;

			nTotal++;
			state = '_';
			switch (pTask->state){

				/* waiting */
				case dg87_task_state_uninitialize:
				case dg87_task_state_wait_for_dep:
				case dg87_task_state_wait_for_condition:
					nWaiting++;
					state = 'W';
					if (showWaiting == FALSE)
						continue;
					break;

				/* terminated */
				case dg87_task_state_terminated:
					nTerminated++;
					state = 'T';
					if (showTerminated == FALSE)
						continue;
					break;

				/* holded */
				case dg87_task_state_abortStopped:
				case dg87_task_state_waitenabled:
					nHolded++;
					state = 'H';
					if (showHolded == FALSE)
						continue;
					break;

				/* running */
				case dg87_task_state_running:
					nRunning++;
					state = 'R';
					if (showRunning == FALSE)
						continue;
					break;

				/* error */
				case dg87_task_state_deleted:
				case dg87_task_state_aborted:
					nError++;
					state = 'E';
					if (showError == FALSE)
						continue;
					break;

				/* cancel */
				case dg87_task_state_ignore:
				case dg87_task_state_ignored:
				case dg87_task_state_canceled:
					nCancel++;
					state = 'C';
					if (showCancel == FALSE)
						continue;
					break;
			}

			/* prints tasks if necessary */
			if (showList == FALSE)
				continue;
			dg87_connection_printf(pConnection,
				"%s%c%s%s%s%s%s%s%s\n",
				pBeg,state,
				pSep,pTask->pDay->name,
				pSep,pTask->name,
				pSep,dg87_task_stateToAscii(pTask->state),
				pEnd);

		} /* next task */
	} /* next day */

	/* print counters if necessary */
	if (showCounters == TRUE){
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Total", pSep, nTotal, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Waiting", pSep, nWaiting, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Holded", pSep, nHolded, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Running", pSep, nRunning, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Error", pSep, nError, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Cancel", pSep, nCancel, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Terminated", pSep, nTerminated, pEnd);
	}

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);

	return 0;
}
/******************************************************************************/
/* ejm
 * eXploitation Jobs Monitor 
 * return statistics on jobs
 * Usage : ejm [options] [DayName]
 */
int cmd_ejm(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	int optch,iopt;
	char* optarg;
	char state;
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_element* pJobElement;
	dg87_element* pTaskElement;
	dg87_element* pDayElement;
	dg87_day *pSelectedDay = NULL;
	dg87_day *pDay;
	dg87_task* pTask;
	dg87_job* pJob;

	int nTotal = 0;
	int nWaiting = 0;
	int nRunning = 0;
	int nHolded = 0;
	int nTerminated = 0;
	int nError = 0;
	int nCancel = 0; 

	BOOL showHolded = FALSE;
	BOOL showRunning = FALSE;
	BOOL showError = FALSE;
	BOOL showCancel = FALSE;
	BOOL showTerminated = FALSE;
	BOOL showWaiting = FALSE;

	/* by default, shows job counters and not job list */
	BOOL showCounters = TRUE;
	BOOL showList = FALSE;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	if (pCommand->argc > 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ejm [options] [DayName]");
		return 1;
	}

	/* if DayName argument was supplied */
	if (pCommand->argc == 1){
		/* find the day */
		pSelectedDay = dg87_util_findDay(pConnection,pCommand->argv[0]);
		if (pSelectedDay == NULL) return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'h': /* holded jobs */
				showHolded = TRUE;
				break;
			case 'r': /* running jobs */
				showRunning = TRUE;
				break;
			case 'e': /* jobs on error */
				showError = TRUE;
				break;
			case 'c': /* canceled jobs */
				showCancel = TRUE;
				break;
			case 't': /* terminated jobs */
				showTerminated = TRUE;
				break;
			case 'w': /* waiting jobs */
				showWaiting = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* if filter option was given then shows job list */
	if (showHolded == TRUE || showRunning == TRUE || 
			showError == TRUE || showCancel == TRUE ||
			showTerminated == TRUE || showWaiting == TRUE)
	{
		showCounters = FALSE;
		showList = TRUE;
	}

	/* for each day */
	pDayElement = pContext->pDayVector->pFirst; 
	while (pDayElement != NULL){
		pDay = pDayElement->pObject;
		pDayElement = pDayElement->pNext;

		/* skip unwanted days if day filter was given */
		if (pSelectedDay != NULL && pSelectedDay != pDay)
			continue;

		/* for each task of the day */
		pTaskElement = pDay->pTaskVector->pFirst;
		while (pTaskElement != NULL){
			pTask = pTaskElement->pObject;
			pTaskElement= pTaskElement->pNext;

			/* for each job of the task */
			pJobElement = pTask->pJobVector->pFirst;
			while (pJobElement != NULL){
				pJob = pJobElement->pObject;
				pJobElement= pJobElement->pNext;

				nTotal++;
				state = '_';
				switch (pJob->state){

					/* waiting */
					case dg87_job_state_uninitialize:
					case dg87_job_state_waitfordep:
						nWaiting++;
						state = 'W';
						if (showWaiting == FALSE)
							continue;
						break;

					/* terminated */
					case dg87_job_state_terminated:
					case dg87_job_state_exit:
						nTerminated++;
						state = 'T';
						if (showTerminated == FALSE)
							continue;
						break;

					/* holded */
					case dg87_job_state_abortStopped:
					case dg87_job_state_deleteStopped:
					case dg87_job_state_waitenabled:
					case dg87_job_state_readytorun:
					case dg87_job_state_readytosubmit:
					case dg87_job_state_submit:
						nHolded++;
						state = 'H';
						if (showHolded == FALSE)
							continue;
						break;

					/* running */
					case dg87_job_state_running:
					case dg87_job_state_waitStartTempo:
					case dg87_job_state_waitEndTempo: 
						nRunning++;
						state = 'R';
						if (showRunning == FALSE)
							continue;
						break;

					/* error */
					case dg87_job_state_deleted:
					case dg87_job_state_aborted:
						nError++;
						state = 'E';
						if (showError == FALSE)
							continue;
						break;

					/* cancel */
					case dg87_job_state_ignore:
					case dg87_job_state_ignored:
					case dg87_job_state_canceled:
						nCancel++;
						state = 'C';
						if (showCancel == FALSE)
							continue;
						break;
				}

				/* prints job if necessary */
				if (showList == FALSE)
					continue;
				dg87_connection_printf(pConnection,
					"%s%c%s%s%s%s%s%s%s%s%s\n",
					pBeg,state,
					pSep,pJob->pTask->pDay->name,
					pSep,pJob->pTask->name,
					pSep,pJob->name, 
					pSep,dg87_job_stateToAscii(pJob->state),
					pEnd);

			} /* next job */
		} /* next task */
	} /* next day */

	/* print counters if necessary */
	if (showCounters == TRUE){
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Total", pSep, nTotal, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Waiting", pSep, nWaiting, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Holded", pSep, nHolded, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Running", pSep, nRunning, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Error", pSep, nError, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Cancel", pSep, nCancel, pEnd);
		dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
			pBeg, "Terminated", pSep, nTerminated, pEnd);
	}

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);

	return 0;
}
/******************************************************************************/
int cmd_save(dg87_connection* pConnection)
{
	dg87_engine *pEngine;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	pEngine = pContext->pEngine;

	/* control the argument number */
	if (pCommand->argc != 0 || pCommand->optc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"save");
		return 1;
	}
	
	dg87_connection_printf(pConnection,"Save\n");
	return 0;
}
/******************************************************************************/
/* End of file */
