/* file cmdqueue.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdqueue.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>


/******************************************************************************/
/* cmd_qk
 * Queue Kill
 * Usage : qk TargetName QueueName
 */
int cmd_qk(dg87_connection* pConnection)
{
	dg87_target* pTarget;
	dg87_command* pCommand;
	if (pConnection == NULL)
		return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qk TargetName QueueName\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;
	
	/* delete the queue */
	dg87_err_clearError();
	if (dg87_target_queueDeleteByName(pTarget, pCommand->argv[1]) 
			== FALSE){
		dg87_connection_printf(pConnection,
			"Can't delete the queue '%s' in the target '%s' ! %s\n",
			pCommand->argv[1], pTarget->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************
 * cmd_qa
 * Queue Add
 * Usage : qa TargetName QueueName [description]
 */
int cmd_qa(dg87_connection* pConnection)
{
	dg87_target* pTarget;
	dg87_queue* pQueue;
	dg87_command* pCommand;
	if (pConnection == NULL)
		return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc < 2 || pCommand->argc > 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qa TargetName QueueName [description]\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* add the queue to the Target */
	dg87_err_clearError();
	pQueue = dg87_target_queueAdd(pTarget, pCommand->argv[1]);
	if (pQueue == NULL){
		dg87_connection_printf(pConnection,
			"Can't add the queue '%s' on the target '%s' ! %s\n",
			pCommand->argv[1], pTarget->name,
			dg87_err_getErrorStr());
		return 1;
	}
	/* change description */
	if (pCommand->argc == 3){ 
		dg87_err_clearError();
		if (dg87_queue_setDescription(pQueue, pCommand->argv[2])
				==FALSE)
		{
			dg87_connection_printf(pConnection,
			"Can't add the queue '%s' on the target '%s' ! %s\n",
				pCommand->argv[1], pTarget->name,
				dg87_err_getErrorStr());
			dg87_target_queueDelete(pTarget, pQueue);
			return 1;
		}
	}
	return 0;
}
/******************************************************************************/
/* cmd_qc
 * Usage : qc TargetName
 * Queue count
 * return the number of queues in the Target.
 */
int cmd_qc(dg87_connection* pConnection)
{
	dg87_target* pTarget;
	dg87_command* pCommand;
	if (pConnection == NULL)
		return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qc TargetName\n");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;
	/* print Queue count */
	dg87_connection_printf(pConnection, "%i\n", 
			pTarget->pQueueVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_ql
	dg87_command* pCommand;
	if (pConnection == NULL)
		return 1;
	pCommand = pConnection->pCommand;
 * Usage : ql [options] TargetName
 * Queue List
 * options
 * -d : print description of queue 
 * -t : print Target name of queue
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first queue to display (first index is 0)
 * -Nnumber : number of queue to display (no limit if -N < 1)
 */
int cmd_ql(dg87_connection* pConnection)
{
	char *pSep,*pBeg,*pEnd; /* separator, begin, end */
	/*static char optstring[] = "dtB:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first queue to display */
	int number = 0;/* number of queues to display */
	int count = 0; /* number of queue actually display */
	dg87_element* pElement = NULL;
	dg87_target* pTarget;
	dg87_queue* pQueue;
	BOOL showDescription = FALSE;
	BOOL showTargetName = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL)
		return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ql [options] TargetName");
		return 1;
	}

	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");
	showDescription = FALSE;
	showTargetName = FALSE;

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); 
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); 
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); 
				break;
			case 'I': /* index of the first queue to display */
				start = atoi(optarg); 
				break;
			case 'N': /* number of queue to display */
				number = atoi(optarg); 
				break;
			case 'd': /* description */
				showDescription = TRUE; 
				break;
			case 't': /* target */
				showTargetName = TRUE; 
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
		
	/* find the first queue to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pTarget->pQueueVector, start);
	else
		pElement = pTarget->pQueueVector->pFirst;


	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of queue to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pQueue = pElement->pObject;

		/* print Target */
		dg87_connection_printf(pConnection, pBeg);
		if (showTargetName ==  TRUE)
			dg87_connection_printf(pConnection, "%s%s", 
				pQueue->pTarget->name, pSep);
		
		/* print Queue Name */
		dg87_connection_printf(pConnection, pQueue->name);
		
		/* print Queue Description */
		if (showDescription == TRUE) 
			dg87_connection_printf(pConnection, "%s%s", 
				pSep, pQueue->description);
		 
		dg87_connection_printf(pConnection, "%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}

/******************************************************************************/
/* qi
 * Usage : qi [options] TargetName QueueName
 * Queue Information 
 * get information for a queue 
 * Options :
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int cmd_qi(dg87_connection* pConnection)
{
	char *pSep,*pBeg,*pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch,iopt;
	char* optarg;
	dg87_target *pTarget;
	dg87_queue *pQueue;
	dg87_command* pCommand;

	if (pConnection == NULL)
		return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			 "qi [options] TargetName QueueName");
		return 1;
	}

	/* find the target */
	if ((pTarget = dg87_util_findTarget(pConnection, 
			pCommand->argv[0])) == NULL){
		return 1;
	}

	/* find the queue */
	if ((pQueue = dg87_util_findQueue(pConnection, 
			pTarget, pCommand->argv[1])) == NULL)
		return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* print target name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg,"Target Name",pSep, pQueue->pTarget->name, pEnd);
	/* print queue name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg,"Queue Name",pSep, pQueue->name, pEnd);
	/* print queue desription */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg,"Description",pSep, pQueue->description, pEnd);
	/* open's property */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Opened", pSep, 
		dg87_boolToAscii(pQueue->open), pEnd);
	/* max's property */
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
		pBeg, "Max", pSep, pQueue->max, pEnd);
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/***************************************************************************/
/* qsd
 * Queue Set Description
 * Usage : qsd TargetName QueueName description
 */
int cmd_qsd(dg87_connection* pConnection)
{
	dg87_target *pTarget;
	dg87_queue *pQueue;

	dg87_command* pCommand;

	if (pConnection == NULL)
		return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qsd TargetName QueueName description");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* find the queue */
	pQueue = dg87_util_findQueue
		(pConnection, pTarget, pCommand->argv[1]);
	if (pQueue == NULL) return 1;

	/* change the description */
	dg87_err_clearError();
	if ( dg87_queue_setDescription(pQueue, pCommand->argv[2]) == FALSE)
	{
		dg87_connection_printf(pConnection,
			"Can't set description for the queue " \
			"'%s' in the target '%s' ! %s\n",
			pQueue->name, pTarget->name,
			dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/***************************************************************************/
/* qso
 * Queue Set Open 
 * Usage : tsd TargetName QueueName BooleanValue 
 */
int cmd_qso(dg87_connection* pConnection)
{
	dg87_target *pTarget;
	dg87_queue *pQueue;
	BOOL value;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qso TargetName QueueName BooleanValue");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* find the queue */
	pQueue = dg87_util_findQueue
		(pConnection, pTarget, pCommand->argv[1]);
	if (pQueue == NULL) return 1;


	/* interpret boolean value */
	if ((value = dg87_asciiToBool(pCommand->argv[2])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[2]);
		return 1;
	}

	/* change the property */
	dg87_err_clearError();
	if (dg87_queue_setOpen(pQueue, value) == FALSE){
		dg87_connection_printf(pConnection,
		"Can't change open's property for the queue '%s' " \
		"of the target '%s' ! %s\n",
			pQueue->name, pTarget->name,
			dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}
/***************************************************************************/
/* qsm
 * Queue Set Max 
 * Usage : tsd TargetName QueueName IntegerValue 
 */
int cmd_qsm(dg87_connection* pConnection)
{
	dg87_target *pTarget;
	dg87_queue *pQueue;
	int value;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 3){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qsm TargetName QueueName IntegerValue");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the target */
	pTarget = dg87_util_findTarget(pConnection, pCommand->argv[0]);
	if (pTarget == NULL) return 1;

	/* find the queue */
	pQueue = dg87_util_findQueue
		(pConnection, pTarget, pCommand->argv[1]);
	if (pQueue == NULL) return 1;


	/* convert into integer */
	value = atoi(pCommand->argv[2]);
	if (value < 1) {
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', must be an integer > 0 !\n",
			pCommand->argv[2]);
		return 1;
	}

	/* change the value */
	dg87_err_clearError();
	if (dg87_queue_setMax(pQueue, value) == FALSE){
		dg87_connection_printf(pConnection,
		"Can't change max's property for the queue '%s' " \
		"of the target '%s' ! %s\n",
			pQueue->name, pTarget->name,
			dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}



/***************************************************************************/
/* cmd_qxtc
 * Usage : qxtc [options] TargetName QueueName
 * return the number of eXp's tasks in the queue.
 */
int cmd_qxtc(dg87_connection* pConnection)
{
	dg87_target* pTarget;
	dg87_queue* pQueue;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qxtc TargetName QueueName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the target */
	if ((pTarget = dg87_util_findTarget(pConnection, 
				pCommand->argv[0])) == NULL)
		return 1;

	/* find the queue */
	if ((pQueue = dg87_util_findQueue(pConnection, pTarget, 
			pCommand->argv[1])) == NULL)
		return 1;
	
	dg87_connection_printf(pConnection, "%i\n", 
			pQueue->pTaskList->count);
	return 0;	
}
/***************************************************************************/
/* cmd_qxtl
 * Usage : qxtl [options] TargetName QueueName
 * Queue eXp Task List
 * give the list of Task of the exp which have this queue 
 * Options :
 * -d : print description of task
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int cmd_qxtl(dg87_connection* pConnection)
{
	char *pSep,*pBeg,*pEnd; /* separator, begin, end */
	/*static char optstring[] = "dB:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_item* pItem = NULL;
	dg87_task* pTask;
	dg87_target* pTarget;
	dg87_queue* pQueue;
	BOOL showDescription = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qxtl [options] TargetName QueueName");
		return 1;
	}

	/* find the target */
	if ((pTarget = dg87_util_findTarget(pConnection, 
				pCommand->argv[0])) == NULL)
		return 1;

	/* find the queue */
	if ((pQueue = dg87_util_findQueue(pConnection, pTarget, 
			pCommand->argv[1])) == NULL)
		return 1;
	
	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg); break;
			case 'N': /* number of task to display */
				number = atoi(optarg); break;
			case 'd': /* description */
				showDescription = TRUE; break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* find the first task to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pQueue->pTaskList, start);
	else
		pItem = pQueue->pTaskList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pTask = pItem->pObject;

		/* print Task's name and day's name */
		dg87_connection_printf(pConnection, "%s%s%s%s",
			pBeg, pTask->pDay->name, pSep, pTask->name);
		
		/* print description */
		if (showDescription == TRUE) {
			dg87_connection_printf(pConnection, "%s%s",
				pSep, pTask->description);
		} 
		dg87_connection_printf(pConnection, "%s\n", pEnd);
		
		pItem = pItem->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/***************************************************************************/
/* cmd_qxjc
 * Usage : qxjc [options] TargetName QueueName
 * return the number of eXp's jobs in the queue.
 */
int cmd_qxjc(dg87_connection* pConnection)
{
	dg87_target* pTarget;
	dg87_queue* pQueue;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qxjc TargetName QueueName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* find the target */
	if ((pTarget = dg87_util_findTarget(pConnection, 
				pCommand->argv[0])) == NULL)
		return 1;

	/* find the queue */
	if ((pQueue = dg87_util_findQueue(pConnection, pTarget, 
			pCommand->argv[1])) == NULL)
		return 1;
	
	dg87_connection_printf(pConnection, "%i\n", 
			pQueue->pRunningJobList->count);
	return 0;	
}

/***************************************************************************/
/* cmd_qxjl
 * Usage : qxjl [options] TargetName QueueName
 * Queue eXp Job List
 * give the list of jobs of the exp which are in this queue 
 * Options :
 * -d : print description of task
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 * -s : state of the jobs
 */
int cmd_qxjl(dg87_connection* pConnection)
{
	char *pSep,*pBeg,*pEnd; /* separator, begin, end */
	/*static char optstring[] = "sdB:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first job to display */
	int number = 0;/* number of job to display */
	int count = 0; /* number of job actually display */
	dg87_item* pItem = NULL;
	dg87_job* pJob;
	dg87_target* pTarget;
	dg87_queue* pQueue;
	BOOL showDescription = FALSE;
	BOOL showState = FALSE;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"qxjl [options] TargetName QueueName");
		return 1;
	}

	/* find the target */
	if ((pTarget = dg87_util_findTarget(pConnection, 
				pCommand->argv[0])) == NULL)
		return 1;

	/* find the queue */
	if ((pQueue = dg87_util_findQueue(pConnection, pTarget, 
			pCommand->argv[1])) == NULL)
		return 1;

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg); break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg); break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg); break;
			case 'I': /* index of the first job to display */
				start = atoi(optarg); break;
			case 'N': /* number of job to display */
				number = atoi(optarg); break;
			case 'd': /* description */
				showDescription = TRUE; break;
			case 's': /* description */
				showState = TRUE; break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option !");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* find the first job to display */
	pItem = NULL;
	if (start > 0)
		pItem = dg87_list_getItem(pQueue->pRunningJobList, start);
	else
		pItem = pQueue->pRunningJobList->pFirst;

	/* create the list */
	count = 0;
	while (pItem != NULL){
		/* reach the limit of job to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pJob = pItem->pObject;

		/* print Job's name Task's name and day's name */
		dg87_connection_printf(pConnection, "%s%s%s%s%s%s",
			pBeg, pJob->pTask->pDay->name, 
			pSep, pJob->pTask->name,
			pSep, pJob->name);
		
		/* print state */
		if (showState == TRUE) {
			dg87_connection_printf(pConnection, "%s%s", pSep, 
				dg87_job_stateToAscii(pJob->state));
		} 
		/* print description */
		if (showDescription == TRUE) {
			dg87_connection_printf(pConnection, "%s%s",
				pSep, pJob->description);
		} 
		dg87_connection_printf(pConnection, "%s\n", pEnd);
		
		pItem = pItem->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/***************************************************************************/
/* End of file */
