/* file cmdvariable.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "engine.h"
#include "cmdvariable.h"
#include "util.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>


/******************************************************************************/
/* vk
 * Variable Kill
 * Usage : vk VariableName
 */
int cmd_vk(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"vk VariableName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	if (dg87_engine_variableDeleteByName(pContext->pEngine, 
				pCommand->argv[0]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't delete the variable '%s' ! %s!\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* va
 * Variable Add
 * Usage : va VariableName [value]
 */
int cmd_va(dg87_connection* pConnection)
{
	dg87_variable* pVariable;
	dg87_context* pContext;
	dg87_command* pCommand;
	char* pValue = "";

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc < 1 || pCommand->argc > 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"va VariableName [description]");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}

	/* initial value as optional argument */
	if (pCommand->argc == 2)
		pValue = pCommand->argv[1];

	/* add the variable */
	dg87_err_clearError();
	pVariable = dg87_engine_variableAdd(pContext->pEngine, 
			pCommand->argv[0], pValue);
	if (pVariable == NULL){
		dg87_connection_printf(pConnection,
				"Can't add the variable '%s'! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	/* set description */
	if (pCommand->argc == 2){
		dg87_err_clearError();
		if (dg87_variable_setValue(pVariable, pCommand->argv[1])
				== FALSE)
		{
			dg87_connection_printf(pConnection,
				"Can't set value for " \
				"the variable '%s' ! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
			/* TODO delete variable before return ? */
			return 1;
		}
	}
	return 0;
}
/******************************************************************************/
/* cmd_vc
 * variable count
 * Usage : vc
 * return the number of variable.
 */
int cmd_vc(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 0){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "vc");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	dg87_connection_printf(pConnection,
		"%i\n", pContext->pEngine->pVariableVector->count);
	return 0;
}
/******************************************************************************/
/* vi
 * Variable information 
 * Usage : ti [options] VariableName
 * Options 
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 */
int cmd_vi(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator begin end */
	/*static char optstring[] = "B:S:E:";*/
	int optch,iopt;
	char* optarg;
	dg87_variable *pVariable;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	
	/* control number of arguments */
	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"vi [Options] VariableName");
		return 1;
	}
	/* find the variable */
	pVariable = dg87_util_findVariable(pConnection, pCommand->argv[0]);
	if (pVariable == NULL) return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
					"Unknown option !\n");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	/* Variable name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Variable name", pSep, pVariable->name, pEnd);
	/* variable value*/
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Value", pSep, pVariable->pValue, pEnd);
	/* variable linkedConditions value*/
	dg87_connection_printf(pConnection,"%s%s%s%i%s\n",
		pBeg, "Linked Conditions", pSep, 
		pVariable->linkedConditions, pEnd);

	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
/* cmd_vl
 * variable list
 * options
 * -v : print value of variable
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first variable to display (first index is 0)
 * -Nnumber : number of variable to display (no limit if -N < 1)
 */
int cmd_vl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator begin end */
	/*static char optstring[] = "dB:S:E:I:N:";*/
	int optch, iopt;
	char* optarg;
	int start = 0; /* index of the first variable to display */
	int number = 0;/* number of variables to display */
	int count = 0; /* number of variable actually display */
	dg87_element* pElement = NULL;
	dg87_variable* pVariable;
	BOOL showValue = FALSE;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* control number of arguments */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"vl [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");
	showValue = FALSE;

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first variable to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of variable to display */
				number = atoi(optarg);
				break;
			case 'v': /* value */
				showValue = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
					"Unknown option !\n");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	
	/* find the first TASK to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pContext->pEngine->pVariableVector, start);
	else
		pElement = pContext->pEngine->pVariableVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pVariable = pElement->pObject;
		dg87_connection_printf(pConnection,"%s%s", 
			pBeg, pVariable->name, pEnd);
		if (showValue == TRUE)
			dg87_connection_printf(pConnection,"%s%s", 
				pSep, pVariable->pValue);
		dg87_connection_printf(pConnection,"%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/***************************************************************************/
/* vsv
 * Variable Set Value
 * Usage : vsv VariableName Variable 
 */
int cmd_vsv(dg87_connection* pConnection)
{
	dg87_variable *pVariable;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"vsv VariableName value");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the variable */
	pVariable = dg87_util_findVariable(pConnection, pCommand->argv[0]);
	if (pVariable == NULL) return 1;

	/* change the description */
	dg87_err_clearError();
	if (dg87_variable_setValue(pVariable, pCommand->argv[1]) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't change value for the variable '%s' ! %s\n",
			pVariable->name, dg87_err_getErrorStr());
		return 1;
	}

	return 0;
}

/***************************************************************************/
/* End of file */
