/* file condition.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <libdg87/global.h>
#include "condition.h"
#include "variable.h"

/****************************************************************************/
dg87_condition* dg87_conditionHmin_new(int minutes)
{
	dg87_condition* pCondition;
	if (minutes < 0 || minutes >= 24*60){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pCondition = dg87_malloc(sizeof(dg87_condition), DG87_CONDITION);
	if (pCondition == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCondition->state = dg87_condition_waiting;
	pCondition->type = condition_hmin;
	pCondition->data.pConditionHmin = 
		dg87_malloc(sizeof(struct dg87_condition_h), DG87_OTHER);
	if (pCondition->data.pConditionHmin == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	pCondition->data.pConditionHmin->minutes = minutes;
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_conditionHmax_new(int minutes)
{
	dg87_condition* pCondition;
	if (minutes < 0 || minutes >= 24*60){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pCondition = dg87_malloc(sizeof(dg87_condition), DG87_CONDITION);
	if (pCondition == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCondition->state = dg87_condition_waiting;
	pCondition->type = condition_hmax;
	pCondition->data.pConditionHmax = 
		dg87_malloc(sizeof(struct dg87_condition_h), DG87_OTHER);
	if (pCondition->data.pConditionHmax == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	pCondition->data.pConditionHmax->minutes = minutes;
	return pCondition;
}

/****************************************************************************/
dg87_condition* dg87_conditionHlate_new(int minutes)
{
	dg87_condition* pCondition;
	if (minutes < 0 || minutes >= 24*60){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pCondition = dg87_malloc(sizeof(dg87_condition), DG87_CONDITION);
	if (pCondition == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCondition->state = dg87_condition_waiting;
	pCondition->type = condition_hlate;
	pCondition->data.pConditionHlate = 
		dg87_malloc(sizeof(struct dg87_condition_h), DG87_OTHER);
	if (pCondition->data.pConditionHlate == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	pCondition->data.pConditionHlate->minutes = minutes;
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_conditionFile_new(char* pPathFile)
{
	dg87_condition* pCondition;
	if (pPathFile == NULL || pPathFile[0] == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pCondition = dg87_malloc(sizeof(dg87_condition), DG87_CONDITION);
	if (pCondition == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCondition->state = dg87_condition_waiting;
	pCondition->type = condition_file;
	pCondition->data.pConditionFile = 
		dg87_malloc(sizeof(struct dg87_condition_file), DG87_OTHER);
	if (pCondition->data.pConditionFile == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	pCondition->data.pConditionFile->pPath = 
		dg87_strdup(pPathFile, DG87_CHAR);
	if (pCondition->data.pConditionFile->pPath == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition->data.pConditionFile, DG87_OTHER);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_conditionManual_new(char* pMessage)
{
	dg87_condition* pCondition;
	if (pMessage == NULL || pMessage[0] == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	pCondition = dg87_malloc(sizeof(dg87_condition), DG87_CONDITION);
	if (pCondition == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCondition->state = dg87_condition_waiting;
	pCondition->type = condition_manual;
	pCondition->data.pConditionManual = 
		dg87_malloc(sizeof(struct dg87_condition_manual), DG87_OTHER);
	if (pCondition->data.pConditionManual == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	pCondition->data.pConditionManual->pMessage = 
		dg87_strdup(pMessage, DG87_CHAR);
	if (pCondition->data.pConditionManual->pMessage == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition->data.pConditionManual, DG87_OTHER);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_conditionIfEqual_new(
		struct dg87_variable_struct* pVariable,
		char* pValue)
{
	dg87_condition* pCondition;
	if (pVariable == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_malloc(sizeof(dg87_condition), DG87_CONDITION);
	if (pCondition == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCondition->state = dg87_condition_waiting;
	pCondition->type = condition_ifequal;
	pCondition->data.pConditionIfEqual = 
		dg87_malloc(sizeof(struct dg87_condition_eq), DG87_OTHER);
	if (pCondition->data.pConditionIfEqual == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	/* copy value */
	pCondition->data.pConditionIfEqual->pValue = 
		dg87_strdup(pValue, DG87_CHAR);
	if (pCondition->data.pConditionIfEqual->pValue == NULL){
		dg87_free(pCondition, DG87_CONDITION);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* increment variable usage count */
	pVariable->linkedConditions++;
	pCondition->data.pConditionIfEqual->pVariable = pVariable;

	return pCondition;
}
/****************************************************************************/
dg87_condition* dg87_conditionIfNotEqual_new(
		struct dg87_variable_struct* pVariable,
		char* pValue)
{
	dg87_condition* pCondition;
	if (pVariable == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pCondition = dg87_malloc(sizeof(dg87_condition), DG87_CONDITION);
	if (pCondition == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pCondition->state = dg87_condition_waiting;
	pCondition->type = condition_ifnequal;
	pCondition->data.pConditionIfNotEqual = 
		dg87_malloc(sizeof(struct dg87_condition_eq), DG87_OTHER);
	if (pCondition->data.pConditionIfNotEqual == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		dg87_free(pCondition, DG87_CONDITION);
		return NULL;
	}
	/* copy value */
	pCondition->data.pConditionIfNotEqual->pValue = 
		dg87_strdup(pValue, DG87_CHAR);
	if (pCondition->data.pConditionIfNotEqual->pValue == NULL){
		dg87_free(pCondition, DG87_CONDITION);
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* increment variable usage count */
	pVariable->linkedConditions++;
	pCondition->data.pConditionIfNotEqual->pVariable = pVariable;

	return pCondition;
}
/****************************************************************************/
BOOL dg87_condition_delete(dg87_condition* pCondition)
{
	if (pCondition == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* delete memory for structure */
	switch(pCondition->type){
		case condition_hmin:
			dg87_free(pCondition->data.pConditionHmin, DG87_OTHER);
			break;
		case condition_hmax:
			dg87_free(pCondition->data.pConditionHmax, DG87_OTHER);
			break;
		case condition_hlate:
			dg87_free(pCondition->data.pConditionHlate, 
					DG87_OTHER);
			break;
		case condition_file:
			dg87_free(pCondition->data.pConditionFile->pPath,
					DG87_CHAR);
			dg87_free(pCondition->data.pConditionFile, DG87_OTHER);
			break;
		case condition_manual:
			dg87_free(pCondition->data.pConditionManual->pMessage,
					DG87_CHAR);
			dg87_free(pCondition->data.pConditionManual, 
					DG87_OTHER);
			break;
		case condition_ifequal:
			/* variable decrement usage count */
			pCondition->data.pConditionIfEqual
				->pVariable->linkedConditions--;
			dg87_free(pCondition->data.pConditionIfEqual
					->pValue, DG87_CHAR);
			dg87_free(pCondition->data.pConditionIfEqual, 
					DG87_OTHER);
			break;
		case condition_ifnequal:
			/* variable decrement usage count */
			pCondition->data.pConditionIfNotEqual
				->pVariable->linkedConditions--;
			dg87_free(pCondition->data.pConditionIfNotEqual
					->pValue, DG87_CHAR);
			dg87_free(pCondition->data.pConditionIfNotEqual, 
					DG87_OTHER);
			break;

		default:
			dg87_err_setError(DG87_ERR_NULLPARAMETER);
			break;
	}
	dg87_free(pCondition, DG87_CONDITION);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_condition_load(dg87_condition* pCondition, dg87_storage* pStorage)
{
	dg87byte b;
	if (pCondition == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read 'state' property */
	if (dg87_storage_readByte(pStorage, &b) == FALSE)
		return FALSE;
	pCondition->state = b;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_condition_save(dg87_condition* pCondition, dg87_storage* pStorage)
{
	if (pCondition == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* 'state' property */
	if (dg87_storage_writeByte(pStorage, pCondition->state) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
dg87_condition_type dg87_condition_asciiToCondition(char* pCondition)
{
	if (pCondition == NULL) return condition_error;

	if ( strcasecmp(pCondition, "hmin") == 0)
		return condition_hmin;
		
	if ( strcasecmp(pCondition,"hmax") == 0)
		return condition_hmax;

	if ( strcasecmp(pCondition,"hlate") == 0)
		return condition_hlate;

	if ( strcasecmp(pCondition,"file") == 0)
		return condition_file;

	if ( strncmp(pCondition,"manual", 6) == 0)
		return condition_manual;

	if ( strncmp(pCondition,"if_equal", 8) == 0)
		return condition_ifequal;
	if ( strncmp(pCondition,"if_not_equal", 13) == 0)
		return condition_ifnequal;

	return condition_error;
}
/****************************************************************************/
char* dg87_condition_conditionToAscii(dg87_condition_type conditionType)
{
	switch(conditionType)
	{
		case condition_hmin:   return "hmin";
		case condition_hmax:   return "hmax";
		case condition_hlate:  return "hlate";
		case condition_file:   return "file";
		case condition_manual: return "manual";
		case condition_ifequal:  return "if_equal";
		case condition_ifnequal:  return "if_not_equal";
		case condition_error:  return "error";
	}
	return "error";
}
/****************************************************************************/
int dg87_condition_hourToMinute(char* hour)
{
	char* pHour,*pMin;
	int result = -1;
      	if (hour == NULL) return -1; 
	pHour = strdup(hour);
	if (pHour == NULL) return -1;
	pMin = strchr(pHour,':');
	if ( pMin == NULL )
		result = atoi(pHour);
	else
	{
		*pMin = '\0';
		result = atoi(pHour) * 60 + atoi(pMin+1);
	}
	free(pHour);
	return result;
}
/****************************************************************************/
char* dg87_condition_minuteToHour(int minutes)
{
	int h,m;
	char* pRes;
	if (minutes < 0 || minutes >= 24*60)
		return NULL;
	pRes = dg87_malloc(sizeof(char) * 5, DG87_CHAR);
	h = minutes / 60;
	m = minutes % 60;
	sprintf(pRes, "%.2i:%.2i",h,m);
	return pRes;
}
/****************************************************************************/
char* dg87_condition_stateToAscii(dg87_condition_state depstate)
{
	switch (depstate){
		case dg87_condition_waiting: return "waiting";
		case dg87_condition_finished: return "finished";
		default: return "unknown"; /* to remove warning */
	}
	return NULL;
}
/****************************************************************************/
int getCurrentTime()
{
	time_t dt;
	struct tm dc;
	time(&dt);
	dc = *localtime(&dt);
	return dc.tm_hour * 60 + dc.tm_min;
}
/****************************************************************************/
dg87_condition_state dg87_condition_getState(dg87_condition* pCondition)
{
	struct stat sf;

	if (pCondition == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* nothing to do if condition is finished or canceled */
	if (pCondition->state != dg87_condition_waiting)
		return pCondition->state;
	       	
	switch (pCondition->type)
	{
		case condition_hmin:
			/* if h > hlate then state=finished*/
			if (getCurrentTime()  
				>= pCondition->data.pConditionHmin->minutes)
				pCondition->state = dg87_condition_finished;
			break;
		case condition_hmax:
			/* if h > hlate then state=finished */
			if (getCurrentTime()  
				>= pCondition->data.pConditionHmax->minutes)
				pCondition->state = dg87_condition_finished;
			break;
		case condition_hlate:
			/* if h > hlate then state=finished */
			if (getCurrentTime()  
				>= pCondition->data.pConditionHlate->minutes)
				pCondition->state = dg87_condition_finished;
			break;
		case condition_file:
			/* if file is present then state = finished */
			if (stat(pCondition->data.pConditionFile->pPath,
						&sf) == 0){
				pCondition->state = dg87_condition_finished;
			}
			break;
		case condition_manual:
			/* nothing to test : this type of condition is
			 * validated by a user with 'xtcv' command */
			break;
		case condition_ifequal:
			if (strcmp(pCondition->data.pConditionIfEqual
				->pVariable->pValue, 
				pCondition->data.pConditionIfEqual->pValue) 
					== 0)
				pCondition->state = dg87_condition_finished;
			break;
		case condition_ifnequal:
			if (strcmp(pCondition->data.pConditionIfNotEqual
				->pVariable->pValue, 
				pCondition->data.pConditionIfNotEqual->pValue) 
					!= 0)
				pCondition->state = dg87_condition_finished;
			break;
		default:
			break;
	}
	return pCondition->state;
}
/****************************************************************************/
BOOL dg87_condition_validate(dg87_condition *pCondition)
{
	if (pCondition == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pCondition->state = dg87_condition_finished;
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_condition_check(dg87_condition* pCondition, int (*print)(char*))
{
	int minutes;
	switch (pCondition->type){
		case condition_hmin:
			minutes = pCondition->data.pConditionHmin->minutes;
			if (minutes < 0 || minutes >= 24*60){
				print("xCondition : incorrect value hmin !\n");
				return 1;
			}
			break;
		case condition_hmax:
			minutes = pCondition->data.pConditionHmax->minutes;
			if (minutes < 0 || minutes >= 24*60){
				print("xCondition : incorrect value hmax !\n");
				return 1;
			}
			break;
		case condition_hlate:
			minutes = pCondition->data.pConditionHlate->minutes;
			if (minutes < 0 || minutes >= 24*60){
				print("xCondition : incorrect value hlate !\n");
				return 1;
			}
			break;
		case condition_file:
			if (pCondition->data.pConditionFile->pPath == NULL){
				print("xCondition : Path file is NULL !\n");
				return 1;
			}
			break;
		case condition_manual:
			if (pCondition->data.pConditionManual->pMessage == NULL)
			{
				print("xCondition : Message is NULL !\n");
				return 1;
			}
			break;
		default:
			print("xCondition : incorrect type !\n");

	}
	return 0;
}
#endif /*DEBUG*/

/****************************************************************************/
/* End of file */
