/* file context.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef CONTEXT_H
#define CONTEXT_H 1

#include <time.h>
#include <libdg87/vector.h>
#include <libdg87/conf.h>
#include <libdg87/storage.h>
#include <libdg87/storage.h>
#include "global.h"
#include "day.h"
#include "context.h"
#include "target.h"

/*
 * dg87_context object
 */
typedef struct dg87_context_struct
{
	int ipPort;
	struct dg87_engine_struct* pEngine;
	dg87_vector* pTargetVector;
	dg87_vector* pDayVector;
	struct dg87_server_struct* pServer;
	dg87_conf *pConf; /* configuration file */
	dg87_storage *pStorage;
	char* pPidFile;
	char* pDiogenePassword;
	time_t saveTime;
	dg87uint saveTempo;
} dg87_context;

/*
 * dg87_context_new
 */
dg87_context* dg87_context_new(char* pConfFile);

/*
 * dg87_context_delete
 */
BOOL dg87_context_delete(dg87_context* pContext);

/*
 * dg87_context_setServer
 */
BOOL dg87_context_setServer(dg87_context* pContext, 
		struct dg87_server_struct* pServer);

/*
 * dg87_context_makePeriodicSave
 */
BOOL dg87_context_makePeriodicSave(dg87_context* pContext);

/*
 * dg87_context_readConfig
 * This function reads (or re-reads) the configuration file variables
 */
BOOL dg87_context_readConfig(dg87_context* pContext);

/*
 * dg87_log
 * write the message in the system log.
 */
void dg87_context_log(char* pMessage);

/*************************************************************************/
/*  functions for targets                                                */
/*************************************************************************/

/*
 * dg87_context_targetAdd
 * create a Target object, add this object in the TargetVector 
 * of the context and return a pointer on this object.
 * Return NULL if the target object can't be create.
 */
dg87_target* dg87_context_targetAdd(dg87_context* pContext, char* pTargetName);

/*
 * dg87_context_TargetCount
 * return the number of Targets.
 * return -1 if pContext is NULL.
 */
int dg87_context_targetCount(dg87_context* pContext);

/*
 * dg87_context_targetGetByName
 * return a pointer on the target which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_target* dg87_context_targetGetByName(dg87_context* pContext, char* pName);

/*
 * dg87_context_targetGetByIndex
 * return a pointer on the target at the given position.
 */
dg87_target* dg87_context_targetGetByIndex(dg87_context* pContext, int index);

/*
 * dg87_context_targetDeleteByName
 * delete the target which has the name given in parameter.
 */
BOOL dg87_context_targetDeleteByName(dg87_context* pContext,char* pName);

/*
 * dg87_context_targetDeleteByIndex
 * delete the target at the given index.
 */
BOOL dg87_context_targetDeleteByIndex(dg87_context* pContext,int index);

/*
 * dg87_context_targetDelete
 */
BOOL dg87_context_targetDelete(dg87_context* pContext, dg87_target* pTarget);

/*************************************************************************/
/*  functions for days */
/*************************************************************************/

/*
 * dg87_context_dayAdd
 * create a Day object, add this object in the DayVector 
 * of the context and return a pointer on this object.
 * Return NULL if the day object can't be create.
 */
dg87_day* dg87_context_dayAdd(dg87_context* pContext, char* pDayName,
		time_t startTime);

/*
 * dg87_context_DayCount
 * return the number of Days.
 * return -1 if pContext is NULL.
 */
int dg87_context_dayCount(dg87_context* pContext);

/*
 * dg87_context_dayGetByName
 * return a pointer on the day which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_day* dg87_context_dayGetByName(dg87_context* pContext, char* pName);

/*
 * dg87_context_dayGetByIndex
 * return a pointer on the day at the given position.
 */
dg87_day* dg87_context_dayGetByIndex(dg87_context* pContext, int index);

/*
 * dg87_context_dayDeleteByName
 * delete the day which has the name given in parameter.
 */
BOOL dg87_context_dayDeleteByName(dg87_context* pContext,char* pName);

/*
 * dg87_context_dayDeleteByIndex
 * delete the day at the given index.
 */
BOOL dg87_context_dayDeleteByIndex(dg87_context* pContext,int index);

/*
 * dg87_context_save
 */
BOOL dg87_context_save(dg87_context* pContext);

/*
 * dg87_context_load
 */
BOOL dg87_context_load(dg87_context* pContext);

#ifdef DEBUG
/*
 * dg87_context_ckeck
 */
int dg87_context_check(dg87_context* pContext, int (*print)(char*));
#endif /*DEBUG*/

#endif /*CONTEXT_H*/
/* End of file */
