/* file day.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef DAY_H 
#define DAY_H 1

#include <time.h>
#include "global.h"
#include <libdg87/vector.h>
#include "task.h"
#include "taskdep.h"

 
typedef struct dg87_day_struct
{
	char name[DG87_DAY_NAME_LENGTH];
	/* locked is TRUE during planning process */
	BOOL locked;
	BOOL enabled;
	dg87_vector* pTaskVector;
	dg87_list* pTaskdepList;
	time_t startTime;
} dg87_day;

/*
 * dg87_day_new
 * create a new day and return a pointer on this.
 */
dg87_day* dg87_day_new(char* name, time_t startTime);

/*
 * dg87_day_delete
 */
BOOL dg87_day_delete(dg87_day* pDay);

/*
 * dg87_day_load
 */
BOOL dg87_day_load(dg87_day* pDay, dg87_storage* pStorage);

/*
 * dg87_day_save
 */
BOOL dg87_day_save(dg87_day* pDay, dg87_storage* pStorage);

/*
 * dg87_day_setEnabled
 */
BOOL dg87_day_setEnabled(dg87_day* pDay, BOOL value);

/*
 * dg87_day_validate
 * when a day is created, the locked property is TRUE.
 * This command set the property to FALSE.
 * This command must be called after that all tasks, jobs and dependences
 * has been copy from the ref to the day.
 */
BOOL dg87_day_validate(dg87_day* pDay);

/*************************************************************************/
/*  functions for tasks                                                    */
/*************************************************************************/

/*
 * dg87_day_taskAdd
 * create an TASK object, add this object in the TaskList
 * of the day and return a pointer on this object.
 * Return NULL if the TASK object can't be create.
 */
dg87_task* dg87_day_taskAdd(dg87_day* pDay, char* pTaskName);

/*
 * dg87_day_TaskCount
 * return the number of TASK.
 * return -1 if pDay is NULL.
 */
int dg87_day_taskCount(dg87_day* pDay);

/*
 * dg87_day_taskGetByName
 * return a pointer on the TASK which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_task* dg87_day_taskGetByName(dg87_day* pDay, char* pName);

/*
 * dg87_taskGetByIndex
 * return a pointer on the TASK at the given position.
 */
dg87_task* dg87_day_taskGetByIndex(dg87_day* pDay, int index);

/*
 * dg87_day_taskDelete
 * remove from vector and delete the task given in parameter.
 */
BOOL dg87_day_taskDelete(dg87_day* pDay,dg87_task* pTask);

/*
 * dg87_day_taskDeleteByName
 * delete the task which has the name given in parameter.
 */
BOOL dg87_day_taskDeleteByName(dg87_day* pDay,char* pName);

/*
 * dg87_day_taskDeleteByIndex
 * delete the task at the given index.
 */
BOOL dg87_day_taskDeleteByIndex(dg87_day* pDay,int index);

/*************************************************************************/
/*  functions for taskdep                                                  */
/*************************************************************************/

/*
 * dg87_day_taskdepAdd
 * add a dependance between two tasks of the day. The name of the tasks must
 * be pass in parameters.
 */
dg87_taskdep* dg87_day_taskdepAdd (dg87_day* pDay, 
	 struct dg87_task_struct* pTaskBefore, 
	 struct dg87_task_struct* pTaskAfter);

/*
 * dg87_day_taskdepCount
 * return the number of taskdep the day.
 * return -1 if pDay is NULL.
 */
int dg87_day_taskdepCount(dg87_day* pDay);

/*
 * dg87_day_taskdepGet
 * return a pointer on the taskdep at the given index.
 */
dg87_taskdep* dg87_day_taskdepGet(dg87_day* pDay, int index);

/*
 * dg87_day_taskdepDeleteByIndex
 * delete the taskdep at the given index.
 */
BOOL dg87_day_taskdepDeleteByIndex(dg87_day* pDay,int index);

/*
 * dg87_day_taskdepDelete
 * delete the taskdep given in parameter 
 */
BOOL dg87_day_taskdepDelete(dg87_day* pDay,dg87_taskdep *pTaskDep);

#ifdef DEBUG
/*
 * dg87_day_ckeck
 */
int dg87_day_check(dg87_day* pDay, int (*print)(char*));
#endif /*DEBUG*/

#endif

/* End of file */
