/* file taskdep.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef TASKDEP_H
#define TASKDEP_H 1
#include "global.h"

typedef enum {
	dg87_taskdep_state_unknown,
	dg87_taskdep_state_waiting,
	dg87_taskdep_state_validated,
	dg87_taskdep_state_canceled
} dg87_taskdep_state;

typedef enum {
	dg87_taskdep_type_unknown,
	dg87_taskdep_type_always,
	dg87_taskdep_type_onSuccess,
	dg87_taskdep_type_onError
} dg87_taskdep_type;

typedef struct dg87_taskdep_struct
{
	struct dg87_task_struct* pTaskBefore;
	struct dg87_task_struct* pTaskAfter;
	dg87_taskdep_type type;
	dg87_taskdep_state state;
} dg87_taskdep;

/*
 * dg87_taskdep_new
 */
dg87_taskdep* dg87_taskdep_new (struct dg87_task_struct* pTaskBefore, 
		struct dg87_task_struct* pTaskAfter);
/*
 * dg87_taskdep_delete
 */
BOOL dg87_taskdep_delete(dg87_taskdep* pTaskDep);

/*
 * dg87_taskdep_load
 */
BOOL dg87_taskdep_load(dg87_taskdep* pTaskDep, dg87_storage* pStorage);

/*
 * dg87_taskdep_save
 */
BOOL dg87_taskdep_save(dg87_taskdep* pTaskDep, dg87_storage* pStorage);

/*
 * dg87_taskdep_setType
 * set the type of dependence for the taskdep.
 */
BOOL dg87_taskdep_setType(dg87_taskdep* pTaskDep, dg87_taskdep_type type);

/*
 * dg87_taskdep_setState
 * set the state of the taskdep.
 */
BOOL dg87_taskdep_setState(dg87_taskdep* pTaskDep, dg87_taskdep_state state);

/*
 * dg87_taskdep_evaluate
 * evaluate dependence's state.
 */
BOOL dg87_taskdep_evaluate(dg87_taskdep* pTaskdep);

/*
 * dg87_taskdep_asciiToType
 * convert a string into a dependence type.
 * return -1 if error.
 */
dg87_taskdep_type dg87_taskdep_asciiToType(char* pStrType);

/*
 * dg87_taskdep_typeToAscii
 * convert a dependence type into a string
 */
char* dg87_taskdep_typeToAscii(dg87_taskdep_type type);

/*
 * dg87_taskdep_asciiToState
 * convert a string into a dependence state.
 * return -1 if error.
 */
dg87_taskdep_state dg87_taskdep_asciiToState(char* pStrState);

/*
 * dg87_taskdep_stateToAscii
 * convert a dependence state into a string
 */
char* dg87_taskdep_stateToAscii(dg87_taskdep_state state);

#ifdef DEBUG
/*
 * dg87_taskdep_ckeck
 */
int dg87_taskdep_check(dg87_taskdep* pTaskdep, int (*print)(char*));
#endif /*DEBUG*/

#endif
/* End of file */
