/* file util.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef UTIL_H
#define UTIL_H

#include "global.h"
#include <libdg87/connection.h>
#include "target.h"
#include "queue.h"
#include "day.h"
#include "variable.h"
#include <stdlib.h>

/******************************************************************************/
/* dg87_util_findDay
 * return the Day which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find.
 */
dg87_day* dg87_util_findDay(dg87_connection *pConnec,char* pName);

/******************************************************************************/
/* dg87_util_findTask 
 * find the task which has the given name (pName) in the given day (pDay).
 * Print an error message and return NULL if it can't be find.
 */
dg87_task* dg87_util_findTask(dg87_connection *pConnec, 
		dg87_day* pDay, char* pName);

/******************************************************************************/
/* dg87_util_findJob
 * find the job which has the given name (pName) in the given task (pTask).
 * Print an error message and return NULL if it can't be find.
 */
dg87_job* dg87_util_findJob(dg87_connection *pConnec, 
		dg87_task* pTask, char* pName);

/******************************************************************************/
/* dg87_util_findTarget
 * return the Target which correspond to the name given in parameter.
 * Print an error message and return NULL if it can't be find.
 */
dg87_target* dg87_util_findTarget(dg87_connection *pConnec,char* pName);

/******************************************************************************/
/* dg87_util_findQueue
 * return the Queue which correspond to the name given in parameter.
 * The Target must be pass as parameter.
 * Print an error message and return NULL if it can't be find.
 */
dg87_queue* dg87_util_findQueue(dg87_connection *pConnec,
		dg87_target* pTarget, char* pQueueName);
	
/******************************************************************************/
/* dg87_util_findVariable
 * find the variable which has the given name (pName).
 * Print an error message and return NULL if it can't be find.
 */
dg87_variable* dg87_util_findVariable(dg87_connection *pConnec, char* pName);

#endif /* UTIL_H */
/* End of file */
