/* file variable.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "context.h"
#include "variable.h"
#include <stdlib.h>
#include <string.h>

/****************************************************************************/
dg87_variable* dg87_variable_new(char* pName, char* pValue)
{
	dg87_variable* pVariable;
	if (pName == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	if (strlen(pName) == 0){
		dg87_err_setError(DG87_ERR_INCORRECTVALUE);
		return NULL;
	}
	if (strlen(pName) >= DG87_VARIABLE_NAME_LENGTH){
		dg87_err_setError(DG87_ERR_NAMETOLONG);
		return FALSE;
	}
	pVariable = dg87_malloc(sizeof(dg87_variable), DG87_VARIABLE);
	if (pVariable == NULL){
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	/* copy value */
	pVariable->pValue = dg87_strdup(pValue, DG87_CHAR);
	if (pVariable->pValue == NULL){
		dg87_free(pVariable, DG87_VARIABLE);
		return NULL;
	}
	strncpy(pVariable->name, pName, DG87_VARIABLE_NAME_LENGTH); 
	pVariable->linkedConditions = 0;
	return pVariable;
}

/****************************************************************************/
BOOL dg87_variable_delete(dg87_variable* pVariable)
{
	if (pVariable == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pVariable->linkedConditions > 0){
		dg87_err_setError(DG87_ERR_VARIABLEUSEDBYCONDITIONS);
		return FALSE;
	}
	if (pVariable->pValue != NULL)
		dg87_free(pVariable->pValue, DG87_CHAR);
	dg87_free(pVariable, DG87_VARIABLE);
	return TRUE;
}
/****************************************************************************/
BOOL dg87_variable_load(dg87_variable* pVariable, dg87_storage* pStorage)
{
	char* pString;
	if (pVariable == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* read variable value */
	if (dg87_storage_readString(pStorage, &pString) == FALSE)
		return FALSE;
	if (dg87_variable_setValue(pVariable, pString) == FALSE){
		dg87_free(pString, DG87_CHAR);
		return FALSE;
	}
	dg87_free(pString, DG87_CHAR);

	return TRUE;
}
/****************************************************************************/
BOOL dg87_variable_save(dg87_variable* pVariable, dg87_storage* pStorage)
{
	if (pVariable == NULL || pStorage == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* write variable value */
	if (dg87_storage_writeString(pStorage, pVariable->pValue) == FALSE)
		return FALSE;
	return TRUE;
}
/****************************************************************************/
BOOL dg87_variable_setValue(dg87_variable* pVariable, char* pValue)
{
	char* pCopy;
	if (pVariable == NULL || pValue == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* copy new value */
	pCopy = dg87_strdup(pValue, DG87_CHAR);
	if (pCopy == NULL)
		return FALSE;
	/* clear previous value */
	dg87_free(pVariable->pValue, DG87_CHAR);
	pVariable->pValue = pCopy;
	return TRUE;
}
/****************************************************************************/
#ifdef DEBUG
int dg87_variable_check(dg87_variable* pVariable, int (*print)(char*))
{
	if (strlen(pVariable->name) == 0){
		print("Variable : name length is zero !\n");
		return 1;
	}

	return 0;
}
#endif /*DEBUG*/
/****************************************************************************/

/* End of file */
