/* file main.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#include <config.h>
#include <libdg87/global.h>
#include "global.h"
#include <libdg87/connection.h>
#include <libdg87/conf.h>


extern char *optarg;
extern int optind;
extern int opterr;

char* pHost = NULL;
char* pUser = NULL;
char* pPassword = NULL;
char* pIpPort = NULL;
BOOL verbose = FALSE;

/*
 * dg87_conf_privateFieldArray
 */
dg87_conf_field privateFieldArray[]=
{
	{"DG87D_ADDRESS",   NULL, NULL},
	{"DG87D_IPPORT",    NULL, NULL},
	{"DG87D_USER",      NULL, NULL},
	{"DG87D_PASSWORD",  NULL, NULL},

	{"DG87X_ADDRESS",   NULL, NULL},
	{"DG87X_IPPORT",    NULL, NULL},
	{"DG87X_USER",      NULL, NULL},
	{"DG87X_PASSWORD",  NULL, NULL},

	{"DG87R_ADDRESS",   NULL, NULL},
	{"DG87R_IPPORT",    NULL, NULL},
	{"DG87R_USER",      NULL, NULL},
	{"DG87R_PASSWORD",  NULL, NULL},

	{NULL, NULL, NULL}
};

/*
 * dg87_conf_publicFieldArray
 */
dg87_conf_field publicFieldArray[]=
{
	{"DG87D_ADDRESS",   NULL, NULL},
	{"DG87D_IPPORT",    NULL, NULL},
	/*{"DG87D_USER",      NULL, NULL}, */
	/*{"DG87D_PASSWORD",  NULL, NULL}, */

	{"DG87X_ADDRESS",   NULL, NULL},
	{"DG87X_IPPORT",    NULL, NULL},
	/* {"DG87X_USER",      NULL, NULL},*/
	/*{"DG87X_PASSWORD",  NULL, NULL}, */

	{"DG87R_ADDRESS",   NULL, NULL},
	{"DG87R_IPPORT",    NULL, NULL},
	/*{"DG87R_USER",      NULL, NULL},*/
	/*{"DG87R_PASSWORD",  NULL, NULL},*/

	{NULL, NULL, NULL}
};

dg87_conf* pPrivateConf = NULL;
dg87_conf* pPublicConf = NULL;


/******************************************************************************/
/*
 * get Value
 *
 * - option ligne de commande
 * - variable d'environnement
 * - fichier de configuration prive
 * - fichier de configuration public 
 */
char* getValue(char* pName, char* pDefaultValue, char* pValue)
{
	if (pName == NULL || pDefaultValue == NULL)
		return NULL;

	/* return if value is already set */
	if (pValue != NULL)
		return pValue; 

	/* free memory of previous value */
	free(pValue);

	/* return environment variable if set */
	if ( (pValue = getenv(pName)) != NULL)
		return pValue;

	/* return value of private configuration file */
	if (pPrivateConf != NULL)
		if ((pValue = dg87_conf_getFieldValue(pPrivateConf, 
						pName)) != NULL)
			return pValue;

	/* return value of public configuration file */
	if (pPublicConf != NULL)
		if ((pValue = dg87_conf_getFieldValue(pPublicConf, 
						pName)) != NULL)
			return pValue;

	/* return default value */
	return pDefaultValue;
}
/******************************************************************************/
BOOL readConfigFiles(char* pConfigFile, char* baseName)
{
	char* ipPortField = NULL;
	char* addressField = NULL;
	char* userField = NULL;
	char* passwordField = NULL;
	char* ipPortDefault = NULL;
	char* pHomeDir;
	struct stat fstat;
	BOOL specific = TRUE;

	if (pConfigFile == NULL) {
		pConfigFile = DIOCMD_PRIVATE_CONFIGFILE;
		specific = FALSE;
	}

	/* get home directory from environment variable */
	pHomeDir = getenv("HOME");

	/* if config file path is relative to home directory */
	if (pConfigFile != NULL && pConfigFile[0] == '~'){
		if (pHomeDir == NULL){
			fprintf(stderr, "Can't open configuration file '%s', "\
				"because HOME environment variable "
				"is not set !\n", pConfigFile);
			return FALSE;
		}
		/* change current directory to home directory */
		chdir(pHomeDir);
		/* make config path relative to current directory (home) */
		pConfigFile++; /* skip '~' character */
		if (pConfigFile[0] == '/') pConfigFile++; /*skip '/' character*/
	}

	/* check if private configuration file exists */
	if (stat(pConfigFile, &fstat) != 0){
		/* don't print error message if '-C' wasn't used */
		if (specific == TRUE){
			fprintf(stderr, 
				"Configuration file '%s' doesn't exist !\n",
				pConfigFile);
			return FALSE;
		}
	} else {
		/* create configure file object */
		if ((pPrivateConf = dg87_conf_new(pConfigFile, 
					privateFieldArray)) == NULL)
			return FALSE;
		/* read configuration file */
		if (dg87_conf_read(pPrivateConf) == FALSE){ 
			fprintf(stderr, "Error while reading private " \
					"configuration file '%s' !\n", 
					pConfigFile);
			dg87_conf_delete(pPrivateConf);
			return FALSE;
		}
	}

	/* check if public configuration file exists */
	if (stat(DIOCMD_PUBLIC_CONFIGFILE, &fstat) == 0){
		/* create configure file object */
		if ((pPublicConf = dg87_conf_new(DIOCMD_PUBLIC_CONFIGFILE, 
						publicFieldArray)) == NULL){
			dg87_conf_delete(pPrivateConf);
			return FALSE;
		}
		/* read configuration file */
		if (dg87_conf_read(pPublicConf) == FALSE){
			fprintf(stderr, 
				"Error while reading public configuration " \
				"file '%s' !\n", DIOCMD_PUBLIC_CONFIGFILE);
			dg87_conf_delete(pPrivateConf);
			dg87_conf_delete(pPublicConf);
			return FALSE;
		}
	}

	/* get address and IP port field names */ 
	addressField =  "DG87D_ADDRESS";
	ipPortField =   "DG87D_IPPORT";
	userField =     "DG87D_USER";
	passwordField = "DG87D_PASSWORD";
	ipPortDefault = DG87D_IPPORT;
	if (strcmp(baseName, "dgr") == 0){
		addressField =  "DG87R_ADDRESS";
		ipPortField =   "DG87R_IPPORT";
		userField =     "DG87R_USER";
		passwordField = "DG87R_PASSWORD";
		ipPortDefault = DG87R_IPPORT;
	}
	else if (strcmp(baseName, "dgx") == 0){
		addressField =  "DG87X_ADDRESS";
		ipPortField =   "DG87X_IPPORT";
		userField =     "DG87X_USER";
		passwordField = "DG87X_PASSWORD";
		ipPortDefault = DG87X_IPPORT;
	}
	/* TODO previous value is not free after call to getValue */
	pHost =     getValue(addressField, "localhost", pHost);
	pIpPort =   getValue(ipPortField,  ipPortDefault, pIpPort);
	pUser =     getValue(userField,    DG87_USER_ADMIN_NAME, pUser);
	pPassword = getValue(passwordField,"", pPassword);

	if (verbose == TRUE){
		fprintf(stderr, "%s\n", baseName);
		fprintf(stderr, "host name : \t%s\n", pHost);
		fprintf(stderr, "IP port : \t%s\n", pIpPort);
		fprintf(stderr, "User name : %s\n", pUser);
		fprintf(stderr, "Password : %s\n", pPassword);
	}
	if (pPrivateConf != NULL) dg87_conf_delete(pPrivateConf);
	if (pPublicConf != NULL) dg87_conf_delete(pPublicConf);
	return TRUE;
}
/******************************************************************************/
static void usage(const char* name){
	fprintf(stderr,"usage: %s [-s] [-H host] [-P port] "\
			"[-u username] [-p password] command \n", name);
}
/******************************************************************************/
static char* getBaseName(const char* path){
	char *baseName = 0;
	if ((baseName = strrchr(path,'/')) != 0)
		++baseName;
	else
		baseName = (char*) path;
	return baseName;
}
/******************************************************************************/
int main(int argc, char* argv[])
{
	static char optstring[] = "C:H:P:u:p:hvsw:V";
	char* pString;
	char* pBanner;
	char* pConfigFile = NULL;
	int optch;
	char* baseName;
	char* daemonName;
	int length,i;
	BOOL showProgress=FALSE;
	struct timeval tv;
	struct dg87_connection_struct * pConnection;
	int exitCode = -1;
	int tempo = 0;

	int ipPort = -1;
	
	baseName =  getBaseName(argv[0]);
       

	/* interpret options */
	while ((optch = getopt(argc, argv, optstring)) != -1){
		switch (optch){
			case 'C': /* configuration file */
				pConfigFile = strdup(optarg);
				break;
			case 'H': /* host */
				pHost = strdup(optarg);
				break;
			case 'P': /* port */
				pIpPort = strdup(optarg);
				break;
			case 'u': /* user name */
				pUser = strdup(optarg);
				break;
			case 'p': /* user password */
				pPassword = strdup(optarg);
				break;
			case 'h': /* help */
				usage(baseName);
				exit(0);
				break;
			case 's': /* show progress */
				showProgress = TRUE;	
				break;
			case 'v': /* version */
				fprintf(stderr,"%s-%s\n", PACKAGE, VERSION);
				exit(0);
				break;
			case 'V': /* verbose mode */
				verbose = TRUE;
				break;
			case 'w': /* tempo */
				tempo = atoi(optarg);
				break;
			default:
				fprintf(stderr,"Use -h for help.\n");
				exit(0);
				break;
		}
	}


	/* read configuration file */
	if (readConfigFiles(pConfigFile, baseName)== FALSE)
		exit(1);
	ipPort = atoi(pIpPort);
	
	/* create connection object */
	pConnection = dg87_connection_out_new(pHost,ipPort);
	if (pConnection == NULL){
		fprintf(stderr, "Can't create connection object !\n");
		exit(1);
	}

	/* read banner */
	while (pConnection->state != dg87_connection_state_messageAvailable &&
		pConnection->state != dg87_connection_state_error)
	{
		if (dg87_connection_update(pConnection))
			if (showProgress){
				fprintf(stderr,"."); fflush(stderr);
			}
		/* wait 0.001 second */
		tv.tv_sec = 0; tv.tv_usec = 1000;
		select(0, NULL, NULL , NULL, &tv);
	}
	if (showProgress == TRUE) fprintf(stderr,"\n");
	if (pConnection->state == dg87_connection_state_error){
		dg87_connection_delete(pConnection);
		fprintf(stderr, "Connection error !\n");
		exit(1);
	}
	pBanner = dg87_connection_read(pConnection);

	/* interpret banner */
	/*diogene87-dg87d-0.8.2cvs-(P01)*/
	/* verify package name */
	if (strncmp(pBanner, PACKAGE, strlen(PACKAGE)) != 0){
		dg87_connection_delete(pConnection);
		fprintf(stderr, "Wrong banner '%s' !\n", pBanner);
		exit(1);
	}
	/* verify daemon name */
	if (strcmp(baseName, "dgd") == 0)
		daemonName = DG87D_NAME; 
	else if (strcmp(baseName, "dgr") == 0)
		daemonName = DG87R_NAME; 
	else if (strcmp(baseName, "dgx") == 0)
		daemonName = DG87X_NAME; 
	else
		daemonName = NULL;
	if (daemonName != NULL && strncmp(pBanner+strlen(PACKAGE)+1, 
				daemonName, strlen(daemonName)) != 0){
		dg87_connection_delete(pConnection);
		fprintf(stderr, "Wrong daemon name in banner '%s' !\n", 
				pBanner);
		exit(1);
	}

	dg87_free(pBanner, DG87_CHAR);
	

	/* send user and password */
	if (pUser != NULL && pPassword != NULL){

		dg87_connection_printf(pConnection, "user \"%s\" \"%s\"",
				pUser, pPassword);
		if (dg87_connection_send(pConnection) == FALSE){
			dg87_connection_delete(pConnection);
			fprintf(stderr, "Can't send message !\n");
			exit(1);
		}
		/* send password and wait response */
		while (pConnection->state == 
				dg87_connection_state_sendingMessage ||
			pConnection->state == 
				dg87_connection_state_readingMessage)
		{
			if (dg87_connection_update(pConnection))
				if (showProgress){
					fprintf(stderr,".");
					fflush(stderr);
				}
			/* wait 0.01 second */
			tv.tv_sec = 0; tv.tv_usec = 10000;
			select(0, NULL, NULL , NULL, &tv);
		}
		if (showProgress == TRUE) fprintf(stderr,"\n");
		if (pConnection->state == dg87_connection_state_error){
			dg87_connection_delete(pConnection);
			fprintf(stderr, "Communication error !\n");
			exit(1);
		}

		/* read response */
		pString = dg87_connection_read(pConnection);
		if ( pString == NULL) { 
			fprintf(stderr, "Response unavailable !\n");
			dg87_connection_delete(pConnection);
			exit(1);
		}

		/* get exit code */
		length = strlen(pString);
		if (length == 2 && strcmp("OK", pString) == 0){
			*pString = '\0';
			exitCode = 0;
		} else if (length > 3 && 
				strcmp("OK", pString + length - 2) == 0){
			*(pString + length -3) = '\0';
			exitCode = 0;
		} else {
			/* find the string ERROR: */
			while (length > 0 && pString[length] != ':')
				length--;
			if (length > 6 && strncmp("ERROR:", 
						pString+length-5 , 6) == 0)
			{
				exitCode = atoi(pString + length + 1);
				*(pString + length -5) = '\0';
				/* ensure that an error code is return */
				if (exitCode == 0) exitCode = 255;
			}
			else
				exitCode = -1;
		}

		/* print the string */
		for (i=0; i < strlen(pString); i++)
			if (pString[i] != '\r')
				fprintf((exitCode==0)?stdout:stderr,
						"%c", pString[i]);
		if (exitCode != 0)
			return exitCode;

	}

	/* tempo */
	if (tempo > 0)
		sleep(tempo);

	/* send command (concat all arguments) */
	for (i = optind; i < argc; ++i){
		if (strlen(argv[i]) == 0)
			/* send an empty string */
			dg87_connection_printf(pConnection, "\"\" ");
		else if (strchr(argv[i], ' ') == NULL 
				&& strchr(argv[i], '\t') == NULL
				&& strchr(argv[i], '\"') == NULL)
			dg87_connection_printf(pConnection, "%s ", argv[i]);
		else
		{
			/* TODO if string contains quote, add a backslash 
			 * before */
			dg87_connection_printf(pConnection, "\"%s\" ", argv[i]);
		}
	}
	/*printf("buffer:%s\n", pConnection->pBuffer);*/

	if (dg87_connection_send(pConnection) == FALSE){
		dg87_connection_delete(pConnection);
		fprintf(stderr, "Can't send message !\n");
		exit(1);
	}

	/* send request and wait response */
	while (pConnection->state == dg87_connection_state_sendingMessage ||
		pConnection->state == dg87_connection_state_readingMessage)
	{
		if (dg87_connection_update(pConnection))
			if (showProgress){
				fprintf(stderr,".");
				fflush(stderr);
			}
		/* wait 0.01 second */
		tv.tv_sec = 0; tv.tv_usec = 10000;
		select(0, NULL, NULL , NULL, &tv);
	}
	if (showProgress == TRUE) fprintf(stderr,"\n");
	if (pConnection->state == dg87_connection_state_error){
		dg87_connection_delete(pConnection);
		fprintf(stderr, "Communication error !\n");
		exit(1);
	}

	/* read response */
	pString = dg87_connection_read(pConnection);
	if ( pString == NULL) { 
		fprintf(stderr, "Response unavailable !\n");
		dg87_connection_delete(pConnection);
		exit(1);
	}

	/* get exit code */
	length = strlen(pString);
	if (length == 2 && strcmp("OK", pString) == 0){
		*pString = '\0';
		exitCode = 0;
	} else if (length > 3 && strcmp("OK", pString + length - 2) == 0){
		*(pString + length -3) = '\0';
		exitCode = 0;
	} else {
		/* find the string ERROR: */
		while (length > 0 && pString[length] != ':')
			length--;
		if (length > 6 && strncmp("ERROR:", pString+length-5 , 6) == 0)
		{
			exitCode = atoi(pString + length + 1);
			*(pString + length -5) = '\0';
			/* ensure that an error code is return */
			if (exitCode == 0) exitCode = 255;
		}
		else
			exitCode = -1;
	}

	/* print the string */
	for (i=0; i < strlen(pString); i++)
		if (pString[i] != '\r')
			fprintf((exitCode==0)?stdout:stderr,
					"%c", pString[i]);

	/*printf("%s\n", pString);*/

	/* free memory */
	free(pString);
	dg87_connection_delete(pConnection);

	return exitCode;
}
/* End of file */
