/* file cmdconnection.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "daemon.h"
#include "command.h"
#include "cmdconnection.h"

/******************************************************************************/
int cmd_cc(dg87_connection* pConnection)
{
	dg87_list* pListConnection;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pListConnection = pConnection->pServer->pListConnection;
	dg87_connection_printf(pConnection,"%i\n", pListConnection->count);
	return 0;
}
/******************************************************************************/
int cmd_ck(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	dg87_item* pItem;
	dg87_connection* pItemConnection=NULL;
	int ipPort;
	char* pHostName;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "ck host port");
		return 1;
	}
	/* find the connection */
	ipPort = atoi(pCommand->argv[1]);
	pHostName = pCommand->argv[0];
	pItem = pConnection->pServer->pListConnection->pFirst;
	while (pItem != NULL){
		pItemConnection = pItem->pObject;
		if (ipPort == pItemConnection->ipPort &&
				strcmp(pHostName, pItemConnection->pHost) == 0)
			break;
		pItem = pItem->pNext;
	}
	if (pItem == NULL){
		dg87_connection_printf(pConnection,
			"No connection with the host '%s' on the port '%i'\n",
			pHostName, ipPort);
		return 1;
	}
	if (pItemConnection != NULL)
		dg87_connection_close(pItemConnection);
	return 0;
}
/******************************************************************************/
/* cmd_cl
 * Connection list
 * Usage : cl [options]
 * options :
 * -u : show user name
 */
int cmd_cl(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	int optch, iopt;
	char* optarg;
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	dg87_item* pItem;
	dg87_connection* pItemConnection;
	BOOL showUser = FALSE;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	/* control number of arguments */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"cl [options]");
		return 1;
	}
	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'u': /* user name */
				showUser = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}


	pItem = pConnection->pServer->pListConnection->pFirst;
	while (pItem != NULL){
		pItemConnection = pItem->pObject;
		pItem = pItem->pNext;
		dg87_connection_printf(pConnection, "%s%s%s%i",
				pBeg, pItemConnection->pHost,
				pSep, pItemConnection->ipPort);
		if (showUser == TRUE){
		 	dg87_connection_printf(pConnection, "%s%s", pSep,
					(pItemConnection->pUser == NULL)?
					"?":pItemConnection->pUser->name);
		}
		dg87_connection_printf(pConnection, "%s\n", pEnd);
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}
/******************************************************************************/
int cmd_user(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	dg87_user* pUser;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 2){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", 
			"user UserName Password");
		return 1;
	}	
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the user */
	pUser = dg87_server_userGetByName(pConnection->pServer,
			pCommand->argv[0]);
	if (pUser == NULL){
		dg87_connection_printf(pConnection,
			"Unknown user '%s'.\n",
			pCommand->argv[0]);
		return 1;
	}
	/* verify password */
	if (dg87_user_verifyPassword(pUser, pCommand->argv[1]) == FALSE){
		dg87_connection_printf(pConnection,
				"Incorrect password.\n");
		return 1;
	}

	/* link user and connection */
	if (dg87_connection_setUser(pConnection, pUser) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't connect user '%s'. %s\n",
				pUser->name,
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
int cmd_password(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	dg87_user* pUser;
	char* pPassword;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc < 1 || pCommand->argc>2 ){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", 
			"password [UserName] NewPassword");
		return 1;
	}	
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the user */
	if ( pCommand->argc == 1 ){
		pUser = pConnection->pUser;
		if (pUser == NULL){
			dg87_connection_printf(pConnection,
				"No user connected.");
			return 1;
		}
		pPassword = pCommand->argv[0];
	} else { 
		/* check user privilege (to change password of another user) */
		if (pConnection->pUser->privileges[dg87_command_user]
				< dg87_command_access_manage){
			dg87_connection_printf(pConnection,
				"Authorization denied.\n");
			return 1;
		}

		pUser = dg87_server_userGetByName(pConnection->pServer,
				pCommand->argv[0]);
		if (pUser == NULL){
			dg87_connection_printf(pConnection,
				"Unknown user '%s'\n",
				pCommand->argv[0]);
			return 1;
		}
		pPassword = pCommand->argv[1];
	}
	dg87_user_setPassword(pUser, pPassword);
	return 0;
}
/******************************************************************************/
/* End of file */
