/* file cmdmessage.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <assert.h>

#include "daemon.h"
#include "command.h"
#include "cmdmessage.h"

/******************************************************************************/
int cmd_mc(dg87_connection* pConnection)
{
	dg87_list* pListMessage;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pListMessage= pConnection->pServer->pListMessage;
	dg87_connection_printf(pConnection,"%i\n", pListMessage->count);
	return 0;
}
/******************************************************************************/
int cmd_mk(dg87_connection* pConnection)
{
	dg87_message* pMessage;
	dg87_command* pCommand;
	int index;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "mk index");
		return 1;
	}
	/* find the connection */
	index = atoi(pCommand->argv[0]);
	pMessage = dg87_list_removeObjectByIndex(
			pConnection->pServer->pListMessage, index - 1);
	if (pMessage == NULL){
		dg87_connection_printf(pConnection,
			"Incorrect value for index '%i'\n", index);
		return 1;
	}
	dg87_message_cancel(pMessage);
	return 0;
}
/******************************************************************************/
/*
 * cmd_ml
 * Usage : ml [options] 
 * options :
 * -s : print message state
 * -m : print message
 */
int cmd_ml(dg87_connection* pConnection)
{
	BOOL showState = FALSE;
	BOOL showMessage = FALSE;
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	dg87_item* pItem;
	dg87_command* pCommand;
	dg87_message* pItemMessage;
	int optch, iopt;
	char* optarg;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	/* control number of arguments */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"ml [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 's': /* state */
				showState = TRUE;
				break;
			case 'm': /* show message */
				showMessage = TRUE;
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	pItem = pConnection->pServer->pListMessage->pFirst;
	while (pItem != NULL){
		pItemMessage = pItem->pObject;
		pItem = pItem->pNext;
		dg87_connection_printf(pConnection, "%s%s%s%i", pBeg,
				pItemMessage->pHost, pSep,
				pItemMessage->ipPort);
		if (showMessage == TRUE)
			dg87_connection_printf(pConnection, "%s%s", pSep,
				pItemMessage->pMsgString);
		if (showState == TRUE)
			dg87_connection_printf(pConnection, "%s%s", pSep,
				dg87_message_stateToAscii(pItemMessage->state));
		dg87_connection_printf(pConnection, "%s\n", pEnd);
	}
	return 0;
}
/******************************************************************************/
/* End of file */
