/* file command.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef COMMAND_H
#define COMMAND_H

#include "global.h"
#include "connection.h"
#include "stdio.h"

#define DG87_COMMAND_ERR_STRNOTCLOSED 1
#define DG87_COMMAND_ERR_UNKNOWNOPTION 2
#define DG87_COMMAND_ERR_NOARGUMENT 3


/*
 * dg87_command object
 */
typedef struct dg87_command_struct
{
	int argc;   /* number of arguemnts */
	char** argv; /* arguments vector */
	int optc;   /* number of options */
	char* optv; /* options vector */
	char** optargv; /* argument's options vector */
	char* buffer; /* buffer which contain the command string */
	char* optstr; /* options accepted by function */
	char* pFunctionName; /* first argument of cmd line */
	int error;
	char errorOption;
} dg87_command;

/*
 * dg87_command_new
 */
dg87_command* dg87_command_new(char* pCmdString);

/*
 * dg87_command_interpretOptions
 */
BOOL dg87_command_interpretOptions(dg87_command* pCommand, char* pOptString);

/*
 * dg87_command_delete
 */
BOOL dg87_command_delete(dg87_command* pCommand);

/*
 * dg87_command_asciiToFamily
 * convert a string to a command family.
 * return -1 if error.
 */
dg87_command_family dg87_command_asciiToFamily(char*);

/*
 * dg87_command_familyToAscii
 * convert a command family into a string
 */
char* dg87_command_familyToAscii(dg87_command_family);

/*
 * dg87_command_asciiToPrivilege
 * convert a string to a command privilege.
 * return -1 if error.
 */
dg87_command_privilege dg87_command_asciiToPrivilege(char*);

/*
 * dg87_command_privilegeToAscii
 * convert a command privilege into a string
 */
char* dg87_command_privilegeToAscii(dg87_command_privilege);



#endif /*COMMAND_H*/
/* End of file */
