/* file conf.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef CONF_H
#define CONF_H 

#include "global.h"

/*
 * dg87_conf_field
 */
typedef struct dg87_conf_field_struct
{
	char* pName;         /* name of field */
	char* pDefaultValue; /* default value */
	char* pValue;        /* actual value (NULL at startup) */
} dg87_conf_field;

/*
 * dg87_conf
 */
typedef struct dg87_conf_struct
{
	char* pConfigFile;
	dg87_conf_field *pFieldArray;
} dg87_conf;

/*
 * dg87_conf_new
 *
 * Create a new dg87_conf file object and return a pointer on this object.
 * Parameters are :
 * pConfigFile : path of the config file ("/etc/diogene87/dg87d.conf")
 * pFieldArray : Array of dg87_conf_field structures. Last element must be
 * NULL.
 * dg87_conf_read must be call to read configuation file.
 */
dg87_conf *dg87_conf_new(char* pConfigFile, dg87_conf_field *pFieldArray);

/*
 * dg87_conf_delete
 */
BOOL dg87_conf_delete(dg87_conf *pConf);

/*
 * dg87_conf_read
 * This function reads (or re-reads) the configuration file.
 * Field values are modified.
 */
BOOL dg87_conf_read(dg87_conf *pConf);

/* 
 * dg87_conf_getFieldValue
 * This functions returns the value of a field in the field array.
 * Returns NULL if field can't be found
 */
char* dg87_conf_getFieldValue(dg87_conf *pConf, char* pFieldName);

#endif /*CONF_H*/
/* End of file */
