/* file daemon.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "global.h"
#include "daemon.h"
#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <sys/stat.h> /*umask*/
#include <sys/types.h> /*getpwnam*/
#include <pwd.h>
#include <config.h>
#include <assert.h>

/*****************************************************************************/
dg87_daemon* dg87_daemon_new(char* pName, char* pUserName)
{
	dg87_daemon* pDaemon;
	assert(pName != NULL);
	if (pName == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pDaemon = dg87_malloc(sizeof(dg87_daemon), DG87_DAEMON);
	if (pDaemon == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	if (pUserName == NULL)
		pDaemon->pUserName = NULL;
	else {
		pDaemon->pUserName = dg87_strdup(pUserName, DG87_CHAR);
		if (pDaemon->pUserName == NULL) {
			dg87_free(pDaemon, DG87_DAEMON);
			dg87_err_setError(DG87_ERR_OUTOFMEMORY);
			return NULL;
		}
	}
	pDaemon->pName = pName;
	return pDaemon; 
}

/*****************************************************************************/
int dg87_daemon_start(dg87_daemon* pDaemon)
{
	int pid;
	int fd;
	struct passwd *pwp;
	assert(pDaemon != NULL);
	if (pDaemon == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}

	/* change user id */
	if (pDaemon->pUserName != NULL){
		/* search user informations */
		if ((pwp = getpwnam(pDaemon->pUserName)) == NULL){
			fprintf(stderr, 
				"Can't get user information '%s'\n",
				pDaemon->pUserName);
			return -1;
		}
		/* change gid */
		if ( setgid(pwp->pw_gid) == -1){
			fprintf(stderr, "Can't change group (GID)\n");
			return -1;
		}
		/* change uid */
		if ( setuid(pwp->pw_uid) == -1){
			fprintf(stderr, "Can't change user (UID)\n");
			return -1;
		}
	}

	pid = fork();
	if (pid < 0){
		/* can't fork */
		fprintf(stderr,"fork error\n");
		return -1;
	}
	if (pid > 0){
		/* return 0 to the parent process */
		return 0;
	}
	/* create a new session to be process group leader */
	setsid();
	pid = fork();
	if (pid < 0){
		/* can't fork */
		fprintf(stderr,"error while fork\n");
		return -1;
	}
	if (pid > 0){
		/* the parent end normaly */
		return 0;
	}
	/* close all stream */
	for (fd = 3/*TODO gettablesize()*/; fd >= 0; --fd)
		close(fd);

	/* redirect stdin, stdout and stderr to /dev/null */
	fd = open("/dev/null", O_RDWR);
	dup(fd);dup(fd);
	umask(022);
	chdir("/tmp");
	
	/* open the log */
	openlog(pDaemon->pName,LOG_PERROR | LOG_PID, LOG_DAEMON);
	syslog(LOG_ALERT, "Daemon start");

	return 1;
}
/*****************************************************************************/
void dg87_daemon_stop(dg87_daemon* pDaemon)
{
	/* the deamon ends here */
	syslog(LOG_ALERT, "Daemon end ");
	closelog();
}
/*****************************************************************************/
void dg87_daemon_delete(dg87_daemon* pDaemon)
{
	assert(pDaemon != NULL);
	if (pDaemon == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return;
	}
	if (pDaemon->pUserName != NULL) 
		dg87_free(pDaemon->pUserName, DG87_CHAR);

	dg87_free(pDaemon, DG87_DAEMON);
}

/****************************************************************************/
void dg87_daemon_log(char* pMessage)
{
	assert(pMessage != NULL);
	fprintf(stderr,"%s\n", pMessage);
	syslog(LOG_ALERT, pMessage);
}
/*****************************************************************************/
/* End of file */
