/* file global.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef LIB_GLOBAL_H
#define LIB_GLOBAL_H 1

#define TRUE 1
#define FALSE 0
#define BOOLERROR 2
#define BOOL unsigned char 

#define dg87uint unsigned short int
#define dg87int short int
#define dg87long int
#define dg87byte unsigned char

#include <stdlib.h>
#include "dgerr.h"

#ifdef DEBUG

#define DG87_OTHER	0
#define DG87_CHAR	1
#define DG87_VECTOR	2
#define DG87_LIST	3
#define DG87_ITEM	4
#define DG87_ELEMENT	5
#define DG87_ENGINE	6
#define DG87_SERVER	7
#define DG87_INTERPRET	8
#define DG87_CONNECTION	9
#define DG87_DAEMON	10
#define DG87_MESSAGE	11
#define DG87_COMMAND	12
#define DG87_CONTEXT	13
#define DG87_TASK 	14
#define DG87_JOB 	15
#define DG87_TARGET	16
#define DG87_QUEUE 	17
#define DG87_TASKDEP	18
#define DG87_JOBDEP	19
#define DG87_CONDITION	20
#define DG87_REF	21
#define DG87_DAY	22
#define DG87_RULE	23
#define DG87_OPERATION	24
#define DG87_STRBUF	25
#define DG87_USER	26
#define DG87_VARIABLE	27
#define DG87_STORAGE	28

#define DG87_ARRAYSIZE	29
/* DON'T FORGET TO UPDATE dg87_refcptobj and dg87_objname in memory.c !!! */

/*void* dg87_memory_malloc(size_t A);*/
void* dg87_memory_malloc(size_t A, int B);

/*void* dg87_memory_strdup(const char* A);*/
void* dg87_memory_strdup(const char* A, int B);

/*extern void dg87_memory_free(void *ptr);*/
extern void dg87_memory_free(void *ptr, int B);

void dg87_memory_report(void);
int dg87_memory_usage(void);
char* dg87_memory_stat(void);

time_t dg87_asciiToDate(char* pString);
char* dg87_dateToAscii(time_t time);

char* dg87_boolToAscii(BOOL value);
BOOL dg87_asciiToBool(char* value);

#define dg87_malloc(A,B) dg87_memory_malloc(A,B)
#define dg87_strdup(A,B) dg87_memory_strdup(A,B)
#define dg87_free(A,B) dg87_memory_free(A,B) 

#define dg87_usage() dg87_memory_usage()
#define dg87_report() dg87_memory_report()
#define dg87_stat() dg87_memory_stat()

#else

#define dg87_malloc(A,B) malloc(A)
#define dg87_strdup(A,B) strdup(A)
#define dg87_free(A,B) free(A)

#define dg87_report() {}
#define dg87_usage() {}
#define dg87_stat() {}

#endif /*DEBUG*/

struct dg87_string_struct;
struct dg87_job_struct;
struct dg87_jobdep_struct;
struct dg87_task_struct;
struct dg87_taskdep_struct;
struct dg87_target_struct;
struct dg87_context_struct;
struct dg87_connection_struct;
struct dg87_engine_struct;
struct dg87_interpret_struct;
struct dg87_server_struct;

struct dg87_xjob_struct;
struct dg87_xjobdep_struct;
struct dg87_xtask_struct;
struct dg87_xtaskdep_struct;
struct dg87_day_struct;
struct dg87_user_struct;
struct dg87_command_struct;


/* command families */
#define DG87_COMMAND_FAM_MAX 9 
typedef enum {
	dg87_command_generic = 0,       /* user, password, help, memory ... */
	dg87_command_communication = 1, /* connection, message */
	dg87_command_user = 2,
	dg87_command_engine = 3,        /* shutdown, ese */
	dg87_command_target = 4,        /* target, queue */
	dg87_command_ref = 5,           /* dg87d referential, task, job... */
	dg87_command_job = 6,           /* dg87r jobs */
	dg87_command_day = 7,           /* dg87x day, task, job */
	dg87_command_variable = 8	/* variables */
}dg87_command_family;

/* privilege to access to functions */
typedef enum{
	dg87_command_access_none,
	dg87_command_access_consult,
	dg87_command_access_manage,
	dg87_command_access_admin
}dg87_command_privilege;



#endif /* LIB_GLOBAL_H */
/* End of file */
