/* file interpret.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#include "global.h"
#include "daemon.h"
#include "server.h"
#include "interpret.h"
#include "cmdconnection.h"
#include "cmdmessage.h"
#include "cmduser.h"

/* local prototypes */
int dg87_interpret_cmd_exit(dg87_connection* pConnection);
int dg87_interpret_cmd_version(dg87_connection* pConnection);
int dg87_interpret_cmd_help(dg87_connection* pConnection);
#ifdef DEBUG
int dg87_interpret_cmd_memory(dg87_connection* pConnection);
#endif

/* array for dg87_interpret's commands */
dg87_interpret_cmd pCmdLocalArray[] = {
	{"cc", cmd_cc, "Connection Count", "", 
		dg87_command_communication, dg87_command_access_consult},
	{"ck", cmd_ck, "Connection Kill", "", 
		dg87_command_communication, dg87_command_access_admin},
	{"cl", cmd_cl, "Connection List", "B:S:E:u", 
		dg87_command_communication, dg87_command_access_consult},
	{"exit", dg87_interpret_cmd_exit,"Exit Diogene", "",
		dg87_command_generic, dg87_command_access_none},
	{"help", dg87_interpret_cmd_help,"Give help on commands", "",
		dg87_command_generic, dg87_command_access_none},
	{"mc", cmd_mc, "Message Count", "", 
		dg87_command_communication, dg87_command_access_consult},
#ifdef DEBUG
	{"memory",dg87_interpret_cmd_memory,
		"Memory usage (debug release only)", "v",
		dg87_command_generic, dg87_command_access_none},
#endif
	{"mk", cmd_mk, "Message Kill", "", 
		dg87_command_communication, dg87_command_access_admin},
	{"ml", cmd_ml, "Message List", "B:S:E:sm", 
		dg87_command_communication, dg87_command_access_consult},

	{"password", cmd_password, "Change user password", "", 
		dg87_command_generic, dg87_command_access_consult},

	{"ua", cmd_ua, "User Add", "", 
		dg87_command_user, dg87_command_access_admin},
	{"uc", cmd_uc, "User Count", "", 
		dg87_command_user, dg87_command_access_consult},
	{"ui", cmd_ui, "User Information", "B:S:E:", 
		dg87_command_user, dg87_command_access_consult},
	{"uk", cmd_uk, "User Kill", "", 
		dg87_command_user, dg87_command_access_admin},
	{"ul", cmd_ul, "User List", "B:S:E:I:N:c", 
		dg87_command_user, dg87_command_access_consult},
	{"usp", cmd_usp, "User Set Privilege", "", 
		dg87_command_user, dg87_command_access_manage},

	{"user", cmd_user, "Identify user", "", 
		dg87_command_generic, dg87_command_access_none},

	{"version", dg87_interpret_cmd_version,
		"Return the name and the version of the program", 
		"",dg87_command_generic, dg87_command_access_none},
	{NULL,NULL,NULL,NULL,0}
};
/******************************************************************************/
int dg87_interpret_cmd_exit(dg87_connection* pConnection)
{
	assert(pConnection != NULL);
	dg87_connection_printf(pConnection,"Bye\n");
	dg87_connection_close(pConnection);
	return 0;
}
/******************************************************************************/
int dg87_interpret_cmd_version(dg87_connection* pConnection)
{
	assert(pConnection != NULL);
	dg87_connection_printf(pConnection,"%s-%s-%s\n", 
			pConnection->pServer->pPackageName,
			pConnection->pServer->pProgramName,
			pConnection->pServer->pProgramVersion);
	return 0;
}
/******************************************************************************/
int dg87_interpret_cmd_help(dg87_connection* pConnection)
{
	int iCmd;
	int brk = 0;
	dg87_interpret_cmd* pCommand;
	dg87_interpret* pInterpret;
	assert(pConnection != NULL);
	if (pConnection == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pInterpret = pConnection->pServer->pInterpret;
	switch (pConnection->pCommand->argc){

		case 0: /* print the command list */
		dg87_connection_printf(pConnection, 
				"Available functions are :\n");
		/* internal functions */
		for (iCmd = 0; pCmdLocalArray[iCmd].pName != NULL; iCmd++){
			if (dg87_user_verifyPrivilege(pConnection->pUser, 
						pCmdLocalArray[iCmd].family,
						pCmdLocalArray[iCmd].privilege)
					== TRUE){
				dg87_connection_printf(pConnection, 
					"%s \t", pCmdLocalArray[iCmd].pName);
				if (brk++ == 8){
					dg87_connection_printf(pConnection, 
							"\n");
					brk = 0;
				}
			}
		}
		/* client functions */
		for (iCmd=0; pInterpret->pCmdArray[iCmd].pName != NULL; iCmd++){
			if (dg87_user_verifyPrivilege(pConnection->pUser, 
					pInterpret->pCmdArray[iCmd].family,
					pInterpret->pCmdArray[iCmd].privilege)
					== TRUE){
				dg87_connection_printf(pConnection, "%s \t", 
					pInterpret->pCmdArray[iCmd].pName);
				if (brk++ == 8){
					dg87_connection_printf(pConnection, 
							"\n");
					brk = 0;
				}
			}
		}
		dg87_connection_printf(pConnection, "\n");
		break;

		case 1: /* help for one command */

		/* search in local array */
		pCommand = pCmdLocalArray;
		while (pCommand->pName != NULL && strcmp(pCommand->pName, 
				pConnection->pCommand->argv[0]) != 0) 
			pCommand++;
		/* search in client array */
		if (pCommand->pName == NULL){
			pCommand = pInterpret->pCmdArray;
			while (pCommand->pName != NULL && strcmp(pCommand->pName
					, pConnection->pCommand->argv[0]) != 0) 
				pCommand++;
		}

		if (pCommand->pName == NULL){
			dg87_connection_printf(pConnection, 
					"Help : Unknown command '%s'\n",
					pConnection->pCommand->argv[0]);
			break;
		}

		if (dg87_user_verifyPrivilege(pConnection->pUser, 
				pCommand->family,
				pCommand->privilege) == FALSE){
			dg87_connection_printf(pConnection, 
					"Help : Unauthorized command '%s'\n",
					pConnection->pCommand->argv[0]);
			break;

		}

		/* print help command */
		dg87_connection_printf(pConnection, 
				"%s %s : %s\n", "Function ",
				pCommand->pName, pCommand->pHelp);
		break;

		default:
		dg87_connection_printf(pConnection, "Incorrect use\n");
	}
	return 0;
}
/******************************************************************************/
#ifdef DEBUG
int dg87_interpret_cmd_memory(dg87_connection* pConnection)
{
	dg87_command* pCommand;
	char* pBuffer;
	assert(pConnection != NULL);
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->optc == 0)
		dg87_connection_printf(pConnection, "%l\n", dg87_usage());
	else{
		if (pCommand->optv[0] != 'v'){
			dg87_connection_printf(pConnection, 
				"Unknown option '%c' for command '%s'\n",
				pCommand->optv[0], pCommand->pFunctionName);
			return 1;
		} 
		pBuffer = dg87_stat();
		if (pBuffer == NULL)
			return 1;
		dg87_connection_printf(pConnection, pBuffer);
		free(pBuffer);
	}
	return 0;
}
#endif
/******************************************************************************/
dg87_interpret* dg87_interpret_new(struct dg87_server_struct* pServer,
		struct dg87_interpret_cmd_struct *pCmdArray)
{
	dg87_interpret* pInterpret;
	assert(pServer != NULL && pCmdArray != NULL);
	if (pServer == NULL || pCmdArray == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	pInterpret = dg87_malloc(sizeof(dg87_interpret), DG87_INTERPRET);
	if (pInterpret == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pInterpret->pServer = pServer;
	pInterpret->pCmdArray = pCmdArray;
	return pInterpret;
}
/******************************************************************************/
BOOL dg87_interpret_delete(dg87_interpret* pInterpret)
{
	assert(pInterpret != NULL);
	if (pInterpret == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_free(pInterpret, DG87_INTERPRET);
	return TRUE;
}
/******************************************************************************/
int dg87_interpret_execute(dg87_interpret* pInterpret,
		dg87_connection* pConnection)
{
	int retval = 0;
	dg87_interpret_cmd* pCmd = NULL;
	assert(pInterpret != NULL && pConnection != NULL);	
	if (pInterpret == NULL || pConnection == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return -1;
	}

	/* read the command and initialize the Command object */
	if (dg87_connection_readCommand(pConnection) == FALSE)
		return -2;

	/* find the function in the local functions array */
	pCmd = pCmdLocalArray;
	while (pCmd->pName != NULL && 
		strcmp(pConnection->pCommand->pFunctionName, 
			pCmd->pName) != 0) pCmd++;

	/* find the function in the server functions array */
	if (pCmd->pName == NULL){
		pCmd = pInterpret->pCmdArray;
		while (pCmd->pName != NULL && 
			strcmp(pConnection->pCommand->pFunctionName, 
				pCmd->pName) != 0) pCmd++;
	}

	if (pCmd->pName == NULL){
		dg87_connection_printf(pConnection, "Unknown command '%s'\n",
			pConnection->pCommand->pFunctionName);
		dg87_connection_printf(pConnection,
			"ERROR:%i", DG87_ERR_UNKNOWNCOMMAND );
		dg87_connection_send(pConnection);	
		return -2;
	}

	/* control access privilege */
	if (pConnection->pUser != NULL &&
			dg87_user_verifyPrivilege(pConnection->pUser, 
				pCmd->family, pCmd->privilege) == FALSE){
		dg87_connection_printf(pConnection,
				"Permission denied\n");
		dg87_connection_printf(pConnection,
			"ERROR:%i", DG87_ERR_PERMDENIED );
		dg87_connection_send(pConnection);	
		return -2;
	}

	/* interpret options */
	if (dg87_command_interpretOptions(
		pConnection->pCommand, pCmd->pOptStr) == FALSE) {
		dg87_connection_printf(pConnection, "Interpretation Error\n",
			pConnection->pCommand->pFunctionName);
		dg87_connection_printf(pConnection,
			"ERROR:%i", DG87_ERR_INTERPRETERROR);
		dg87_connection_send(pConnection);	
		return -2;
	}
	switch (pConnection->pCommand->error){
		case DG87_COMMAND_ERR_STRNOTCLOSED:
			dg87_connection_printf(pConnection,
				"Quoted string not closed.\n");
			dg87_connection_send(pConnection);	
			return -2;
		case DG87_COMMAND_ERR_UNKNOWNOPTION:
			dg87_connection_printf(pConnection,
				"Unknown option '-%c' for " \
				"command '%s'.\n",
				pConnection->pCommand->errorOption,
				pConnection->pCommand->pFunctionName);

			dg87_connection_send(pConnection);	
			return -2;
		case DG87_COMMAND_ERR_NOARGUMENT:
			dg87_connection_printf(pConnection,
				"No argument for option '-%c' in " \
				"command '%s'.\n",
				pConnection->pCommand->errorOption,
				pConnection->pCommand->pFunctionName);
			dg87_connection_send(pConnection);	
			return -2;
	}

	if ( pConnection->pCommand->error != 0)
		dg87_connection_printf(pConnection,
			"ERROR:%i", pConnection->pCommand->error);

	/* call the function */
	retval = pCmd->pFunction(pConnection);
	if (retval == 0 || retval == -1)
		dg87_connection_printf(pConnection,"OK");
	else
		dg87_connection_printf(pConnection,
				"ERROR:%i", retval);

	dg87_connection_send(pConnection);	

	return retval;
}
/******************************************************************************/
/* End of file */
