/* file memory.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifdef DEBUG

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>
#include <assert.h>
#include "global.h"

int dg87_refcptnew=0;
int dg87_refcptfre=0;
int dg87_refcptobj[DG87_ARRAYSIZE]={0,0,0,0,0, 0,0,0,0,0, 0,0,0,0,0, 
	0,0,0,0,0, 0,0,0,0,0, 0,0,0,0};

char* dg87_objname[DG87_ARRAYSIZE]=
	{ "Other", "Char", "Vector", "List", "Item", "Element", "Engine",
	"Server", "Interpret", "Connection", "Daemon", "Message", "Command",
	"Context", "Task", "Job", "Target", "Queue", "TaskDep", "JobDep", 
	"Condition", "Ref", "Day", "Rule", "Operation", "Strbuf", "User",
	"Variable", "Storage"};
/******************************************************************************/
void* dg87_memory_malloc(size_t A, int B)
{
	dg87_refcptobj[B]++;
	dg87_refcptnew++;
	return malloc(A);
}
/******************************************************************************/
void* dg87_memory_strdup(const char* A, int B)
{
	assert(A != NULL);
	dg87_refcptobj[B]++;
	dg87_refcptnew++; 
	return strdup(A);
}
/******************************************************************************/
extern void dg87_memory_free(void *ptr, int B)
{
	assert(ptr != NULL);
	dg87_refcptobj[B]--;
	dg87_refcptfre++;
	free(ptr);
}
/******************************************************************************/
int dg87_memory_usage(void)
{
	return dg87_refcptnew - dg87_refcptfre;
}
/******************************************************************************/
char* dg87_memory_stat(void){
	int i;
	char str[20];
	char* newbuffer;
	int size;
	char* buffer;

	buffer = strdup("Memory usage:\n");
	if (buffer == NULL)
		return NULL;
 	size = strlen(buffer) + 1;
	for (i=0; i<DG87_ARRAYSIZE ; i++){
		sprintf(str,"%i", dg87_refcptobj[i]);
		size = size + strlen(dg87_objname[i]) + strlen(str) + 4;
		newbuffer = realloc(buffer,size);
		if (newbuffer == NULL){
			free(buffer);
			return NULL;
		}
		buffer = newbuffer;
		strcat(buffer, dg87_objname[i]);
		strcat(buffer," : ");
		strcat(buffer, str);
		strcat(buffer, "\n");
	}
	return buffer;
}
/******************************************************************************/
void dg87_memory_report(void)
{	
	syslog(LOG_ALERT,"reference count: new(%i) - free(%i)=%i\n",
			dg87_refcptnew, dg87_refcptfre, 
			dg87_refcptnew-dg87_refcptfre);
}
#endif /*DEBUG*/
/******************************************************************************/
/* dg87_asciiToDate
 * convert a string to a time_t.
 * The format of the string is YYYY-MM-DD hh:mm where YYYY is year,
 * MM is month, DD is month day, hh is hour and mm is minutes
 * The function return -1 on error.
 */
time_t dg87_asciiToDate(char* pString)
{
	time_t td;
	struct tm dc;
	int year, month, day, hour, minute;
	char buffer[17];
	assert(pString != NULL);
	if (pString == NULL || strlen(pString) != 16)
		return -1;
	if (pString[4] != '-' || pString[7] != '-' || pString[10] != ' '
			|| pString[13] != ':')
		return -1;
	strncpy(buffer, pString,16);
	buffer[4] = '\0'; buffer[7] = '\0'; buffer[10] = '\0'; 
	buffer[13] = '\0'; buffer[16] = '\0';
	
	year = atoi(buffer); month = atoi(buffer+5); day = atoi(buffer+8); 
	hour = atoi(buffer+11); minute = atoi(buffer+14);

	dc.tm_year = year - 1900;
	dc.tm_mon = month - 1;
	dc.tm_mday = day;
	dc.tm_hour = hour;
	dc.tm_min = minute;
	dc.tm_sec = 0;
	dc.tm_isdst = -1;
	td = mktime(&dc);
	return td;
}
/******************************************************************************/
/* dg87_dateToAscii
 * return a string witch represent the time given in parameter. 
 * The format of the string is YYYY-MM-DD hh:mm where YYYY is year,
 * MM is month, DD is month day, hh is hour and mm is minutes
 * The function return -1 on error.
 * The string must be free with a call to dg87_free
 */
char* dg87_dateToAscii(time_t time)
{
	struct tm dc;
	char format[] = "%Y-%m-%d %H:%M";
	char* pString = dg87_malloc(sizeof(char) * 20, DG87_CHAR);
	if (pString == NULL) return NULL;
	dc = *localtime(&time);
	strftime(pString,20, format,&dc);
	return pString;
}
/******************************************************************************/
/* dg87_dateToAscii
 * return a string witch represents the boolean value. 
 * user don't have to free memory.
 */
char* dg87_boolToAscii(BOOL value)
{
	return (value == TRUE)?"true":"false";
}
/******************************************************************************/
/* dg87_asciiToBool
 * convert a string to a BOOL.
 * this function returns BOOLERROR if value is incorrect.
 */
BOOL dg87_asciiToBool(char* pValue)
{
	assert(pValue != NULL);
	if (pValue == NULL)
		return BOOLERROR;
	if (strcmp(pValue, "true") == 0)
		return TRUE;
	if (strcmp(pValue, "false") == 0)
		return FALSE;
	return BOOLERROR;
}
/******************************************************************************/
/* End of file */
