/* file server.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef SERVER_H
#define SERVER_H 1

#include <fcntl.h>
#include "interpret.h"
#include "global.h"
#include "connection.h"
#include "user.h"
#include "list.h"
#include "vector.h"

typedef struct dg87_server_struct{
	int ipPort;
	BOOL (*work_function)(struct dg87_server_struct*);
	int (*signal_function)(struct dg87_server_struct*, int signo);
	void* pData;
	BOOL quit;
	BOOL shutdown;
	dg87_interpret* pInterpret;
	dg87_list* pListConnection;
	dg87_list* pListMessage;
	dg87_vector* pUserVector;
	char* pProgramName;
	char* pProgramVersion;
	char* pPackageName;
	int fdPidFile; /* file descriptor of PID file */
	char* pPidFile; /* path of the lock file */
	char* pDiogenePassword; /* password of diogene user */
} dg87_server;

/*
 * dg87_server_new
 */
dg87_server* dg87_server_new(
	char* pPackageName,
	char* pProgramName,
	char* pProgramVersion,
	int ipPort,
	BOOL (*work_function)(dg87_server*),
	int (*signal_function)(dg87_server*, int signo),
	void* pData,
	struct dg87_interpret_cmd_struct *pCmdArray,
	char* pPidFile,
	char* pDiogenePassword /* password of diogene user */
	);

/*
 * dg87_server_run
 */
BOOL dg87_server_run(dg87_server* pServer);

/*
 * dg87_server_delete
 */
BOOL dg87_server_delete(dg87_server* pServer);

/*
 * dg87_server_postMessage
 */
BOOL dg87_server_postMessage(dg87_server* pServer, 
		char* pHost, int ipPort, char* pMessage);

/* 
 * dg87_server_reload
 * This function is used to change PID file or IP port or
 * a running server.
 * pPidFile is the string of the PID file. If pPidFile is NULL,
 * no PID file is used.
 */
BOOL dg87_server_reload(dg87_server* pServer, char* pPidFile, int ipPort,
	char* pDiogenePassword);

/*************************************************************************/
/*  functions for users                                                  */
/*************************************************************************/

/*
 * dg87_server_userAdd
 * create an user object, add this object in the UserVector 
 * of the server and return a pointer on this object.
 * Return NULL if the user object can't be create.
 */
dg87_user* dg87_server_userAdd(dg87_server* pServer, char* pName);

/*
 * dg87_server_userCount
 * return the number of users.
 * return -1 if pServer is NULL.
 */
int dg87_server_userCount(dg87_server* pServer);

/*
 * dg87_server_userGetByName
 * return a pointer on the user which has the name
 * given in parameter. Return NULL if it can't be find.
 */
dg87_user* dg87_server_userGetByName(dg87_server* pServer, char* pName);

/*
 * dg87_server_userGetByIndex
 * return a pointer on the user at the given position.
 */
dg87_user* dg87_server_userGetByIndex(dg87_server* pServer, int index);

/*
 * dg87_server_userDeleteByName
 * delete the user which has the name given in parameter.
 */
BOOL dg87_server_userDeleteByName(dg87_server* pServer,char* pName);

/*
 * dg87_server_userDeleteByIndex
 * delete the user at the given index.
 */
BOOL dg87_server_userDeleteByIndex(dg87_server* pServer,int index);

#endif /*SERVER_H*/

/* End of file */
