/* file storage.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef STORAGE_H 
#define  STORAGE_H 1

#include <time.h>
#include "global.h"

typedef enum{
	dg87_storage_mode_closed,
	dg87_storage_mode_read,
	dg87_storage_mode_write
} dg87_storage_mode;

typedef struct{
	char* pDataFile;
	char* pBackupFile;
	char* pSignature;
	int version;
	dg87_storage_mode mode;
	int fd; /* file descriptor */
} dg87_storage;

/*
 * dg87_storage_new
 */
dg87_storage *dg87_storage_new(char* pDataFile, char* pBackupFile,
		char* pSignature, int version);

/*
 * dg87_storage_delete
 */
BOOL dg87_storage_delete(dg87_storage* pStorage);

/*
 * dg87_storage_changeDataFile
 */
BOOL dg87_storage_changeDataFile(dg87_storage* pStorage, 
		char* pDataFile, char* pBackupFile);

/*
 * dg87_storage_open
 */
BOOL dg87_storage_open(dg87_storage* pStorage, dg87_storage_mode mode);

/*
 * dg87_storage_close
 */
BOOL dg87_storage_close(dg87_storage* pStorage);

/*
 * dg87_storage_writeString
 */
BOOL dg87_storage_writeString(dg87_storage* pStorage, char* pString);

/*
 * dg87_storage_readString
 */
BOOL dg87_storage_readString(dg87_storage* pStorage, char** ppString);

/*
 * dg87_storage_writeLong
 * 32 bits
 */
BOOL dg87_storage_writeLong(dg87_storage* pStorage, dg87long value);

/*
 * dg87_storage_readLong
 */
BOOL dg87_storage_readLong(dg87_storage* pStorage, dg87long *pValue);

/*
 * dg87_storage_writeUInt
 * 16 bits
 */
BOOL dg87_storage_writeUInt(dg87_storage* pStorage, dg87uint value);

/*
 * dg87_storage_readUInt
 */
BOOL dg87_storage_readUInt(dg87_storage* pStorage, dg87uint *pValue);

/*
 * dg87_storage_writeInt
 * 16 bits
 */
BOOL dg87_storage_writeInt(dg87_storage* pStorage, dg87int value);

/*
 * dg87_storage_readInt
 */
BOOL dg87_storage_readInt(dg87_storage* pStorage, dg87int *pValue);

/*
 * dg87_storage_writeByte
 * 32 bits
 */
BOOL dg87_storage_writeByte(dg87_storage* pStorage, dg87byte value);

/*
 * dg87_storage_readByte
 */
BOOL dg87_storage_readByte(dg87_storage* pStorage, dg87byte *pValue);


#endif /*DG87_STORAGE*/
/* End of file */
