/* file strbuf.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <assert.h>
#include "strbuf.h"

/******************************************************************************/
dg87_strbuf* dg87_strbuf_new(void)
{
	dg87_strbuf * pStrbuf;
	pStrbuf = dg87_malloc(sizeof(dg87_strbuf), DG87_STRBUF);
	if (pStrbuf == NULL) {
		dg87_err_setError(DG87_ERR_OUTOFMEMORY);
		return NULL;
	}
	pStrbuf->pBuffer = NULL;
	pStrbuf->bufferSize = 0;
	pStrbuf->size = 0;
	if (dg87_strbuf_cat(pStrbuf, "") == FALSE){
		dg87_strbuf_delete(pStrbuf);
		return NULL;
	}
	return pStrbuf;
}

/******************************************************************************/
BOOL dg87_strbuf_delete(dg87_strbuf* pStrbuf)
{	
	assert(pStrbuf != NULL);
	if (pStrbuf == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	if (pStrbuf->pBuffer != NULL)
		dg87_free(pStrbuf->pBuffer, DG87_CHAR);
	dg87_free(pStrbuf, DG87_STRBUF);

	return TRUE;
}
/******************************************************************************/
char* dg87_strbuf_getBuffer(dg87_strbuf* pStrbuf)
{
	char* pBuffer;
	assert(pStrbuf != NULL);
	if (pStrbuf == NULL){
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return NULL;
	}
	pBuffer = pStrbuf->pBuffer;
	pStrbuf->pBuffer = NULL;
	if (dg87_strbuf_cat(pStrbuf, "") == FALSE){
		return NULL;
	}
	return pBuffer;
}
/******************************************************************************/
BOOL dg87_strbuf_cat(dg87_strbuf* pStrbuf, char* pBuffer)
{
	assert(pStrbuf != NULL && pBuffer != NULL);
	if (pStrbuf == NULL || pBuffer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	dg87_strbuf_ncat(pStrbuf, pBuffer, strlen(pBuffer));
	return TRUE;
}
/******************************************************************************/
BOOL dg87_strbuf_ncat(dg87_strbuf* pStrbuf, char* pBuffer, int stringSize)
{
	int newSize;
	char* pNewBuffer;
	assert(pStrbuf != NULL && pBuffer != NULL);
	if (pStrbuf == NULL || pBuffer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* initialize the buffer */
	if (pStrbuf->pBuffer == NULL){
		newSize = DG87_STRBUF_BUFFERSIZE;
		pNewBuffer = dg87_malloc(newSize, DG87_CHAR);
		if (pNewBuffer == NULL){
			dg87_err_setError(DG87_ERR_OUTOFMEMORY);
			return FALSE;
		}
		pStrbuf->pBuffer = pNewBuffer; 
		pStrbuf->bufferSize = newSize; 
		pStrbuf->size = 0;
		pStrbuf->pBuffer[0] = 0;
	}
	/* if the buffer is to small to concat the string */
	while (pStrbuf->size+stringSize >= pStrbuf->bufferSize)
	{
		/* then expand the buffer */
		newSize = pStrbuf->bufferSize + DG87_STRBUF_BUFFERSIZE;
		pNewBuffer = realloc(pStrbuf->pBuffer, newSize);
		if (pNewBuffer == NULL){
			dg87_err_setError(DG87_ERR_OUTOFMEMORY);
			return FALSE;
		}
		pStrbuf->pBuffer = pNewBuffer;
		pStrbuf->bufferSize = newSize;
	}
	/* append the string to the buffer */
	memcpy(pStrbuf->pBuffer + pStrbuf->size,
			pBuffer, stringSize + 1);
	pStrbuf->size += stringSize;

	return TRUE;

}

/******************************************************************************/
BOOL dg87_strbuf_cpy(dg87_strbuf* pStrbuf, char* pBuffer)
{
	assert(pStrbuf != NULL && pBuffer != NULL);
	if (pStrbuf == NULL || pBuffer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* reinitialize string buffer */
	dg87_free(pStrbuf->pBuffer, DG87_CHAR);
	pStrbuf->pBuffer = NULL;
	pStrbuf->size = 0;
	pStrbuf->bufferSize = 0;
	/* concat new string to an empty string */
	dg87_strbuf_ncat(pStrbuf, pBuffer, strlen(pBuffer));
	return TRUE;
}

/******************************************************************************/
BOOL dg87_strbuf_ncpy(dg87_strbuf* pStrbuf, char* pBuffer, int size)
{
	assert(pStrbuf != NULL && pBuffer != NULL);
	if (pStrbuf == NULL || pBuffer == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}
	/* reinitialize string buffer */
	dg87_free(pStrbuf->pBuffer, DG87_CHAR);
	pStrbuf->pBuffer = NULL;
	pStrbuf->size = 0;
	pStrbuf->bufferSize = 0;
	/* concat new string to an empty string */
	dg87_strbuf_ncat(pStrbuf, pBuffer, size);
	return TRUE;
}
/******************************************************************************/
BOOL dg87_strbuf_printf(dg87_strbuf* pStrbuf, char* pFormat, ...)
{
	char buffer[60];
	char c[2];
	va_list ap;
	int ival;
	long lval;
	char* pString;
	
	assert(pStrbuf != NULL && pFormat != NULL);
	if (pStrbuf == NULL || pFormat == NULL) {
		dg87_err_setError(DG87_ERR_NULLPARAMETER);
		return FALSE;
	}

	va_start(ap,pFormat); /* ap points on the first optional argument */
	while (*pFormat)
	{
		if (*pFormat == '\n'){
			/* print CR LF */
			dg87_strbuf_cat(pStrbuf,"\n\r");
		} else if (*pFormat != '%'){
			/* not a format character (%), print it */
			c[0] = *pFormat; c[1] = 0;
			dg87_strbuf_cat(pStrbuf, c);
		}
		/* interpret character after % */
		else switch(*++pFormat)
		{
			case 's': /* replace '%s' by a string */
				pString = va_arg(ap, char*);
				if (pString != NULL)
					dg87_strbuf_cat( pStrbuf, pString);
				break;
			case 'c': /* replace '%c' by a character */
				ival = va_arg(ap, int);
				sprintf(buffer,"%c", ival);
				dg87_strbuf_cat( pStrbuf, buffer);
				break;
			case 'i': /* replace '%i' by a integer */
				ival = va_arg(ap, int);
				sprintf(buffer,"%i", ival);
				dg87_strbuf_cat( pStrbuf, buffer);
				break;
			case 'l': /* replace '%l' by a long */
				lval = va_arg(ap, long);
				sprintf(buffer,"%li", lval);
				dg87_strbuf_cat( pStrbuf, buffer);
				break;
			default:
				c[0] = *pFormat; c[1] = 0;
				dg87_strbuf_cat(pStrbuf, c);
		} /* end case */
		pFormat++; /* next character */

	} /* end while */
	va_end(ap);	
	return TRUE;
}

/******************************************************************************/

/* End of file */
