/* file user.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef USER_H
#define USER_H

#include <config.h>
#include "global.h"
#include "storage.h"

/*
 * dg87_user object
 */
typedef struct dg87_user_struct
{
	char name[DG87_USER_NAME_LENGTH];
	char password[DG87_USER_PASSWORD_LENGTH];
	int connectionCount;  /* number of active connections */
	dg87_command_privilege privileges[DG87_COMMAND_FAM_MAX];
	BOOL systemAccount;
} dg87_user;

/*
 * dg87_user_new
 */
dg87_user* dg87_user_new(char* pName);

/*
 * dg87_user_delete
 */
BOOL dg87_user_delete(dg87_user* pUser);

/*
 * dg87_user_load
 */
BOOL dg87_user_load(dg87_user* pUser, dg87_storage* pStorage);

/*
 * dg87_user_save
 */
BOOL dg87_user_save(dg87_user* pUser, dg87_storage* pStorage);


/*
 * dg87_user_setPassword
 */
BOOL dg87_user_setPassword(dg87_user* pUser, char* pPassword);

/*
 * dg87_user_verifyPassword
 */
BOOL dg87_user_verifyPassword(dg87_user* pUser, char* pPassword);

/*
 * dg87_user_updateConnexionCount
 */
BOOL dg87_user_updateConnectionCount(dg87_user* pUser, int delta);

/*
 * dg87_user_setPrivilege
 */
BOOL dg87_user_setPrivilege(dg87_user* pUser, 
		dg87_command_family family,
		dg87_command_privilege privilege);

/*
 * dg87_user_setAdministrator
 */
BOOL dg87_user_setAdministrator(dg87_user* pUser);

/*
 * dg87_user_verifyPrivilege
 */
BOOL dg87_user_verifyPrivilege(dg87_user* pUser,
		dg87_command_family family,
		dg87_command_privilege privilege);

#endif /*USER_H*/
/* End of file */
