/* file vector.h */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#ifndef VECTOR_H
#define VECTOR_H 1

#include "global.h"
#include "element.h"

typedef struct{
	dg87_element* pFirst;
	dg87_element* pLast;
	dg87uint count;
} dg87_vector;

/*
 * dg87_vector_new
 */
dg87_vector* dg87_vector_new(void);

/*
 * dg87_vector_delete
 */
BOOL dg87_vector_delete(dg87_vector *pVector);

/*
 * dg87_vector_addObject
 */
BOOL dg87_vector_addObject(dg87_vector *pVector, void *pObject, char *pKey);

/*
 * dg87_vector_removeObject
 * This function remove the object from the vector.
 * It return FALSE if the object can't be find.
 */
BOOL dg87_vector_removeObject(dg87_vector *pVector, void* pObject);

/*
 * dg87_vector_removeObjectByKey
 * remove the object which correspond to the key given in parameter.
 * This function return a pointer on the object which was remove.
 * It return NULL if no object has a key witch match the given key.
 */
void* dg87_vector_removeObjectByKey(dg87_vector *pVector, char* pKey);

/*
 * dg87_vector_removeObjectByIndex
 * remove the object at the given index in parameter.
 * This function return a pointer on the object which was remove.
 */
void* dg87_vector_removeObjectByIndex(dg87_vector *pVector, dg87uint index);
/*
 * dg87_vector_count
 */
dg87uint dg87_vector_count(dg87_vector *pVector);

/*
 * dg87_vecotr_getObjectByIndex
 */
void* dg87_vector_getObjectByIndex(dg87_vector *pVector, dg87uint index);

/*
 * dg87_vector_getObjectByKey
 */
void* dg87_vector_getObjectByKey(dg87_vector *pVector, char* key);

/*
 * dg87_vector_getElementByIndex
 */
dg87_element* dg87_vector_getElementByIndex(dg87_vector *pVector, dg87uint index);

/*
 * dg87_vector_getElementByKey
 */
dg87_element* dg87_vector_getElementByKey(dg87_vector *pVector, char* key);

/*
 * dg87_vector_dump
 */
void dg87_vector_dump(dg87_vector *pVector);

/*
 * dg87_vector_findElement
 * find the element associate with the given Object.
 * Return NULL if Object is not in the list.
 */
dg87_element* dg87_vector_findElement(dg87_vector* pVector, void* pObject);

#ifdef DEBUG
/*
 * dg87_vector_ckeck
 */
int dg87_vector_check(dg87_vector* pVector, int (*print)(char*));
#endif /*DEBUG*/

#endif
/* End of file */
