/* file cmdday.c */
/*
 *      Diogene87
 *	Copyright (C) 2003-2004 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */

#include "cmdday.h"
#include "util.h"
#include "context.h"

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <fcntl.h>

/******************************************************************************/
/* xdv
 * eXploitation Day Validate
 * Usage : xdv DayName
 */
int cmd_xdv(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	dg87_day* pDay;
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xdv DayName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;
	/* validate the day */
	if (dg87_day_validate(pDay) == FALSE)
		return 1;
	return 0;
}
/******************************************************************************/
/* xdk
 * eXploitation Day Kill
 * Usage : xdk DayName
 */
int cmd_xdk(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 1){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xdk DayName");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* delete the day */
	dg87_err_clearError();
	if (dg87_context_dayDeleteByName(pContext, 
				pCommand->argv[0]) == FALSE){
		dg87_connection_printf(pConnection,
				"Can't delete the day '%s' ! %s\n",
				pCommand->argv[0],
				dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* xda
 * eXploitation Day Add
 * Usage : xda DayName
 */
int cmd_xda(dg87_connection* pConnection)
{
	dg87_day* pDay;
	dg87_context* pContext;
	dg87_command* pCommand;
	time_t startTime = 0;
	if (pConnection == NULL) return 1;
	pContext = pConnection->pServer->pData;
	pCommand = pConnection->pCommand;
	if (pCommand->argc != 2) {
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xda DayName Date");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	/* get start time from parameter */
	startTime = dg87_asciiToDate(pCommand->argv[1]); 
	if (startTime == -1){
		dg87_connection_printf(pConnection, 
			"Can't create day '%s', because date '%s' "\
			"is not valid !\n",
			pCommand->argv[0], pCommand->argv[1]);
		return 1;
	}

	/* add the day */
	dg87_err_clearError();
	pDay = dg87_context_dayAdd(pContext, pCommand->argv[0], startTime);
	if (pDay == NULL){
		dg87_connection_printf(pConnection,
				"Can't add a day '%s' ! %s\n",
				pCommand->argv[0], dg87_err_getErrorStr());
		return 1;
	}
	return 0;
}
/******************************************************************************/
/* cmd_xdc
 * eXploitation Day Count
 * return the number of Days.
 * Usage : xdc 
 */
int cmd_xdc(dg87_connection* pConnection)
{
	dg87_context* pContext;
	dg87_command* pCommand;
	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;
	if (pCommand->argc != 0){
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "xdc");
		return 1;
	}
	if (pCommand->optc != 0){
		dg87_connection_printf(pConnection, 
			"Unknown option '%c' for command '%s'\n",
			pCommand->optv[0], pCommand->pFunctionName);
		return 1;
	}
	dg87_connection_printf(pConnection,"%i\n", 
			pContext->pDayVector->count);
	return 0;
}
/******************************************************************************/
/* cmd_xdl
 * eXploitation Day List
 * Usage : xdl [options] 
 * options
 * -Svalue : set the separator use between to fields
 * -Bvalue : set the begin of the line
 * -Evalue : set the end of the line 
 * -Inumber : index of the first task to display (first index is 0)
 * -Nnumber : number of task to display (no limit if -N < 1)
 */
int cmd_xdl(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:I:N:";*/
	int optch,iopt;
	char* optarg;
	int start = 0; /* index of the first task to display */
	int number = 0;/* number of TASKs to display */
	int count = 0; /* number of TASK actually display */
	dg87_element* pElement = NULL;
	dg87_day* pDay;
	dg87_context* pContext;
	dg87_command* pCommand;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;
	pContext = pConnection->pServer->pData;

	/* control number of arguments */
	if (pCommand->argc != 0){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n", "xdl [options]");
		return 1;
	}

	/* default separator */
	pSep = strdup("\t");
	pBeg = strdup("");
	pEnd = strdup("");

	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			case 'I': /* index of the first task to display */
				start = atoi(optarg);
				break;
			case 'N': /* number of task to display */
				number = atoi(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}

	/* find the first Task to display */
	pElement = NULL;
	if (start > 0)
		pElement = dg87_vector_getElementByIndex(
			pContext->pDayVector, start);
	else
		pElement = pContext->pDayVector->pFirst;

	/* create the list */
	count = 0;
	while (pElement != NULL){
		/* reach the limit of task to display*/
		count++;
		if (number > 0 && count > number)
			break;
		pDay = pElement->pObject;
		/* print day's name */
		dg87_connection_printf(pConnection, "%s%s",
			pBeg, pDay->name);
		dg87_connection_printf(pConnection, "%s\n", pEnd);
		pElement = pElement->pNext;
	}
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;	
}
/******************************************************************************/
/* xdi
 * eXploitation Day Information
 * get information for a day
 * Usage : xdi DayName
 */
int cmd_xdi(dg87_connection* pConnection)
{
	char *pSep, *pBeg, *pEnd; /* separator, begin, end */
	/*static char optstring[] = "B:S:E:";*/
	int optch,iopt;
	char* optarg;
	dg87_day *pDay;
	dg87_command* pCommand;
	char* pString;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 1){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xdi [options] DayName");
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* default separator */
	pSep = strdup(":\t");
	pBeg = strdup("");
	pEnd = strdup("");
	
	/* interpret options */
	for (iopt = 0; iopt < pCommand->optc; iopt++){
		optch = pCommand->optv[iopt];
		optarg = pCommand->optargv[iopt];
		switch (optch){
			case 'S': /* separator */
				free(pSep); pSep = strdup(optarg);
				break;
			case 'B': /* begin separator */
				free(pBeg); pBeg = strdup(optarg);
				break;
			case 'E': /* end separator */
				free(pEnd); pEnd = strdup(optarg);
				break;
			default:  /* unknown option */
				dg87_connection_printf(pConnection, 
						"Unknown option");
				free(pSep); free(pBeg); free(pEnd);
				return 1;
		}
	}
	
	/* Task name */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Day's Name", pSep, pDay->name, pEnd);
	/* start time */
	if ((pString = dg87_dateToAscii(pDay->startTime)) != NULL){
		dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
			pBeg, "Start Time", pSep, pString, pEnd);
		dg87_free(pString, DG87_CHAR);
	}
	/* enabled property */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Enabled", pSep, 
		dg87_boolToAscii(pDay->enabled), pEnd);
	/* locked property */
	dg87_connection_printf(pConnection,"%s%s%s%s%s\n",
		pBeg, "Locked", pSep, 
		dg87_boolToAscii(pDay->locked), pEnd);
	
	/* free memory */
	free(pSep); free(pBeg); free(pEnd);
	return 0;
}

/******************************************************************************/
/* xdse
 * eXploitation Day Set Enabled 
 * Usage : xdse DayName booleanValue
 */
int cmd_xdse(dg87_connection* pConnection)
{
	dg87_day *pDay;
	dg87_command* pCommand;
	BOOL value;

	if (pConnection == NULL) return 1;
	pCommand = pConnection->pCommand;

	if (pCommand->argc != 2){ 
		dg87_connection_printf(pConnection,
			"Incorrect call.\nUsage : %s\n",
			"xdse DayName BooleanValue");
		return 1;
	}
	/* find the day */
	pDay = dg87_util_findDay(pConnection, pCommand->argv[0]);
	if (pDay == NULL) return 1;

	/* interpet enable argument */
	if ((value = dg87_asciiToBool(pCommand->argv[1])) == BOOLERROR){
		dg87_connection_printf(pConnection,
			"Incorrect value '%s', use true or false !\n",
			pCommand->argv[1]);
		return 1;
	}
	if (dg87_day_setEnabled(pDay, value) == FALSE){
		dg87_connection_printf(pConnection,
			"Can't change Enabled property for day '%s' !\n",
			pDay->name);
		return 1;
	}
	return 0;
}

/******************************************************************************/
/* End of file */
